!{\src2tex{textfont=tt}}
!!****p* creation_chi0/assemblychi0
!! NAME
!! assemblychi0
!!
!! FUNCTION
!! assembly chi0 from rhotw(G) and den(w)
!!  chi0(G,G',io) = chi0(G,G',io) + (rhotwg(G) * rhotwg*(G')) * den(io)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_chi0, creation_X, creation_chi0drude
!!
!! CHILDREN
!! cgerc
!!
!! SOURCE

  subroutine assemblychi0(npwmat,nomega,chi0,rhotwg,den)

  use ggwdef
  use dpdef
  
  implicit none
  
  integer npwmat, nomega
  complex chi0(npwmat,npwmat,nomega) 
  complex rhotwg(npwmat)
  complex (kind(0.0d0)) :: den(nomega)
  integer io
  complex sden

  do io = 1, nomega
    sden = den(io)
    call cgerc(npwmat,npwmat,sden,rhotwg,1,rhotwg,1,chi0(:,:,io),npwmat)
  enddo
  end
!! NOTES
!! it used to be calculated directly, without making use of the blas routine
!! cgerc, because there were bugs on the NEC blas (no vectorization)
!!  complex rhotwgs(npwmat)
!!  integer ig,igp
!!  rhotwgs(:) = conjg(rhotwg(:))
!!  do ig = 1, npwmat
!!    do igp = 1, npwmat
!!      do io = 1, nomega
!!        chi0(ig,igp,io) = chi0(ig,igp,io)+rhotwg(ig) * rhotwgs(igp) * den(io)
!!      enddo
!!    enddo
!!  enddo
!!***


!{\src2tex{textfont=tt}}
!!****p* creation_chi0/assemblywings
!! NAME
!! assemblywings
!!
!! FUNCTION
!! assembly the upper and left wings from rhotw(G) and den(w)
!!  uwing(q,G',io) = uwing(q,G',io) + (rhotwg(q) * rhotwg*(G')) * den(io)
!!  lwing(q,G ,io) = lwing(q,G ,io) + (rhotwg(G) * rhotwg*(q )) * den(io)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_chi0
!!
!! SOURCE
! ==================================================================
  subroutine assemblywings(npwmat,nomega,isppol,res,iq,rhotwg,den)
! ==================================================================
! ------------------------------------------------------------------
  use ggwdef
  use dpdef
  
  implicit none
  
  integer npwmat, nomega
  type(response) res
  complex rhotwg(npwmat), rhotwgs(npwmat)
  complex rhotwg1, rhotwgs1
  complex (kind(0.0d0)) :: den(nomega)
  integer io,iq
  integer isppol

! calculate rhotwg*(G)

  rhotwgs(:) = conjg(rhotwg(:))
  rhotwg1 = rhotwg(1)
  rhotwgs1 = rhotwgs(1)

   do io = 1, nomega
    res%lwing(:,io,iq,isppol) = res%lwing(:,io,iq,isppol) + rhotwg(:) * rhotwgs1 * den(io)
    res%uwing(:,io,iq,isppol) = res%uwing(:,io,iq,isppol) + rhotwg1 * rhotwgs(:) * den(io)
   enddo

  return
  end
!!***

