!{\src2tex{textfont=tt}}
!!****p* read_structure/bcast_input_parameters
!! NAME
!! bcast_input_parameters
!!
!! FUNCTION
!! MPI broadcast of input parameters
!! 
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! read_structure
!!
!! SOURCE

subroutine bcast_input_parameters(iparam)

use dpdef
use ggwdef
use verbose

implicit none

type(dpparam) iparam

#ifdef MPI
include 'mpif.h'
integer ik,ib,isppol
integer ierr,rang,nprocs
call mpi_comm_size(mpi_comm_world,nprocs,ierr)
call mpi_comm_rank(mpi_comm_world,rang,ierr)

!TODO most of these communications should be suppressed

call mpi_barrier(mpi_comm_world,ierr) 
call mpi_bcast(iverbose,1,mpi_integer,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%nomega,1,mpi_integer,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%domega,1,mpi_real,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%omegai,1,mpi_real,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%omegae,1,mpi_real,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%broad,1,mpi_real,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%nopr,1,mpi_integer,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%nop,1,mpi_integer,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%nkibz,1,mpi_integer,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%npwvec,1,mpi_integer,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%npwmat,1,mpi_integer,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%nbnds,1,mpi_integer,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%nbndqps,1,mpi_integer,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%lomo,1,mpi_integer,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%npwwfn,1,mpi_integer,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%td,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%vkb,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%soenergy,1,mpi_real,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%stretchvalence,1,mpi_real,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%stretchconduction,1,mpi_real,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%alphadiv,1,mpi_real,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%beta,1,mpi_real,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%KP_THEORY,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%JDOS,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%FXCMT,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%FXCALPHA,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%FXCALPHABETA,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%FXCEPS,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%FXCEPSDIAGO,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%FXCVBAR,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%NOFXC,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%LDA,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%qps,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%qpsenergy,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%TESTELECTRON,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%hilbert,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%nonlinear,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%nsppol,1,mpi_integer,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%savememory,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%doublegrid,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%antieps,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%testrun,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%matrixel,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%matelanalysis,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%exc,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%exciton,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%nqibz,1,mpi_integer,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%haydock,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%niter,1,mpi_integer,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%encutoff,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%ircut,1,mpi_real,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%uvcut,1,mpi_real,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%stripecut,1,mpi_real,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%RESONANT,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%COUPLING,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%COUPLPERT,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%WDIAG,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%WFULL,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%GWTERM,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%EXCHANGETERM,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%COULOMBTERM,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%dielconstscreen,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%renormdielconst,1,mpi_real,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%coulcutoff,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%rlcut,1,mpi_real,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%fulldiago,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%blockpert,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%pert,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%deltadeg,1,mpi_real,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%maxblocksize,1,mpi_integer,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%order,1,mpi_integer,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%lifetime,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%ninvq,1,mpi_integer,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%nopq,1,mpi_integer,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%pes,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%ephoton,1,mpi_real,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%ONLYDRUDE,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%DRUDE,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%testrun,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%nqpol,1,mpi_integer,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%kBT,1,mpi_real,0,mpi_comm_world,ierr)
call mpi_bcast(iparam%oldqplda,1,mpi_logical,0,mpi_comm_world,ierr)
#endif

end
!!***


!{\src2tex{textfont=tt}}
!!****p* read_structure/bcast_kb_parameters
!! NAME
!! bcast_kb_parameters
!!
!! FUNCTION
!! MPI broadcast of kb parameters
!! 
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! read_structure
!!
!! SOURCE
subroutine bcast_kb_parameters(kb)

use dpdef
use ggwdef
use verbose

implicit none
type(kb_potential) kb

#ifdef MPI

include 'mpif.h'
integer ik,ib,isppol
integer ierr,rang,nprocs
call mpi_comm_size(mpi_comm_world,nprocs,ierr)
call mpi_comm_rank(mpi_comm_world,rang,ierr)

call mpi_barrier(mpi_comm_world,ierr)
call mpi_bcast(kb%exists,1,mpi_logical,0,mpi_comm_world,ierr)
call mpi_bcast(kb%natom,1,mpi_integer,0,mpi_comm_world,ierr)
call mpi_bcast(kb%mpsang,1,mpi_integer,0,mpi_comm_world,ierr)
call mpi_bcast(kb%ntype,1,mpi_integer,0,mpi_comm_world,ierr)

#endif

end
!!***

!{\src2tex{textfont=tt}}
!!****p* read_structure/bcast_kss_data
!! NAME
!! bcast_kss_data
!!
!! FUNCTION
!! MPI broadcast of input parameters
!! 
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! read_structure
!!
!! SOURCE

subroutine bcast_kss_data(iparam,latt,chemstr,kb,symstr,bzstruct,elecstr,trspace)

use dpdef
use ggwdef
use verbose

implicit none

type(dpparam) iparam
type(chemistry) chemstr
type(symmetry) symstr
type(bzstructure) bzstruct 
type(electronic_structure) elecstr
type(transposed_space) trspace
type(lattstruct) latt
type(kb_potential) kb

#ifdef MPI

include 'mpif.h'
integer ik,ib,isppol
integer ierr,rang,nprocs
call mpi_comm_size(mpi_comm_world,nprocs,ierr)
call mpi_comm_rank(mpi_comm_world,rang,ierr)

 call mpi_barrier(mpi_comm_world,ierr)
 call mpi_bcast(latt%a1,3,mpi_real,0,mpi_comm_world,ierr)
 call mpi_bcast(latt%a2,3,mpi_real,0,mpi_comm_world,ierr)
 call mpi_bcast(latt%a3,3,mpi_real,0,mpi_comm_world,ierr)
 call mpi_bcast(chemstr%typeat,kb%natom,mpi_integer,0,mpi_comm_world,ierr)
 call mpi_bcast(chemstr%xred,kb%natom*3,mpi_double_precision,0,mpi_comm_world,ierr)
 if (.not.iparam%JDOS .and. kb%inclvkb) then
  call mpi_bcast(kb%vkbsign,kb%mpsang*kb%ntype,mpi_real,0,mpi_comm_world,ierr)
  call mpi_bcast(kb%vkb,iparam%npwvec*kb%ntype*kb%mpsang*iparam%nkibz,mpi_real,0,mpi_comm_world,ierr)
  call mpi_bcast(kb%vkbd,iparam%npwvec*kb%ntype*kb%mpsang*iparam%nkibz,mpi_real,0,mpi_comm_world,ierr)
 endif

 call mpi_bcast(trspace%gvec,3*iparam%npwvec,mpi_integer,0,mpi_comm_world,ierr)

if(.not.iparam%JDOS) then
!call mpi_bcast(elecstr%wfg,iparam%npwvec*iparam%nbnds*iparam%nkibz*iparam%nsppol,mpi_complex,0,mpi_comm_world,ierr)
 do isppol=1,iparam%nsppol
 do ik =1,iparam%nkibz
  do ib  = iparam%nbndmin, iparam%nbndmax
   call mpi_bcast(elecstr%wfg(:,ib,ik,isppol),iparam%npwvec,mpi_complex,0,mpi_comm_world,ierr)
  enddo
 enddo
 enddo
endif

call mpi_bcast(elecstr%nsppol,1,mpi_integer,0,mpi_comm_world,ierr)
call mpi_bcast(elecstr%energy,iparam%nkibz*iparam%nbndmax*iparam%nsppol,mpi_double_precision,0,mpi_comm_world,ierr) 
call mpi_bcast(symstr%op,3*3*iparam%nopr,mpi_real,0,mpi_comm_world,ierr)
call mpi_bcast(symstr%symafm,iparam%nopr,mpi_integer,0,mpi_comm_world,ierr)
call mpi_bcast(bzstruct%kibz,3*iparam%nkibz,mpi_real,0,mpi_comm_world,ierr)

#endif

end
!!***
