!{\src2tex{textfont=tt}}
!!****p* wreps/calc_eps0
!! NAME
!! calc_eps0
!!
!! FUNCTION
!! Calculates the electronic eps(0). 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! wreps
!!
!! SOURCE
 subroutine calc_eps0(iparam,res)
 use dpdef
 use ggwdef
 use verbose
 implicit none
 type(dpparam) iparam
 type(response) res

 integer :: nnq, iq
 real, allocatable :: epsmlf0(:), epsmnlf0(:)
 
 if(iverbose>2)call uverbose('b','calc_eps0')
 
 if(iverbose > 0) then
  headline='Evaluation of the Dielectric Constant'
  call print_headline()
 endif

 if(iparam%nqpol == 1) then
  nnq = 1
 else
  nnq = 8        
 endif
 allocate(epsmlf0(nnq))
 allocate(epsmnlf0(nnq))
 
 do iq = 1, nnq
  epsmlf0(iq) = real(res%epsmlf(1,iq))
  epsmnlf0(iq) = real(res%epsmnlf(1,iq))
 enddo      
 do iq = 1, iparam%nqpol
  write(uout,*) 'q-direction ', iq
  write(uout,1975) epsmlf0(iq), epsmnlf0(iq)
1975       format(' dielectric constant epsilon0 = ',f8.4,/, &
                  ' dielectric constant without local fields = ',f8.4)
  if(abs(aimag(iparam%omega(1))) > 0.001) write(uout,1980) aimag(iparam%omega(1))*HARTREE
1980       format(' warning: broadening = ',f5.2,' [eV]')
  write(uout,*)  
 enddo       
 if(nnq>1) then
  write(uout,*) 'q average of reciprocal coordinates directions'
  write(uout,1975) epsmlf0(7), epsmnlf0(7)
  write(uout,*)  
  write(uout,*) 'q average of cartesian directions'
  write(uout,1975) epsmlf0(8), epsmnlf0(8)
  write(uout,*)  
 endif      


 deallocate(epsmlf0,epsmnlf0)

 if(iverbose > 0) then
  call print_footline()
 endif
 if(iverbose>2)call uverbose('e','calc_eps0')

 end subroutine calc_eps0
!!***
