!{\src2tex{textfont=tt}}
!!****p* main/pclock
!! NAME
!! timing point function
!!
!! FUNCTION
!! print the timing point number
!! itimpt: timing point number
!! if itimpt = 0 start the clock
!! else print the cpu time elapsed since clock started
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! header, read_structure, setmesh, identk, fftwfn, cden, prep_rhotw, creation_chi0, creation_chi0drude, 
!! creation_kernelmt, checks, solve_exciton, prep_rhotw
!!
!! CHILDREN
!! get_time
!!
!! SOURCE 

 subroutine pclock(itimpt)

  use dpdef
  implicit none

  integer itimpt
  real, save ::  cpustart=0.
  real, save ::  wallstart=0.
  real cputime, walltime
  character cdate*8, ctime*10, czone*5
  integer datetimezone(8)
  external get_time

 if (itimpt.eq.0) then !unit must be explicitly 6 here (uout not yet known)
   call date_and_time(cdate,ctime,czone,datetimezone)
   write(uout,20) datetimezone(1:3), datetimezone(5:7)
20  format(' clock started    ',i4,'-',i2,'-',i2,'   ',i2,':',i2,':',i2)
   call get_time(cpustart,wallstart) ! get start time
!   write(uout,*)
!   write(uout,*)'   ******* time point *************'
!   write(uout,50) itimpt, 0., 0.
!   write(uout,*)'   ********************************'
 else
   call get_time(cputime,walltime)   ! get current time
   write(uout,*)
   write(uout,*)'   ******* time point *************'
   write(uout,50) itimpt, cputime-cpustart, walltime-wallstart
   write(uout,*)'   ********************************'
 endif

50 format ('    * timing point number ',i4/       &
           '    * cpu time  = ',f10.2,' seconds'/ &
           '    * real time = ',f10.2,' seconds')
 write(uout,*)
 
 
 return
 end
!!***


!{\src2tex{textfont=tt}}
!!****p* pclock/get_time
!! NAME
!! get timing point
!!
!! FUNCTION
!! returns current cpu- and walltime in seconds (real)
!! 
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! pclock
!!
!! SOURCE 

subroutine get_time(cputime, walltime)
  implicit none
  double precision, intent(OUT):: cputime, walltime
  double precision,    save    :: cpuclock=0., wallclock=0.
  integer, save    :: occount=0,  owcount=0
  integer ccount,crate,cmax
  integer wcount,wrate,wmax

  call cpu_clock(ccount,crate,cmax)
  if(ccount-occount >= 0) then
    cpuclock = cpuclock + float((ccount-occount))/crate
  else
    cpuclock = cpuclock + float((ccount-occount+cmax))/crate
  endif
  occount = ccount

  call system_clock(wcount,wrate,wmax)
  if(wcount-owcount >= 0) then
    wallclock = wallclock + dfloat((wcount-owcount))/wrate
  else
    wallclock = wallclock + dfloat((wcount-owcount+wmax))/wrate
  endif
  owcount = wcount

  cputime=cpuclock
  walltime=wallclock
end subroutine get_time
!!***




!{\src2tex{textfont=tt}}
!!****p* main/pclock2
!! NAME
!! timing point function v2
!!
!! FUNCTION
!! print the timing point number
!! itimpt: timing point number
!! if itimpt = 0 start the clock
!! else print the cpu time elapsed since clock started
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! CHILDREN
!! giorni 
!!
!! SOURCE 
subroutine pclock2(itimpt)
use dpdef
implicit none
integer :: itimpt
double precision :: timestamp
double precision, save :: start = 0.
integer :: YY, MM, DD, HH, m, s

call get_time2(timestamp,YY, MM, DD, HH, m, s)

if (itimpt==0) then
 write(uout,20) YY,MM,DD,HH,m,s
20  format(' clock started    ',i4,'-',i2,'-',i2,'   ',i2,':',i2,':',i2)
 start = timestamp
else if(itimpt==1000) then
  write(uout,30) YY,MM,DD,HH,m,s
30  format(' clock stopped    ',i4,'-',i2,'-',i2,'   ',i2,':',i2,':',i2)
else
   write(uout,*)
   write(uout,*)'   ******* time point *************'
   write(uout,50) itimpt, timestamp-start
   write(uout,*)'   ********************************'
endif

50 format ('    * timing point number ',i5/       &
           '    * elapsed time = ',f10.2,' seconds')
 write(uout,*)


end subroutine pclock2
!!***

subroutine get_time2(timestamp,YY,MM,DD,HH,m,s)

integer :: values(8)
integer :: YY, MM, DD, HH, m, s, ms
double precision :: correzione_bisestile
double precision :: timestamp
double precision, external :: giorni


call date_and_time(values=values)

YY = values(1)
MM = values(2)
DD = values(3)
HH = values(5)
m = values(6)
s = values(7)
ms = values(8)

! years since 1970 x nsec in a (365 days) year
timestamp=float(YY-1970)*31536000.

! add the days of leap year
correzione_bisestile=float((YY-1969)/4) * 3600. * 24.
timestamp = timestamp + correzione_bisestile

! add the days since the beginning of the year
timestamp = timestamp + giorni(MM,DD)*3600*24

! add the leap year correction if it is the case
correzione_bisestile = 0.
if (MM .gt. 2 .and. mod(YY,4).eq.0) then
  correzione_bisestile = float(3600*24)
endif
timestamp = timestamp + correzione_bisestile

! add the seconds since the beginning of the day
timestamp = timestamp + float(3600*HH + 60*m + s)

! add the milliseconds
timestamp = timestamp + float(ms)/1000.

end subroutine get_time2

!{\src2tex{textfont=tt}}
!!****p* timestamp/giorni
!! NAME
!! giorni
!!
!! FUNCTION
!! find the number of days since the beginning of the year 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! timestamp
!!
!! SOURCE
double precision function giorni(MM,DD)
  integer MM,DD

  select case (MM)
   case (1)
     giorni = 0. + float(DD-1)
   case (2)
     giorni = 31. + float(DD-1)
   case (3)
     giorni = 59. + float(DD-1)
   case (4)
     giorni = 90. + float(DD-1)
   case (5)
     giorni = 120. + float(DD-1)
   case (6)
     giorni = 151. + float(DD-1)
   case (7)
     giorni = 181. + float(DD-1)
   case (8)
     giorni = 212. + float(DD-1)
   case (9)
     giorni = 243. + float(DD-1)
   case (10)
     giorni = 273. + float(DD-1)
   case (11)
     giorni = 304. + float(DD-1)
   case (12)
     giorni = 334. + float(DD-1)
  end select
end function giorni
!!***

