!{\src2tex{textfont=tt}}
!!****p* main/createVc
!! NAME
!! createVc
!!
!! FUNCTION
!! set up table of length of |q+G| for coulombian potential
!! and for coulombian potential Vc
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! CHILDREN
!! vlngth
!!
!! SOURCE

  subroutine createVc(iparam,latt,trspace,coul)

  use ggwdef
  use verbose
  use dpdef
  implicit none
  type(dpparam) iparam
  type(coulomb) coul
  type(transposed_space) trspace
  type(lattstruct) latt

  integer iq, ig, istat 
  real, external :: vlngth
  real gpq(3)
  

  if(iverbose > 2) call uverbose('b','createVc')


  allocate(coul%Vc(iparam%npwmat,iparam%nqpol),stat=istat)
  if(istat.ne.0) stop 'out of memory, qplus'

! calculates qplusG(G) = |q+G| table for coulombian potential

  allocate(coul%qplusG(iparam%npwmat,iparam%nqpol),stat=istat)
  if(istat.ne.0) stop 'out of memory, qplus'

  if (iverbose > 0) then
   headline= 'Calculating the Coulomb potential'
   call print_headline()
  endif     

  do iq = 1, iparam%nqpol
   if(all(abs(iparam%qpol(:,iq))<1.e-3)) then
    coul%qplusG(1,iq) = vlngth(iparam%qpol(1,iq),latt%b1,latt%b2,latt%b3)
    do ig = 2, iparam%npwmat
      gpq(1) = trspace%gvec(1,ig)
      gpq(2) = trspace%gvec(2,ig)
      gpq(3) = trspace%gvec(3,ig)
      coul%qplusG(ig,iq) = vlngth(gpq,latt%b1,latt%b2,latt%b3)
    enddo
   else
    do ig = 1, iparam%npwmat
      gpq(1) = trspace%gvec(1,ig) + iparam%qpol(1,iq)
      gpq(2) = trspace%gvec(2,ig) + iparam%qpol(2,iq)
      gpq(3) = trspace%gvec(3,ig) + iparam%qpol(3,iq)
      coul%qplusG(ig,iq) = vlngth(gpq,latt%b1,latt%b2,latt%b3)
    enddo
   endif
  enddo ! iq
  
  coul%Vc(:,:) = 4 * PI / (coul%qplusG(:,:)**2)


  if (.not. (iparam%FXCMT .or.  iparam%exciton)) deallocate(coul%qplusG)

  if (iverbose > 0) call print_footline()

  if(iverbose > 2) call uverbose('e','createVc')

  
 if(.not.iparam%FXCMT) deallocate(trspace%gvec)

  end
!!***

