!{\src2tex{textfont=tt}}
!!****p* creation_chi0/create_den
!! NAME
!! create_den
!!
!! FUNCTION
!! Create the denominator of chi0 for this (it) transition
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_chi0
!!
!! SOURCE
subroutine create_den(iparam,it,trans,tablekp,elecstr,elecstrp,Wd,iet,den)

 use ggwdef
 use dpdef

 implicit none
 
 type(dpparam) iparam
 type(transition) trans(iparam%nt)
 type(polarizationq) tablekp
 type(electronic_structure) elecstr,elecstrp
 integer ikbz, ikibz, iinv, ibv, ibc, it, ikpibz, isppol, iet
 complex (kind(0.0d0)) :: den(iparam%nomega),egwdiff
 double precision factocc, Wd, sumlifetime

 ikbz = trans(it)%k
 ibv = trans(it)%v
 ibc = trans(it)%c
 ikibz = tablekp%ikibz(ikbz)
 ikpibz=tablekp%ikpibz(ikbz)
 iinv=tablekp%iinv(ikbz)
 isppol =  trans(it)%s
 factocc = trans(it)%factocc
 egwdiff = trans(it)%gwen + Wd * factocc 


 if (iparam%lifetime) then
  sumlifetime = abs(elecstr%imsigma(ikibz,ibc,isppol))+abs(elecstrp%imsigma(ikpibz,ibv,isppol))
 else
  sumlifetime = 0.0
 endif 

 if (iparam%hilbert .or. iparam%matrixel) then
   iet = nint(real((egwdiff - iparam%omegai))/iparam%domega) + 1 
 else

   if (iparam%pes) then !photoemission
    den(:)=-factocc /(iparam%omega(:)-(elecstr%gwenergy(ikibz,ibc,isppol)-elecstr%fermie ))&
          *aimag( 1./(-elecstr%gwenergy(ikibz,ibc,isppol)+elecstrp%gwenergy(ikpibz,ibv,isppol)&
          +iparam%ephoton+1.*cmplx(0.,iparam%broad) )  )
   else !!!this is the normal case:
    den(:)=factocc/(iparam%omega(:)-egwdiff+cmplx(0.0,sumlifetime))
   endif

 endif

end subroutine create_den
!!***
