!{\src2tex{textfont=tt}}
!!****p* creation_chi0/create_rhotw
!! NAME
!! create_rhotw
!!
!! FUNCTION
!! Create the oscillator strength of chi0 for this (it) transition
!!
!!  rhotw(kp,vc) = <v,kp| e^-i(q+G)r |c,k>;    kp = k - q - G0
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_chi0, creation_chi0drude
!!
!! CHILDREN
!! get_wfr, rho_tw_g
!!
!! SOURCE
subroutine create_rhotw(iparam,it,trans,tablekp,elecstr,elecstrp,fftmesh,bzstruct,bzstructp,rhotwg,drudetr,ntrans)

 use ggwdef
 use dpdef

 implicit none
 
 type(dpparam) iparam
 type(transition) trans(ntrans)
 type(bzstructure) bzstruct, bzstructp
 type(polarizationq) tablekp
 type(electronic_structure) elecstr, elecstrp
 type(mesh) fftmesh
 integer iinv2, ig01a,ig02a,ig03a, ntrans
 integer ikbz, ikibz, iinv, ibv, ibc, it, ikpbz, ikpibz, isppol
 complex :: wfr2(fftmesh%nr), wfr1(fftmesh%nr)
 complex rhotwg(iparam%npwmat)
 logical drudetr
  ikbz = trans(it)%k
  ibv = trans(it)%v
  ibc = trans(it)%c
  ikibz = tablekp%ikibz(ikbz)
  ikpibz=tablekp%ikpibz(ikbz)
  iinv=tablekp%iinv(ikbz)
  if (drudetr) iinv=mod(iinv,2)+1
  isppol =  trans(it)%s
  ikpbz=tablekp%ikp(ikbz)
  iinv2=tablekp%iinv2(ikbz)
  if (drudetr) iinv2=mod(iinv2,2)+1
  ig01a=tablekp%ig01a(ikbz)
  ig02a=tablekp%ig02a(ikbz)
  ig03a=tablekp%ig03a(ikbz)

  call get_wfr(iparam,elecstr,elecstrp,fftmesh,ibv,ikpibz,ibc,ikibz,isppol,wfr1,wfr2)

  call rho_tw_g(iparam%npwmat,fftmesh%nr,fftmesh%nr, &
                fftmesh%nrr(1),fftmesh%nrr(1),fftmesh%nrr(2),fftmesh%nrr(3), &
                fftmesh%igfft(:,ig01a,ig02a,ig03a), &
                wfr1,iinv,bzstructp%ktabr(:,ikpbz), &
                wfr2,iinv2,bzstruct%ktabr(:,ikbz),fftmesh%menoplan,rhotwg) 

end subroutine create_rhotw
!!***

