!{\src2tex{textfont=tt}}
!!****p* creation_kernelmt/creation_X
!! NAME
!! creation_X
!!
!! FUNCTION
!! creation of X (generalization of chi0) for kernel Mapping Theory
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_kernelmt
!! 
!! CHILDREN
!! assemblychi0, makeT, makeTcoup
!!
!! SOURCE
 subroutine creation_X(iparam,hd,ucvol,gwenergy,Wd,trans, &
     ktab,res,tr,Tkernel)

 use dpdef
 use ggwdef
 use verbose
 use interfaces

 implicit none
 
 type(dpparam) iparam
 
 type(transition) trans(iparam%nt)

 type(transgen) tr

 type(response) res


 complex (kind(0.0d0)) :: hd(iparam%nt,iparam%nt)
 complex (kind(0.0d0)) :: Tkernel(iparam%npwmat,iparam%npwmat,iparam%nomega)
 complex (kind(0.0d0)) :: den(iparam%nomega)
 complex (kind(0.0d0)) :: den2(iparam%nomega)
! this is called den2 because it defines X not chi0 
 complex (kind(0.0d0)) :: egwdiff
 complex (kind(0.0d0)) :: den_vckR(iparam%nt,iparam%nomega)
 complex (kind(0.0d0)) :: den_vckR2(iparam%nt,iparam%nomega)
! complex (kind(0.0d0)), allocatable :: den_vckA(:,:)
! complex (kind(0.0d0)), allocatable :: den_vckA2(:,:)
 
 double precision gwenergy(iparam%nkibz,iparam%nbnds)
 real :: factocc
 real (kind(0.0d0)) :: Wd
 real :: weight, ucvol

 logical tkernelggp
 
 integer istat
 
 integer :: ktab(iparam%nkbz)
 integer :: ikbz, ikibz, ibv, ibc, it,ig, io

 complex, external :: pdtqrc
 
#ifndef MPI
 allocate(tr%aux2(iparam%nt,iparam%npwmat))
 tr%aux2=tr%rhotwgen
#endif
 
 if(iverbose > 2) call uverbose('b','creation_X')
 if(iverbose > 0) then
   headline="Creation T = rho_t W_ttp rho_tp"
   call print_headline()
 endif

 allocate(res%chi0X(iparam%npwmat,iparam%npwmat,iparam%nomega,1),stat=istat)
 if(istat.ne.0) stop 'out of memory, chi0X'
 if(iverbose>2) then
  call memstat('(creation_X) res%chi0X allocated =')
 endif

 res%chi0X(:,:,:,:) = 0.0

! if(.not.iparam%td) then
!   allocate(den_vckA(iparam%nt,iparam%nomega))
!   allocate(den_vckA2(iparam%nt,iparam%nomega))
! endif


 
!loop over k-points in BZ
 do it =1,iparam%nt
   ikbz=trans(it)%k
   ikibz = ktab(ikbz) 
   
     ibc = trans(it)%c
     ibv = trans(it)%v 
     factocc = trans(it)%factocc
       egwdiff = gwenergy(ikibz,ibc) - gwenergy(ikibz,ibv) &
                    + Wd * factocc
       den(:)=factocc/(iparam%omega(:)-egwdiff)
       den2(:)=factocc/(iparam%omega(:)-egwdiff)
!       den2(:)=factocc/(egwdiff)
!       den2(:)=factocc
!       den2(:)=factocc/(iparam%omega(:)+ediff)
!       den2(:)=factocc ! /(iparam%omega(:)+egwdiff-5)
!       do io=1,iparam%nomega
!           omega2(io)=cmplx(real(iparam%omega(io)),iparam%eta2)
!       enddo
!       omega2(:)=cmplx(real(iparam%omega(:)),iparam%eta2)
!       den2(:)=aimag( factocc/(omega2(:)+egwdiff) )
!       den2(:)=-aimag(iparam%omega(:))/((real(iparam%omega(:)) + &
!                egwdiff)**2 + aimag(iparam%omega(:))**2)

!       if(ibv<=iparam%homo.and.ibc>iparam%homo) then
!         it=transtab(ikbz,ibv-iparam%lomo+1,ibc-iparam%homo)
!         it=transtab(ikbz,ibv,ibc)
!       else 
!         it=transtab(ikbz,ibc-iparam%lomo+1,ibv-iparam%homo)
!       endif

       call assemblychi0(iparam%npwmat,iparam%nomega,res%chi0X,tr%aux2(it,:),den2)
 
       den_vckR(it,:)=den(:)
       den_vckR2(it,:)=den2(:)

!       if(.not.iparam%td) then
!         den_vckA(it,:)=den(:)
!         den_vckA2(it,:)=den2(:)
!       endif
!     enddo ! ibv
!   enddo ! ibc
 enddo ! ikbz

 weight = 2.0 / iparam%nkbz
 res%chi0X(:,:,:,:) = weight * res%chi0X(:,:,:,:) / ucvol

 if(iverbose > 0) then
  write(uout,*)
  write(uout,*)'  --->   X = rho_t  f_t(G) created'      
 endif
 
 Tkernel(:,:,:)=(0,0)

 if(iparam%td) then
   if(iparam%xx) then
     if(iverbose > 0) then
      write(uout,*)
      write(uout,*)'  --->   insertion of X^-1 X in T'      
     endif
!    this will put a further X-1X term in the formula for chi     
     inquire(file='Tkernel',exist=tkernelggp)
     if(tkernelggp) then
      open(unit=51,file='Tkernel',form='unformatted')
      read(51)(((Tkernel(ig,it,io),ig=1,iparam%npwmat),it=1,iparam%npwmat),io=1,iparam%nomega)
      close(51)
     else
      open(unit=51,file='Tkernel',form='unformatted')
      call makeT(iparam,hd,tr%aux2,den_vckR2,den_vckR2,Tkernel)
      write(51)(((Tkernel(ig,it,io),ig=1,iparam%npwmat),it=1,iparam%npwmat),io=1,iparam%nomega)
      close(51)
     endif
   else
      call makeT(iparam,hd,tr%aux2,den_vckR,den_vckR2,Tkernel) 
   endif  
 else
!   call makeTcoup(iparam,hd,hdc,tr%aux2,tr%aux2,den_vckR,den_vckA,Tkernel)
 endif
 Tkernel(:,:,:) = weight * Tkernel(:,:,:) / ucvol

 if(iverbose > 0) then
  call print_footline()
 endif
 if(iverbose > 2) call uverbose('e','creation_X')
 end 
!!***
