!{\src2tex{textfont=tt}}
!!****p* main/creation_chi0drude
!! NAME
!! creation_chi0drude
!!
!! FUNCTION
!! Calculate drude contribution (w->0 and q->0) for chi0
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! CHILDREN
!! pdtqrc, create_rhotw, create_den2, assemblychi0, printcm
!!
!! SOURCE

subroutine creation_chi0drude(transdrude,iparam,bzstruct,tablekp,elecstr,res,ucvol,fftmesh)

 use ggwdef
 use dpdef
 use verbose

 implicit none

 type(dpparam) iparam
 type(transition) transdrude(iparam%ndtmax,iparam%nqpol)
 type(response) res
 type(bzstructure) bzstruct
 type(polarizationq) tablekp
 type(electronic_structure) elecstr
 type(mesh) fftmesh

 complex (kind(0.0d0)) :: den(iparam%nomega)
 complex rhotwg(iparam%npwmat)

 real weight,ucvol

 complex, external :: pdtqrc

 integer iq, it
 integer istat
 logical drudetr
 integer isppol

#ifdef MPI
 integer ierr,rang,nprocs,startcycle,stopcycle
 complex, allocatable :: aux(:,:,:,:,:)
 include 'mpif.h'
 call mpi_comm_size(mpi_comm_world,nprocs,ierr)
 call mpi_comm_rank(mpi_comm_world,rang,ierr)
 if (rang==0) then
#endif

 if(iverbose > 2) call uverbose('b','creation_chi0drude')

 if(iverbose > 0) then
  headline= 'Calculating chi0drude'
  call print_headline()
! If (iparam%td) call mesg('warning','antiresonant term excluded',idum,idum)
 endif

#ifdef MPI
 endif
#endif


 allocate(res%chi0drude(iparam%npwmat,iparam%npwmat,iparam%nomega,elecstr%nsppol,iparam%nqpol),stat=istat)
 if(istat.ne.0) stop 'out of memory, chi0drude'


res%chi0drude(:,:,:,:,:)=cmplx(0.0,0.0)

 do iq=1,iparam%nqpol
#ifdef MPI
  call split_trans(1,iparam%ndrudet(iq),startcycle,stopcycle)
  do it=startcycle,stopcycle
#else
  do it=1,iparam%ndrudet(iq)
#endif
   isppol = transdrude(it,iq)%s
   drudetr=.false.

   if ((.not.iparam%td) .and. (mod(it,2)==0))drudetr=.true.
   call create_rhotw(iparam,it,transdrude(:,iq),tablekp,elecstr,elecstr,fftmesh,bzstruct,bzstruct,rhotwg, &
                     drudetr,iparam%ndrudet(iq))

   call create_den2(iparam,it,iq,transdrude,den)
   call assemblychi0(iparam%npwmat,iparam%nomega,res%chi0drude(:,:,:,isppol,iq),rhotwg,den)
  enddo
 enddo

 deallocate(iparam%ndrudet)

#ifdef MPI
 call mpi_barrier(mpi_comm_world,ierr)
 allocate(aux(iparam%npwmat,iparam%npwmat,iparam%nomega,elecstr%nsppol,iparam%nqpol))
 call mpi_reduce(res%chi0drude,aux,iparam%npwmat*iparam%npwmat*iparam%nomega*elecstr%nsppol*iparam%nqpol,mpi_complex,mpi_sum,0,&
                 mpi_comm_world,ierr)
 
 res%chi0drude=aux
 deallocate(aux)
 if (rang == 0) then
#endif


 if (elecstr%nsppol == 1) then
   weight = 2.0 / iparam%nkbz
 elseif (elecstr%nsppol == 2) then
   weight = 1.0 / iparam%nkbz
 endif
 res%chi0drude(:,:,:,:,:) = weight * res%chi0drude(:,:,:,:,:) / ucvol


 if(iverbose > 1 .and. iparam%npwmat>=9) then
  write(uout,*)
  write(uout,*) '    chi0drude(G,G",isppol) at the first omega and first q'

   do isppol=1, elecstr%nsppol
    call printcm(res%chi0drude(:,:,1,isppol,1),iparam%npwmat,iparam%npwmat)
   enddo !isppol
 endif
 if(iverbose > 0) then
  call print_footline()
 endif

 if(iverbose > 2) call uverbose('e','creation_chi0drude')
#ifdef MPI
 endif
#endif
 end
!!***

