!{\src2tex{textfont=tt}}
!!****p* main/creation_Wtt
!! NAME
!! creation_Wtt
!!
!! FUNCTION
!! interface for the creation of W matrix elements for mt kernel
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!! 
!! CHILDREN
!! makeWq, makeWqcoup, remove_diagonal, evaluate_memory
!!
!! SOURCE

subroutine creation_Wtt(iparam,bzstruct,elecstr,latt,mtkernel, &
                        symstr,trspace,rot,fftmesh)

use dpdef
use ggwdef
use verbose
use interfaces
implicit none
type(dpparam) iparam
type(bzstructure) bzstruct
type(electronic_structure) elecstr
type(lattstruct) latt
type(mappingkernel) mtkernel 
type(symmetry) symstr
type(transposed_space) trspace
type(rot_table) rot
type(mesh) fftmesh

real qpol(3)

#ifdef MPI
include 'mpif.h'
integer ierr,rang,nprocs
call mpi_comm_size(mpi_comm_world,nprocs,ierr)
call mpi_comm_rank(mpi_comm_world,rang,ierr)
if (rang==0) then
#endif
!!! SPIN !!!
allocate(mtkernel%hd(iparam%nt,iparam%nt))
qpol(:)=0
call makeWq(iparam,elecstr,latt%b1,latt%b2,latt%b3,latt%ucvol,trspace%gvec,symstr%op,&
            rot%grottb,fftmesh%igfft,fftmesh, &
            bzstruct%kbz,bzstruct%ktab,bzstruct%ktabi, &
            bzstruct%ktabo,bzstruct%ktabr,mtkernel%hd,qpol)
if(.not.iparam%td) then
  allocate(mtkernel%hdc(iparam%nt,iparam%nt))
  call makeWqcoup(iparam,elecstr,latt%b1,latt%b2,latt%b3,latt%ucvol,trspace%gvec,symstr%op,&
            rot%grottb,fftmesh%igfft,fftmesh,&
            bzstruct%kbz,bzstruct%ktab,&
            bzstruct%ktabi,bzstruct%ktabo,bzstruct%ktabr,&
            mtkernel%hdc,qpol)
endif
call remove_diagonal(iparam%nt,mtkernel%hd,mtkernel%Wd) 

call evaluate_memory(iparam)
if(iverbose>2) then
 call memstat('exc Hamiltonian allocated =')
endif

deallocate(rot%grottb)

#ifdef MPI
endif
call mpi_barrier(mpi_comm_world,ierr)
call mpi_bcast(mtkernel%Wd,1,mpi_double_precision,0,mpi_comm_world,ierr)
#endif

end
!!***




!{\src2tex{textfont=tt}}
!!****p* creation_epsilon/creation_kernelmt
!! NAME
!! creation_kernelmt
!!
!! FUNCTION
!! creation of the Mapping Theory Kernel
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_epsilon
!! 
!! CHILDREN
!! split_trans, creation_X
!!
!! SOURCE

subroutine creation_kernelmt(iparam,elecstr,bzstruct,latt,trans,tr, &
                             res,mtkernel)
use dpdef
use ggwdef
use verbose
use interfaces
implicit none
type(dpparam) iparam
type(bzstructure) bzstruct
type(electronic_structure) elecstr
type(lattstruct) latt
type(transition) trans(iparam%nt)
type(transgen) tr
type(response) res 
type(mappingkernel) mtkernel 
integer istat

#ifdef MPI
include 'mpif.h'
integer ierr,rang,nprocs,istart,istop
call mpi_comm_size(mpi_comm_world,nprocs,ierr)
call mpi_comm_rank(mpi_comm_world,rang,ierr)
write(uout,*)'entra in kernelmt'
call split_trans(1,iparam%nt,istart,istop)
allocate(tr%aux(iparam%nt,iparam%npwmat),stat=istat)
if(istat.ne.0) stop 'out of memory, aux'
tr%aux = 0
allocate(tr%aux2(iparam%nt,iparam%npwmat),stat=istat)
if(istat.ne.0) stop 'out of memory, aux2'
tr%aux(istart:istop,:)=tr%rhotwgen(istart:istop,:)
call mpi_barrier(mpi_comm_world,ierr)
call mpi_reduce(tr%aux,tr%aux2,iparam%nt*iparam%npwmat,mpi_complex,mpi_sum,0,mpi_comm_world,ierr)
deallocate(tr%aux)
if(iverbose>2) then
 write(uout,*)'proc =',rang+1 ,' / ',nprocs
 call memstat(' tr%aux2 allocated =')
endif
if (rang == 0)  then
#endif

allocate(mtkernel%Tkernel(iparam%npwmat,iparam%npwmat,iparam%nomega),stat=istat)
if(istat.ne.0) stop 'out of memory, Tkernel'
if(iverbose>2) then
 call memstat(' Tkernel allocated =')
endif
if(iparam%td) then  !!SPIN!!
 call creation_X(iparam,mtkernel%hd,latt%ucvol,elecstr%gwenergy(:,:,iparam%nsppol), & 
          mtkernel%Wd,trans,bzstruct%ktab,res,tr,mtkernel%Tkernel)
else
 deallocate(mtkernel%hdc)
endif
deallocate(mtkernel%hd)
if(iverbose>2) then
 call memstat('exc Hamiltonian deallocated =')
endif

deallocate(tr%rhotwgen)

#ifdef MPI
endif
call mpi_barrier(mpi_comm_world,ierr)
#endif

deallocate(elecstr%gwenergy,elecstr%oc)


end
!!***
