!{\src2tex{textfont=tt}}
!!****p* creation_exciton/create_tablekmkp
!! NAME
!! create_tablekmkp
!!
!! FUNCTION
!! Create table of point q = k - kp
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_exciton
!!
!! SOURCE

 subroutine create_tablekmkp(iparam,qstruct,bzstruct,tablekmkp)

 use dpdef
 use ggwdef
 use verbose

 implicit none

 type(dpparam) iparam
 type(qstructure) qstruct
 type(bzstructure) bzstruct
 type(kmkpq) tablekmkp

 integer :: ik,ikp,I,ig01,ig02,ig03,iq
 real :: kmkp(3)

 integer, external :: mnint

 if (iverbose > 2) call uverbose('b','create_tablekmkp')


 allocate(tablekmkp%ig01(iparam%nkbz,iparam%nkbz))
 allocate(tablekmkp%ig02(iparam%nkbz,iparam%nkbz))
 allocate(tablekmkp%ig03(iparam%nkbz,iparam%nkbz))
 allocate(tablekmkp%iqkkp(iparam%nkbz,iparam%nkbz))
!   Find q = K - KP - G0
 do ik = 1, iparam%nkbz 
  do ikp = 1, iparam%nkbz 
    kmkp(:) = bzstruct%kbz(:,ik) - bzstruct%kbz(:,ikp)
    kmkp(:) = kmkp(:) * qstruct%qdenom
    I = qstruct%kmkptab(mnint(kmkp(1)), mnint(kmkp(2)), mnint(kmkp(3)))
    if (I == 0) stop 'create_tablekmkp: q = k-kp-G0 not found'
    iq = I / 1000 ; I = I - 1000*iq
    ig01 = I / 100 ; I = I - 100*ig01 ; ig01 = ig01 - 2
    ig02 = I / 10  ; I = I - 10*ig02  ; ig02 = ig02 - 2
    ig03 = I - 2
    tablekmkp%ig01(ik,ikp) = ig01
    tablekmkp%ig02(ik,ikp) = ig02
    tablekmkp%ig03(ik,ikp) = ig03
    tablekmkp%iqkkp(ik,ikp)= iq
  enddo
 enddo
 if (iverbose > 2) call uverbose('e','create_tablekmkp')

 end

!!***


!{\src2tex{textfont=tt}}
!!****p* creation_exciton/create_tablek2
!! NAME
!! create_tablek2
!!
!! FUNCTION
!! given  k (and k+qpol) this routine finds k2 such that:
!! k2 -qpol = k+qpol => k2 = k+ 2qpol
!! so: in order to have tableantikp(k) one can use tablekp(k2)
!! this is useful for rhotwq0 to get antireso vv' from reso vv'
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_exciton
!!
!! CHILDREN
!! findkp
!!
!! SOURCE

 subroutine create_tablek2(iparam,bzstruct,tablek2)

use dpdef
use ggwdef
use verbose

implicit none
type(dpparam) iparam
type(bzstructure) bzstruct

integer :: tablek2(iparam%nkbz)

integer ikbz,ik2,i
real k2(3),qpol(3)
integer g0(3)

logical, external :: findkp


if(iverbose > 2)call uverbose('b','creation_tablek2')

 if (iparam%KP_THEORY) then

  do ikbz=1,iparam%nkbz 
    tablek2(ikbz) = ikbz
  enddo

 else

  qpol(:) = iparam%qpol(:,1)

  do ikbz=1,iparam%nkbz 
   k2(:) = bzstruct%kbz(:,ikbz) + 2*qpol(:)
   if(.not. findkp(iparam%nkbz,bzstruct%kbz,ik2,k2,g0)) then
    write(uout,*) 'create_tablek2 : k2 = k + 2qpol + G0 not found;'
    write(uout,1927) ikbz, (bzstruct%kbz(i,ikbz),i=1,3), (k2(i),i=1,3)
1927     format(' ik = ',i4,' k'' = ',3f12.6/' k + 2qpol = ',6x,3f12.6)
    stop
   endif
   tablek2(ikbz) = ik2 
   if(iverbose > 2) then
    write(uout,'(a,i4,3f10.5)') ' ik, k ', ikbz,bzstruct%kbz(:,ikbz)
    write(uout,'(a,i4,3f10.5)') ' ik2,k2', ik2,bzstruct%kbz(:,ik2)
   endif

   enddo

 endif 


if(iverbose > 2)call uverbose('e','creation_tablek2')

 end subroutine
!!***

