!{\src2tex{textfont=tt}}
!!****p* main/creation_tablekp
!! NAME
!! creation_tablekp
!!
!! FUNCTION
!! Creation of the kpoints table. This table permits to know everything
!! connected to the kpoint index ikbz, like ikibz, ikpibz, ikp, iinv, etc. 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! CHILDREN
!! findkp
!!
!! SOURCE
subroutine  creation_tablekp(iparam,bzstruct,bzstructp,tablekp)

use dpdef
use ggwdef
use verbose

implicit none
type(dpparam) iparam
type(bzstructure) bzstruct, bzstructp
type(polarizationq) tablekp

real qpol(3),kmq(3)
integer g0(3)
integer ikbz,ikp,i,ikibz,iinv

logical, external :: findkp


if(iverbose > 2)call uverbose('b','creation_tablekp')


qpol(:) = iparam%qpol(:,1)

allocate(tablekp%ik(iparam%nkbz))    !arrivo
allocate(tablekp%ikp(iparam%nkbz))   !partenza
allocate(tablekp%ikibz(iparam%nkbz)) 
allocate(tablekp%ikpibz(iparam%nkbz)) 
allocate(tablekp%iinv(iparam%nkbz))  !kp (partenza)
allocate(tablekp%iinv2(iparam%nkbz)) !k (arrivo)
allocate(tablekp%ig01a(iparam%nkbz)) 
allocate(tablekp%ig02a(iparam%nkbz)) 
allocate(tablekp%ig03a(iparam%nkbz)) 


do ikbz=1,iparam%nkbz
 tablekp%ik(ikbz)= ikbz 
 ikibz =  bzstruct%ktab(ikbz)              ! final point k
 tablekp%ikibz(ikbz) = ikibz
 if(iparam%KP_THEORY) then
  tablekp%ikp(ikbz) = ikbz
  tablekp%ikpibz(ikbz) = ikibz
  tablekp%ig01a(ikbz) = 3
  tablekp%ig02a(ikbz) = 3
  tablekp%ig03a(ikbz) = 3
  iinv = (3-bzstruct%ktabi(ikbz))/2        ! bzstructp=bzstruct in this case
  tablekp%iinv(ikbz) = iinv
  tablekp%iinv2(ikbz) = iinv
 else
  kmq(:) = bzstruct%kbz(:,ikbz) - qpol(:)  ! initial point kp = k - q - G0
  if(.not. findkp(iparam%nkbz,bzstructp%kbz,ikp,kmq,g0)) then
   write(uout,*) 'creation_tablekp: k'' = k - qpol - G0 not found;'
   write(uout,1927) ikbz, (bzstructp%kbz(i,ikbz),i=1,3), (kmq(i),i=1,3)
1927     format(' ik = ',i4,' k'' = ',3f12.6/' k - qpol = ',6x,3f12.6)
   write(uout,*) '( NOTE: the precision of the comparison (q_precision) is ',q_precision,' )'
   stop
  endif
  tablekp%ikp(ikbz)=ikp
  tablekp%ikpibz(ikbz) = bzstructp%ktab(ikp)
  tablekp%ig01a(ikbz) = g0(1) + 3
  tablekp%ig02a(ikbz) = g0(2) + 3
  tablekp%ig03a(ikbz) = g0(3) + 3
  tablekp%iinv(ikbz) = bzstructp%ktabi(ikp)
  tablekp%iinv2(ikbz) = bzstruct%ktabi(ikbz)
 endif
enddo


if(iverbose > 2)call uverbose('e','creation_tablekp')
 
end
!!***



!{\src2tex{textfont=tt}}
!!****p* main/creation_tableantikp
!! NAME
!! creation_tableantikp
!!
!! FUNCTION
!! Creation of the kpoints table. This table permits to know everything
!! connected to the kpoint index ikbz, like ikibz, ikpibz, ikp, iinv, etc. 
!! This is needed for transition in antiresonant case for excitonic hamiltonian.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! CHILDREN
!! findkp
!!
!! SOURCE
subroutine  creation_tableantikp(iparam,bzstruct,tableantikp)

use dpdef
use ggwdef
use verbose

implicit none
type(dpparam) iparam
type(bzstructure) bzstruct
type(polarizationq) tableantikp

real qpol(3),kmq(3)
integer g0(3)
integer ikbz,ikp,i,ikibz,iinv

logical, external :: findkp

if(iverbose > 2)call uverbose('b','creation_tableantikp')

qpol(:) = iparam%qpol(:,1)

allocate(tableantikp%ik(iparam%nkbz))     !arrivo
allocate(tableantikp%ikp(iparam%nkbz))    !partenza
allocate(tableantikp%ikibz(iparam%nkbz)) 
allocate(tableantikp%ikpibz(iparam%nkbz)) 
allocate(tableantikp%iinv(iparam%nkbz))    !kp -> partenza
allocate(tableantikp%iinv2(iparam%nkbz))   !k  -> arrivo
allocate(tableantikp%ig01a(iparam%nkbz))   !k+q
allocate(tableantikp%ig02a(iparam%nkbz)) 
allocate(tableantikp%ig03a(iparam%nkbz)) 


do ikbz=1,iparam%nkbz

 tableantikp%ikp(ikbz)  = ikbz              !partenza
 ikibz =  bzstruct%ktab(ikbz)
 tableantikp%ikpibz(ikbz) = ikibz

 if(iparam%KP_THEORY) then
  tableantikp%ik(ikbz) = ikbz      
  tableantikp%ikibz(ikbz) = ikibz
  tableantikp%ig01a(ikbz) = 3
  tableantikp%ig02a(ikbz) = 3
  tableantikp%ig03a(ikbz) = 3
  iinv = (3-bzstruct%ktabi(ikbz))/2
  tableantikp%iinv(ikbz) = iinv
  tableantikp%iinv2(ikbz) = iinv
 else
  kmq(:) = bzstruct%kbz(:,ikbz) + qpol(:)
  if(.not. findkp(iparam%nkbz,bzstruct%kbz,ikp,kmq,g0)) then
   write(uout,*) 'creation_tableantikp: k'' = k + qpol + G0 not found;'
   write(uout,1927) ikbz, (bzstruct%kbz(i,ikbz),i=1,3), (kmq(i),i=1,3)
1927     format(' ik = ',i4,' k'' = ',3f12.6/' k + qpol = ',6x,3f12.6)
   stop
  endif

  tableantikp%ik(ikbz)=ikp             !arrivo
  tableantikp%ikibz(ikbz) = bzstruct%ktab(ikp)

  tableantikp%ig01a(ikbz) = -g0(1) + 3
  tableantikp%ig02a(ikbz) = -g0(2) + 3
  tableantikp%ig03a(ikbz) = -g0(3) + 3

  tableantikp%iinv(ikbz) = bzstruct%ktabi(ikbz) !partenza
  tableantikp%iinv2(ikbz) = bzstruct%ktabi(ikp) !arrivo

 endif
enddo

if(iverbose > 2)call uverbose('e','creation_tableantikp')
 
end
!!***

