/*
      dp program
      
      calculate Dielectric Properties
      TDDFT code in Frequency-Reciprocal space on a PlaneWaves basis

      input:
        input file: parameters of calculation (q, omega)
        kss  file:  Kohn-Sham band structure and material info (KSS)
        kssp file:  (opt) separate KS-bs for initial states (k-qpol)
        gw   file:  GW corrections
        gwp  file:  (opt) separate GW file for initial states (k-qpol)
        qps  file:  quasiparticle band structure info  (QPS)
        qpsp file:  (opt) separate QPS file for initial states (k-qpol)
        scr  file:  screening file

      obsolete input file:
        qplda file:  use it as kss file with -k
        vkb file:    non-local potential (Kleinman-Bylander parameters)
        em1 file:    old screening file format 
         
      output:
        ou*.mdf: macroscopic dielectric function epsilon

      this is the main dp.c file used to run the real fortan main file dp.f

!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html

*/

#include <stdio.h>
#include <string.h>

main(argc,argv)
int argc;
char *argv[];
{
  int il, kl, klp, gl, glp, ql, qlp, sl,  vl, ll, rl; /* file lengths */
  /* filenames */
  char *inputfile=NULL, *kssfile=NULL,  *gwfile=NULL,  *qpsfile=NULL, *scrfile=NULL  ; 
  char *kssfilep=NULL, *gwfilep=NULL,  *qpsfilep=NULL, *root=NULL;
  char *lambdafile=NULL;
  /* obsolete files */
  char *vkbfile=NULL; 
  char *program; /* program name */
 
 
#ifdef MPI
/* #include "mpi.h" */
  MPI_Init(&argc,&argv);
#endif

  program = argv[0];

  read_command_line(argc,argv,&inputfile,&kssfile,&kssfilep,&gwfile,&gwfilep, &qpsfile,&qpsfilep, &scrfile, &vkbfile, &lambdafile, &root);

  il = check_file(&inputfile,"dp.in",program);
  kl = check_file(&kssfile,"in.kss",program);
  klp= check_file(&kssfilep,"indq.kss",program);
  gl = check_file(&gwfile,"in.gw",program);
  glp= check_file(&gwfilep,"indq.gw",program);
  ql = check_file(&qpsfile,"in.qps",program);
  qlp= check_file(&qpsfilep,"indq.qps",program);
  sl = check_file(&scrfile,"in.scr",program);
  vl = check_file(&vkbfile,"in.vkb",program);
  ll = check_file(&lambdafile,"in.exeig",program);
  if (NULL != root) { 
     rl = strlen(root);
  }  else {
     rl = 0;
  }

#ifdef GFOR
_gfortran_set_record_marker(4); /* default is 4 */
#ifdef GFOR_BIG
_gfortran_set_convert(2);         /* 2: big endian, 3: little endian */
#endif
#ifdef GFOR_LITTLE
_gfortran_set_convert(3);         /* 2: big endian, 3: little endian */
#endif
#endif

  /*  printf("files %s %s %s %s %d \n",inputfile,kssfile,vkbfile,root,rl); */

  /* call the fortran main */
  dp_(inputfile,&il,kssfile,&kl,kssfilep,&klp,gwfile,&gl,gwfilep,&glp,qpsfile,&ql,qpsfilep,&qlp,scrfile,&sl,vkbfile,&vl,lambdafile,&ll,root,&rl);

#ifdef MPI
  MPI_Finalize();
#endif

  return 0;
}
