!{\src2tex{textfont=tt}}
!!****h* Mod/params
!! NAME
!! params
!!
!! FUNCTION
!! Defines some parameters
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!!***

module params
 integer, parameter  :: ILONG = selected_int_kind(16)
 character (len=62) headline
 integer ::  uout=6  ! output unit, 6 by default, it could be put to 0 for debugging purposes
 real    ::  q_precision= 1.0e-4  ! precision of q-vector and k-points 
 real    ::  kp_limit   = 1.0e-2  ! limit for k.p theory
!! used in lcompv, icompv_fraction, is_zero, is_small
!! (the criterion is that each component (in rl units) should be < e)

 double precision, parameter :: PI = 3.14159265358979323846
 double precision, parameter :: zero = 0.0d0
 double precision, parameter :: H2eV = 27.2113845d0 ! standard uncertainty 0.0000023 (NIST)
 double precision, parameter :: eV2H = 1.0d0 / H2eV
 real, parameter :: HARTREE = 27.2113845
 real, parameter :: ANGSTROM = 0.529177

!exciton
 integer, parameter :: qdenommx = 32

end module params

!!***


!{\src2tex{textfont=tt}}
!!****h* Mod/dpdef
!! NAME
!! dpdef
!!
!! FUNCTION
!! dpdef module only contains the dpparam type
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!!***
module dpdef

use params

implicit none

!{\src2tex{textfont=tt}}
!!****t* dpdef/dpparam
!! NAME
!! dpparam
!!
!! FUNCTION
!! dpparam structure (essentially devoted to input paramenters)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
type dpparam
  logical TESTPARTICLE, TESTELECTRON
  logical JDOS, RPA, LDA, GW, SO, NLDA
  logical qps,qpsenergy
  logical pes
  logical FXCALPHA, FXCALPHABETA, FXCEPS, FXCEPSDIAGO, FXCVBAR, CUTOFF
  logical FXCMT, FXCCONTACT
  logical NOFXC
  logical KP_THEORY
  logical kernel
  logical quick
  logical nonlinear
  logical doublegrid  ! use two kss-input files shifted by q
  logical antieps     ! calculate eps*(-q,-w) instead of eps(q,w)
  logical testrun     ! perform check on time and memory needs
  logical mpilocal    ! in case only master reads (MPI)
  logical gwfilenew
  logical savememory
  logical matrixel
  logical matelanalysis
  logical lifetime
  logical DRUDE, ONLYDRUDE
  logical writechi0
  logical readchi0
!  character (len=12) :: ttype
!  character (len=7)  :: kxct
  integer npwmat, npwwfn, npwvec
  integer matsh, wfnsh
  integer nsh
  integer, pointer :: shlim(:)
  integer ngw
  integer nsppol
  integer lomo, homo, lumo, nbnds
  integer nbndmin,nbndmax
  integer nbndqpmin,nbndqpmax
  integer nbndqps 
  integer nbndv, nbndc
  integer nt,ntspin
  integer(ILONG) :: ntres
  integer, pointer :: ntblock(:,:)
  integer nkibz, nkbz
  integer nel
  integer nop, ninv, nopr
  integer nopq, ninvq
  integer nopqpol, ninvqpol
  integer ndtmax
  integer, pointer :: ndrudet(:)
  integer wropt
  logical inv
  real qpg(3)
  integer nqpol
  real qpol(3,6)
  real qpolcc(3,6)
  real qpolcc2pioa(3,6)
  integer ig
  integer g(3)
  real gcc(3)
  real gcc2pioa(3)
  integer alias
  real soenergy
  real stretchvalence,stretchconduction
  logical somult
  real alphadiv
  real rcut, beta
  real V0, fcut
  real omegai, omegae, domega
  real broad
  integer nomega
  complex, pointer :: omega(:)
  real eta2
  logical td
  logical xx
  logical vkb
  logical hilbert
  logical metal
  integer occopt   ! ~equivalent to occopt in ABINIT (only available: 3 = Fermi-Dirac is now available)
                   !                                                  0 = Temperature zero
  real kBT         ! electronic temperature (equivalent to tsmear in ABINIT) in Hartree

  real omegaplasma
  real ephoton
!exciton
  logical exciton
  logical exc
  character (len=50) lftype
  character (len=30) wtype
  character (len=30) htype
  logical  haydock,haydock_restart
  integer  niter
  integer nqibz
  logical RESONANT, COUPLING, COUPLPERT
  logical WDIAG, WFULL
  logical GWTERM, EXCHANGETERM, COULOMBTERM
  logical encutoff
  real ircut,uvcut,stripecut
  logical dielconstscreen
  real renormdielconst 
  logical coulcutoff
  real rlcut
  logical :: fulldiago 
  logical :: blockpert 
  logical :: pert 
  real :: deltadeg 
  integer :: maxblocksize 
  integer ::  order 
!end exciton 

!plexc
  logical analysis_exciton
  logical plot_exciton
  real enmin_analysis, enmax_analysis
  integer ntrans_analysis
!  integer lamb
  integer lambda(2)
  real enedeg
  integer ncell(3)
  logical hole,atoms,bonds
  integer nhole,natoms,nbonds
  double precision xredhole(3),xcarthole(3)
!end plexc


  character (len=60) :: calctype, dieltype, entype, wftype, approxtype, solvertype
  logical linear, current
  logical enks
  logical kss

  logical oldqplda 
  logical wf_distrib

endtype
!!***

!{\src2tex{textfont=tt}}
!!****t* dpdef/dpfiles
!! NAME
!! dpfiles
!!
!! FUNCTION
!! dpfiles structure contains all the info about the input files 
!!  kss 10, vkb 11, em1 12, gw 13
!!  exh 55, exc 56
!!  exeig 25, exceg 26, exovl 27
!!  trans 28
!!  wfg 30, wfr 31
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! SOURCE
type dpfiles
  integer il, kl, gl, ql,sl, vl, ll, rl
  integer hl, cl, eigl, cegl, ol  ! length of the filenames
  character (len=300) :: input, kss, gw, qps, scr, vkb,lambda, root
  character (len=30) :: exeig, exceg, exovl
  logical eigx, cegx, ox ! existence of the files
  integer :: eigu=25, cegu=26, ou=27
!old
  logical hx, cx  ! existence of the files
  integer :: hu=55, cu=56
  character (len=30) :: exh, exc
!endold
!new (for hamiltonian blocks)
  integer :: uni(2,2,2,2) 
  character (len=30) :: in_blockfile_name(2,2,2,2), out_blockfile_name(2,2,2,2)
  logical :: blockfile_uncompleted(2,2,2,2)
!endnew
  integer :: sizeofcomplex, kstart, kcstart
  logical :: HMEMO
  integer :: nrblock,ncblock,nspinblock
  integer(ILONG), pointer :: lastirec(:,:)
  integer(ILONG) :: dimham, storeham
  logical :: square
!oldqplda
  integer i1
endtype
!!***


end module

!{\src2tex{textfont=tt}}
!!****h* Mod/verbose
!! NAME
!! verbose
!!
!! FUNCTION
!! verbose module only contains the debug type
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!!***
module verbose

use params

implicit none
!{\src2tex{textfont=tt}}
!!****t* verbose/debug
!! NAME
!! debug
!!
!! FUNCTION
!! debug structure (devoted to the output verbosity)
!! 
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! NOTES
!! verbose module carries the integer 'iverbose' that can assume several values:
!! * 0) minimum amount of info written down  
!! * 1) default values. essential things are written down 
!! * 2) verbose mode: many more things are written down
!! * 3) debug mode: to use only in debugging case, all the loop indices are written (tree file created) 
!!
!! SOURCE
 integer :: spaces = -1
 integer :: iverbose
 integer :: loopperc = 5
 integer(ILONG) :: loopnmatel

!!***
end module verbose


