!{\src2tex{textfont=tt}}
!!****p* main/fftwfn
!! NAME
!! fftwfn
!!
!! FUNCTION
!! calculate KS wavefunctions in real space using FFT
!! Input:  wfg = KS wavefunctions in PW format
!! Output: wfr = KS wavefunctions in real space
!! 
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! CHILDREN
!! c3dfft
!!
!! SOURCE

  subroutine fftwfn(iparam,fftmesh,elecstr)

  use verbose
  use dpdef
  use ggwdef
  use interfaces
  implicit none
  type(dpparam) iparam
  type(electronic_structure) elecstr
  type(mesh) fftmesh

  integer ig, ik, ib, istat,is

#ifdef MPI
  include 'mpif.h'
  integer ierr,rang,nprocs,ir
  call mpi_comm_size(mpi_comm_world,nprocs,ierr)
  call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif

  if(iverbose > 2) call uverbose('b','fftwfn')

  allocate(elecstr%wfr(fftmesh%nr,iparam%nbndmin:iparam%nbnds,iparam%nkibz,iparam%nsppol),stat=istat)
  if(istat.ne.0) stop 'wavefunctions(r) out of memory'
  if(iverbose>2) then
   call memstat('wfr allocated =')
  endif
  
#ifdef MPI
 if(.not.(iparam%mpilocal.and.rang/=0)) then
#endif


  if(iverbose > 0) then
   headline='FFT wavefunctions to real space'
   call print_headline()
  endif
  elecstr%wfr(:,:,:,:) = 0.0
  do is =  1, iparam%nsppol
   do ik = 1, iparam%nkibz
     do ib = iparam%nbndmin, iparam%nbnds
       if(iverbose>2)  write (uout,*) 'is,ik,ib',is,ik,ib 
!      fill FFT array from PW array
       do ig = 1, iparam%npwvec
         elecstr%wfr(fftmesh%igfft(ig,3,3,3),ib,ik,is) = elecstr%wfg(ig,ib,ik,is)
       enddo
!      take FFT to give wfn in real space
       call c3dfft(elecstr%wfr(:,ib,ik,is), &
                   fftmesh%nrr(1),fftmesh%nrr(2),fftmesh%nrr(3),& 
                   fftmesh%nrr(1),fftmesh%nrr(2),fftmesh%nrr(3),1,fftmesh%plan)
      enddo
    enddo
  enddo
  if(iverbose > 0) then
   call print_footline()
  endif

  deallocate(elecstr%wfg)
  if(iverbose>2) then
    call memstat('wfg deallocated =  ')
  endif

  if(iverbose > 2) call uverbose('e','fftwfn')
  
#ifdef MPI
endif 
 if(iparam%mpilocal) then
  call mpi_barrier(mpi_comm_world,ierr)
  do ir=1,fftmesh%nr
   call mpi_bcast(elecstr%wfr(ir,:,:,:),(iparam%nbnds-iparam%nbndmin+1)*iparam%nkibz*iparam%nsppol,& 
                  mpi_complex,0,mpi_comm_world,ierr)
  enddo

 endif !mpilocal
#endif


  end
!!***
