!{\src2tex{textfont=tt}}
!!****p* coulombterm/coul_kpt_indeces
!! NAME
!! coul_kpt_indeces
!!
!! FUNCTION
!! find the right indeces iq,ig01,ig02,ig03 once in the loop over ik,ik' 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! coulombterm
!!
!! SOURCE
subroutine coul_kpt_indeces(ikbz,ikpbz,nres1,nres2,tablekmkp,tablekp,tableantikp,Windeces)

 use ggwdef
 use dpdef
 implicit none
 type(kmkpq) tablekmkp
 type(polarizationq) tablekp,tableantikp
 type (Wmatrixel) Windeces
 
 integer,intent(in) :: ikbz,ikpbz,nres1,nres2

! => see definitions of objects in creation_tablekp and creation_tableantikp 

! k indeces
 if(nres1 ==1 ) then
  Windeces%ikibz1=tablekp%ikpibz(ikbz) !partenza (v,k-qpol)
  Windeces%iik1=tablekp%iinv(ikbz)
  Windeces%ikbz1=tablekp%ikp(ikbz)
  Windeces%ikibz2=tablekp%ikibz(ikbz)  !arrivo  (c,k)
  Windeces%iik2=tablekp%iinv2(ikbz)
  Windeces%ikbz2=tablekp%ik(ikbz)
 else
  Windeces%ikibz1=tableantikp%ikpibz(ikbz) !partenza (c,k)
  Windeces%iik1=tableantikp%iinv(ikbz)
  Windeces%ikbz1=tableantikp%ikp(ikbz)
  Windeces%ikibz2=tableantikp%ikibz(ikbz)  !arrivo (v,k+qpol)
  Windeces%iik2=tableantikp%iinv2(ikbz)
  Windeces%ikbz2=tableantikp%ik(ikbz)
 endif
! Windeces%iopk = bzstruct%ktabo(ikbz)

! k' indeces
 if(nres2 ==1 ) then
  Windeces%ikibz3=tablekp%ikpibz(ikpbz)  !partenza (v',k'-qpol)
  Windeces%iik3=tablekp%iinv(ikpbz)
  Windeces%ikbz3=tablekp%ikp(ikpbz) 
  Windeces%ikibz4=tablekp%ikibz(ikpbz)   !arrivo   (c',k')
  Windeces%iik4=tablekp%iinv2(ikpbz)
  Windeces%ikbz4=tablekp%ik(ikpbz)
 else
  Windeces%ikibz3=tableantikp%ikpibz(ikpbz) !partenza (c',k')
  Windeces%iik3=tableantikp%iinv(ikpbz)
  Windeces%ikbz3=tableantikp%ikp(ikpbz) 
  Windeces%ikibz4=tableantikp%ikibz(ikpbz)  !arrivo (v',k'+qpol)
  Windeces%iik4=tableantikp%iinv2(ikpbz)
  Windeces%ikbz4=tableantikp%ik(ikpbz)
 endif
! Windeces%iopkp = bzstruct%ktabo(ikpbz)



!matrixelements: rhotw(1<->3) W rhotw(2<->4) 
  Windeces%iq = tablekmkp%iqkkp(Windeces%ikbz2,Windeces%ikbz4)
  Windeces%left_ig01=tablekmkp%ig01(Windeces%ikbz2,Windeces%ikbz4)+3 
  Windeces%left_ig02=tablekmkp%ig02(Windeces%ikbz2,Windeces%ikbz4)+3 
  Windeces%left_ig03=tablekmkp%ig03(Windeces%ikbz2,Windeces%ikbz4)+3 

  Windeces%right_ig01=tablekmkp%ig01(Windeces%ikbz1,Windeces%ikbz3)+3 
  Windeces%right_ig02=tablekmkp%ig02(Windeces%ikbz1,Windeces%ikbz3)+3 
  Windeces%right_ig03=tablekmkp%ig03(Windeces%ikbz1,Windeces%ikbz3)+3 

end subroutine coul_kpt_indeces
!!***

!{\src2tex{textfont=tt}}
!!****p* coulombterm/coul_bnd_indeces
!! NAME
!! coul_bnd_indeces
!!
!! FUNCTION
!! Find band indeces for beginning and end of loop in create_Wmatrixelement  
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! coulombterm
!!
!! SOURCE
subroutine coul_bnd_indeces(nres1,nres2,iparam,Windeces)

 use ggwdef
 use dpdef
 implicit none
 type (Wmatrixel) Windeces
 type(dpparam) iparam
 
 integer,intent(in) :: nres1,nres2


! band indeces:
 if (nres1 .eq. 1) then !resonant
  Windeces%ibini1= iparam%lomo  !ib1 valence
  Windeces%ibend1 = iparam%homo
  Windeces%ibini2= iparam%lumo  !ib2 conduction
  Windeces%ibend2 = iparam%nbnds
 else !antiresonant
  Windeces%ibini1= iparam%lumo  !ib1 conduction
  Windeces%ibend1 = iparam%nbnds
  Windeces%ibini2= iparam%lomo  !ib2 valence
  Windeces%ibend2 = iparam%homo
 endif

 if (nres2 .eq. 1) then !resonant
  Windeces%ibini3= iparam%lomo  !ib3 valence     
  Windeces%ibend3 = iparam%homo                 
  Windeces%ibini4= iparam%lumo  !ib4 conduction
  Windeces%ibend4 = iparam%nbnds
 else  !antiresonant                            
  Windeces%ibini3= iparam%lumo  !ib3 conduction
  Windeces%ibend3 = iparam%nbnds
  Windeces%ibini4= iparam%lomo  !ib4 valence
  Windeces%ibend4 = iparam%homo
 endif


end subroutine coul_bnd_indeces
!!***
