!{\src2tex{textfont=tt}}
!!****p* creation_X/makeT
!! NAME
!! makeT
!!
!! FUNCTION
!! creation of T (nX W Xn) for kernel Mapping Theory
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_X
!! 
!! SOURCE
subroutine makeT(iparam,hd,rho_vcks,den_vck,den_vck2,Tkernel)
 use dpdef
 use verbose
 implicit none
 type(dpparam) iparam
 integer nh
 complex (kind(0.0d0)) :: hd(iparam%nt,iparam%nt)
 complex (kind(0.0d0)) :: rho_vck(iparam%nt,iparam%npwmat)
 complex :: rho_vcks(iparam%nt,iparam%npwmat)
 complex (kind(0.0d0)) :: den_vck(iparam%nt,iparam%nomega)
 complex (kind(0.0d0)) :: den_vck2(iparam%nt,iparam%nomega)
 complex (kind(0.0d0)) :: Tkernel(iparam%npwmat,iparam%npwmat,iparam%nomega)
 complex (kind(0.0d0)), allocatable :: rho_vckstar(:,:)
 complex (kind(0.0d0)), allocatable :: phit(:)
 complex (kind(0.0d0)), allocatable :: phitstar(:)
 complex (kind(0.0d0)), allocatable :: phitaux(:)
 complex (kind(0.0d0)), external :: zdotu
 integer io,ig,igp,istat

 if(iverbose > 2) call uverbose('b','makeT')

 rho_vck(:,:)=rho_vcks(:,:)
 nh = iparam%nt
 allocate(rho_vckstar(nh,iparam%npwmat),stat=istat)
 if(istat.ne.0) stop 'phit not allocated'
 rho_vckstar(:,:)=conjg(rho_vck(:,:))
 
 allocate(phit(nh),stat=istat)
 if(istat.ne.0) stop 'phit not allocated'
 allocate(phitstar(nh),stat=istat)
 if(istat.ne.0) stop 'phitstar not allocated'
 allocate(phitaux(nh),stat=istat)
 if(istat.ne.0) stop 'phitaux not allocated'

 do io=1,iparam%nomega
   do igp=1,iparam%npwmat
     phitstar(:)=rho_vckstar(:,igp)*den_vck2(:,io)
     call zgemv('N',nh,nh,(1.d0,0.d0),hd(:,:),nh,phitstar,1,(0.d0,0.d0),phitaux,1)
!     call cgemv('N',nh,nh,(1.d0,0.d0),hd(:,:),nh,phitstar,1,(0.d0,0.d0),phitaux,1)
     do ig=1,iparam%npwmat
       phit(:)=rho_vck(:,ig)*den_vck(:,io)
       Tkernel(ig,igp,io) = - zdotu(nh,phit,1,phitaux,1)
!       Tkernel(ig,igp,io) = - cdotu(nh,phit,1,phitaux,1)
     enddo
   enddo
 enddo 

 deallocate(rho_vckstar,phit,phitstar,phitaux)

 if(iverbose > 0) write(uout,*) '  -->   T kernel obtained'
 if(iverbose > 2) call uverbose('e','makeT')
end
!!***

!{\src2tex{textfont=tt}}
!!****p* creation_X/makeTcoup
!! NAME
!! makeTcoup
!!
!! FUNCTION
!! creation of T (nX W Xn) for kernel Mapping Theory
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_X
!! 
!! SOURCE
subroutine makeTcoup(iparam,hd,hdc,rho_vckR,rho_vckA,den_vckR,den_vckA,&
   Tkernel)
 use dpdef
 use verbose
 implicit none
 type(dpparam) iparam
 integer nh
 complex (kind(0.0d0)) :: hd(iparam%nt,iparam%nt),hdc(iparam%nt,iparam%nt)
 complex (kind(0.0d0)) :: rho_vckR(iparam%nt,iparam%npwmat)
 complex (kind(0.0d0)) :: rho_vckA(iparam%nt,iparam%npwmat)
 complex (kind(0.0d0)) :: den_vckR(iparam%nt,iparam%nomega)
 complex (kind(0.0d0)) :: den_vckA(iparam%nt,iparam%nomega)
 complex (kind(0.0d0)) :: Tkernel(iparam%npwmat,iparam%npwmat,iparam%nomega)
 complex (kind(0.0d0)) :: rho_vckstarR(iparam%nt,iparam%npwmat),rho_vckstarA(iparam%nt,iparam%npwmat)
 complex (kind(0.0d0)) :: phitR(iparam%nt,iparam%npwmat),phitA(iparam%nt,iparam%npwmat)
 complex (kind(0.0d0)) :: phitstarR(iparam%nt,iparam%npwmat),phitstarA(iparam%nt,iparam%npwmat)
 complex (kind(0.0d0)) :: phitauxRR(iparam%nt),phitauxRA(iparam%nt),phitauxAA(iparam%nt),phitauxAR(iparam%nt)

 complex (kind(0.0d0)), external :: zdotu
 integer io,ig,igp

 if(iverbose > 2) call uverbose('b','makeTcoup')

 nh = iparam%nt
 write(uout,*) 'make T kernel'

 rho_vckstarR(:,:)=conjg(rho_vckR(:,:))
 rho_vckstarA(:,:)=conjg(rho_vckA(:,:))

 do io=1,iparam%nomega
  write(uout,'(a,1x,i5,a,1x,i5)') 'frequency no',io,'/',iparam%nomega
!  write(0,'(a,x,i5,a,x,i5)') 'frequency no',io,'/',iparam%nomega
  do ig=1,iparam%npwmat
   phitR(:,ig)=rho_vckR(:,ig)*den_vckR(:,io)
   phitA(:,ig)=rho_vckA(:,ig)*den_vckA(:,io)
   phitstarR(:,ig)=rho_vckstarR(:,ig)*den_vckR(:,io)
   phitstarA(:,ig)=rho_vckstarA(:,ig)*den_vckA(:,io)
  enddo

  do igp=1,iparam%npwmat
! reso - reso & anti - anti
   call zgemv('N',nh,nh,(1.d0,0.d0),      hd(:,:)  ,nh,phitstarR(:,igp),1,(0.d0,0.d0),phitauxRR,1)
   call zgemv('N',nh,nh,(1.d0,0.d0),conjg(hd(:,:)) ,nh,phitstarA(:,igp),1,(0.d0,0.d0),phitauxAA,1)
! reso - anti & anti - reso
   call zgemv('N',nh,nh,(1.d0,0.d0),      hdc(:,:) ,nh,phitstarA(:,igp),1,(0.d0,0.d0),phitauxRA,1)
   call zgemv('N',nh,nh,(1.d0,0.d0),conjg(hdc(:,:)),nh,phitstarR(:,igp),1,(0.d0,0.d0),phitauxAR,1)
   phitauxRR(:)=phitauxRR(:)+phitauxRA(:)
   phitauxAA(:)=phitauxAA(:)+phitauxAR(:)
   do ig=1,iparam%npwmat
    Tkernel(ig,igp,io) = - zdotu(nh,phitR(:,ig),1,phitauxRR,1) &
                         - zdotu(nh,phitA(:,ig),1,phitauxAA,1)
   enddo
  enddo
 enddo

 write(uout,*) 'T kernel obtained'
 if(iverbose > 2) call uverbose('e','makeTcoup')
 return
end
!!***

!{\src2tex{textfont=tt}}
!!****p* creation_Wtt/makeWq
!! NAME
!! makeWq
!!
!! FUNCTION
!! creation of W matrix elements for mt kernel at q=0 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_Wtt
!!
!! CHILDREN
!! testscr, rdscr, rdepsm1, identq_old, clcqpg, qtabg0, iqg0, get_wfr, rho_tw_g 
!!
!! SOURCE
subroutine makeWq(iparam,elecstr,b1,b2,b3,ucvol,gvec,op,grottb,igfft,fftmesh,&
 kbz,ktab,ktabi,ktabo,ktabr,hd,qpol)

 use ggwdef
 use dpdef
 use verbose
 
 implicit none
 
 type(dpparam) iparam
 type(mesh) fftmesh
 type(electronic_structure) elecstr
 
 real :: b1(3),b2(3),b3(3)
 integer :: gvec(3,iparam%ngw)
 integer nh
 complex (kind(0.0d0)) :: hd(iparam%nt,iparam%nt)
 integer :: transtab(iparam%nkbz,iparam%nbndv,iparam%nbndc)

      
 real ucvol

 real op(3,3,iparam%nopq)
 integer :: igfft(iparam%npwvec,5,5,5)
!      integer grottb(iparam%npwvec,2,iparam%nop)
 integer grottb(iparam%npwvec,2,iparam%nopq)
 real kbz(3,iparam%nkbz)
 integer ktab(iparam%nkbz), ktabi(iparam%nkbz), ktabo(iparam%nkbz)
 integer :: ktabr(fftmesh%nr,iparam%nkbz)
 integer nqbzem1
 real, allocatable :: qbz(:,:), qpg(:,:), qibz(:,:)
 integer, allocatable :: qtab(:), qtabi(:), qtabo(:)
 real KMK1(3)

      
 real Q0VOL, FAQ, RTW
 real fcc_const
 logical VCUT
 real rlcut

 complex http
 complex, allocatable :: h(:)
 complex, allocatable :: hk(:,:,:,:,:)
 integer fform
 integer nomegaem1, nqibzem1, npweps, npwwfnem1,nbndsem1
 integer nqbzmax
 complex omegaem1(1)
 complex, allocatable :: epsm1q(:,:,:), Wibz(:,:,:)
 complex W(iparam%ngw,iparam%ngw), Wstar(iparam%ngw,iparam%ngw)
 complex Wstardiag(iparam%ngw)

 complex rhxtwg_v(iparam%ngw,iparam%nbnds,iparam%nbnds)
 complex rhxtwg_c(iparam%ngw,iparam%nbnds,iparam%nbnds)  
 complex ctccp(iparam%ngw)
 
 integer igfftg0(iparam%ngw)
 integer ktabrk(fftmesh%nr), ktabrkp(fftmesh%nr)
 integer ig, igp, igr, igpr
 integer ir
 integer ik,ikibz,iik,ikp,ikpibz,iikp
 integer iopk,iopkp
 integer iq,iqbz,iqibz,iopq,iiq
 integer iqbz_0
 integer g0(3)
 integer it, itp
 integer iv, ivp, ic, icp, ivr, icr, ikbz
 integer irv, irc, irvp, ircp


 integer itemp
 complex ctemp
 logical ltemp
 integer istat
 real qpgtemp

 logical, parameter :: TIMEREVERSAL = .false.
 logical HMEMO
 integer lastirec
 integer :: kstart = 1
 integer :: kcstart = 1
 integer sizeofcomplex

 complex, external :: pdtqrc
 integer, external :: lrec
 complex, external :: cdotu
 logical :: WDIAG=.true.
 

 complex, allocatable :: bigomegatwsq(:,:,:)
 complex, allocatable :: omegatw(:,:,:)


! integer, parameter :: qdenommx = 32
 integer qdenom
 integer kmkptab(-3*qdenommx:3*qdenommx, -3*qdenommx:3*qdenommx, &
                  -3*qdenommx:3*qdenommx)


 complex :: wfr2(fftmesh%nr), wfr1(fftmesh%nr)
 integer :: isppol

! NEW for q!=0

 logical, external :: findkp
 integer ikpol, ikppol, ikpolibz, ikppolibz, iikpol, iikppol, iqpol
 integer ktabrkpol(fftmesh%nr), ktabrkppol(fftmesh%nr)
 integer g0pol(3), g0ppol(3), igfftg0pol(iparam%ngw), g0qpol(3),g0tmp(3)
 real kmq(3), kmqp(3), qpol(3)
 real kmk1pol(3)
 real kmq_all(3,iparam%nkbz)
 integer ikpol_all(iparam%nkbz)
 integer g0pol_all(3,iparam%nkbz)
 integer :: icount_qpol=0
 integer iqibzpol,iopqpol,iiqpol
  
 if(iverbose > 2)  call uverbose('b','makeWq')


 if(iverbose > 0)  then
   headline='Preparing W(q)'
   call print_headline()
 endif
 nh = iparam%nt
 
 sizeofcomplex = 2*lrec()     

 hd(:,:) = 0
!Some constants  
 FAQ = 4*PI/(ucvol*iparam%nkbz)
 RTW = (6.*PI**2/(ucvol*iparam%nkbz))**(1./3.)
 Q0VOL = (8.0*PI**3) / (ucvol*iparam%nkbz)
!If q=0 and C=V then set up rho-twiddle(G=0) to reflect an
!analytic integration of q**-2 over the volume element:
!<q**-2> = 7.44 V**(-2/3)   (for fcc cell)
 fcc_const = SQRT(7.44*Q0VOL**(-2.0/3.0))

!Average of (q+q')**-2 integration for head of Coulomb matrix 
!INTRTW(QL) = (2*PI*RTW + PI*(RTW**2/QL-QL)*LOG((QL+RTW)/(QL-RTW))) * & 
!             (ucvol*iparam%nkbz)/(2*PI)**3. * QL*QL


 write(uout,*)iparam%nkbz,iparam%nbndv,iparam%nbndc,nh
 it = 0
 do ikbz = 1, iparam%nkbz 
  do ivr = 1, iparam%nbndv
   do icr = 1, iparam%nbndc
     it = it + 1 
     transtab(ikbz,ivr,icr) = it
   enddo 
  enddo 
 enddo 
 if(it/=iparam%nt) stop 'internal error on transtab'

! NEW for q!=0
! prepares all kmq 
 do ik = 1, iparam%nkbz
  kmq_all(:,ik) = kbz(:,ik) - qpol(:)
  if(.not. findkp(iparam%nkbz,kbz,ikpol_all(ik),kmq_all(:,ik),g0pol_all(:,ik))) then
    write(uout,*) 'warning: cchi0 2: k" = k - q - G0 not found;'
    stop 'qpol not found'
  endif
 enddo

!-----------------------------------------------------------------------

 lastirec = (nh*nh + nh)/2


 inquire(file='in.exh',exist=ltemp)
 if(ltemp) then  ! if already exists
  open(55,file='in.exh',access='direct',recl=sizeofcomplex)
  read(55,rec=lastirec+1) itemp
  if(itemp == nh) then
   if(iverbose > 0) write(uout,*) 'reading existing (resonant) W(q) from in.exh'
   read(55,rec=lastirec+2) kstart
   if(kstart < iparam%nkbz) then
     kstart = kstart + 1
     if(iverbose > 1) write(uout,*) 'starting from k ', kstart
   else
     if(iverbose > 1) write(uout,*) 'reading'
     do itp = 1, nh
       do it = 1, itp
	 ir = it + itp * (itp - 1) / 2
	 read(55,rec=ir) ctemp
	 hd(it,itp) = ctemp
	 hd(itp,it) = conjg(ctemp)
       enddo
       hd(itp,itp) = real(hd(itp,itp))
     enddo
     close(55)
     if(iverbose > 1) write(uout,*) 'read'
     return
   endif
  else
   write(uout,*) 'ERROR: wrong in.exh'
   stop 'wrong in.exh'
  endif
 else
   close(55)
   open(55,file='out.exh',access='direct',recl=sizeofcomplex)
   kstart = 1
 endif
     
!----------------------------------------------------------------------
!     make W
!----------------------------------------------------------------------

 call testscr('in.em1',6,nqibzem1,nomegaem1,npweps,npwwfnem1,nbndsem1, &
              fform,iparam%mpilocal)
 allocate(qibz(3,nqibzem1))
 allocate(epsm1q(iparam%ngw,iparam%ngw,nqibzem1),stat=istat)
if (fform==1002) then
   nomegaem1=1
   call rdscr('in.em1',6,iparam%ngw,nqibzem1,nomegaem1,qibz,omegaem1,epsm1q, iparam%mpilocal)
 else if(fform==1003) then
   nomegaem1=1
   write(uout,*)'epsilon file = epsm1'
   allocate(bigomegatwsq(iparam%npwmat,iparam%npwmat,nqibzem1),stat=istat)
   if(istat .ne. 0) stop 'out of memory: epsm1q'
   allocate(omegatw(iparam%npwmat,iparam%npwmat,nqibzem1),stat=istat)
   if(istat .ne. 0) stop 'out of memory: omegatw'
   call rdepsm1('in.em1',6,iparam%npwmat,nqibzem1,qibz,epsm1q,&
                bigomegatwsq,omegatw)
   deallocate(bigomegatwsq,omegatw)
 endif


 nqbzmax=nqibzem1*iparam%nopq*iparam%ninvq
 allocate(qbz(3,nqbzmax),qtab(nqbzmax),qtabo(nqbzmax),qtabi(nqbzmax))
 call identq_old(qibz,nqibzem1,nqbzmax,op,iparam%nopq,iparam%ninvq,qbz,qtab,qtabi,qtabo, &
             nqbzem1)
 allocate(qpg(iparam%npwvec,nqibzem1))
 iparam%nqibz=nqibzem1
 call clcqpg(iparam,gvec,qibz,b1,b2,b3,qpg)
 deallocate(qibz) 

 allocate(Wibz(iparam%ngw,iparam%ngw,nqibzem1),stat=istat)
 do iqibz = 1, nqibzem1
  do ig = 1, iparam%ngw
    do igp = 1, iparam%ngw
      Wibz(ig,igp,iqibz) = epsm1q(ig,igp,iqibz) / &
                           (qpg(ig,iqibz) * qpg(igp,iqibz))
    enddo
  enddo
 enddo
 deallocate(epsm1q)

 inquire(file='cutoff',exist=VCUT)
 if(VCUT) then
 open(97,file='cutoff')
 read(97,*) rlcut
 close(97)
 write(uout,'("cut-off on the coulombian interaction ",f6.2)') rlcut

 do iq = 1, nqibzem1
  qpg(1,iq) = sqrt(2.0)/rlcut
  do ig = 2, iparam%ngw
    qpgtemp = qpg(ig,iq)
    qpg(ig,iq) = qpgtemp * sqrt(1.0/(1.0-cos(qpgtemp*rlcut)))
  enddo
 enddo
 endif

      
!----------------------------------------------------------------------
!     try to allocate excitonic hamiltonian
!----------------------------------------------------------------------

 allocate(hk(iparam%nbndv,iparam%nbndc,iparam%nkbz,iparam%nbndv,iparam%nbndc),stat=istat)
 if(istat> 0) stop 'buy more memory'
 if(iverbose > 1) write(uout,*) '    memory size ', lastirec, ' sizeof(complex)'
 allocate(h(lastirec),stat=istat)
 if(istat > 0) then
   HMEMO = .false.
   if(iverbose > 1)  write(uout,*) '    not enough memory, write h to disk'
 else
   HMEMO = .true.
   if(iverbose > 1) write(uout,*) '    matrix W_ttp allocated'
   kstart = 1
   kcstart = 1
   h(:) = (0.0,0.0)
 endif


!----------------------------------------------------------------------
!     resonant hamiltonian
!----------------------------------------------------------------------
 
  if(iverbose > 0)  then
   write(uout,*) '    calculate resonant W'
  endif
! append the size of hamiltonian matrix to check
  write(55,rec=lastirec+1) nh
  

!-----------------------------------------------------------------------
!     coulomb term
!-----------------------------------------------------------------------
  
 if(WDIAG) then
   if(iverbose > 0) write(uout,*) '    W diagonal only'
 else
   if(iverbose > 0) write(uout,*) '    W full'
 endif
 write(uout,*)

! NEW for q!=0
! identify q == 0
 iqbz_0 = 0
 do iqbz = 1, nqbzem1
  if(all(abs(qbz(:,iqbz))<1.0e-3)) iqbz_0 = iqbz
 enddo
 if(iqbz_0 == 0) then
  write(uout,*) '**ERROR excheig 1: q=0 not found'
  stop 'excheig 1: q=0 not found'
 endif
! end NEW for q!=0

 call qtabg0(nqbzem1,qbz,qdenom,kmkptab)
 deallocate(qbz)

 do ik = kstart, iparam%nkbz        ! loop over k
   if (iverbose >1)  write(uout,*)'    status ',ik,'/',iparam%nkbz,' completed'
       ikibz = ktab(ik)
       iik = (3-ktabi(ik))/2
       iopk = ktabo(ik)
       ktabrk(:) = ktabr(:,ik)
!      if(iik==2) cycle             ! time-reversal or not       
! NEW for q!=0

!        kmq(:) = kbz(:,ik) - qpol(:)
!!       find k' = k - q - G0 and G0 also
!        if(.not. findkp(iparam%nkbz,kbz,ikpol,kmq,g0pol)) then
!          write(uout,*) 'warning: cchi0 2: k" = k - q - G0 not found;'
!          write(uout,1927) ik, (kbz(i,ik),i=1,3), (kmq(i),i=1,3)
!1927      format(' ik = ',i4,' k = ',3f12.6/' k - q = ',6x,3f12.6)
!          stop'qpol not found'
!        endif

          kmq(:) = kmq_all(:,ik)
          ikpol =  ikpol_all(ik)
          g0pol(:) = g0pol_all(:,ik)

       
        ikpolibz = ktab(ikpol)
        iikpol = (3-ktabi(ikpol))/2
        ktabrkpol(:) = ktabr(:,ikpol)

! end NEW for q!=0


       do ikp = ik, iparam%nkbz           ! loop over kp
        ikpibz = ktab(ikp)
        iikp = (3-ktabi(ikp))/2
        iopkp = ktabo(ikp)
        ktabrkp(:) = ktabr(:,ikp)

!      Find q = K - KP - G0
       kmk1(:) = kbz(:,ik) - kbz(:,ikp)

  call iqg0(kmk1,qdenom,kmkptab,iq,g0)

       iqibz = qtab(iq) ; iopq = qtabo(iq) ; iiq = (3-qtabi(iq))/2
       igfftg0(1:iparam%ngw) = igfft(1:iparam%ngw,3+g0(1),3+g0(2),3+g0(3))

! NEW for q!=0

!        kmqp(:) = kbz(:,ikp) - qpol(:)
!!       find k' = k - q - G0 and G0 also
!        if(.not. findkp(iparam%nkbz,kbz,ikppol,kmqp,g0ppol)) then
!          write(uout,*) 'warning: cchi0 2: k" = k - q - G0 not found;'
!          write(uout,1928) ik, (kbz(i,ik),i=1,3), (kmq(i),i=1,3)
!1928      format(' ik = ',i4,' k = ',3f12.6/' k - q = ',6x,3f12.6)
!          stop'qpol not found'
!        endif

          kmqp(:) = kmq_all(:,ikp)
          ikppol =  ikpol_all(ikp)
          g0ppol(:) = g0pol_all(:,ikp)
       
        ikppolibz = ktab(ikppol)
        iikppol = (3-ktabi(ikppol))/2
        ktabrkppol(:) = ktabr(:,ikppol)

        
!      Find q = K - KP - G0
       kmk1pol(:) = kbz(:,ikpol) - kbz(:,ikppol)

  call iqg0(kmk1pol,qdenom,kmkptab,iqpol,g0qpol)

       iqibzpol = qtab(iqpol) ; iopqpol = qtabo(iqpol) ; iiqpol = (3-qtabi(iqpol))/2
       if (iqpol /= iq) stop 'iqpol != iq'

       g0tmp(:) =  g0qpol(:)   + ( g0pol(:) - g0ppol(:) )
       if (any(abs(g0tmp(:)) > 2 ) ) then
          icount_qpol=icount_qpol + 1
          write(uout,*) 'WARNING g0qpol greater than 2', g0qpol(:) 
          write(uout,*) 'this has happend ', icount_qpol,' times' 
          g0tmp(:) = 0
        endif
       igfftg0pol(1:iparam%ngw) = igfft(1:iparam%ngw,3+g0tmp(1),3+g0tmp(2),3+g0tmp(3))

! end NEW for q!=0

!      create W (omega=0)
       if(iiq == 1) then
         do ig = 1, iparam%ngw
           igr = grottb(ig,iiq,iopq)
           do igp = 1, iparam%ngw
             igpr = grottb(igp,iiq,iopq)
             W(igr,igpr) = Wibz(ig,igp,iqibz)
           enddo
         enddo
       else
         do ig = 1, iparam%ngw
           igr = grottb(ig,iiq,iopq)
           do igp = 1, iparam%ngw
             igpr = grottb(igp,iiq,iopq)
             W(igr,igpr) = conjg(Wibz(ig,igp,iqibz))
           enddo
         enddo
       endif
       Wstar(:,:) = conjg(W(:,:))
       do ig = 1, iparam%ngw
         Wstardiag(ig) = Wstar(ig,ig)
       enddo

!for spin compatibility
   isppol =1 
       
       
        do iv = iparam%lomo, iparam%homo         !      loop over band V
         do ivp = iparam%lomo, iparam%homo       !       Loop over band VP

                 
              call get_wfr(iparam,elecstr,elecstr,fftmesh,ivp,ikppolibz,iv,ikpolibz,isppol,wfr1,wfr2)
              call rho_tw_g(iparam%ngw,fftmesh%nr,fftmesh%nr,&
     &                     fftmesh%nrr(1),fftmesh%nrr(1),fftmesh%nrr(2),fftmesh%nrr(3),&
     &                    igfftg0pol,&
     &                    wfr1,iikppol,ktabrkppol,&
     &                    wfr2,iikpol,ktabrkpol,fftmesh%menoplan,&
     &                    rhxtwg_v(:,ivp,iv))

            if (ik==ikp) then
                    if (iv==ivp) then 
                      rhxtwg_v(1,ivp,iv) = fcc_const * qpg(1,iqbz_0)
                    else  
                      rhxtwg_v(1,ivp,iv) = 0
                    endif
            endif
         enddo ! ivp
        enddo ! iv

        do icp = iparam%lumo, iparam%nbnds     !      Loop over band C_prime
         do ic = iparam%lumo,  iparam%nbnds    !       Loop over band C

! NEW for q!=0 -- now the indeces inside rho_tw_g are different
            call get_wfr(iparam,elecstr,elecstr,fftmesh,icp,ikpibz,ic,ikibz,isppol,wfr1,wfr2)
            call rho_tw_g(iparam%ngw,fftmesh%nr,fftmesh%nr,&
            fftmesh%nrr(1),fftmesh%nrr(1),fftmesh%nrr(2),fftmesh%nrr(3),&
     &                    igfftg0,&
     &                    wfr1,iikp,ktabrkp,&
     &                    wfr2,iik,ktabrk,fftmesh%menoplan,&
     &                    rhxtwg_c(:,icp,ic))


            if (ik==ikp) then
                    if (ic==icp) then 
                      rhxtwg_c(1,icp,ic) = fcc_const * qpg(1,iqbz_0)
                    else  
                      rhxtwg_c(1,icp,ic) = 0
                    endif
            endif
! end NEW for q!=0
         enddo ! ic
        enddo ! icp

!        write(89,*)ik,ikp,rhxtwg_v(:,iparam%lomo:iparam%homo,iparam%lomo:iparam%homo)
!        write(90,*)ik,ikp,rhxtwg_c(:,iparam%lumo:iparam%nbnds,iparam%lumo:iparam%nbnds)
!       if( (ik==1.and.ikp==1) .or.&
!     &    (ik==1.and.ikp==5) .or.&
!     &    (ik==1.and.ikp==7) ) then
!        write(89,'(2(i4,2x),2x,4(f7.5,2x))') ik,ikp,rhxtwg_v(1:2,3,3)
!        write(89,'(2(i4,2x),2x,4(f7.5,2x))') ik,ikp,rhxtwg_v(1:2,3,4)
!        write(90,'(2(i4,2x),2x,4(f7.5,2x))') ik,ikp,rhxtwg_c(1:2,5,5)
!        write(90,'(2(i4,2x),2x,4(f7.5,2x))') ik,ikp,rhxtwg_c(1:2,5,6)
!       endif


!      now calculate:
!      sum_G,G' rhotw(G)* W(G,G') rhotw(G')
      do ic = iparam%lumo, iparam%nbnds
        irc = ic - iparam%lumo + 1
        do icp = iparam%lumo,  iparam%nbnds
         ircp = icp - iparam%lumo + 1

!        sum_GG' rho_c'c*(G) W(G,G') rho_v'v(G')
!        first sum on G: sum_G rho_c'c(G) W*(G,G')    (W conjgated)
         if(WDIAG) then
           ctccp(:) = rhxtwg_c(:,icp,ic) * Wstardiag(:)
         else
           ctccp(:) = matmul(rhxtwg_c(:,icp,ic),Wstar(:,:))
         endif
         
         do iv = iparam%lomo, iparam%homo
          irv = iv - iparam%lomo + 1
          
          
          do ivp = iparam%lomo, iparam%homo
            irvp = ivp - iparam%lomo + 1


!           then sum on G': sum_G' (rho_c'c(G) W*(G,G'))* rho_v'v(G')
            http = -faq * dot_product(ctccp(:),rhxtwg_v(:,ivp,iv))
            hk(irv,irc,ikp,irvp,ircp) = http
                          
          enddo ! ivp
         enddo ! iv
        enddo ! icp
       enddo ! ic
       
      enddo ! ikp
      
!      now store hamiltonian in memory or disc
       do ikp = ik, iparam%nkbz           ! loop over kp
       if(HMEMO) then
         do irc = 1, iparam%nbndc
           do ircp = 1, iparam%nbndc
        do irv = 1, iparam%nbndv
          it = transtab(ik,irv,irc)
          do irvp = 1, iparam%nbndv
            itp = transtab(ikp,irvp,ircp)
            hd(it,itp)=hk(irv,irc,ikp,irvp,ircp)
            hd(itp,it)=conjg(hk(irv,irc,ikp,irvp,ircp))
            if(itp >= it) then
             ir = it + itp * (itp - 1) / 2
             h(ir) = hk(irv,irc,ikp,irvp,ircp)
            else
             ir = itp + it * (it - 1) / 2
             h(ir) = conjg(hk(irv,irc,ikp,irvp,ircp))
            endif
           enddo
          enddo
         enddo
        enddo
       else
        do irvp = 1, iparam%nbndv
         do ircp = 1, iparam%nbndc
          itp = transtab(ikp,irvp,ircp)
          do irv = 1, iparam%nbndv
           do irc = 1, iparam%nbndc
            it = transtab(ik,irv,irc)
            if(itp >= it) then
             ir = it + itp * (itp - 1) / 2
             write(55,rec=ir) hk(irv,irc,ikp,irvp,ircp)
            else
             ir = itp + it * (it - 1) / 2
             write(55,rec=ir) conjg(hk(irv,irc,ikp,irvp,ircp))
            endif
           enddo
          enddo
         enddo
        enddo 
       endif
       enddo ! ikp
       
      if(.not. HMEMO) then
        write(55,rec=lastirec+2) ik
      endif


      enddo ! ik

 if(iverbose > 0) then
  write(uout,*) '             done '
  write(uout,*)
 endif
 if(HMEMO .and. iverbose > 1) then
   write(uout,'(" -W(1,1) = [eV] ",f7.2)') real(h(1))*HARTREE
   write(uout,'(" -W(3,2) = [eV] ",f7.2)') real(hd(3,2))*HARTREE
   write(uout,*)
 endif
 
 deallocate(hk,qtab,qtabo,qtabi,Wibz,qpg)

!----------------------------------------------------------------------
!     write hamiltonian 
!----------------------------------------------------------------------

 if(HMEMO) then
   if (iverbose > 0) then
    write(uout,*) '    write resonant W_ttp(q) on disk (out.exh)'
   endif
   do ir = 1, lastirec
     write(55,rec=ir) h(ir)
   enddo
   write(55,rec=lastirec+1) nh
   write(55,rec=lastirec+2) iparam%nkbz
 endif
 
 close(55)
 deallocate(h)      

 if(iverbose > 0)  then
  call print_footline()
 endif
 if(iverbose > 2)  call uverbose('e','makeWq')
end
!!***

!{\src2tex{textfont=tt}}
!!****p* creation_Wtt/makeWqcoup
!! NAME
!! makeWqcoup
!!
!! FUNCTION
!! creation of W matrix elements for mt kernel at q=/0
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_Wtt
!!
!! CHILDREN
!! testscr, rdscr, rdepsm1, identq_old, clcqpg, qtabg0, iqg0, get_wfr,rho_tw_g 
!!
!! SOURCE
subroutine makeWqcoup(iparam,elecstr,b1,b2,b3,ucvol,gvec,op,grottb,igfft,fftmesh,&
 kbz,ktab,ktabi,ktabo,ktabr,&
 hd,qpol)
 use ggwdef
 use dpdef
 use verbose
 
 implicit none

 type(dpparam) iparam
 type(mesh) fftmesh
 type(electronic_structure) elecstr

 real :: b1(3),b2(3),b3(3)
 integer :: gvec(3,iparam%ngw)
 integer nh
 complex (kind(0.0d0)) :: hd(iparam%nt,iparam%nt)
 integer :: transtab(iparam%nkbz,iparam%nbndv,iparam%nbndc)

      
 real ucvol

 real op(3,3,iparam%nopq)
      integer :: igfft(iparam%npwvec,5,5,5)
!      integer grottb(iparam%npwvec,2,iparam%nop)
      integer grottb(iparam%npwvec,2,iparam%nopq)
      real  kbz(3,iparam%nkbz)
      integer ktab(iparam%nkbz), ktabi(iparam%nkbz), ktabo(iparam%nkbz)
      integer :: ktabr(fftmesh%nr,iparam%nkbz)
      integer nqbzem1
 real, allocatable :: qbz(:,:), qpg(:,:), qibz(:,:)
 integer, allocatable :: qtab(:), qtabi(:), qtabo(:)
      real KMK1(3)

      
      real Q0VOL, FAQ, RTW
      real fcc_const
      logical VCUT
      real rlcut

      complex http
      complex, allocatable :: h(:)
      complex, allocatable :: hk(:,:,:,:,:)
      integer fform
      integer nomegaem1, nqibzem1, npweps, npwwfnem1,nbndsem1
      integer nqbzmax
      complex omegaem1(1)
 complex, allocatable :: epsm1q(:,:,:), Wibz(:,:,:)
 complex W(iparam%ngw,iparam%ngw), Wstar(iparam%ngw,iparam%ngw)
 complex Wstardiag(iparam%ngw)

      complex rhxtwg_v(iparam%ngw,iparam%nbnds,iparam%nbnds)
      complex rhxtwg_c(iparam%ngw,iparam%nbnds,iparam%nbnds)  
      complex ctccp(iparam%ngw)
      
      integer igfftg0(iparam%ngw)
      integer ktabrk(fftmesh%nr), ktabrkp(fftmesh%nr)
      integer ig, igp, igr, igpr
      integer ir
      integer ik,ikibz,iik,ikp,ikpibz,iikp
      integer iopk,iopkp
      integer iq,iqbz,iqibz,iopq,iiq
      integer iqbz_0
      integer g0(3)
      integer it, itp
      integer iv, ivp, ic, icp, ivr, icr, ikbz
      integer irv, irc, irvp, ircp


      integer itemp
      complex ctemp
      logical ltemp
      integer istat
      real qpgtemp

      logical, parameter :: TIMEREVERSAL = .false.
      logical HMEMO
      integer lastirec
      integer :: kstart = 1
      integer :: kcstart = 1
      integer sizeofcomplex

      complex, external :: pdtqrc
      integer, external :: lrec
      complex, external :: cdotu
      logical :: WDIAG=.true.
      
      complex, allocatable :: bigomegatwsq(:,:,:)
      complex, allocatable :: omegatw(:,:,:)

!  integer, parameter :: qdenommx = 32
  integer qdenom
  integer kmkptab(-3*qdenommx:3*qdenommx, -3*qdenommx:3*qdenommx, &
                  -3*qdenommx:3*qdenommx)


 complex :: wfr2(fftmesh%nr), wfr1(fftmesh%nr)
 integer :: isppol



! NEW for q!=0

 logical, external :: findkp
 integer ikpol, ikppol, ikpolibz, ikppolibz, iikpol, iikppol, iqpol
 integer ktabrkpol(fftmesh%nr), ktabrkppol(fftmesh%nr)
 integer g0pol(3), g0ppol(3), igfftg0pol(iparam%ngw), g0qpol(3),g0tmp(3)
 real kmq(3), kmqp(3), qpol(3)

 real kmq_all(3,iparam%nkbz)
 integer ikpol_all(iparam%nkbz)
 integer g0pol_all(3,iparam%nkbz)

 real kmk1pol(3)
 integer :: icount_qpol=0
 integer iqibzpol,iopqpol,iiqpol
  

! end  NEW for q!=0

 if(iverbose > 2)  call uverbose('b','makeWqcoupl')

!-----------------------------------------------------------------------
      nh = iparam%nt
 
      sizeofcomplex = 2*lrec()     

      hd(:,:) = 0
!     Some constants  
      FAQ = 4*PI/(UCVOL*iparam%nkbz)
      RTW = (6.*PI**2/(UCVOL*iparam%nkbz))**(1./3.)
      Q0VOL = (8.0*PI**3) / (UCVOL*iparam%nkbz)
!     If q=0 and C=V then set up rho-twiddle(G=0) to reflect an
!     analytic integration of q**-2 over the volume element:
!     <q**-2> = 7.44 V**(-2/3)   (for fcc cell)
      fcc_const = SQRT(7.44*Q0VOL**(-2.0/3.0))

!     Average of (q+q')**-2 integration for head of Coulomb matrix 
!     INTRTW(QL) = (2*PI*RTW + PI*(RTW**2/QL-QL)*LOG((QL+RTW)/(QL-RTW)))
!    &              * (UCVOL*iparam%nkbz)/(2*PI)**3. * QL*QL


      it = 0
      do ikbz = 1, iparam%nkbz 
        do ivr = 1, iparam%nbndv
          do icr = 1, iparam%nbndc
              it = it + 1 
              transtab(ikbz,ivr,icr) = it
          enddo 
        enddo 
      enddo 
      if(it/=iparam%nt) stop 'internal error on transtab'

! NEW for q!=0
!    prepares all kmq
               do ik = 1, iparam%nkbz
                kmq_all(:,ik) = kbz(:,ik) - qpol(:)
                if(.not. findkp(iparam%nkbz,kbz,ikpol_all(ik),kmq_all(:,ik),g0pol_all(:,ik))) then
                 write(uout,*) 'warning: cchi0 2: k" = k - q - G0 not found;'
                 stop 'qpol not found'
                endif
               enddo



!-----------------------------------------------------------------------

 lastirec = (nh*nh + nh)/2

 write(uout,*) 'excitonic hamiltonian'
 inquire(file='in.exc',exist=ltemp)
 if(ltemp) then  ! if already exists
  open(56,file='in.exc',access='direct',recl=sizeofcomplex)
  read(56,rec=lastirec+1) itemp
  if(itemp == nh) then
   write(uout,*) 'reading coupling exciton hamiltonian from in.exc'
   read(56,rec=lastirec+2) kstart
   if(kstart < iparam%nkbz) then
     kstart = kstart + 1
     write(uout,*) 'starting from k ', kstart
   else
     write(uout,*) 'reading'
     do itp = 1, nh
       do it = 1, itp
         ir = it + itp * (itp - 1) / 2
         read(56,rec=ir) ctemp
         hd(it,itp) = ctemp
         hd(itp,it) = conjg(ctemp)
       enddo
       hd(itp,itp) = real(hd(itp,itp))
     enddo
     close(56)
     write(uout,*) 'read'
     return
   endif
  else
   write(uout,*) 'ERROR: wrong in.exc'
   stop 'wrong in.exc'
  endif
 else
   close(56)
   open(56,file='out.exc',access='direct',recl=sizeofcomplex)
   write(uout,*) 'coupling term'
   kstart = 1
 endif
      
!----------------------------------------------------------------------
!     make W coupling
!----------------------------------------------------------------------

 call testscr('in.em1',6,nqibzem1,nomegaem1,npweps,npwwfnem1,nbndsem1,fform,iparam%mpilocal)
 allocate(qibz(3,nqibzem1))
 allocate(epsm1q(iparam%ngw,iparam%ngw,nqibzem1),stat=istat)
if (fform==1002) then
   nomegaem1=1
   call rdscr('in.em1',6,iparam%ngw,nqibzem1,nomegaem1,qibz,omegaem1,epsm1q,iparam%mpilocal)
 else if(fform==1003) then
   nomegaem1=1
   write(uout,*)'epsilon file = epsm1'
   allocate(bigomegatwsq(iparam%npwmat,iparam%npwmat,nqibzem1),stat=istat)
   if(istat .ne. 0) stop 'out of memory: epsm1q'
   allocate(omegatw(iparam%npwmat,iparam%npwmat,nqibzem1),stat=istat)
   if(istat .ne. 0) stop 'out of memory: omegatw'
   call rdepsm1('in.em1',6,iparam%npwmat,nqibzem1,qibz,epsm1q,&
                bigomegatwsq,omegatw,iparam%mpilocal)
   deallocate(bigomegatwsq,omegatw)
 endif

 nqbzmax=nqibzem1*iparam%nopq*iparam%ninvq
 allocate(qbz(3,nqbzmax),qtab(nqbzmax),qtabo(nqbzmax),qtabi(nqbzmax))
 call identq_old(qibz,nqibzem1,nqbzmax,op,iparam%nopq,iparam%ninvq,qbz,qtab,qtabi,qtabo,nqbzem1)
 allocate(qpg(iparam%ngw,nqibzem1))
 call clcqpg(iparam%ngw,gvec,qibz,nqibzem1,b1,b2,b3,qpg)
 deallocate(qibz)

 allocate(Wibz(iparam%ngw,iparam%ngw,nqibzem1),stat=istat)
        do iqibz = 1, nqibzem1
          do ig = 1, iparam%ngw
            do igp = 1, iparam%ngw
              Wibz(ig,igp,iqibz) = epsm1q(ig,igp,iqibz) / &
                                  (qpg(ig,iqibz) * qpg(igp,iqibz))
            enddo
          enddo
        enddo
 deallocate(epsm1q)

      inquire(file='cutoff',exist=VCUT)
      if(VCUT) then
        open(97,file='cutoff')
        read(97,*) rlcut
        close(97)
        write(uout,'("cut-off on the coulombian interaction ",f6.2)') rlcut
        
        do iq = 1, nqibzem1
          qpg(1,iq) = sqrt(2.0)/rlcut
          do ig = 2, iparam%ngw
            qpgtemp = qpg(ig,iq)
            qpg(ig,iq) = qpgtemp * sqrt(1.0/(1.0-cos(qpgtemp*rlcut)))
          enddo
        enddo
      endif

      

      
!----------------------------------------------------------------------
!     try to allocate excitonic hamiltonian
!----------------------------------------------------------------------

      write(uout,*) 'try to allocate W_ttp'
      allocate(hk(iparam%nbndv,iparam%nbndc,iparam%nkbz,iparam%nbndv,iparam%nbndc),stat=istat)
      if(istat> 0) stop 'buy more memory'
      write(uout,*) 'memory size ', lastirec, ' sizeof(complex)'
      allocate(h(lastirec),stat=istat)
      if(istat > 0) then
        HMEMO = .false.
        write(uout,*) 'not enough memory, write h to disk'
      else
        HMEMO = .true.
        write(uout,*) 'allocated'
        kstart = 1
        kcstart = 1
        h(:) = (0.0,0.0)
      endif


!----------------------------------------------------------------------
!     coupling hamiltonian
!----------------------------------------------------------------------
 
      write(uout,*) 'calculate coupling W'

      write(uout,*) 'disk space ', lastirec, ' sizeof(complex)'
!     append the size of hamiltonian matrix to check
      write(56,rec=lastirec+1) nh
      

!-----------------------------------------------------------------------
!     coulomb term
!-----------------------------------------------------------------------
            
 if(WDIAG) then
   write(uout,*) 'W diagonal only'
 else
   write(uout,*) 'W full'
 endif
 write(uout,*)

! NEW for q!=0
 !     identify q == 0
      iqbz_0 = 0
      do iqbz = 1, nqbzem1
        if(all(abs(qbz(:,iqbz))<1.0e-3)) iqbz_0 = iqbz
      enddo
      if(iqbz_0 == 0) then
        write(uout,*) '**ERROR excheig 1: q=0 not found'
        stop 'excheig 1: q=0 not found'
      endif
! end NEW for q!=0



  call qtabg0(nqbzem1,qbz,qdenom,kmkptab)
  deallocate(qbz)


 do ik = kstart, iparam%nkbz        ! loop over k
!   write(unit=6,fmt='(t3,"status:",i4,"/",i5," completed")',advance='no') ik-1, iparam%nkbz
!       write(0,*)'status ',ik,'/',iparam%nkbz,' completed'
       write(uout,*)'status ',ik,'/',iparam%nkbz,' completed'
       ikibz = ktab(ik)
       iik = (3-ktabi(ik))/2
       iopk = ktabo(ik)
       ktabrk(:) = ktabr(:,ik)
!      if(iik==2) cycle             ! time-reversal or not       

! NEW for q!=0

!        kmq(:) = kbz(:,ik) - qpol(:)
!!       find k' = k - q - G0 and G0 also
!        if(.not. findkp(iparam%nkbz,kbz,ikpol,kmq,g0pol)) then
!          write(uout,*) 'warning: cchi0 2: k" = k - q - G0 not found;'
!          write(uout,1927) ik, (kbz(i,ik),i=1,3), (kmq(i),i=1,3)
!1927      format(' ik = ',i4,' k = ',3f12.6/' k - q = ',6x,3f12.6)
!          stop'qpol not found'
!        endif

          kmq(:) = kmq_all(:,ik)
          ikpol =  ikpol_all(ik)
          g0pol(:) = g0pol_all(:,ik)

       
        ikpolibz = ktab(ikpol)
        iikpol = (3-ktabi(ikpol))/2
        ktabrkpol(:) = ktabr(:,ikpol)

! end NEW for q!=0


       do ikp = ik, iparam%nkbz           ! loop over kp
        ikpibz = ktab(ikp)
        iikp = (3-ktabi(ikp))/2
        iopkp = ktabo(ikp)
        ktabrkp(:) = ktabr(:,ikp)

!        kmqp(:) = kbz(:,ikp) + qpol(:)
!!       find k' = k - q - G0 and G0 also
!        if(.not. findkp(iparam%nkbz,kbz,ikppol,kmqp,g0ppol)) then
!          write(uout,*) 'warning: cchi0 2: k" = k - q - G0 not found;'
!          write(uout,1928) ik, (kbz(i,ik),i=1,3), (kmq(i),i=1,3)
!1928      format(' ik = ',i4,' k = ',3f12.6/' k - q = ',6x,3f12.6)
!          stop'qpol not found'
!        endif

          kmqp(:) = kmq_all(:,ikp)
          ikppol =  ikpol_all(ikp)
          g0ppol(:) = g0pol_all(:,ikp)

       
        ikppolibz = ktab(ikppol)
        iikppol = (3-ktabi(ikppol))/2
        ktabrkppol(:) = ktabr(:,ikppol)

!      Find q = K - KP - G0
! following line changed with respect to resonant case
       kmk1(:) = kbz(:,ik) - kbz(:,ikp)
       
       call iqg0(kmk1,qdenom,kmkptab,iq,g0)

       iqibz = qtab(iq) ; iopq = qtabo(iq) ; iiq = (3-qtabi(iq))/2
       igfftg0(1:iparam%ngw) = igfft(1:iparam%ngw,3+g0(1),3+g0(2),3+g0(3))

! NEW for q!=0
        
!      Find q = K - KP - G0
! following line changed with respect to resonant case
       kmk1pol(:) = kbz(:,ikpol) - kbz(:,ikppol)
!       kmk1pol(:) = kbz(:,ikpol) + g0pol(:) - kbz(:,ikppol) - g0ppol(:)
 
       call iqg0(kmk1pol,qdenom,kmkptab,iqpol,g0qpol)

       iqibzpol = qtab(iqpol) ; iopqpol = qtabo(iqpol) ; iiqpol = (3-qtabi(iqpol))/2
       if (iqpol /= iq) stop 'iqpol != iq'

       g0tmp(:) =  g0qpol(:)   + ( g0pol(:) - g0ppol(:) )
       if (any(abs(g0tmp(:)) > 2 ) ) then
          icount_qpol=icount_qpol + 1
          write(uout,*) 'WARNING g0qpol greater than 2', g0tmp(:) 
          write(uout,*) 'this has happend ', icount_qpol,' times' 
          g0tmp(:) = 0
        endif
       igfftg0pol(1:iparam%ngw) = igfft(1:iparam%ngw,3+g0tmp(1),3+g0tmp(2),3+g0tmp(3))


! end NEW for q!=0

!      create W (omega=0)
       if(iiq == 1) then
         do ig = 1, iparam%ngw
           igr = grottb(ig,iiq,iopq)
           do igp = 1, iparam%ngw
             igpr = grottb(igp,iiq,iopq)
             W(igr,igpr) = Wibz(ig,igp,iqibz)
           enddo
         enddo
       else
         do ig = 1, iparam%ngw
           igr = grottb(ig,iiq,iopq)
           do igp = 1, iparam%ngw
             igpr = grottb(igp,iiq,iopq)
             W(igr,igpr) = conjg(Wibz(ig,igp,iqibz))
           enddo
         enddo
       endif
       Wstar(:,:) = conjg(W(:,:))
       do ig = 1, iparam%ngw
         Wstardiag(ig) = Wstar(ig,ig)
       enddo

! for spin compatibility
     isppol =1
       
! new indeces for coupling       
        do iv = iparam%lomo, iparam%homo         !      loop over band V
         do icp = iparam%lumo, iparam%nbnds      !       Loop over band CP

            
            call get_wfr(iparam,elecstr,elecstr,fftmesh,icp,ikppolibz,iv,ikpolibz,isppol,wfr1,wfr2)

            call rho_tw_g(iparam%ngw,fftmesh%nr,fftmesh%nr,&
     &                    fftmesh%nrr(1),fftmesh%nrr(1),fftmesh%nrr(2),fftmesh%nrr(3),&
     &                    igfftg0pol,&
     &                    wfr1,iikppol,ktabrkppol,&
     &                    wfr2,iikpol,ktabrkpol,fftmesh%menoplan,&
     &                    rhxtwg_v(:,icp,iv))

!            call rho_tw_g(iparam%ngw,fftmesh%nr,fftmesh%nr,&
!            fftmesh%nrr(1),fftmesh%nrr(1),fftmesh%nrr(2),fftmesh%nrr(3),&
!     &                    igfftg0pol,&
!     &                    wfr(:,icp,ikppolibz),iikppol,ktabrkppol,&
!     &                    wfr(:,iv,ikpolibz),iikpol,ktabrkpol,&
!     &                    rhxtwg_v(:,icp,iv))

            if (ik==ikp) then
                    if (iv==icp) then 
                      rhxtwg_v(1,icp,iv) = fcc_const * qpg(1,iqbz_0)
                    else  
                      rhxtwg_v(1,icp,iv) = 0
                    endif
            endif
         enddo ! icp
        enddo ! iv

! new indeces for coupling       
        do ivp = iparam%lomo, iparam%homo      !      Loop over band VP
         do ic = iparam%lumo,  iparam%nbnds    !       Loop over band C

            call get_wfr(iparam,elecstr,elecstr,fftmesh,ivp,ikpibz,ic,ikibz,isppol,wfr1,wfr2)

            call rho_tw_g(iparam%ngw,fftmesh%nr,fftmesh%nr,&
     &                    fftmesh%nrr(1),fftmesh%nrr(1),fftmesh%nrr(2),fftmesh%nrr(3),&
     &                    igfftg0,&
     &                    wfr1,iikp,ktabrkp,&
     &                    wfr2,iik,ktabrk,fftmesh%menoplan,&
     &                    rhxtwg_c(:,ivp,ic))


            if (ik==ikp) then
                    if (ic==ivp) then 
                      rhxtwg_c(1,ivp,ic) = fcc_const * qpg(1,iqbz_0)
                    else  
                      rhxtwg_c(1,ivp,ic) = 0
                    endif
            endif
! end NEW for q!=0
         enddo ! ic
        enddo ! ivp


!      now calculate:
!      sum_G,G' rhotw(G)* W(G,G') rhotw(G')
      do ic = iparam%lumo, iparam%nbnds
        irc = ic - iparam%lumo + 1
        do ivp = iparam%lomo, iparam%homo  
         irvp = ivp - iparam%lomo + 1

!        sum_GG' rho_c'c*(G) W(G,G') rho_v'v(G')
!        first sum on G: sum_G rho_c'c(G) W*(G,G')    (W conjgated)
         if(WDIAG) then
           ctccp(:) = rhxtwg_c(:,ivp,ic) * Wstardiag(:)
         else
           ctccp(:) = matmul(rhxtwg_c(:,ivp,ic),Wstar(:,:))
         endif
         
         do iv = iparam%lomo, iparam%homo
          irv = iv - iparam%lomo + 1
          
          
          do icp = iparam%lumo, iparam%nbnds
            ircp = icp - iparam%lumo + 1


!           then sum on G': sum_G' (rho_c'c(G) W*(G,G'))* rho_v'v(G')
            http = -faq * dot_product(ctccp(:),rhxtwg_v(:,icp,iv))
            hk(irv,irc,ikp,irvp,ircp) = http
                          
          enddo ! icp
         enddo ! iv
        enddo ! ivp
       enddo ! ic
       
      enddo ! ikp
      
!      now store hamiltonian in memory or disc
       do ikp = ik, iparam%nkbz           ! loop over kp
       if(HMEMO) then
         do irc = 1, iparam%nbndc
           do ircp = 1, iparam%nbndc
        do irv = 1, iparam%nbndv
          it = transtab(ik,irv,irc)
          do irvp = 1, iparam%nbndv
            itp = transtab(ikp,irvp,ircp)
            hd(it,itp)=hk(irv,irc,ikp,irvp,ircp)
            hd(itp,it)=hk(irv,irc,ikp,irvp,ircp)
            if(itp >= it) then
             ir = it + itp * (itp - 1) / 2
             h(ir) = hk(irv,irc,ikp,irvp,ircp)
            else
             ir = itp + it * (it - 1) / 2
             h(ir) = hk(irv,irc,ikp,irvp,ircp)
            endif
           enddo
          enddo
         enddo
        enddo
       else
        do irvp = 1, iparam%nbndv
         do ircp = 1, iparam%nbndc
          itp = transtab(ikp,irvp,ircp)
          do irv = 1, iparam%nbndv
           do irc = 1, iparam%nbndc
            it = transtab(ik,irv,irc)
            if(itp >= it) then
             ir = it + itp * (itp - 1) / 2
             write(56,rec=ir) hk(irv,irc,ikp,irvp,ircp)
            else
             ir = itp + it * (it - 1) / 2
             write(56,rec=ir) hk(irv,irc,ikp,irvp,ircp)
            endif
           enddo
          enddo
         enddo
        enddo 
       endif
       enddo ! ikp
       
      if(.not. HMEMO) then
        write(56,rec=lastirec+2) ik
      endif


      enddo ! ik

      write(uout,*)
      write(uout,*) 'coulomb term completed'
      write(uout,*)
      if(HMEMO) then
        write(uout,'(" -W(1,1) = [eV] ",f7.2)') real(h(1))*HARTREE
        write(uout,'(" -W(3,2) = [eV] ",f7.2)') real(hd(3,2))*HARTREE
        write(uout,*)
      endif
      
      deallocate(hk,qtab,qtabo,qtabi,Wibz,qpg)

!----------------------------------------------------------------------
!     write hamiltonian 
!----------------------------------------------------------------------

      if(HMEMO) then
        write(uout,*) 'write coupling excitonic hamiltonian on disk'
        do ir = 1, lastirec
          write(56,rec=ir) h(ir)
        enddo
        write(56,rec=lastirec+1) nh
        write(56,rec=lastirec+2) iparam%nkbz
      endif
      
      close(56)
      deallocate(h)

 if(iverbose > 2)  call uverbose('e','makeWqcoupl')
end 


subroutine remove_diagonal(nh,hd,Wd)
 use verbose
 use dpdef
 use ggwdef
 implicit none
 integer nh
 complex (kind(0.0d0)) :: hd(nh,nh)
 real (kind(0.0d0)) :: Wd
 integer it
 real (kind(0.0d0)) :: rt, msd
 
 if(iverbose > 2)  call uverbose('b','remove_diagonal')
 
 rt=0
 do it=1,nh
   rt=rt+hd(it,it)
 enddo
 Wd=rt/nh

 if (iverbose > 1) then 
  write(uout,*) 'removing diagonal from W by a value of [eV] ',Wd*HARTREE
  write(uout,*)
 endif

 rt=0
 do it=1,nh
   rt=rt+(hd(it,it)-Wd)*(hd(it,it)-Wd)
 enddo
 msd=sqrt(real(rt))
 if(msd>0.05*HARTREE) then
   write(uout,*) 'WARNING: the diagonal of W is not constant'
   write(uout,*) 'taking average value'
 endif
   
 do it=1, nh
   hd(it,it) = 0
 enddo
 
 if(iverbose > 2)  call uverbose('e','remove_diagonal')
 end
!!***

