!{\src2tex{textfont=tt}}
!!****p* create_rhotwq0/get_wfr
!! NAME
!! get_wfr
!!
!! FUNCTION
!! get the ks wave function with the right index. used in the excitonic part
!! 
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! create_rhotw,create_rhotwq0, create_rhotwt, create_rhxtwg_coul, fxcterm, makeWq, makeWqcoup, prep_rhotw
!!
!! CHILDREN
!! c3dfft
!!
!! SOURCE

subroutine get_wfr(iparam,elecstr,elecstrp,fftmesh,ib_ini,ikibz_ini,ib_end,ikibz_end,isppol,wfr1,wfr2)

  use dpdef
  use ggwdef
#ifdef OMP
!$ use OMP_LIB
#endif
  implicit none

  type(dpparam) iparam
  type(electronic_structure) elecstr,elecstrp
  type(mesh) fftmesh

  integer, intent(in) :: ib_ini,ikibz_ini !initial state
  integer, intent(in) :: ib_end,ikibz_end !final state
  integer, intent(in) :: isppol
  complex, intent(out) :: wfr2(fftmesh%nr), wfr1(fftmesh%nr)
  integer ig,ir1,ir2

  if(iparam%savememory) then
    wfr1(:)=0  ! initial state |v,kp>
    wfr2(:)=0  ! final state   |c,k>

    if (.not. iparam%wf_distrib) then
     do ig = 1, iparam%npwvec
      wfr1(fftmesh%igfft(ig,3,3,3)) = elecstrp%wfg(ig,ib_ini,ikibz_ini,isppol)
      wfr2(fftmesh%igfft(ig,3,3,3)) = elecstr%wfg(ig,ib_end,ikibz_end,isppol)
     enddo
    else
     ir1=elecstrp%wf_count(ikibz_ini,ib_ini,isppol)
     ir2=elecstr%wf_count(ikibz_end,ib_end,isppol)
     do ig = 1, iparam%npwvec
      wfr1(fftmesh%igfft(ig,3,3,3)) = elecstrp%wfg2(ig,ir1)
      wfr2(fftmesh%igfft(ig,3,3,3)) = elecstr%wfg2(ig,ir2)
     enddo
    endif

    call c3dfft(wfr1,fftmesh%nrr(1),fftmesh%nrr(2),fftmesh%nrr(3),&
               fftmesh%nrr(1),fftmesh%nrr(2),fftmesh%nrr(3),1,fftmesh%plan)
    call c3dfft(wfr2,fftmesh%nrr(1),fftmesh%nrr(2),fftmesh%nrr(3),&
               fftmesh%nrr(1),fftmesh%nrr(2),fftmesh%nrr(3),1,fftmesh%plan)
  else
    wfr1(:)=elecstrp%wfr(:,ib_ini,ikibz_ini,isppol)
    wfr2(:)=elecstr%wfr(:,ib_end,ikibz_end,isppol)
  endif

end
!!***

