!{\src2tex{textfont=tt}}
!!****p* creation_exciton/blocks
!! NAME
!! handling_files
!!
!! FUNCTION
!!  
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_exciton
!!
!! CHILDREN
!! lrec
!!
!! SOURCE


 subroutine blocks(iparam,files)

 use ggwdef
 use dpdef
 use verbose

 implicit none

 type(dpparam) iparam
 type(dpfiles) files

 integer :: rblock,cblock,ispinblock,jspinblock,spinunit
 integer, external :: lrec
 
 if(iverbose > 2) call uverbose('b','blocks')

!1. count number of blocks

! spin structure
! ----------------------------------- for qpol=0 v-+ can be obtained by v+-so:
! | [diag-W+v]++     |      v+-     |  qpol = 0 nsppol = 1 -> nspinblock=1 
! -----------------------------------  qpol = 0 nsppol = 2 -> nspinblock=3
! |     v-+          | [diag-W+v]-- |  qpol \=0 nsppol = 2 -> nspinblock=4
! -----------------------------------   

 files%nspinblock = iparam%nsppol

!for each spin block:
!count the number of subblocks of the exciton hamiltonian to calculate
!
!          ! cblock=1  | cblock=2 |
! ---------------------------------
! rblock=1 | RESONANT  | COUPLING |  
! ---------------------------------
! rblock=2 | ANTICOUP  | ANTIRESO |
! ---------------------------------
! note: anticoup block is not used

! dimension info for direct access files 
 files%sizeofcomplex = 2*lrec()
 if (iparam%td) then
  files%nrblock=1
  files%ncblock=1
  allocate(files%lastirec(files%nrblock,files%ncblock))
  files%lastirec(:,:) = (iparam%ntres*iparam%ntres + iparam%ntres)/2
 else
   if(iparam%KP_THEORY .and. iparam%ig==1) then
    files%nrblock=1
    files%ncblock=2
    allocate(files%lastirec(files%nrblock,files%ncblock))
    files%lastirec(:,:) = (iparam%ntres*iparam%ntres + iparam%ntres)/2
   else
    files%nrblock=2
    files%ncblock=2
    allocate(files%lastirec(files%nrblock,files%ncblock))
    files%lastirec(1,1) = (iparam%ntres*iparam%ntres + iparam%ntres)/2
    files%lastirec(2,2) = (iparam%ntres*iparam%ntres + iparam%ntres)/2
    files%lastirec(1,2) = iparam%ntres*iparam%ntres 
    files%lastirec(2,1) = iparam%ntres*iparam%ntres !useless?
   endif  
 endif


!block ordering: 
!spin unit
! ---------
! | 1 | 2 | 
! ---------
! | 3 | 4 |
! ---------

!2. defines file unit numbers
!spin up up (ispinblock=1):
!51(reso), 52 (coupling), 54 (antireso)
!spin up dn (ispinblock=2):
!61(reso), 62 (coupling), 64 (antireso)
! etc.
!for other spin components: add multiples of 10 to each unit according to spinunit ordering
  do ispinblock=1,files%nspinblock
   do jspinblock=1,files%nspinblock
    spinunit = (ispinblock-1)*2+jspinblock
    do rblock =1,files%nrblock
     do cblock = rblock,files%ncblock
      files%uni(ispinblock,jspinblock,rblock,cblock) = 41+spinunit*10 + 2*(rblock-1)+(cblock-1)
     enddo
    enddo
   enddo
  enddo



!3. defines file names 

   if (iparam%nsppol == 1) then
 
      files%out_blockfile_name(1,1,1,1) = 'out.exh'
      files%in_blockfile_name(1,1,1,1) = 'in.exh'
      files%out_blockfile_name(1,1,1,2) = 'out.exc'
      files%in_blockfile_name(1,1,1,2) = 'in.exc'
      files%out_blockfile_name(1,1,2,2) = 'out.antiexh'
      files%in_blockfile_name(1,1,2,2) = 'in.antiexh'
      !files%out_blockfile_name(1,1,2,1) = 'out.antiexc'
      !files%in_blockfile_name(1,1,2,1) = 'in.antiexc'

   else

      files%out_blockfile_name(1,1,1,1) = 'out.exh.upup'
      files%in_blockfile_name(1,1,1,1) = 'in.exh.upup'
      files%out_blockfile_name(1,1,1,2) = 'out.exc.upup'
      files%in_blockfile_name(1,1,1,2) = 'in.exc.upup'
      files%out_blockfile_name(1,1,2,2) = 'out.antiexh.upup'
      files%in_blockfile_name(1,1,2,2) = 'in.antiexh.upup'
      !files%out_blockfile_name(1,1,2,1) = 'out.antiexc.upup'
      !files%in_blockfile_name(1,1,2,1) = 'in.antiexc.upup'

      files%out_blockfile_name(1,2,1,1) = 'out.exh.updn'
      files%in_blockfile_name(1,2,1,1) = 'in.exh.updn'
      files%out_blockfile_name(1,2,1,2) = 'out.exc.updn'
      files%in_blockfile_name(1,2,1,2) = 'in.exc.updn'
      files%out_blockfile_name(1,2,2,2) = 'out.antiexh.updn'
      files%in_blockfile_name(1,2,2,2) = 'in.antiexh.updn'
      !files%out_blockfile_name(1,2,2,1) = 'out.antiexc.updn'
      !files%in_blockfile_name(1,2,2,1) = 'in.antiexc.updn'

      files%out_blockfile_name(2,2,1,1) = 'out.exh.dndn'
      files%in_blockfile_name(2,2,1,1) = 'in.exh.dndn'
      files%out_blockfile_name(2,2,1,2) = 'out.exc.dndn'
      files%in_blockfile_name(2,2,1,2) = 'in.exc.dndn'
      files%out_blockfile_name(2,2,2,2) = 'out.antiexh.dndn'
      files%in_blockfile_name(2,2,2,2) = 'in.antiexh.dndn'
      !files%out_blockfile_name(2,2,2,1) = 'out.antiexc.dndn'
      !files%in_blockfile_name(2,2,2,1) = 'in.antiexc.dndn'

      files%out_blockfile_name(2,1,1,1) = 'out.exh.dnup'
      files%in_blockfile_name(2,1,1,1) = 'in.exh.dnup'
      files%out_blockfile_name(2,1,1,2) = 'out.exc.dnup'
      files%in_blockfile_name(2,1,1,2) = 'in.exc.dnup'
      files%out_blockfile_name(2,1,2,2) = 'out.antiexh.dnup'
      files%in_blockfile_name(2,1,2,2) = 'in.antiexh.dnup'
      !files%out_blockfile_name(2,1,2,1) = 'out.antiexc.dnup'
      !files%in_blockfile_name(2,1,2,1) = 'in.antiexc.dnup'
   

   endif
!old and deprecated
      files%exh=files%out_blockfile_name(1,1,1,1)
      files%exc=files%out_blockfile_name(1,1,1,2) 
!endold and deprecated
      files%exeig='out.exeig'
      inquire(file='in.exeig',exist=files%eigx)
      if(files%eigx)  files%exeig = 'in.exeig'
      files%exceg = 'out.exceg'
      inquire(file='in.exceg',exist=files%cegx)
      if(files%cegx)  files%exceg = 'in.exceg'
      files%exovl = 'out.exovl'
      inquire(file='in.exovl',exist=files%ox)
      if(files%ox)  files%exovl = 'in.exovl'

 if(iverbose > 2) call uverbose('e','blocks')
 end subroutine blocks
!!***

!{\src2tex{textfont=tt}}
!!****p* creation_exciton/handling_files
!! NAME
!! handling_files
!!
!! FUNCTION
!! Handle (if existent) the restart files: in.exh in.exc 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_exciton
!!
!! SOURCE

subroutine handling_files(ispinblock,jspinblock,rblock,cblock,iparam,files)
use dpdef
use verbose
implicit none
type(dpparam) iparam
type(dpfiles) files
integer,intent(in) :: ispinblock,jspinblock,rblock,cblock

integer :: fileunit 
integer(ILONG) :: endrec
character (len=30) :: filename
logical :: in_blockfile_exist
integer(ILONG) :: itemp


if (iverbose > 2) call uverbose('b','handling_files')


fileunit = files%uni(ispinblock,jspinblock,rblock,cblock)
filename = files%out_blockfile_name(ispinblock,jspinblock,rblock,cblock)
inquire(file=files%in_blockfile_name(ispinblock,jspinblock,rblock,cblock),exist=in_blockfile_exist)
if(in_blockfile_exist) filename=files%in_blockfile_name(ispinblock,jspinblock,rblock,cblock)

open(fileunit,file=filename,access='direct',recl=files%sizeofcomplex)
if (iverbose > 2)  write(uout,*) 'file ', filename,' unit ',fileunit,' open'

if(in_blockfile_exist) then  ! if already exists
  endrec=files%lastirec(rblock,cblock) 
  read(fileunit,rec=(endrec+1)) itemp
  if(itemp == iparam%ntres) then
    write(uout,*) 'reading exciton hamiltonian from ',filename
    read(fileunit,rec=(endrec+2)) files%kstart
    if(files%kstart < iparam%nkbz) then
      files%blockfile_uncompleted(ispinblock,jspinblock,rblock,cblock) = .true.
      files%kstart = files%kstart + 1
      write(uout,*) 'starting from k ', files%kstart
    else
      files%blockfile_uncompleted(ispinblock,jspinblock,rblock,cblock) = .false.
      close(fileunit)
      if (iverbose > 2) write(uout,*) 'file on unit ',fileunit,' close'
    endif
  else
    write(uout,*) 'ERROR: wrong file '
    write(uout,*) 'block ',ispinblock,jspinblock,rblock,cblock,' file ', filename
    write(uout,*) 'transitions found ',itemp, ' expected ', iparam%ntres
    stop 'wrong input file'
  endif
else
  if(iverbose>2) write(uout,*) 'starting block ',ispinblock,jspinblock,rblock,cblock,' from beginning'
  files%blockfile_uncompleted(ispinblock,jspinblock,rblock,cblock) = .true.
  files%kstart = 1
endif

if (iverbose > 2)  write(uout,*)

if (iverbose > 2) call uverbose('e','handling_files')

end 
!!***


