!{\src2tex{textfont=tt}}
!!****p* creation_chi0/mk_chi0_delta_body
!! NAME
!! mk_chi0_delta_body
!!
!! FUNCTION
!! Compute the delta-like parts of chi0 (body)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!! Lucia Caramella
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_chi0
!!
!! CHILDREN
!! cgerc
!!
!! SOURCE

  subroutine mk_chi0_delta_body (iet,nomega,npwmat,factocc,rhotwg,body)
  
  implicit none
  
  integer iet
  integer nomega, npwmat
  double precision factocc
  complex  factocc_c_single
  complex, dimension(npwmat) :: rhotwg
  complex, dimension(npwmat,npwmat,nomega) :: body
  
  factocc_c_single = cmplx(factocc,0)

  if(iet.ge.1 .and. iet.le.nomega)then
   call cgerc(npwmat,npwmat,factocc_c_single,rhotwg,1,rhotwg,1,body(:,:,iet),npwmat)
  endif 
  
  
  end subroutine mk_chi0_delta_body
!!***
  
  
!{\src2tex{textfont=tt}}
!!****p* creation_chi0/mk_chi0_delta_wings
!! NAME
!! mk_chi0_delta_wings
!!
!! FUNCTION
!! Compute the delta-like parts of chi0 (wings: lwg, uwg)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!! Lucia Caramella
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_chi0
!!
!! SOURCE

  subroutine mk_chi0_delta_wings(iet,iq,nomega,npwmat,nq,factocc,rhotwg,lwg,uwg)
  
  implicit none
  
  integer iet, iq
  integer nomega, npwmat, nq
  double precision factocc
  complex, dimension(npwmat) :: rhotwg
  complex, dimension(npwmat,nomega,nq) :: lwg
  complex, dimension(npwmat,nomega,nq) :: uwg
  
! local variables
  complex :: rhotwg1, rhotwgs1
  complex, dimension(npwmat) :: rhotwgs
  
  
  rhotwgs(:)  = conjg( rhotwg(:) )
  
  if(iet.ge.0 .and. iet.le.nomega)then
   rhotwg1=rhotwg(1)
   rhotwgs1=rhotwgs(1)
   lwg(:,iet,iq) = lwg(:,iet,iq) + factocc*rhotwg(:)*rhotwgs1
   uwg(:,iet,iq) = uwg(:,iet,iq) + factocc*rhotwg1*rhotwgs(:)
  endif  
  

  end subroutine mk_chi0_delta_wings
!!***

!{\src2tex{textfont=tt}}
!!****p* creation_chi0/make_hilbert
!! NAME
!! make_hilbert
!!
!! FUNCTION
!! Create the non-delta part of the chi0 via hilbert transform
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!! Lucia Caramella
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_chi0
!!
!! CHILDREN
!! mk_chi0_Hlbrt_body, mk_chi0_Hlbrt_wings
!!
!! SOURCE

  subroutine make_hilbert(iparam,res)

  use dpdef
  use ggwdef
  use verbose
  
  implicit none
  
  type(dpparam) iparam
  type(response) res

  if(iverbose > 2) call uverbose('b','make_hilbert')
  
  if(iverbose > 0) then
   write(uout,*) ' calculating the Hilbert transform '
   write(uout,*)
  endif
  
  res%chi0(:,:,:,:)  = res%chi0(:,:,:,:)*pi/iparam%domega ! normalization
  res%uwing(:,:,:,:) = res%uwing(:,:,:,:)*pi/iparam%domega !
  res%lwing(:,:,:,:) = res%lwing(:,:,:,:)*pi/iparam%domega !

  call mk_chi0_Hlbrt_body(iparam%nomega,iparam%omegai,iparam%domega,&
                          iparam%omega,iparam%npwmat,iparam%broad,res%chi0)
  call mk_chi0_Hlbrt_wings(iparam%nomega,iparam%npwmat,iparam%nqpol,iparam%omegai,&
                           iparam%domega,iparam%omega,iparam%broad,res%lwing,res%uwing)
 
  if(iverbose > 2) call uverbose('e','make_hilbert')
  
 end subroutine make_hilbert
!!***
  
  
!{\src2tex{textfont=tt}}
!!****p* make_hilbert/mk_chi0_Hlbrt_body
!! NAME
!! mk_chi0_Hlbrt_body
!!
!! FUNCTION
!! Compute the body of complete chi0 from the delta-like part
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!! Lucia Caramella
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! make_hilbert
!!
!! CHILDREN
!! Hlbrt
!!
!! SOURCE

  subroutine mk_chi0_Hlbrt_body(nomega,omegai,domega,omega,npwmat,broad,body)

  implicit none

  integer, intent(in) :: nomega, npwmat
  real, intent(in) :: omegai, domega, broad
  complex, dimension(nomega), intent(in) :: omega
  complex, dimension(npwmat,npwmat,nomega) :: body

  !local variables
  integer :: ie, ig, igp, iflag, icount
  double precision :: omega_zero 
  double complex f_zero
  double precision, dimension(nomega) :: romega
  complex, dimension(nomega) :: chi_new
  double complex :: cauxo, weta
  double complex, dimension(nomega) :: cauxi
  double complex, dimension(nomega*nomega) :: clogtab

  omega_zero = omegai - domega
  do ie=1,nomega
  romega(ie) = real(omega(ie))
  enddo

  iflag=0
  do igp = 1, npwmat
   do ig = 1, npwmat
  
    chi_new(:) = cmplx(0.,0.)               ! initialize chi_0
    f_zero = dcmplx(real(body(ig,igp,1)),0) ! starting value of 
                                            ! argument in the H.T.
    do ie=1,nomega
     cauxi(ie) = body(ig,igp,ie)
    enddo

    if (iflag.eq.0) then
     icount = 0
     do ie=1,nomega
      weta = cmplx( romega(ie) , broad )
      call Hlbrt(weta,cauxi(:),romega(:),nomega,omega_zero,&
                 f_zero,cauxo,nomega, clogtab, icount, iflag)
      chi_new(ie) = chi_new(ie) + cauxo
     enddo ! ie
    if (icount .gt. (nomega*nomega)) STOP 'icount too large!'
     iflag = 1
    else
     icount = 0
      do ie=1,nomega
       weta = cmplx( romega(ie) , broad )
       call Hlbrt(weta,cauxi(:),romega(:),nomega,omega_zero,&
                  f_zero,cauxo,nomega, clogtab, icount, iflag)
       chi_new(ie) = chi_new(ie) + cauxo
      enddo 
    if (icount .gt. (nomega*nomega)) STOP 'icount too large!'
    endif

    do ie=1,nomega
     body(ig,igp,ie) = -chi_new(ie) 
    enddo

   enddo ! ig 
  enddo ! igp

  end subroutine  mk_chi0_Hlbrt_body
!!***



!{\src2tex{textfont=tt}}
!!****p* make_hilbert/mk_chi0_Hlbrt_wings
!! NAME
!! mk_chi0_Hlbrt_wings
!!
!! FUNCTION
!! Compute the wings of complete chi0 from the delta-like part
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!! Lucia Caramella
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! make_hilbert
!!
!! CHILDREN
!! Hlbrt
!!
!! SOURCE

  subroutine mk_chi0_Hlbrt_wings(nomega,npwmat,nq,omegai,domega,omega,broad,lwg,uwg)

  implicit none

  integer, intent(in) :: nomega, npwmat, nq
  real, intent(in) :: omegai, domega, broad
  complex, intent(in) :: omega(nomega)
  complex, dimension(npwmat,nomega,nq), intent(inout) :: lwg
  complex, dimension(npwmat,nomega,nq), intent(inout) :: uwg

! local variables
  integer :: ie, ig, iq, icount
  double precision :: omega_zero 
  double complex lwg_zero, uwg_zero
  double precision, dimension(nomega) :: romega
  complex, dimension(nomega) :: lwg_new
  complex, dimension(nomega) :: uwg_new
  double complex :: lwauxo, uwauxo, weta
  double complex, dimension(nomega) :: lwauxi, uwauxi
  double complex, dimension(nomega*nomega) :: clogtab

  omega_zero = omegai - domega
  do ie=1,nomega
   romega(ie) = real( omega(ie) )
  enddo

  do iq = 1, nq
   do ig = 1, npwmat

    lwg_new(:) = cmplx(0.,0.) ! inizializzazione lwg_new
    uwg_new(:) = cmplx(0.,0.) ! inizializzazione uwg_new
    
    lwg_zero = dcmplx(real(lwg(ig,1,iq)),0) ! starting value of arguments
    uwg_zero = dcmplx(real(uwg(ig,1,iq)),0) ! in the Hilbert transform

    do ie=1,nomega
     lwauxi(ie) = lwg(ig,ie,iq)
     uwauxi(ie) = uwg(ig,ie,iq)
    enddo 

    do ie=1,nomega
     weta = cmplx( romega(ie) , broad )
     call Hlbrt(weta,lwauxi(:),romega(:),nomega, &
                omega_zero,lwg_zero,lwauxo,nomega,&
                clogtab, icount, 2)
     call Hlbrt(weta,uwauxi(:),romega(:),nomega,&
                omega_zero,uwg_zero,uwauxo,nomega,&
                clogtab, icount, 2)
     lwg_new(ie) = lwg_new(ie) + lwauxo
     uwg_new(ie) = uwg_new(ie) + uwauxo
    enddo   


    do ie = 1, nomega
     lwg(ig,ie,iq) = -lwg_new(ie)  
     uwg(ig,ie,iq) = -uwg_new(ie) 
    enddo 

   enddo ! ig 
  enddo ! iq

 end subroutine  mk_chi0_Hlbrt_wings
!!***


!{\src2tex{textfont=tt}}
!!****p* make_hilbert/Hlbrt
!! NAME
!! Hlbrt
!!
!! FUNCTION
!! computes the integral of (1/pi)*f(y)dy/(y-x) from a to xp(n)
!! following the definition of hilbert transform
!! the result is fc and the integrand is in am(i)
!! its value at a is fz  and must be zero --> lc this is a problem
!! it must be dimag(x).gt.0 --> lc this is +ieta
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!! Lucia Caramella
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! mk_chi0_Hlbrt_wings,mk_chi0_Hlbrt_body
!!
!! SOURCE

 subroutine Hlbrt(x,am,xp,n,a,fz,fc,nmax,clogtab,icount,iflag)

 use dpdef
 implicit none

 integer :: n,m,j,nmax, iflag, icount
 real*8 pig,ev,e,a
 real*8 xp(nmax)
 complex*16 am(nmax)
 complex*16 fc,x,b,c,d,f,h,fz
 complex*16 clogtab(nmax*nmax)
!double complex, external :: zlog
 data pig/3.14159265358979323846d0/

 if (n.gt.nmax) stop 'n gt nmax'
 m=n-1
 f=x-a
 ev=xp(1)-a
 if(abs(ev).lt.1.e-10) then 
   write(uout,*)' xp(1), a', xp(1), A
   stop 'subroutine Hlbrt; EV TOO SMALL : STOPPING'
 endif
 
 b=log(1.-ev/f)
 fc=fz*(-1.+log((xp(1)-x)/(a-x))*(xp(1)-x)/ev)

 if (iflag.eq.0) then
  do j=1,m
   e=xp(j+1)-xp(j)
   d=xp(j+1)-x
   h=x-xp(j)
! tabulate clog
   icount = icount + 1
   c=log(1.-e/h)
   clogtab(icount) = c
   fc=fc+am(j)*(c*d/e+b*f/ev)
   b=c
   f=h
   ev=e
  enddo
 endif

 if (iflag.eq.1) then
  do j=1,m
   e=xp(j+1)-xp(j)
   d=xp(j+1)-x
   h=x-xp(j)
   icount = icount + 1
   c = clogtab(icount)
   fc=fc+am(j)*(c*d/e+b*f/ev)
   b=c
   f=h
   ev=e
  enddo
 endif

 if (iflag.eq.2) then
  do j=1,m
   e=xp(j+1)-xp(j)
   d=xp(j+1)-x
   h=x-xp(j)
   c=log(1.-e/h)
   fc=fc+am(j)*(c*d/e+b*f/ev)
   b=c
   f=h
   ev=e
  enddo
 endif

 fc=fc+am(n)*(1.+b*f/ev)
 fc=fc/pig     ! lc

 end
!!***

