!{\src2tex{textfont=tt}}
!!****p* main/fftw3interfaces
!!
!!***


!{\src2tex{textfont=tt}}
!!****f* fftw3interfaces/allowed_fftsize
!! NAME
!! allowed_fftsize
!!
!! FUNCTION
!!  calculate the size of the smallest allowed FFT size that will encompass n
!!  in FFTW all is allowed
!! 
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! setmesh
!!
!! SOURCE

integer function allowed_fftsize(n)
  implicit none
  integer n
  allowed_fftsize = n
end
!!***

!{\src2tex{textfont=tt}}
!!****p* fftw3interfaces/c3dfft
!! NAME
!! c3dfft
!!
!! FUNCTION
!! computes the forward fourier transform of a complex
!! function c. the fourier transform is
!! returned in c in output (the input c is overwritten)
!! single precision fourier transform
!! isig <0  inverse
!! isig >0  direct
!! 
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! fftwfn,create_rhotw,cden,rho_tw_g,ckxclda, create_rhotw, get_wfr
!!
!! SOURCE

subroutine c3dfft_oldold(c,id,ie,ig,nl,nm,nn,isig)

  use dpdef
#ifdef OMP
!$ use OMP_LIB
#endif

  implicit none
#include FFTWINCLUDEDIR
  integer id, ie, ig
  integer nl, nm, nn
  integer isig
  complex c(id,ie,ig)
  integer, save :: nlo=0, nmo=0, nno=0, isigo=1
  complex (kind(0.0d0)), save, allocatable :: b(:,:,:)
  integer(ILONG), save :: plan
#ifdef OMP
!$OMP THREADPRIVATE(nlo,nmo,nno,isigo)
!$OMP THREADPRIVATE(plan)
!$OMP THREADPRIVATE(b)
#endif


  if(nl.ne.nlo .or. nm.ne.nmo .or. nn.ne.nno .or. isig.ne.isigo) then
    ! initialize
    if(nlo/=0 .or. nmo/=0 .or. nno/=0) then
      call dfftw_destroy_plan(plan)
      deallocate(b)
    endif
    allocate(b(nl,nm,nn))
    if(isig<0) then
      call dfftw_plan_dft_3d(plan,nl,nm,nn,b,b,FFTW_BACKWARD,FFTW_MEASURE)
    else 
      call dfftw_plan_dft_3d(plan,nl,nm,nn,b,b,FFTW_FORWARD,FFTW_MEASURE)
    endif
    nlo = nl ; nmo = nm ; nno = nn; isigo = isig
  endif
  b(:,:,:) = c(1:nl,1:nm,1:nn)
  call dfftw_execute_dft(plan,b,b)
  c(1:nl,1:nm,1:nn) = b(:,:,:)
end
!!***

subroutine c3dfft_old(c,id,ie,ig,nl,nm,nn,isig)

  use dpdef
#ifdef OMP
!$ use OMP_LIB
#endif

  implicit none
#include FFTWINCLUDEDIR
  integer id, ie, ig
  integer nl, nm, nn
  integer isig
  complex c(id,ie,ig)
  integer, save :: nlo=0, nmo=0, nno=0, isigo=1
  complex (kind(0.0d0)), save, allocatable :: b(:,:,:)
  integer(ILONG), save :: plan_menouno
  integer(ILONG), save :: plan_uno
#ifdef OMP
!$OMP THREADPRIVATE(nlo,nmo,nno,isigo)
!$OMP THREADPRIVATE(plan_uno)
!$OMP THREADPRIVATE(plan_menouno)
!$OMP THREADPRIVATE(b)
#endif


  if(nl.ne.nlo .or. nm.ne.nmo .or. nn.ne.nno) then
    ! initialize
    if(nlo/=0 .or. nmo/=0 .or. nno/=0) then
      call dfftw_destroy_plan(plan_menouno)
      call dfftw_destroy_plan(plan_uno)
      deallocate(b)
    endif
    allocate(b(nl,nm,nn))
    call dfftw_plan_dft_3d(plan_menouno,nl,nm,nn,b,b,FFTW_BACKWARD,FFTW_MEASURE)
    call dfftw_plan_dft_3d(plan_uno,nl,nm,nn,b,b,FFTW_FORWARD,FFTW_MEASURE)
    nlo = nl ; nmo = nm ; nno = nn; isigo = isig
  endif
  b(:,:,:) = c(1:nl,1:nm,1:nn)
  if(isig<0) then
    call dfftw_execute_dft(plan_menouno,b,b)
  else 
    call dfftw_execute_dft(plan_uno,b,b)
  endif
  c(1:nl,1:nm,1:nn) = b(:,:,:)
end

subroutine c3dfft_plan(nl,nm,nn,plan,menoplan)

  use params
  implicit none
#include FFTWINCLUDEDIR
  integer nl, nm, nn
  integer isig
  complex (kind(0.0d0)), allocatable :: b(:,:,:)
  integer(ILONG) :: plan
  integer(ILONG) :: menoplan

  allocate(b(nl,nm,nn)) 
  call dfftw_plan_dft_3d(menoplan,nl,nm,nn,b,b,FFTW_BACKWARD,FFTW_MEASURE)
  call dfftw_plan_dft_3d(plan,nl,nm,nn,b,b,FFTW_FORWARD,FFTW_MEASURE)
  deallocate(b)

end 


subroutine c3dfft(c,id,ie,ig,nl,nm,nn,isig,plan)

  use dpdef

  implicit none
#include FFTWINCLUDEDIR
  integer nl, nm, nn
  integer id, ie, ig
  integer isig
  complex c(id,ie,ig)
  complex (kind(0.0d0)), allocatable :: b(:,:,:)
  integer(ILONG) :: plan
 
  allocate(b(nl,nm,nn))
  b(:,:,:) = c(1:nl,1:nm,1:nn)
  call dfftw_execute_dft(plan,b,b)
  c(1:nl,1:nm,1:nn) = b(:,:,:)
  deallocate(b)

end 

