!{\src2tex{textfont=tt}}
!!****p* main/lapackinterfaces
!!
!!***

!{\src2tex{textfont=tt}}
!!****p* lapackinterfaces/chevc
!! NAME
!! chevc
!!
!! FUNCTION
!! Compute the eigenvalues and the eigenvectors of a complex Hermitian matrix
!! only the upper triangle of a is used

!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!!
!! SOURCE

 subroutine chevc(n,a,lda,w,z,ldz)

 implicit none
 
 integer n, lda, ldz
 complex a(lda,n)
 complex z(ldz,n)
 real w(n)
 
!upper-packed storage mode
 character, parameter :: uplo = 'U'
 character, parameter :: jobz = 'V'
 integer lwork
 real, allocatable :: rwork(:)
 complex, allocatable :: work(:)
 integer info
 
 integer istat,i,j
 
 allocate(rwork(max(1,3*n-2)),stat=istat)
 if(istat.ne.0) stop 'chevc 1: out of memory!'
 
 lwork = max(1,2*n-1)
 allocate(work(lwork),stat=istat)
 if(istat.ne.0) stop 'chevc 2: out of memory!'

!the 6 next lines: lwork query (does not go on nec?)
!lwork = -1
!call cheev(jobz,uplo,n,a,lda,w,work,lwork,rwork,info)
!lwork = work(1)
!deallocate(work)
!allocate(work(lwork),stat=istat)
!if(istat.ne.0) stop 'chevc 3: out of memory!'
 
 call cheev(jobz,uplo,n,a,lda,w,work,lwork,rwork,info)

 do i = 1, n
   do j = 1, n
     z(i,j) = a(i,j)
   enddo
 enddo

 deallocate(rwork)
 deallocate(work)
 
 end
!!***

!{\src2tex{textfont=tt}}
!!****p* lapackinterfaces/cgevc
!! NAME
!! cgevc
!!
!! FUNCTION
!! Compute the eigenvalues and the (right) eigenvectors of a complex 
!! general matrix 
!! a and z must have no common elements
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!!
!! SOURCE

 subroutine cgevc(n,a,lda,w,z,ldz)

 implicit none
 
 integer n, lda, ldz
 complex a(lda,n)
 complex z(ldz,n)
 complex w(n)
 
 character, parameter :: jvl = 'N'
 character, parameter :: jvr = 'V'
 integer info
 integer lwork
 complex, allocatable :: work(:)
 real, allocatable :: rwork(:)
 integer, parameter :: ldvl = 1
 complex vl(1,1)

 integer istat
       
 lwork = max(1,2*n)
 allocate(work(lwork),stat=istat)
 if(istat.ne.0) stop 'cgevc 1: out of memory!'
 allocate(rwork(2*n),stat=istat)
 if(istat.ne.0) stop 'cgevc 2: out of memory!'

 call cgeev(jvl,jvr,n,a,lda,w,vl,ldvl,z,ldz,work,lwork,rwork,info)
 
 deallocate(work,rwork)

 end
!!***


!{\src2tex{textfont=tt}}
!!****p* lapackinterfaces/cginv
!! NAME
!! cginv
!!
!! FUNCTION
!! invert a complex general matrix
!! CGETRF computes an LU factorization of a general M-by-N matrix A
!! using partial pivoting with row interchanges.
!! CGETRI computes the inverse of a matrix using the LU factorization
!! computed by CGETRF.

!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_altchi, creation_epsrpa
!!
!! SOURCE

 subroutine cginv(n,a,lda)
 
 implicit none 

 integer n, lda
 complex a(lda,n)
 
 integer ipiv(n)
 complex work(n)
 integer info
 
 call cgetrf(n,n,a,lda,ipiv,info)
 if(info.ne.0) then
!   write(uout,*) info
   stop 'cgetrf failed'
 endif
 call cgetri(n,a,n,ipiv,work,n,info)
 if(info.ne.0) then
!   write(uout,*) info
   stop 'cgetri failed'
 endif
 
 end
!!***

!{\src2tex{textfont=tt}}
!!****p* lapackinterfaces/zginv
!! NAME
!! zginv
!!
!! FUNCTION
!! invert a double precision complex general matrix
!! ZGETRF computes an LU factorization of a general M-by-N matrix A
!! using partial pivoting with row interchanges.
!! ZGETRI computes the inverse of a matrix using the LU factorization
!! computed by ZGETRF.

!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_epsmt
!!
!! SOURCE

 subroutine zginv(n,a,lda)

 implicit none
 
 integer n, lda
 complex (kind(0.0d0)) :: a(lda,n)
 
 integer ipiv(n)
 complex (kind(0.0d0)) :: work(n)
 integer info
 
 call zgetrf(n,n,a,lda,ipiv,info)
 if(info.ne.0) then
!   write(uout,*) info
   stop 'cgetrf failed'
 endif
 call zgetri(n,a,n,ipiv,work,n,info)
 if(info.ne.0) then
!   write(uout,*) info
   stop 'cgetri failed'
 endif
 
 end
!!***


!{\src2tex{textfont=tt}}
!!****p* lapackinterfaces/rginv
!! NAME
!! rginv
!!
!! FUNCTION
!! invert a real general matrix
!! SGETRF computes an LU factorization of a general M-by-N matrix A
!! using partial pivoting with row interchanges.
!! SGETRI computes the inverse of a matrix using the LU factorization
!! computed by SGETRF.

!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! calccomm, latt_and_q
!!
!! SOURCE
 
 subroutine rginv(n,a,lda)

 implicit none
 
 integer n, lda
 real a(lda,n)
 
 integer ipiv(n)
 real work(n)
 integer info
 
 call sgetrf(n,n,a,lda,ipiv,info)
 if(info.ne.0) stop 'sgetrf failed'
 call sgetri(n,a,n,ipiv,work,n,info)
 if(info.ne.0) stop 'sgetri failed'
 
 end
!!***

!{\src2tex{textfont=tt}}
!!****p* lapackinterfaces/cgle
!! NAME
!! cgle
!!
!! FUNCTION
!! solve the general complex problem A * X = B
!! where A is a complex matrix N x N and X, B are complex vectors


!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_epsrpa, creation_epslda, creation_altchi 
!!
!! SOURCE

 subroutine cgle(n,a,lda,b,x)

 implicit none
 
 integer n, lda
 complex a(lda,n), x(n), b(n)
 
 integer, parameter :: nrhs=1
 integer info
 integer ipiv(n)
 complex bb(n,nrhs)
 
 bb(:,1) = b(:)
 call cgesv(n,nrhs,a,lda,ipiv,bb,n,info)
 if(info/=0) then
!   write(uout,*) info
   stop 'cgesv failed'
 endif
 x(:) = bb(:,1)
 
 end
!!***

!{\src2tex{textfont=tt}}
!!****p* lapackinterfaces/zgle
!! NAME
!! zgle
!!
!! FUNCTION
!! solve the general double presicion complex problem A * X = B
!! where A is a complex matrix N x N and X, B are complex vectors


!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_epsmt
!!
!! SOURCE

 subroutine zgle(n,a,lda,b,x)

 implicit none

 integer n, lda
 complex (kind(0.0d0)) :: a(lda,n), x(n), b(n)
 
 integer, parameter :: nrhs=1
 integer info
 integer ipiv(n)
 complex (kind(0.0d0)) :: bb(n,nrhs)
 
 bb(:,1) = b(:)
 call zgesv(n,nrhs,a,lda,ipiv,bb,n,info)
 if(info/=0) then
!   write(uout,*) info
   stop 'zgesv failed'
 endif
 x(:) = bb(:,1)
 
 end
!!***


!{\src2tex{textfont=tt}}
!!****p* lapackinterfaces/cmatxmat
!! NAME
!! cmatxmat
!!
!! FUNCTION
!! Performs a complex matrix-matrix multiplication
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! calc_wf_qp
!!
!! SOURCE

 subroutine cmatxmat(m,n,k,a,lda,b,ldb,c,ldc)

 implicit none

 Integer  m, n, k, lda, ldb, ldc
 complex  a(lda,k), b(ldb,n), c(ldc,n)


 call cgemm('n','n',m,n,k,(1.,0.),a,lda,b,ldb,(0.,0.),c,ldc)

 end 
!!***

