!{\src2tex{textfont=tt}}
!!****p* main/rddpin
!! NAME
!! rddpin
!!
!! FUNCTION
!! read all the parameters of the dp calculation
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! SOURCE

  subroutine rddpin(inputfile,il,iparam)

  use ggwdef
  use dpdef
  use verbose
  
  implicit none
  integer il
  character (len=il) :: inputfile
  type(dpparam) iparam
  
  integer, external :: read_int, read_string, read_float, read_file
  integer, external :: read_float_array
  logical, external :: is_small
  integer icount, idum
  logical shiftk
  real    omegai
!plexc
  integer ncella,ncellb,ncellc,firstexciton,lastexciton
  real pos(3)



  if(il==0) then
    write(uout,*) 'reading parameters from stdin'
    write(uout,*) '(type CTRL^D when finished)'
    if(read_file('stdin',5)/=1) stop 'error'
  else
    if (il<20) then
     headline="Reading parameters from file "//inputfile
    else
     headline="Reading parameters from input file"
    endif
    call print_headline()
    if(read_file(inputfile,il) /= 1) stop 'error in inputfile'
  endif
  

! =======================
! = Type of calculation =
! =======================

!DEFAULT
 iparam%linear=.true.
 iparam%calctype='Linear TDDFT: Dyson equation in reciprocal space'
 iparam%exciton=.false.
 iparam%pes=.false.
 iparam%nonlinear = .false.
 iparam%current=.false.
 iparam%analysis_exciton=.false.
 iparam%plot_exciton=.false.

 icount=0
 if(read_string('linear',6)>0) icount=icount+1
 if(read_string('exciton',7)>0) then
  iparam%exciton=.true. ; iparam%linear=.false.
  icount = icount +1
  iparam%calctype='Excitonic hamiltonian in transition space'
 endif
 if(read_string('photoemission',13)>0)  then
  iparam%pes=.true. ; iparam%linear=.false.
  icount = icount +1
  iparam%calctype='Photoemission spectrum'
 endif
 if(read_string('nonlinear',9)>0) then
  iparam%nonlinear = .true. ; iparam%linear=.false.
  icount = icount +1
  iparam%calctype='Non-linear TDDFT: Dyson equation in reciprocal space'
 endif
 if(read_string('current',7)>0) then
  iparam%current = .true. ; iparam%linear=.false.
  icount=icount+1
  iparam%calctype='TDCDFT: Dyson equation in reciprocal space'
 endif
 if(read_string('analysis_exciton',16)>0) then
  iparam%analysis_exciton=.true. ; iparam%linear=.false.
  icount=icount+1
  iparam%calctype='Analysis of excitons'
 endif
 if(read_string('plot_exciton',12)>0) then
  iparam%plot_exciton=.true. ; iparam%linear=.false.
  icount=icount+1
  iparam%calctype='Plot of excitons'
 endif

 if(icount > 1) call mesg('error','please specify only one calculation type',idum,idum)

! =======================
! = Dielectric function =
! =======================

!DEFAULT
 iparam%TESTPARTICLE = .true.
 iparam%TESTELECTRON = .false.
 iparam%dieltype = 'testparticle'

 icount=0
 if(read_string('testparticle',12)>0) icount=icount+1
 if(read_string('testelectron',12)>0) then
  iparam%TESTELECTRON = .true.
  iparam%dieltype = 'testelectron'
  icount=icount+1
 endif
 if(icount > 1) call mesg('error','please specify only one dielectric function type',idum,idum)
 if(iparam%TESTELECTRON) iparam%TESTPARTICLE = .false.



! ==========================
! = Fundamental parameters =
! ==========================
!DEFAULT
 iparam%metal =.false.


 if(read_int('npwmat',6,iparam%npwmat)==0) iparam%npwmat = -1
 if(read_int('matsh',5,iparam%matsh)==0) iparam%matsh = -1
 if (iparam%npwmat > 0 .and. iparam%matsh > 0) &
   call mesg('error','please specify only one between npwmat and matsh',idum,idum)

 if(read_int('npwwfn',6,iparam%npwwfn)==0) iparam%npwwfn = -1
 if(read_int('wfnsh',5,iparam%wfnsh)==0) iparam%wfnsh = -1
 if (iparam%npwwfn > 0 .and. iparam%wfnsh > 0) &
   call mesg('error','please specify only one between npwwfn and wfnsh',idum,idum)

 if(read_int('nbands',6,iparam%nbnds)==0) iparam%nbnds = 100
 if(read_int('lomo',4,iparam%lomo)==0) iparam%lomo = 1
 if(read_float_array('q',1,iparam%qpg,3)==0) iparam%qpg(:) = 0.0
 if(read_float('omegai',6,iparam%omegai)==0) iparam%omegai = 0.0
 if(read_float('omegae',6,iparam%omegae)==0) iparam%omegae = 30.0
 if(read_float('domega',6,iparam%domega)==0) iparam%domega = 0.1
 if(read_float('broad',5,iparam%broad)==0) iparam%broad = iparam%domega  
 if(read_float('eta2',4,iparam%eta2)==0) iparam%eta2=iparam%broad
 if(read_int('alias',5,iparam%alias)==0) iparam%alias = 2
 if(read_string('aliasfree',9)>0) iparam%alias = 2

!deprecated
 if(read_int('occopt',6,iparam%occopt)==0) iparam%occopt = 0
!metals
 if(read_string('metal',5)>0) then
  iparam%metal =.true.
  iparam%occopt=1
 endif
 if(read_float('kBT',3,iparam%kBT)==0) iparam%kBT = 0.001*HARTREE
 
    
! if not specified in input file these values are read from kss or scr:
 if(read_int('nkibz',5,iparam%nkibz)==0) iparam%nkibz = -1
 if(read_int('nsppol',6,iparam%nsppol)==0) iparam%nsppol = -1
 if(read_int('nel',3,iparam%nel)==0) iparam%nel = -1
 if(read_int('nqibz',5,iparam%nqibz)==0) iparam%nqibz = -1 !from scr

 if(read_int('ngw',3,iparam%ngw)==0) iparam%ngw = 1 !? what is this


! drude term
   iparam%DRUDE=.false.
   iparam%ONLYDRUDE=.false.
  if (iparam%metal) then
   if(read_string('drude',5)>0) iparam%DRUDE=.true.
   if(read_string('onlydrude',9)>0) iparam%ONLYDRUDE = .true.
  endif



! ============
! = Energies =
! ============

!DEFAULT
 iparam%enks = .true.
 iparam%entype='Kohn-Sham eigenvalues'
 iparam%GW = .false.
 iparam%SO = .false.
 iparam%qpsenergy = .false.
 iparam%somult = .false.
 iparam%lifetime = .false.

 iparam%gwfilenew = .true. !now new GW file is default
 
 icount=0
 if(read_string('enks',4)>0) icount=icount+1
 if(read_string('gw',2)>0) then
  iparam%GW = .true.
  icount=icount+1
  iparam%entype='perturbative GW energies'
 endif
 if(read_string('so',2)>0) then
  iparam%SO = .true.
  icount=icount+1
  iparam%entype='scissor operator corrections'
 endif
 if(read_string('qpsenergy',9)>0) then
  iparam%qpsenergy = .true.
  icount=icount+1
  iparam%entype='self-consistent quasiparticle energies'
 endif
 if(read_string('somult',6)>0) then
  iparam%somult = .true.
  icount=icount+1
  iparam%entype='multiscissor operator corrections'
 endif
 if(icount > 1) call mesg('error','please specify only one energy type',idum,idum)
 if(iparam%GW .or. iparam%SO .or. iparam%qpsenergy .or. iparam%somult) iparam%enks=.false. 

 if(read_string('lifetime',8)>0) iparam%lifetime = .true.  !only Dyson tddft 
                                                           !possible to combine with any energy type
 if(read_string('oldgwfile',9)>0) iparam%gwfilenew = .false. ! use of new gw file
 
 if(read_float('soenergy',8,iparam%soenergy)==0) iparam%soenergy = 0.0 !? could set by default also so true ?
 if(read_float('stretch_valence',15,iparam%stretchvalence)==0) iparam%stretchvalence = 1.0 
 if(read_float('stretch_conduction',18,iparam%stretchconduction)==0) iparam%stretchconduction = 1.0 


 !!!? scissor on qps?

! =================
! = Wavefunctions =
! =================

!DEFAULT
 iparam%kss = .true.
 iparam%qps = .false.
 iparam%wftype='Kohn-Sham wavefunctions'

 icount=0
 if(read_string('kss',3)>0) icount=icount+1
 if(read_string('qps',3)>0) then
  iparam%qps = .true.
  icount=icount+1
  iparam%wftype='self-consistent quasiparticle wavefunctions'
 endif
 if(icount > 1) call mesg('error','please specify only one wavefunction type',idum,idum)
 if(iparam%qps) iparam%kss = .false.



! ==========
! = Kernel =
! ==========

! DEFAULT
  iparam%JDOS = .false.
  iparam%matrixel = .false.
  iparam%RPA = .true.
  iparam%LDA = .false.
  iparam%NLDA = .false.
  iparam%FXCALPHA = .false.
  iparam%FXCALPHABETA= .false.
  iparam%FXCEPS = .false.
  iparam%FXCEPSDIAGO = .false.
  iparam%CUTOFF=.false.
  iparam%FXCVBAR=.false.
  iparam%FXCMT=.false.
  iparam%FXCCONTACT=.false.
  iparam%quick = .false.
   
  iparam%EXC = .false.
  iparam%COULOMBTERM = .false.

  iparam%approxtype='RPA'


  icount=0
  if(read_string('rpa',3)>0) icount=icount+1
  if(read_string('jdos',4)>0) then
   iparam%JDOS = .true.
   icount=icount+1
   iparam%approxtype='JDOS'
  endif 
  if(read_string('printmatrixelements',19)>0) then
   iparam%matrixel = .true.
   icount=icount+1
   iparam%approxtype='Matrixelements'   
  endif
  if(read_string('alda',4)>0) then
   iparam%LDA = .true.
   icount=icount+1
   iparam%approxtype='Adiabatic lda (TDLDA)'   
  endif
  if(read_string('longrange',9)>0) then
   iparam%FXCALPHA =  .true.
   iparam%approxtype='Long-range kernel alpha/q^2'   
  endif
  if(read_string('dyn_longrange',13)>0) then
   iparam%FXCALPHABETA= .true.
   icount=icount+1
   iparam%approxtype='Long-range dynamical kernel (alpha+beta w)/q^2'
  endif
  if(read_string('fxcmt',5)>0) then
   iparam%FXCMT= .true.
   icount=icount+1
   iparam%approxtype='Nanoquanta kernel (mapping theory)'
  endif
!TODO insert definition of these other kernels; how do they work? what do they require?
  if(read_string('fxccontact',10)>0) then
   iparam%FXCCONTACT= .true.
   icount=icount+1
   iparam%approxtype='\?\?\?'
  endif
  if(read_string('nlda',4)>0) then
   iparam%NLDA = .true.
   icount=icount+1
   iparam%approxtype='\?\?\?'
  endif
  if(read_string('eps',3)>0) then
   iparam%FXCEPS = .true.
   icount=icount+1
   iparam%approxtype='\?\?\?'
  endif
  if(read_string('epsdiago',8)>0) then
    iparam%FXCEPSDIAGO = .true.
    icount=icount+1
    iparam%approxtype='\?\?\?'
  endif
  if(read_string('vbar',4)>0) then
   iparam%FXCVBAR = .true.
   icount=icount+1
   iparam%approxtype='\?\?\?'
 endif
  if(read_string('cutoff',6)>0) then
   iparam%CUTOFF=.true.
   icount=icount+1  
   iparam%approxtype='\?\?\?'
  endif
  if(read_string('quick',5)>0) then
   iparam%quick = .true.
   icount=icount+1
   iparam%approxtype='\?\?\?'
  endif

  if(read_string('exc',3)>0) then
   iparam%EXC = .true.
   iparam%COULOMBTERM = .true.
   icount=icount+1
   iparam%approxtype='Bethe-Salpeter equation with excitons'
  endif

  if(icount > 1) call mesg('error','please specify only one approximation type',idum,idum)
  if(iparam%JDOS .or. iparam%matrixel .OR. iparam%LDA .or. iparam%LDA .or. iparam%FXCALPHA .or. &
     iparam%FXCALPHABETA .or. iparam%FXCMT .or. iparam%FXCCONTACT .or. &
     iparam%NLDA .or. iparam%FXCEPS .or. iparam%FXCEPSDIAGO .or. &
     iparam%FXCVBAR .or. iparam%CUTOFF .or. iparam%quick .or. &
     iparam%EXC) iparam%RPA = .false.

  if(read_float('alpha',5,iparam%alphadiv)==0) iparam%alphadiv = 0.0
  if(read_float('rcut',4,iparam%rcut)==0) iparam%rcut = 3.0
  if(read_float('beta',4,iparam%beta)==0) iparam%beta = 0.0
! the next 2s for contact exciton
  if(read_float('V0',2,iparam%V0)==0) iparam%V0 = 0.0
  if(read_float('fcut',4,iparam%fcut)==0) iparam%fcut = 3.0


! ============
! = Coupling =
! ============

!DEFAULT (tddft Dyson)
 iparam%td = .false.

 if(read_string('tammdancoff',11)>0) iparam%td = .true.

!DEFAULT (excitonic hamiltonian)
 iparam%RESONANT = .true.
 iparam%COUPLING = .false.
 iparam%COUPLPERT = .false.
 iparam%htype = 'resonant'

 icount=0 
 if(read_string('resonant',8)>0) icount=icount+1
 if(read_string('coupling',8)>0) then
  iparam%COUPLING = .true.  
  iparam%htype = 'coupling'
  icount=icount+1
 endif
 if(read_string('couplper',8)>0) then
  iparam%COUPLPERT = .true.  
  iparam%htype = 'couplper'
  icount=icount+1
 endif
 if(icount > 1) call mesg('error','please specify only one coupling type',idum,idum)
 if(iparam%COUPLING .or. iparam%COUPLPERT) iparam%RESONANT = .false.
 if(iparam%COUPLPERT) then 
    !iparam%COUPLING = .true.
    call mesg('error','Couplpert not implemented in the stable version.',idum,idum)
 endif
!keep default value for tddft dyson and use variable iparam%td for excitonic hamiltonian
 if(iparam%exciton) then
  iparam%td = iparam%RESONANT 
 endif

! ====================
! = Shifted k points =
! ====================

!DEFAULT
 shiftk = .false. !not used ?
 iparam%inv = .true.
 iparam%ninv = 2

!deprecated
 if(read_int('nsym',4,iparam%nop)==0) iparam%nop = -1
 if(read_string('noinv',5)>0) then
  iparam%inv = .false.
  iparam%ninv = 1
 endif
! if(iparam%nop == 1 .and. iparam%ninv == 1) shiftk = .true.

 if(read_string('shiftk',6)>0) then
   shiftk = .true.
   iparam%nop = 1
   iparam%ninv = 1
   iparam%inv = .false. !check
 endif


! ===================
! = Pseudopotential =
! ===================

!DEFAULT
  iparam%vkb = .true.

  if(read_string('novkb',5)>0) iparam%vkb = .false.


! ===========================
! = Options for Dyson tddft =
! ===========================


!DEFAULT
  iparam%savememory = .false.
  iparam%hilbert = .false.
  iparam%doublegrid = .false.
  iparam%antieps = .false.
  iparam%xx = .false. 

 if(read_string('savememory',10)>0) iparam%savememory = .true. ! use savememory algorithm (this imply 3 FFT / band instead of 2) 
 if(read_string('hilbert',7)>0) iparam%hilbert = .true. ! use the hilbert transform for the chi0
 if(read_string('doublegrid',10)>0)  iparam%doublegrid = .true. ! read second kss files that is shifted by q with respect to the first one
 if(read_string('antieps',7)>0)   iparam%antieps = .true. ! calculate eps*(-w,-q) instead of eps(w,q) which should give same result
 if(read_string('x-1x',4)>0) iparam%xx = .true. ! use also the second insertion of X-1X


! =======================
! = Solver for excitons =
! =======================

!DEFAULT
 iparam%fulldiago=.true.
 iparam%haydock=.false.
 iparam%pert=.false.
 iparam%blockpert=.false.
 iparam%solvertype='Full diagonalisation'

 icount=0
 if(read_string('fulldiago',9)>0) icount=icount+1
 if(read_string('haydock',7)>0) then
   iparam%haydock=.true.
   icount=icount+1
   iparam%solvertype='Haydock iterative algorithm'
 endif
 if(read_string('pert',4)>0) then
   iparam%pert=.true.
   icount=icount+1
   iparam%solvertype='\?\?\?' !explanation?
 endif
 if(read_string('blockpert',9)>0) then
  iparam%blockpert=.true.
  icount=icount+1
  iparam%solvertype='\?\?\?' !explanation?
 endif
 if(icount > 1) call mesg('error','please specify only one solver type',idum,idum)
 if(iparam%haydock .or. iparam%pert .or. iparam%blockpert) iparam%fulldiago=.false.

 iparam%haydock_restart=.false.
 if(read_string('haydock_restart',15)>0) iparam%haydock_restart=.true.
 if(read_int('niter',5,iparam%niter)==0) iparam%niter = 100
 if(read_float('deltadeg',8,iparam%deltadeg)==0) iparam%deltadeg = 0.05
 if(read_int('maxblocksize',12,iparam%maxblocksize)==0) iparam%maxblocksize = 300
 if(read_int('order',5,iparam%order)==0) iparam%order = 1  



! ========================
! = Options for excitons =
! ========================

!DEFAULT 
 iparam%WDIAG = .true.
 iparam%WFULL = .false. 
 iparam%wtype = 'wdiag'
 iparam%EXCHANGETERM = .true.
 iparam%lftype = 'with local fields (exchange term)'
 iparam%encutoff=.false.
 iparam%dielconstscreen=.false.
 iparam%coulcutoff=.false.


 icount=0
 if(read_string('wdiag',5)>0) icount=icount+1
 if(read_string('wfull',5)>0) then
   iparam%WFULL = .true.
   icount=icount+1
   iparam%wtype = 'wfull'
 endif
 if(icount > 1) call mesg('error','please specify only one W (diago/full) type',idum,idum)
 if(iparam%WFULL) iparam%WDIAG = .false.

 icount=0
 if(read_string('lf',2)>0) icount=icount+1 
 if(read_string('nlf',3)>0) then
   iparam%EXCHANGETERM = .false.
   icount=icount+1
   iparam%lftype = 'without local fields (exchange term)'
 endif
 if(icount > 1) call mesg('error','please specify only one v (lf/nlf) type',idum,idum)
 

 if(read_string('encutoff',8)>0) iparam%encutoff=.true.
 if(read_string('dielconstscreen',15)>0) iparam%dielconstscreen=.true.
 if(read_string('coulcutoff',15)>0) iparam%coulcutoff=.true.

 if(read_float('ircut',5,iparam%ircut)==0) iparam%ircut = 0.0
 if(read_float('uvcut',5,iparam%uvcut)==0) iparam%uvcut = 100000.0
 if(read_float('stripecut',5,iparam%stripecut)==0) iparam%stripecut = 100000.0
 if(read_float('renormdielconst',15,iparam%renormdielconst)==0) iparam%renormdielconst = 1.
 if(read_float('rlcut',5,iparam%rlcut)==0) iparam%rlcut = 1000.


!DEFAULT
  iparam%kernel = .false.
  iparam%matelanalysis = .false.

  if(read_string('kernel',6)>0) iparam%kernel = .true. ! write out mt kernel
  if(read_string('matelanalysis',13)>0) iparam%matelanalysis = .true. !print detailed info about matrixelements

  if(read_int('wropt',5,iparam%wropt)==0) iparam%wropt = 0

! =========
! = Tests =
! =========

!DEFAULT
  iparam%testrun = .false.

  if(read_string('testrun',7)>0) iparam%testrun = .true. ! perform testrun


! =================
! = Photoemission =
! =================

 if(read_float('ephoton',7,iparam%ephoton)==0) iparam%ephoton = 0.


! =======
! = MPI =
! =======

  if(read_string('mpilocal',8)>0) iparam%mpilocal = .true. 

! ========
! = chi0 =
! ========

  iparam%readchi0 = .false.
  iparam%writechi0 = .false.
  if(read_string('readchi0',8)>0) iparam%readchi0 = .true.
  if(read_string('writechi0',9)>0) iparam%writechi0 = .true.


! ===============
! = Plotexciton =
! ================

!FIXME define read_int_array for lambda
  if(read_int('firstexciton',12,firstexciton)==0) firstexciton = 1
  if(read_int('lastexciton',11,lastexciton)==0) lastexciton = 100
  if(lastexciton < firstexciton) lastexciton=firstexciton  
  iparam%lambda(1)=firstexciton
  iparam%lambda(2)=lastexciton

  if(read_float('enedegenere',11,iparam%enedeg)==0) iparam%enedeg = 0.001

!  if(read_int('nstates',7,iparam%nstates)==0) iparam%nstates = 3
  if(read_float('enmin_analysis',14,iparam%enmin_analysis)==0) iparam%enmin_analysis=0.0
  if(read_float('enmax_analysis',14,iparam%enmax_analysis)==0) iparam%enmax_analysis=100.0
  if(read_int('ntrans_analysis',15,iparam%ntrans_analysis)==0) iparam%ntrans_analysis=10

!FIXME define read_int_array for ncell
  if(read_int('ncella',7,ncella)==0) ncella = 1
  if(read_int('ncellb',7,ncellb)==0) ncellb = 1
  if(read_int('ncellc',7,ncellc)==0) ncellc = 1
  iparam%ncell(1)=ncella
  iparam%ncell(2)=ncellb
  iparam%ncell(3)=ncellc

  iparam%atoms = .false. ; iparam%bonds = .false. ; iparam%hole = .false.
  if(read_string('atoms',5)>0) iparam%atoms = .true.
  if(read_string('bonds',5)>0) iparam%bonds = .true.
  if(read_string('hole',4)>0) iparam%hole = .true.
  if(read_float_array('xcarthole',9,pos,3)==0) pos(:) = 0.0
  iparam%xcarthole(:)=dble(pos(:))
  if(read_float_array('xredhole',8,pos,3)==0) pos(:) = 0.0
  iparam%xredhole(:)=dble(pos(:))
  if(read_float_array('xangsthole',10,pos,3)==0) pos(:) = 0.0
  iparam%xredhole(:)=dble(pos(:))/ANGSTROM !transform into bohr
  if(iparam%plot_exciton) then
   if(.not.(iparam%atoms.or.iparam%bonds.or.iparam%hole)) then
    write(6,*) 'choose at least one among atoms, bonds and hole'
    stop
   endif
  endif

!end plotexciton  



!=== End READING INPUT ======


! ===================
! == REDEFINITIONS == 
! ===================

!redefines some parameters according to previous choices


  iparam%NOFXC = .true.
  if(iparam%LDA .or. iparam%TESTELECTRON) iparam%NOFXC = .false.
  if(iparam%FXCALPHA .or. iparam%FXCALPHABETA .or. iparam%FXCEPS .or. iparam%FXCEPSDIAGO .or. iparam%FXCMT  &
    .or. iparam%CUTOFF .or. iparam%FXCVBAR .or. iparam%FXCCONTACT) iparam%NOFXC=.false.

  if(read_int('nq',2,iparam%nqpol)==0) iparam%nqpol = 1 ! ?
  iparam%nqpol = 1
  iparam%nomega = (iparam%omegae - iparam%omegai) / iparam%domega + 0.5 + 1

! conversion to Hartree
  iparam%omegai = iparam%omegai / HARTREE
  iparam%omegae = iparam%omegae / HARTREE
  iparam%domega = iparam%domega / HARTREE
  iparam%broad  = iparam%broad  / HARTREE
  iparam%soenergy = iparam%soenergy / HARTREE
  iparam%kBT = iparam%kBT / HARTREE
  iparam%ephoton = iparam%ephoton / HARTREE
!exciton
  iparam%ircut = iparam%ircut / HARTREE
  iparam%uvcut = iparam%uvcut / HARTREE
  iparam%stripecut = iparam%stripecut / HARTREE
!end exciton


!BSE requires excitonic hamiltonian
  if(iparam%exc) iparam%exciton=.true. 

!photoemission
  if(iparam%pes) then
   iparam%RPA = .true.
   iparam%npwmat = 1
   iparam%matsh = -1
   iparam%td = .true.
   iparam%qpg(:) = 0.
  endif
 
!jdos (note: npwmat set only after check qpol - see ?)
  if(iparam%JDOS) then
   iparam%td=.true.
   iparam%npwwfn=1 !check
   iparam%wfnsh=-1 !check
  endif
 
!printmatrixelements
  if(iparam%matrixel) then
   iparam%td=.true.
  endif

! read second kss files that is shifted by q with respect to the first one
  if(iparam%doublegrid) then
    iparam%nop  = 1
    iparam%inv  = .false.
    iparam%ninv = 1          ! no symmetries
    q_precision = 1.0e-6     ! machine precision for comparison of k-points
    kp_limit    = 1.0e-6     ! no k.p perturbation theory in rhotwg
  endif 

! calculate eps*(-w,-q) instead of eps(w,q) which should give same result
  if(iparam%antieps) then
    omegai = iparam%omegai
    iparam%omegai = - iparam%omegae
    iparam%omegae = - omegai
    iparam%qpg(:) = - iparam%qpg(:)
  endif

!testrun
  if(iparam%testrun)   iparam%vkb = .false.  ! skip calculation of commutator

!mapping theory  => savememory false
!  if(iparam%FXCMT) iparam%savememory = .false.
  

!k.p theory if |q| < kp_limit
  iparam%KP_THEORY = .false.
  if(is_small(iparam%qpg(:)-nint(iparam%qpg(:)))) iparam%KP_THEORY = .true.

!plotexciton
  if(iparam%plot_exciton .or. iparam%analysis_exciton) then
    iparam%td = .true.
    iparam%vkb = .false.
  endif

! ==============
! === CHECKS ===
! ==============

!manybody kernels (or BSE) require explicit corrections to ks
  if(iparam%FXCALPHA .or. iparam%FXCALPHABETA .or. iparam%FXCMT .or. iparam%quick .or. &
     iparam%EXC) then
   if(iparam%enks)  call mesg('warning','you are using Kohn-Sham eigenvalues with a many-body-derived kernel',idum,idum)
  endif

!strictly alda should be done with kohn-sham ingredients
  if (iparam%LDA) then
   if(.not. iparam%enks .or. .not. iparam%kss) &
    call mesg('warning','you are doing TDLDA with else than Kohn-Sham LDA eigenvalues/wavefunctions',idum,idum)
  endif

 if (iparam%nbnds < iparam%lomo ) call mesg('error','lomo cannot be greater than nbands',iparam%lomo,iparam%nbnds)

! check for consistency
  if(iparam%KP_THEORY .and. iparam%doublegrid) &
    call mesg('error','q is below q_precision (doublegrid calculation)',idum,idum);  

end subroutine rddpin

!!***


!{\src2tex{textfont=tt}}
!!****p* main/logfile
!! NAME
!! logfile
!!
!! FUNCTION
!! read all the parameters of the dp calculation
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! SOURCE

subroutine logfile(inputfile,il)

 use dpdef
 use verbose

 implicit none

 integer il
 character (len=il) :: inputfile
 integer, external :: read_int,read_file
 character(len=4) :: tag
 character(len=10) :: namelog
 character(len=11) :: nametree
 character(len=11) :: namemem

#ifdef MPI
 include 'mpif.h'
 integer ierr,rang,nprocs
 call mpi_comm_size(mpi_comm_world,nprocs,ierr)
 call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif

!TODO still to think about interactive
  if(il==0) return 
  if(read_file(inputfile,il) /= 1) stop 'error in inputfile'

! ==========
! = Output =
! ==========

  if(read_int('uout',4,uout)==0) uout= 6 ! output unit
  if(read_int('verboselevel',12,iverbose)==0) iverbose = 1 ! verbosity level 

#ifdef MPI
  if(rang/=0)then
   if(rang<10) write(tag,'(i1)') rang
   if(rang >= 10 .and. rang < 100) write(tag,'(i2)') rang
   if(rang >= 100 .and. rang < 1000) write(tag,'(i3)') rang
   namelog=trim('log_'//tag)
   close(uout) !necessary ?
   open(unit=uout,file=namelog)
  end if
  write(uout,*) 'starting on proc ',rang
#endif

  nametree='tree'
#ifdef MPI
  if(rang/=0)then
   nametree=trim('tree_'//tag)
  endif
#endif
  if(iverbose>2) open(unit=77,file=nametree)

  namemem='mem'
#ifdef MPI
  if(rang/=0)then
   namemem=trim('mem_'//tag)
  endif
#endif
  if(iverbose>2) then
   open(unit=78,file=namemem)
   write(78,20)
   write(78,30)
  endif
#ifdef MPI
20 format(60x,'     mem1        mem2         total     proc         just allocated')
30 format(60x,' __________________________________________________________________')
#else
20 format(60x,'     mem1        mem2         total          just allocated')
30 format(60x,' __________________________________________________________')
#endif

end

!!***


!{\src2tex{textfont=tt}}
!!****p* main/filenames
!! NAME
!! filenames
!!
!! FUNCTION
!! determines the name of the input (binary and text) files and create the
!! object files
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! SOURCE

  subroutine filenames(files,il,kl,gl,ql,sl,vl,klp,glp,qlp,ll,rl,inputfile,kssfile,gwfile, &
                       qpsfile,scrfile,vkbfile,kssfilep,gwfilep,qpsfilep,lambdafile,root) 

  use dpdef
  implicit none
  type(dpfiles) files

  integer il, kl, gl, ql, sl, vl, klp, glp, qlp, ll,rl

  character (len=il) :: inputfile
  character (len=kl) :: kssfile
  character (len=gl) :: gwfile
  character (len=ql) :: qpsfile
  character (len=sl) :: scrfile
  character (len=vl) :: vkbfile
  character (len=klp) :: kssfilep
  character (len=glp) :: gwfilep
  character (len=qlp) :: qpsfilep
  character (len=ll) :: lambdafile
  character (len=rl) :: root


  files%il = il
  files%kl = kl
  files%gl = gl
  files%ql = ql
  files%sl = sl
  files%vl = vl
  files%ll = ll  

  files%input=inputfile
  files%kss=kssfile
  files%gw=gwfile
  files%qps=qpsfile
  files%scr=scrfile 
  files%vkb=vkbfile
  files%lambda=lambdafile
  if (rl == 0) then
   files%root=''
   files%rl = rl
  else
   files%root=trim(root)//'_'
   files%rl = rl+1
  endif  
  

!write file used in the calculation
headline="Files concerned"
call print_headline()

if(files%il/=0) write(uout,*) 'input file: ', files%input(1:files%il)
if(files%kl/=0) then
  write(uout,*) 'kss file: ', files%kss(1:files%kl)
else
  stop 'missing kss file'
endif
if(files%gl/=0) write(uout,*) ' gw file: '//files%gw(1:files%gl)
if(files%ql/=0) write(uout,*) 'qps file: '//files%qps(1:files%ql)
if(files%sl/=0) write(uout,*) 'scr file: '//files%scr(1:files%sl)
if(files%vl/=0) write(uout,*) 'vkb file: '//files%vkb(1:files%vl)
if(files%ll/=0) write(uout,*) 'lambda file:'//files%lambda(1:files%ll)

if(klp/=0) write(uout,*)      'kss file: '//kssfilep(1:klp)
if(glp/=0) write(uout,*)      ' gw file: '//gwfilep(1:glp)
if(qlp/=0) write(uout,*)      'qps file: '//qpsfilep(1:qlp)
call print_footline()

  end
!!***


!{\src2tex{textfont=tt}}
!!****p* read_structure/print_parameters
!! NAME
!! print_parameters
!!
!! FUNCTION
!! write the parameters of the dp calculation
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! read_structure
!!
!! SOURCE
! ==================================================================
  subroutine print_parameters(iparam)
! ==================================================================
! write all the parameters of the dp calculation
! ------------------------------------------------------------------
  use ggwdef
  use dpdef
  use verbose
  implicit none
  type(dpparam) iparam
  integer i
  if(iverbose>2)call uverbose('b','print_parameters')

  headline="Parameters chosen"
  call print_headline()

  write(uout,*) iparam%calctype
  write(uout,*) 
  if(iparam%pes) then
   write(uout,*) 'Photoemission spectrum'
   write(uout,*) 'Photon energy [eV] ', iparam%ephoton*HARTREE
   if(iparam%jdos) write(uout,*) 'using constant matrixelements'
  else
   write(uout,*) 'Epsilon type:  ', iparam%dieltype
   write(uout,*) 'Approximation: ',iparam%approxtype
  endif
  write(uout,*)

  write(uout,*) 'Using ', iparam%entype
  write(uout,*) 'Using ', iparam%wftype
  write(uout,*)


! exciton
  if (iparam%exciton) then

    write(uout,*) 'using excitonic hamiltonian'
         
      if(iparam%SO .or. iparam%somult) write(uout,*) 'with scissor operator'
      if(iparam%EXCHANGETERM) then
        write(uout,*) 'local fields effects (v) included'
      else
        write(uout,*) 'local fields effects (v) excluded'
      endif
      if(iparam%COULOMBTERM) then
        write(uout,*) 'excitonic effects (W) included'
        if(iparam%WDIAG) then 
          write(uout,*) 'only W diagonal included'
        else
          write(uout,*) 'full W included'
        endif
      else
        write(uout,*) 'excitonic effects (W) excluded'
      endif
      if (iparam%LDA) then
        write(uout,*) 'excitonic effects (fxc) included in tdlda'
      endif
      if(iparam%COUPLING) then
        if(iparam%COUPLPERT) then
          write(uout,*) 'O(1) perturbative coupling term included'
        else
          write(uout,*) 'coupling term included'
        endif
      else
        write(uout,*) 'resonant term only'
      endif
      if(.not.iparam%COULOMBTERM .and. .not.iparam%COUPLING) then
        write(uout,*) 'WARNING: ANTIRESONANT EXCLUDED'
      endif

      if(iparam%haydock) then
        write(uout,*) 'solving through haydock recursion'
        write(uout,*) ' number of iteration ', iparam%niter
      endif

    write(uout,*)

  endif
!end exciton
   

! print parameters
  write(uout,1000) iparam%npwmat, iparam%matsh, iparam%npwwfn, iparam%wfnsh, iparam%nbnds, iparam%lomo, &
                iparam%nel, iparam%nkibz,iparam%nsppol, iparam%nop, iparam%inv
1000  format(' Fundamental parameters:'/          &
  ' dimension of the matrices,     npwmat = ',i5/ &
  '   corresponding to the shell   matsh  = ',i5/ &
  ' cut-off on the wavefunctions,  npwwfn = ',i5/ &
  '   corresponding to the shell   wfnsh  = ',i5/ &
  ' number of bands included,      nbands = ',i5/ &
  ' lowest occupied band included, lomo   = ',i5/ &
  ' number of electrons            nel    = ',i5/ &
  ' max number of k-points in IBZ, nkibz  = ',i5/ &
  ' number of spin polarizations,  nsppol = ',i5/ &
  ' number of symmetry on k-set,   nsym   = ',i5/ &
  ' inversion on k-set,            inv    = ',l5)


  write(uout,2000) (iparam%qpg(i),i=1,3)
2000 format( & 
  ' polarization vector [r.l.u.]   polvec = ',(3f12.6))

  write(uout,3000) iparam%omegai*HARTREE, iparam%omegae*HARTREE, &
                iparam%domega*HARTREE, iparam%broad*HARTREE
3000  format(&
  ' minimum energy [eV]            Emin   = ',f6.2/ &
  ' maximum energy [eV]            Emax   = ',f6.2/ &
  ' energy step    [eV]            dE     = ',f6.2/ &
  ' lorentzian broadening [eV]     eta    = ',f6.2) 
  
  if(iparam%occopt == 0) then
    write(uout,*) 'insulating occupations at T = 0'
  else if (iparam%occopt == 99) then
    write(uout,*) 
  else 
    write(uout,*)  'metallic occupations'
    write(uout,'(" Fermi-Dirac smearing  [eV]     kBT    = ",f8.4)') iparam%kBT*HARTREE
  endif

  if(iparam%SO)  then
   write(uout,'(" scissor operator energy ",f5.2," [eV]")') iparam%soenergy*HARTREE
   write(uout,'(" valence stretching      ",f5.2)') iparam%stretchvalence
   write(uout,'(" conduction stretching   ",f5.2)') iparam%stretchconduction
  endif
  if(iparam%FXCALPHA) write(uout,'(" alpha ",f6.2)') iparam%alphadiv
  if(iparam%FXCALPHABETA) write(uout,'(" alpha =",f6.2," beta =",f6.2)') iparam%alphadiv, iparam%beta
  if(iparam%FXCCONTACT) write(uout,*) 'V0 = ',iparam%V0
  write(uout,*)
  if(iparam%savememory) write(uout,*)'savememory flag activated '

!exciton
  if(iparam%exciton) then
   if (iparam%encutoff) then 
   write(uout,*) 'using energy cutoff'
   write(uout,1100) iparam%ircut, iparam%uvcut, iparam%stripecut
1100  format( &
   & ' ir cutoff [eV]                 ircut  = ',f6.2/&
   & ' uv cutoff [eV]                 uvcut  = ',f6.2/&
   & ' stripe cutoff [eV]          stripecut = ',f6.2/)
   endif
  endif
!endexciton



  write(uout,*)
  write(uout,*)'verbosity level',iverbose
  call print_footline()

  if(iverbose>2)call uverbose('e','print_parameters')
  return
  end
!!***

