!{\src2tex{textfont=tt}}
!!****p* read_structure/testscr
!! NAME
!! Test SCR screening file
!!
!! FUNCTION
!! This is a first reading to the scr file in order to determine
!! some preliminary values (headers, dimensions, etc.)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! read_structure, makeWq, makeWqcoup
!!
!! TODO
!! This should disappear (or at least made deprecated) once the netcdf
!! structure is definitely in place
!!
!! SOURCE
 subroutine testscr(filename,fns,nq,nomega,npwvece,npwwfne,nbndse,fform,mpilocal)

 use verbose
 use dpdef
 implicit none

 integer fns
 character (len=fns) :: filename
 integer, parameter :: iunit=12
 integer nq,nomega
 integer npwvece,npwwfne,nbndse
 integer fform,headform
 character (len=6) codvsn
 integer bantot,date,intxc,ixc,natom,ngfft(3),nkpt,&
&        nspden,nspinor,nsppol,nsym,ntype
 character (len=80) titem1(2)
 integer itype
 logical mpilocal

#ifdef MPI
include 'mpif.h'
integer ierr,rang,nprocs
call mpi_comm_size(mpi_comm_world,nprocs,ierr)
call mpi_comm_rank(mpi_comm_world,rang,ierr)
if(mpilocal .and. rang/=0) return
#endif

 if(iverbose > 2) call uverbose('b','testscr')


 if(iverbose > 0) then
  if (fns < 20) then
   headline='Testing and reading epsilon^{-1} file '//filename
  else
  headline='Testing and reading epsilon^{-1} file'
  endif
  call print_headline()
 endif
!try before em1 format
 if(fns/=0) then
   open(iunit,file=filename(1:fns),status='old',form='unformatted')
 else
   open(iunit,status='old',form='unformatted')
 endif
 rewind(iunit)
 read(iunit,err=100) codvsn,headform,fform
 if(fform/=1002) goto 100
 read(iunit,err=100) bantot,date,intxc,ixc,natom,ngfft(1:3),nkpt,&
&    nspden,nspinor,nsppol,nsym,ntype
 read(iunit,err=100)
 do itype=1,ntype
   read(iunit,err=100)
 enddo
 read(iunit,err=100)
 read(iunit,err=100) titem1
 read(iunit,err=100) npwvece,npwwfne,nbndse,nq,nomega
 close(iunit)
 if (iverbose > 0) then 
 write(uout,*) '   epsilon^-1 file in the format em1'
 write(uout,'(4x,a79/4x,a79)') titem1(1)(:79), titem1(2)(:79)
 write(uout,*) '   dimension of epsilon^-1 matrix ',npwvece
 write(uout,*) '   number of planewaves used      ',npwwfne
 write(uout,*) '   number of bands used           ',nbndse
 write(uout,*) '   number of q-points             ',nq
 write(uout,*) '   number of omega                ',nomega
 endif
 if(iverbose > 2) call uverbose('e','testscr')
  call print_footline()
 return

 100 rewind(iunit)

!now try old epsm1 format in single and double precision
 read(iunit,err=200) titem1
 read(iunit,err=200) nq
!skip q-points
 read(iunit)
 read(iunit,err=200) npwvece,npwwfne,nbndse
 close(iunit)
 fform=1003
 nomega=2
 if (iverbose > 0) then 
 write(uout,*) '   epsilon^-1 file in the format epsm1'
 write(uout,'(4x,a79/4x,a79)') titem1(1)(:79), titem1(2)(:79)
 write(uout,*) '   dimension of epsm1 matrix ',npwvece
 write(uout,*) '   number of planewaves used ',npwwfne
 write(uout,*) '   number of bands used      ',nbndse
 write(uout,*) '   number of q-points        ',nq
 endif
 if(iverbose > 2) call uverbose('e','testem1')     
  call print_footline()
 return

 200 close(iunit)
 write(uout,*) 'epsilon^-1 file format unknown'
 stop 'epsilon^-1 file format unknown'
 end
!!***


!{\src2tex{textfont=tt}}
!!****p* read_structure/rdscr
!! NAME
!! Read SCR screening file
!!
!! FUNCTION
!! Read the SCR screening abinit file
!! 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! read_structure, makeWq, makeWqcoup
!!
!! TODO
!! This should disappear (or at least made deprecated) once the netcdf
!! structure is definitely in place
!!
!! SOURCE

 subroutine rdscr(em1file,es,npwvec,nq,nomega,q,omega,epsm1,mpilocal)

 use verbose
 use dpdef
 implicit none

 real, parameter :: Ha_eV = 27.2113961
 integer es
 character (len=es) :: em1file
 integer, parameter :: iunit=12
 integer npwvece,npwwfne,nbndse,nqe,nomegae
 integer nq,nomega,npwvec,npwvecaux
 real q(3,nq)
 complex omega(nomega)
 complex epsm1(npwvec,npwvec,nomega,nq)
 real (kind=8) :: qd(3,nq)
 complex (kind=8) :: omegad(nomega)
 complex (kind=8), allocatable :: epsm1d(:,:)
 real dielconst
 integer fform,headform
 character (len=6) codvsn
 character (len=80) titem1(2)
 integer bantot,date,intxc,ixc,natom,ngfft(3),nkpt,&
         nspden,nspinor,nsppol,nsym,ntype
 integer itype,ii,io,iq,ipwe
 logical mpilocal
#ifdef MPI
include 'mpif.h'
integer ierr,rang,nprocs
call mpi_comm_size(mpi_comm_world,nprocs,ierr)
call mpi_comm_rank(mpi_comm_world,rang,ierr)
if(mpilocal .and. rang/=0) return
#endif


 if(iverbose > 2) call uverbose('b','rdscr')
 
 if(iverbose > 0) then 
  headline='Reading epsilon-twiddle^-1 file in the scr format'
  call print_headline()
 endif

 epsm1(:,:,:,:)=(0.0,0.0)

 if(es/=0) then
   open(iunit,file=em1file(1:es),status='old',form='unformatted')
 else
   open(iunit,status='old',form='unformatted')
 endif
 read(iunit) codvsn,headform,fform
 read(iunit) bantot,date,intxc,ixc,natom,ngfft(1:3),nkpt,&
     nspden,nspinor,nsppol,nsym,ntype
 read(iunit)
 do itype=1,ntype
   read(iunit)
 enddo
 read(iunit)
 read(iunit) titem1
 read(iunit) npwvece,npwwfne,nbndse,nqe,nomegae
 npwvecaux=npwvec
 if(npwvece>npwvec) then
   write(uout,*) '   dimension of epsm1 matrix         ', npwvece
   write(uout,*) '   warning: reading a smaller matrix ', npwvec
 else if(npwvece<npwvec) then
   write(uout,*) '   dimension of epsm1 matrix         ', npwvece
   write(uout,*) '   warning: requiring a bigger matrix ', npwvec
   write(uout,*) '         0-padding the rest of the matrix'
   npwvecaux=npwvece
 endif
 if(nqe<nq) stop 'requiring too much q-points'
 if(nomegae<nomega) stop 'requiring too much omega-points'
 read(iunit) ! skip gvec
 read(iunit) qd(1:3,1:nq)
 read(iunit) omegad(1:nomega)
 q(:,:) = qd(:,:)
 omega(:) = omegad(:)

 if(iverbose > 0) then
 write(uout,'(a,/,(3x,i3,3f14.6))')&
  '    q-points [reciprocal lattice units]:',(iq,(q(ii,iq),ii=1,3),iq=1,nq)
 write(uout,'(a,/,(3x,i3,2f7.2))')&
  '    frequencies omega [eV]:',(io,omega(io)*Ha_eV,io=1,nomega)
 endif

!  read epsilon^-1
 if (codvsn=='6.2.0'.or.codvsn=='6.5.0')  then
! this is the new way of abinit to write the scr file
  allocate(epsm1d(npwvece,1))
  do iq=1,nq
    do io=1,nomega
      do ipwe=1,npwvecaux
        read(iunit) epsm1d(1:npwvece,1)
        epsm1(1:npwvecaux,ipwe,io,iq)=epsm1d(1:npwvecaux,1)
      end do
      do ipwe=npwvecaux+1,npwvece
        read(iunit)
      enddo
    end do
    do io=nomega+1,nomegae
      do ipwe=1,npwvece
        read(iunit)
      enddo
    end do
  end do
 else
! this is the old way of abinit to write the scr file
  allocate(epsm1d(npwvece,npwvece))
  do iq=1,nq
    do io=1,nomega
      read(iunit) epsm1d(1:npwvece,1:npwvece)
      epsm1(1:npwvecaux,1:npwvecaux,io,iq) = epsm1d(1:npwvecaux,1:npwvecaux)
    enddo
    do io=nomega+1,nomegae
      read(iunit) ! skip other omegas
    enddo
  enddo
 endif


close(iunit)
 deallocate(epsm1d)
 
 if(abs(real(omega(1)))<0.01 .and. abs(aimag(omega(1)))<0.01) then
   do iq=1,nq
     if(all(abs(q(:,iq))<0.001)) then
     dielconst = 1.0/real(epsm1(1,1,1,iq))
     if(iverbose > 0)  & 
           write(uout,'(a,f6.2)') '    dielectric constant with LF = ', dielconst
     endif
   enddo
 endif

 if(iverbose > 0) then
  call print_footline()
 endif

 if(iverbose > 2) call uverbose('e','rdscr')
 end
!!***


!{\src2tex{textfont=tt}}
!!****p* read_structure/rdepsm1
!! NAME
!! rdepsm1
!!
!! FUNCTION
!! Read the old format epsm1 screening file
!!
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! read_structure, makeWq, makeWqcoup
!!
!! TODO
!! This should disappear (or at least made deprecated) once the netcdf
!! structure is definitely in place
!!
!! SOURCE
 subroutine rdepsm1(em1file,es,npwmat,nq,q,epsm1,bigomegatwsq,omegatw,mpilocal)

 use verbose
 use dpdef
 implicit none

 integer es
 character (len=es) :: em1file
 integer, parameter :: iunit=12
 integer npwvece,npwwfne,nbndse
 integer npwmat
 integer nq, nqr
 real q(3,nq)
 real (kind(0.0d0)) qd(3,nq)
 real qs(3,nq)
 complex cdum
 complex,allocatable :: a(:,:),omegatwsq(:,:)
 character (len=80) titem1(2)
 complex bigomegatwsq(npwmat,npwmat,nq),omegatw(npwmat,npwmat,nq)
 complex epsm1(npwmat,npwmat,nq)

 integer ii,iq,istat,jj,ig,igp
 logical mpilocal

#ifdef MPI
include 'mpif.h'
integer ierr,rang,nprocs
call mpi_comm_size(mpi_comm_world,nprocs,ierr)
call mpi_comm_rank(mpi_comm_world,rang,ierr)
if(mpilocal .and. rang/=0) return
#endif

 if(iverbose > 2) call uverbose('e','rdepsm1')
 write(uout,*) '   reading epsilon-twiddle^-1 file in the epsm1 format'

 allocate(a(npwmat,npwmat),stat=istat)
 if(istat/=0) stop 'out of memory'
 allocate(omegatwsq(npwmat,npwmat),stat=istat)
 if(istat/=0) stop 'out of memory'

 if(es/=0) then
   open(iunit,file=em1file(1:es),status='old',form='unformatted')
 else
   open(iunit,status='old',form='unformatted')
 endif
 rewind(iunit)
! try first double precision

 read(iunit) titem1
 read(iunit) nqr
 read(iunit,err=400) ((qd(ii,iq),ii=1,3),iq=1,nq)
 q(:,:) = qd(:,:)
 read(iunit) npwvece,npwwfne,nbndse

 if(nqr/=nq) then
   write(uout,*) '**error rdepsm1 1: number of q-point error'
   stop
 endif
 if(npwvece<npwmat) then
   write(uout,*) '**error rdepsm1 2: required more plane-wave vectors'
   write(uout,*) '                  than available in epsilon^-1 file'
   write(uout,*) '                  npwmat = ',npwmat
   stop
 endif

 if (iverbose > 0) then
 write(uout,'(a,/,a80)') '    title:',titem1
 write(uout,*) '   number of q-points ',nqr
 write(uout,'(a,/,(3x,i3,3f14.6))')&
  '    q-points [reciprocal lattice units]:',(iq,(q(ii,iq),ii=1,3),iq=1,nq)

 write(uout,'(a,3(/,a,i5))') '    parameters in epsilon^-1 calculation:',&
  '    number of plane wave vectors, NPWVEC = ',npwvece,&
  '    number of plane waves for wavefunctions, NPWWFN = ',npwwfne,&
  '    number of bands, NBNDS  = ',nbndse
 endif

!read plasmon-pole parameters (A and omega-twiddle-squared)
!and write to direct-access scratch file
 do iq=1,nq

!  skip over table of |q+G|: this will be calculated separately
   read(iunit)

!  read A parameters
   read(iunit) ((a(ii,jj),ii=1,npwmat),(cdum,ii=npwmat+1,npwvece),&
    jj=1,npwmat)
   
   epsm1(:,:,iq) = a(:,:)     
   do ii = 1, npwmat
     epsm1(ii,ii,iq) = 1.0 + epsm1(ii,ii,iq)
   enddo
   if (iverbose > 0) then
   if(all(abs(q(:,iq))<1.e-3)) then
     write(uout,'(a,f5.2)') &
      '    macroscopic dielectric constant = ',real(1.0 / epsm1(1,1,iq))
   endif
   endif

!  read omega-twiddle-squared parameters
   read(iunit) ((omegatwsq(ii,jj),ii=1,npwmat),(cdum,ii=npwmat+1,npwvece),&
    jj=1,npwmat)
   
!  If omega-twiddle-squared is negative,
!  set omega-twiddle-squared to 1.0 (a reasonable way of treating
!  such terms, in which epsilon**-1 was originally increasing
!  along this part of the imaginary axis)
!  (note: originally these terms were ignored in Sigma; this was
!  changed on 6 March 1990.)
   do ig=1,npwmat
     do igp=1,npwmat
       if(real(omegatwsq(ig,igp))<=0.0) omegatwsq(ig,igp)=1.0
     enddo
   enddo
!  calculate omega-twiddle
!  we neglect the imag part (if one) in omega-twiddle-squared         
   omegatw(:,:,iq)=sqrt(real(omegatwsq(:,:)))

!  calculate big-omega-twiddle-squared = -omega-twiddle-squared A
   bigomegatwsq(:,:,iq)=-a(:,:)*omegatw(:,:,iq)**2

 enddo

 close(iunit)
 deallocate(a,omegatwsq)

 if(iverbose > 0) then 
  call print_footline()
 endif

 if(iverbose > 2) call uverbose('e','testepsm1')
 return
 
 400 rewind(iunit)
! now try first single precision

 read(iunit) titem1

 read(iunit) nqr

 read(iunit,err=400) ((qs(ii,iq),ii=1,3),iq=1,nq)
 q(:,:) = qs(:,:)
 read(iunit) npwvece,npwwfne,nbndse
 if (iverbose > 0) then
 write(uout,'(a,/,(3x,i3,3f14.6))')&
  '    q-points [reciprocal lattice units]:',(iq,(q(ii,iq),ii=1,3),iq=1,nq)

 write(uout,'(a,3(/,a,i5))') ' parameters in epsilon^-1 calculation:',&
  '    number of plane wave vectors, NPWVEC = ',npwvece,&
  '    number of plane waves for wavefunctions, NPWWFN = ',npwwfne,&
  '    number of bands, NBNDS  = ',nbndse
 endif

 if(npwvece<npwmat) then
   write(uout,*) '**error rdepsm1 2: required more plane-wave vectors'
   write(uout,*) '                  than available in epsilon^-1 file'
   write(uout,*) '                  npwmat = ',npwmat
   stop
 endif

!read plasmon-pole parameters (A and omega-twiddle-squared)
!and write to direct-access scratch file
 do iq=1,nq

!  skip over table of |q+G|: this will be calculated separately
   read(iunit)

!  read A parameters
   read(iunit) ((a(ii,jj),ii=1,npwmat),(cdum,ii=npwmat+1,npwvece),&
&   jj=1,npwmat)

   epsm1(:,:,iq) = a(:,:)     
   do ii = 1, npwmat
     epsm1(ii,ii,iq) = 1.0 + epsm1(ii,ii,iq)
   enddo
   if (iverbose > 0) then
   if(all(abs(q(:,iq))<1.e-3)) then
     write(uout,'(a,f5.2)') &
&     '    macroscopic dielectric constant = ',real(1.0 / epsm1(1,1,iq))
   endif
   endif

!  read omega-twiddle-squared parameters
   read(iunit) ((omegatwsq(ii,jj),ii=1,npwmat),(cdum,ii=npwmat+1,npwvece),&
&   jj=1,npwmat)

!  If omega-twiddle-squared is negative,
!  set omega-twiddle-squared to 1.0 (a reasonable way of treating
!  such terms, in which epsilon**-1 was originally increasing
!  along this part of the imaginary axis)
!  (note: originally these terms were ignored in Sigma; this was
!  changed on 6 March 1990.)
   do ig=1,npwmat
     do igp=1,npwmat
       if(real(omegatwsq(ig,igp))<=0.0) omegatwsq(ig,igp)=1.0
     enddo
   enddo
!  calculate omega-twiddle
!  we neglect the imag part (if one) in omega-twiddle-squared         
   omegatw(:,:,iq)=sqrt(real(omegatwsq(:,:)))

!  calculate big-omega-twiddle-squared = -omega-twiddle-squared A
   bigomegatwsq(:,:,iq)=-a(:,:)*omegatw(:,:,iq)**2

 enddo

 close(iunit)
 deallocate(a,omegatwsq)

 if(iverbose > 0) then 
  call print_footline()
 endif
 if(iverbose > 2) call uverbose('e','rdepsm1')
 end
!!***
