!{\src2tex{textfont=tt}}
!!****p* read_structure/testlda
!! NAME
!! Test Kohn Sham Structure
!!
!! FUNCTION
!! This is a first reading to the kss or qplda files in order to determine
!! some preliminary values (headers, dimensions, etc.)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! NOTE
!! After verifying the fileformat, this subroutine gets the abinit header
!! (call rdhdr), if fileformat is compatible to some abinit version, otherwise
!! it gets the essential information to read the old qplda file (this comes
!! from the cp code, or from some particular interfaces).
!!
!! PARENTS
!! read_structure
!!
!! CHILDREN
!! mesg, rdhdr, reorder_shells, fileform 
!!
!! NOTE
!! This subroutine also contains the reorder_shells subroutine (internal)
!! in order to put the G vectors in the right order (sometimes this is not
!! the case from recent abinit version, and anyway this will be needed
!! once the netcdf format is in place)
!!
!! TODO
!! This should disappear (or at least made deprecated) once the netcdf 
!! structure is definitely in place
!!
!! SOURCE

!=======================================================================
subroutine testlda(iparam,kssfile,kl,nkpt,nbnd,nsppol,npw,i1,i2,&
                   kb,nel,elecstr)
!=======================================================================
! test QPLDA type file
use ggwdef
use dpdef
use verbose
implicit none
type(dpparam) iparam
type(crystal) crystr
type(symmetry) symstr
type(chemistry) chemstr
type(transposed_space) foustr
type(electronic_structure) elecstr
type(kb_potential) kb
type(lattstruct) latt
integer kl
character (len=kl) :: kssfile
integer, parameter :: u=10
integer fform
integer nsym, nkpt, nbnd, npw, nsh, nel, nsym2, nsppol
integer i1, i2, ig
integer, external :: fileform
integer, allocatable :: tmpvec(:,:), shlimtmp(:)
double precision, allocatable :: energyd(:)
real, allocatable :: energys(:)
integer i,j,idum,il,is,isppol,ib,ik


#ifdef MPI
include 'mpif.h'
integer ierr,rang,nprocs
call mpi_comm_size(mpi_comm_world,nprocs,ierr)
call mpi_comm_rank(mpi_comm_world,rang,ierr)
if(iparam%mpilocal .and. rang/=0) return
#endif

if (iverbose > 2)call uverbose('b','testlda')

!default is kss file
iparam%oldqplda = .false.
if (kl <20) then
 headline= 'Checking kss file '//kssfile
else
 headline='Checking kss file'
endif
call print_headline()
fform = fileform(kssfile,kl)
if(fform == 502) then
  write(uout,*) 'kss ABINIT double precision form'
else if(fform == 602) then
  write(uout,*) 'kss ABINIT single precision form'
else if(fform==2) then
  write(uout,*) 'sta double precision form'
else if(fform==1) then
  write(uout,*) 'qplda LSI-CP, PWSCF, FHIMD single precision form'
  iparam%oldqplda=.true.
else
  call mesg('error','unrecognized file format - check the endianness',idum,idum)
endif

i2 = fform
if(fform < 0) stop 'unrecognized file'
if(fform == 502 .or. fform == 602) then
  open(u,form='unformatted',file=kssfile)
  call rdhdr(u,crystr,symstr,chemstr,foustr,elecstr)
    
  latt%a1(1:3) = crystr%rprimd(1:3,1)
  latt%a2(1:3) = crystr%rprimd(1:3,2)
  latt%a3(1:3) = crystr%rprimd(1:3,3)

  read(u) !tit(1)
  read(u) !tit(2)
  read(u) nsym2, nbnd, npw, nsh, kb%mpsang
  nsym = nsym2
  kb%ntype = chemstr%ntype
  kb%natom = chemstr%natom
  nel = chemstr%nel
  nkpt = foustr%nk
  nsppol = elecstr%nsppol
  write(uout,*) 'number of atomic species      ', kb%ntype
  write(uout,*) 'number of atoms               ', kb%natom
  write(uout,*) 'number of ang mom components  ', kb%mpsang
  write(uout,*) 'number of symmetry operations ', nsym
  write(uout,*) 'number of plane waves         ', npw
  write(uout,*) 'number of k-points            ', nkpt
  write(uout,*) 'number of bands               ', nbnd
  write(uout,*) 'number of electrons           ', nel
  write(uout,*) 'number of spin polarizations  ', nsppol

 read(u)  !(((symrel2(jj,ii,isym),ii=1,3),jj=1,3),isym=1,nsym2)
 read(u)  !((tnons(i,isym),i=1,3),isym=1,nsym2)
 allocate(tmpvec(3,npw))
 read(u) ((tmpvec(i,j),i=1,3),j=1,npw)
 read(u)  !(shlim(i),i=1,nsh)
 read(u)  !((vkbsignd(il,is),il=1,mpsang),is=1,ntypat)


! here we reorder the shell, because most likely are badly written by abinit
! write(uout,*) 'reorder shells'
  allocate(shlimtmp(npw))
  do ig=1,npw
  shlimtmp(ig)=ig
  enddo
  call reorder_shells()
  deallocate(tmpvec)
  iparam%nsh = nsh
  allocate(iparam%shlim(nsh))
  iparam%shlim(1:nsh)=shlimtmp(1:nsh)
  deallocate(shlimtmp)

! let's go on reading up to the energies
!      allocate(elecstr%energytmp(foustr%nk,nbnd,elecstr%nsppol))
      if (fform==502) allocate(energyd(1:nbnd))
      if (fform==602) allocate(energys(1:nbnd))
       do isppol=1, elecstr%nsppol
         do ik=1,foustr%nk
           do is = 1, kb%ntype
            do il = 1, kb%mpsang
             read(u) !vkbdb(:,is,il)
             read(u) !vkbdd(:,is,il)
            end do 
          end do 
          if (fform==502) then
           read(u) energyd(1:nbnd)
!          elecstr%energytmp(ik,1:nbnd,isppol)= energyd(1:nbnd)
          else if (fform==602) then
           read(u) energys(1:nbnd)
!           elecstr%energytmp(ik,1:nbnd,isppol)= dble(energys(1:nbnd))
          end if
          do ib=1,nbnd
           read(u) !wfgd(1:npwkss)
          end do
         end do !ik 
        end do !isppol
 
  close(u)

else if(fform==1 .or. fform==2) then

  open(u,form='unformatted',file=kssfile)
  read(u) !tit(1)
  read(u) !tit(2)
  read(u) i1,i2
  read(u) !     skip real-space lattice vectors
  read(u) nsym
  write(uout,*) 'number of symmetry operations ', nsym
  read(u) !     skip symmetry operations
  read(u) npw
  write(uout,*) 'number of plane waves         ', npw
  read(u) !     skip g-vector
  read(u) nkpt
  write(uout,*) 'number of k-points            ', nkpt
  read(u) !     skip k-points
  read(u) nbnd
  write(uout,*) 'number of bands               ', nbnd
  write(uout,*) 
  close(u)
  nel = -1
  nsppol = 1

endif


if(i2/=0.and.i2/=1.and.i2/=2.and.i2/=502.and.i2/=602) then
 call mesg('error','unknown file form',idum,idum)
endif 
if(i2==502 .or. i2==602) then
 kb%exists =.true.
else
 kb%exists = .false.
endif

iparam%nopr = nsym

call print_footline()

if (iverbose > 2)call uverbose('e','testlda')

contains
  subroutine reorder_shells()
    integer :: ish, jsh, osl
    real :: mvec1, mvec2
    real :: ucvol, b1(3), b2(3), b3(3), g(3)
    real, external :: vlngth

    !calculate brillouin zone and unit cell volumes
    ucvol = latt%a1(1)* (latt%a2(2)*latt%a3(3)-latt%a2(3)*latt%a3(2)) &
      + latt%a1(2)* (latt%a2(3)*latt%a3(1)-latt%a2(1)*latt%a3(3))     &      
      + latt%a1(3)* (latt%a2(1)*latt%a3(2)-latt%a2(2)*latt%a3(1))
               

    b1(1) = 2.0 * PI * (latt%a2(2)*latt%a3(3)-latt%a2(3)*latt%a3(2))/ucvol
    b1(2) = 2.0 * PI * (latt%a2(3)*latt%a3(1)-latt%a2(1)*latt%a3(3))/ucvol
    b1(3) = 2.0 * PI * (latt%a2(1)*latt%a3(2)-latt%a2(2)*latt%a3(1))/ucvol
    b2(1) = 2.0 * PI * (latt%a3(2)*latt%a1(3)-latt%a3(3)*latt%a1(2))/ucvol
    b2(2) = 2.0 * PI * (latt%a3(3)*latt%a1(1)-latt%a3(1)*latt%a1(3))/ucvol
    b2(3) = 2.0 * PI * (latt%a3(1)*latt%a1(2)-latt%a3(2)*latt%a1(1))/ucvol
    b3(1) = 2.0 * PI * (latt%a1(2)*latt%a2(3)-latt%a1(3)*latt%a2(2))/ucvol
    b3(2) = 2.0 * PI * (latt%a1(3)*latt%a2(1)-latt%a1(1)*latt%a2(3))/ucvol
    b3(3) = 2.0 * PI * (latt%a1(1)*latt%a2(2)-latt%a1(2)*latt%a2(1))/ucvol

    mvec1 = 0.0
    jsh   = 1
    do ish = 2, npw
      osl   = shlimtmp(ish - 1) + 1
!      g(:)  = trspace%gvec(:, osl)
      g(:)  = tmpvec(:, osl)
      mvec2 = vlngth(g, b1, b2, b3)
!      write(uout,*) ish, mvec2, g

      if(abs(mvec2 - mvec1) > 1e-5) then
        jsh = jsh + 1
        mvec1 = mvec2
      end if
      
      shlimtmp(jsh) = shlimtmp(ish)
    end do
    nsh = jsh

  end subroutine reorder_shells

end subroutine
!!***

!{\src2tex{textfont=tt}}
!!****p* read_structure/rdkss
!! NAME
!! Read Kohn-Sham Structure
!!
!! FUNCTION
!! Read _KSS abinit file (wfn, eigenvalues, occupations, etc.)  
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! read_structure
!!
!! CHILDREN
!! rdhdr, prhdr, fileform 
!!
!! SOURCE
!=======================================================================
subroutine rdkss(kssfile,kl,iparam,&
&                tit,latt,symstr,trspace,bzstruct,elecstr,&
&                kb,chemstr)
!=======================================================================
use dpdef
use ggwdef
use verbose
implicit none
type(dpparam) iparam
type(crystal) crystr
type(symmetry) symstr
type(chemistry) chemstr
type(transposed_space) trspace
type(bzstructure) bzstruct
type(transposed_space) foustr
type(electronic_structure) elecstr
type(lattstruct) latt
type(kb_potential) kb
integer, parameter :: u = 10
integer kl
character (len=kl) :: kssfile
character (len=80) :: tit(2)

integer mpsang


integer :: isppol
character(len=7) :: stag(2)=(/' UP   ',' DOWN '/) 

integer :: nsym2, nbndsto, ncomsto, nsh
integer, allocatable :: symrel2(:,:,:)
double precision, allocatable :: tnons2(:,:)


real, allocatable :: vkbsign(:,:)
double precision, allocatable :: vkbsignd(:,:)
double precision, allocatable :: vkb(:,:,:), vkbd(:,:,:)
double precision, allocatable :: energy(:)
      
real, allocatable :: vkbs(:,:,:), vkbds(:,:,:)
real, allocatable :: energys(:)
     
integer i, j, ik, ib, ig, is, isym, isymin, isymend, il
integer ish, osl
real dum
integer fform
integer, external :: fileform
#ifdef MPI
include 'mpif.h'
integer ierr,rang,nprocs
call mpi_comm_size(mpi_comm_world,nprocs,ierr)
call mpi_comm_rank(mpi_comm_world,rang,ierr)
if(iparam%mpilocal .and. rang/=0) return
#endif


if (iverbose > 2)call uverbose('b','rdkss')

if(kl < 20) then
headline="Reading kss file "//kssfile
else
headline="Reading kss file"
endif
call print_headline()

fform = fileform(kssfile,kl)
if(fform /= 502 .and. fform /= 602) stop 'not kss form file'

open(u,form='unformatted',file=kssfile)
call rdhdr(u,crystr,symstr,chemstr,foustr,elecstr)
write (uout,*)
call prhdr(crystr,chemstr,foustr,elecstr)

latt%a1(1:3) = crystr%rprimd(1:3,1)
latt%a2(1:3) = crystr%rprimd(1:3,2)
latt%a3(1:3) = crystr%rprimd(1:3,3)
if(kb%natom /= chemstr%natom) stop 'error natom'
! modif per silvana-per prendere primo k
! matteo: should be OK from testvar even without this redefinition
!         instead gives segmentation fault  at deallocation vkb .. wfg
!if(iparam%nkibz /= foustr%nk) stop 'error nk'
if(iparam%nkibz /= foustr%nk) then
   write(uout,*)'WARNING : ONLY 1st K-POINT - DANGER'
   foustr%nk = iparam%nkibz
endif

bzstruct%kibz(1:3,1:foustr%nk) = foustr%k(1:3,1:foustr%nk)
elecstr%oc(:,:,:) = 0.0
deallocate(foustr%k)

do isppol = 1,iparam%nsppol
 do ik = 1, iparam%nkibz
!note:
! elecstr%occ is defined only up to elecstr%nb (see in rdhdr)
! elecstr%nb is the number of bands in scf calculation in abinit (variable nband)
! so it can be less than the number of bands stored in KSS (and less than iparam%nbnds)
! safest definition (the rest of elecstr%oc is anyway 0):
  do ib = 1, elecstr%nb 
    if(ib>iparam%nbnds) cycle  ! elecstr%nb can be greater than iparam%nbnds
    elecstr%oc(ik,ib,isppol) = elecstr%occ(ik,ib,isppol)
  enddo
 enddo
enddo 

if(iverbose > 2) then 
 write(uout,*) 'occupations:'
  do isppol = 1, iparam%nsppol
   do ik = 1, iparam%nkibz
     write(uout,'(1x,12f6.2/60(1x,12f6.2/))') &
&      (elecstr%oc(ik,ib,isppol),ib=1,iparam%nbnds)
   enddo
  enddo
 write(uout,*) 
endif

 if(iverbose > 2) then
   call print_symmetry_structure(symstr)
 endif


read(u) tit(1)
read(u) tit(2)

read(u) nsym2, nbndsto, ncomsto, nsh, mpsang

if(mpsang/=kb%mpsang) stop 'mpsang/=kb%mpsang'
if(iparam%nopr>nsym2) stop 'iparam%nopr>nsym2'
if(iparam%npwvec>ncomsto) stop 'iparam%npwvec>ncomsto'
if(iparam%npwwfn>ncomsto) stop 'iparam%npwwfn>ncomsto'
if(iparam%nbnds>nbndsto) stop 'iparam%nbnds>nbndsto'
      
allocate(symrel2(3,3,nsym2))
! pay attention: the indeces i,j are inverted because a transposition
! is needed to obtain symmetries in transposed k-space
! in the file symmetries in real space are stored, identical to what is
! read in the header
read(u) (((symrel2(j,i,isym),i=1,3),j=1,3),isym=1,nsym2) ! the index is inverted
symstr%op= symrel2
allocate(tnons2(3,nsym2))
read(u) ((tnons2(i,isym),i=1,3),isym=1,nsym2)
      
! the inversion could be taken into account
! but not the non-symmorphic
if(iverbose > 1) then
 write(uout,*) 'symmetries of the system in transposed space (must be symmorphyc): '
 write(uout,'("  number of symmetry operations ",i2)') iparam%nopr
 write(uout,*) ' symmetry operations (R t)'
 do isymin=1,iparam%nopr,4
   isymend=isymin+3
   if(isymend>iparam%nopr) isymend=iparam%nopr
   do i=1,3
    write(uout,'(4(3i3,f8.3,3x))') &
    ((symrel2(i,j,isym),j=1,3),tnons2(i,isym),isym=isymin,isymend)
   enddo
   write(uout,*)
 enddo
endif
deallocate(symrel2)

if(.not.iparam%nop==1 .and. .not.iparam%ninv==1) then
 if(any(abs(tnons2(:,1:iparam%nopr))>0.0001)) then
  write(uout,*) 'ERROR: non-symmorphyc symmetries found'
  write(uout,*) 'try to change the atomic position'
  stop 'non-symmorpyc symmetries'
 endif
endif
deallocate(tnons2)

read(u) ((trspace%gvec(i,ig),i=1,3),ig=1,iparam%npwvec)
!allocate(tmpvec(3,ncomsto))
!read(u) ((tmpvec(i,ig),i=1,3),ig=1,ncomsto)


read(u) !(shlim(i),i=1,nsh)

if(iverbose > 1) then
write(uout,'(" shell limit G-vectors [reduced coordinates]")')
osl = 1
do ish = 1, iparam%nsh
  if(iparam%shlim(ish)>iparam%npwvec) osl = iparam%shlim(ish)+1
  write(uout,'(i6,i6,4(i5," (",3i3,")"), &
&         20(/12x,4(i5," (",3i3,")")))') &
&    ish, iparam%shlim(ish), (ig,(trspace%gvec(i,ig),i=1,3),ig=osl,iparam%shlim(ish))
  osl = iparam%shlim(ish)+1
enddo
write(uout,*)
endif

ish = 1
do while((iparam%shlim(ish) < iparam%npwwfn) .and. (ish < iparam%nsh))
  ish = ish + 1
enddo
if(iparam%shlim(ish) > iparam%npwwfn .or. ish > iparam%nsh) then
  write(uout,*) 'WARNING: npwwfn not on a shell'
  write(uout,*) 'chose instead ', iparam%shlim(ish-1)
endif
ish = 1
do while((iparam%shlim(ish) < iparam%npwvec) .and. (ish < iparam%nsh))
  ish = ish + 1
enddo
if(iparam%shlim(ish) > iparam%npwvec .or. ish > iparam%nsh) then
  write(uout,*) 'WARNING: npwmat not on a shell'
  write(uout,*) 'chose instead ', iparam%shlim(ish-1)
endif

if(fform==502) then
  allocate(vkbsignd(mpsang,chemstr%ntype))
  read(u) ((vkbsignd(il,is),il=1,mpsang),is=1,chemstr%ntype)
  if(.not.iparam%JDOS .and. kb%inclvkb) kb%vkbsign(1:mpsang,1:chemstr%ntype) = vkbsignd(1:mpsang,1:chemstr%ntype)
  deallocate(vkbsignd)
else if(fform==602) then
  allocate(vkbsign(mpsang,chemstr%ntype))
  read(u) ((vkbsign(il,is),il=1,mpsang),is=1,chemstr%ntype)
  if(.not.iparam%JDOS .and. kb%inclvkb) kb%vkbsign(1:mpsang,1:chemstr%ntype) = vkbsign(1:mpsang,1:chemstr%ntype)
  deallocate(vkbsign)
endif
 
if(fform==502) then      
  allocate(vkb(ncomsto,chemstr%ntype,mpsang))
  allocate(vkbd(ncomsto,chemstr%ntype,mpsang))
  allocate(energy(nbndsto))
else if(fform==602) then
  allocate(vkbs(ncomsto,chemstr%ntype,mpsang))
  allocate(vkbds(ncomsto,chemstr%ntype,mpsang))
  allocate(energys(nbndsto))
endif


write(uout,*) ' k       eigenvalues [eV]'
do isppol = 1, elecstr%nsppol 
 do ik = 1, foustr%nk
! write(uout,*) 'ik ', ik
  do is = 1, chemstr%ntype
!   write(uout,*) 'is ', is
    do il = 1, mpsang
!     write(uout,*) 'il ', il
      if(fform==502) then      
        read(u) vkb(:,is,il)
        read(u) vkbd(:,is,il)
        if(.not.iparam%JDOS .and. kb%inclvkb) kb%vkb(1:iparam%npwwfn,is,il,ik) = vkb(1:iparam%npwwfn,is,il)
        if(.not.iparam%JDOS .and. kb%inclvkb) kb%vkbd(1:iparam%npwwfn,is,il,ik) = vkbd(1:iparam%npwwfn,is,il)
      else if(fform==602) then
        read(u) vkbs(:,is,il)
        read(u) vkbds(:,is,il)
        if(.not.iparam%JDOS .and. kb%inclvkb) kb%vkb(1:iparam%npwwfn,is,il,ik) = vkbs(1:iparam%npwwfn,is,il)
        if(.not.iparam%JDOS .and. kb%inclvkb) kb%vkbd(1:iparam%npwwfn,is,il,ik) = vkbds(1:iparam%npwwfn,is,il)
      endif
    enddo !il
  enddo !is 
        
! check that the reference be coherent with vkb vkbd
  if(.not.iparam%JDOS .and. kb%inclvkb) then
   do is = 1, chemstr%ntype
    if(chemstr%psa(is)%lloc /= -1) then
      il = chemstr%psa(is)%lloc + 1
      dum = sum(kb%vkb(:,is,il,ik))
      dum = dum + sum(kb%vkbd(:,is,il,ik))
      if(dum/=0.0) then
        write(uout,*) '** WARNING: lloc not coherent with vkb'
        write(uout,*) 'specie ', is
        write(uout,*) 'lloc ', chemstr%psa(is)%lloc
        write(uout,*) 'sum(vkb + vkbd) ', dum
      endif
    else
      ! found lloc on the basis of vkb
    endif
   enddo
  endif               

 
  if(fform==502) then      
    read(u) energy(1:nbndsto)
    if(ik .le. iparam%nkibz .and. isppol .le. iparam%nsppol) &
     elecstr%energy(ik,1:iparam%nbndmax,isppol) =  energy(1:iparam%nbndmax)
  else if(fform==602) then
    read(u) energys(1:nbndsto)
    if(ik .le. iparam%nkibz .and. isppol .le. iparam%nsppol) &
    elecstr%energy(ik,1:iparam%nbndmax,isppol) = dble(energys(1:iparam%nbndmax))
  endif
        
   if(iparam%nsppol==2) then   
     write(uout,'(i5,a,10f7.2/50(10x,10f7.2/))') &
&        ik, stag(isppol), (HARTREE*elecstr%energy(ik,ib,isppol),ib=1,iparam%nbnds) 
   else
      write(uout,'(i5,7x,10f7.2/50(10x,10f7.2/))') &
&        ik, (HARTREE*elecstr%energy(ik,ib,isppol),ib=1,iparam%nbnds)
   endif  

     do ib = 1 , nbndsto
       read(u)  ! wfn
     enddo

 enddo !ik
enddo !isppol  


if(fform==502) then
  deallocate(vkb,vkbd,energy)      
else if(fform==602) then
  deallocate(vkbs,vkbds,energys)
endif

deallocate(iparam%shlim)
close(u)

call print_footline()      
if (iverbose > 2)call uverbose('e','rdkss')

end subroutine
!!***




!{\src2tex{textfont=tt}}
!!****p* testlda/rdhdr
!! NAME
!! Read abinit Header
!!
!! FUNCTION
!! To read abinit header and grab all the useful info. 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! testlda,rdkss
!!
!! CHILDREN
!! mesg, reorder_shells
!!
!! NOTE
!! Updated up to Abinit header version 5.6
!! This subroutine also contains the reorder_shells subroutine (internal)
!! in order to put the G vectors in the right order (sometimes this is not
!! the case from recent abinit version, and anyway this will be needed
!! once the netcdf format is in place
!!
!! TODO
!! This should disappear (or at least made deprecated) once the netcdf 
!! structure is definitely in place
!!
!! SOURCE
!=======================================================================
subroutine rdhdr(u,crystr,symstr,chemstr,foustr,elecstr)
!=======================================================================
! read ABINIT file header
use ggwdef
use dpdef
use verbose
implicit none
integer u
type(crystal) crystr
type(symmetry) symstr
type(chemistry) chemstr
type(transposed_space) foustr
type(electronic_structure) elecstr


character (len=6) :: codvsn
integer :: headform, fform
integer :: date, year, month, day

integer :: bantot, intxc, ixc, natom, ngfft(3), nkpt
integer :: nspden, nspinor, nsppol, nsym, npsp, ntype, occopt, pertcase
integer, allocatable :: nband(:), npwarr(:), symrel(:,:,:), symafm(:)
integer, allocatable :: typeat(:)
integer, allocatable :: istwfk(:), so_typat(:)
double precision :: acell(3), ecut, ecutsm, ecut_eff, rprimd(3,3), qptn(3)
double precision :: stmbias, tphysel, tsmear
double precision :: residm,etotal,fermie
double precision, allocatable :: xred(:,:) ! (3,natom)
double precision, allocatable :: kpt(:,:), occ(:), tnons(:,:)
double precision, allocatable :: znucltypat(:)
! pseudopotential
double precision, allocatable :: znuclpsp(:), zionpsp(:)
character, allocatable :: title(:,:)
integer, allocatable :: pspso(:),pspdat(:),pspcod(:),pspxc(:),lmax(:),lloc(:),mmax(:)

integer i,ik,ib,isymin,isymend,ia,ipa,it, nel
integer usepaw,usewvl

integer isppol

integer, allocatable :: lmn_size(:)
double precision :: ecutdg
double precision, allocatable :: wtk(:)

if(iverbose > 2)call uverbose('b','rdhdr')

read(u) codvsn,headform,fform

if(iverbose > 2) write(uout,'(" ABINIT unformatted file, format ",i3)') fform
write(uout,'(" generated by ABINIT version ",a6," header version ",i1,".",i1)')&
&     codvsn, headform/10, headform-(headform/10)*10

if(headform /= 23 .and. headform /= 34 .and. headform /= 40&
&    .and. headform /= 41 .and. headform /= 42 .and.  headform /= 44 &
&  .and. headform /= 53 .and. headform/=56 .and. headform/=57) then
  write(uout,*) '** ERROR: unrecognized abinit header version', headform
  stop 'unrecognized abinit header version'
endif

! 1st record
if(headform==23) then
  read(u) bantot, date, intxc, ixc, natom, ngfft(1:3), nkpt,&
&         nspden, nspinor, nsppol, nsym, ntype, occopt,&
&         acell(1:3), ecut_eff, rprimd(1:3,1:3)
else if(headform==34) then
  read(u) bantot, date, intxc, ixc, natom, ngfft(1:3), nkpt,&
&         nspden, nspinor, nsppol, nsym, npsp, ntype, occopt,&
&         ecut_eff, rprimd(1:3,1:3)
else if(headform==40) then
  read(u) bantot, date, intxc, ixc, natom, ngfft(1:3), nkpt,&
&         nspden, nspinor, nsppol, nsym, npsp, ntype, occopt,&
&         ecut, ecutsm, ecut_eff, rprimd(1:3,1:3),&
&         tphysel, tsmear
else if(headform==41) then
  read(u) bantot, date, intxc, ixc, natom, ngfft(1:3), nkpt,&
&         nspden, nspinor, nsppol, nsym, npsp, ntype, occopt, pertcase,&
&         ecut, ecutsm, ecut_eff, qptn(1:3), rprimd(1:3,1:3),&
&         tphysel, tsmear
else if(headform==42) then
  read(u) bantot, date, intxc, ixc, natom, ngfft(1:3), nkpt,&
&         nspden, nspinor, nsppol, nsym, npsp, ntype, occopt,pertcase,&
&         ecut, ecutsm, ecut_eff, qptn(1:3), rprimd(1:3,1:3),&
&         stmbias, tphysel, tsmear
else if(headform==44 .or. headform==53 .or. headform==56) then
  read(u) bantot, date, intxc, ixc, natom, ngfft(1:3), nkpt,&
&         nspden, nspinor, nsppol, nsym, npsp, ntype, occopt, pertcase,&
&         usepaw,ecut, ecutdg, ecutsm, ecut_eff, qptn(1:3), rprimd(1:3,1:3),&
&         stmbias,tphysel, tsmear
else if(headform==57) then
  read(u) bantot, date, intxc, ixc, natom, ngfft(1:3), nkpt,&
&         nspden, nspinor, nsppol, nsym, npsp, ntype, occopt, pertcase,&
&         usepaw,ecut, ecutdg, ecutsm, ecut_eff, qptn(1:3),rprimd(1:3,1:3),&
&         stmbias,tphysel, tsmear !, usewvl -  added usewvl only with abinit 5.7.3
endif

year = date / 10000
month = mod(date,10000) / 100
day = mod(date,100)
write(uout,'(" date (yyyy mm dd)",i5,i3,i3)') year, month, day
write(uout,*)

allocate(nband(nkpt*nsppol))
allocate(npwarr(nkpt))
allocate(symrel(3,3,nsym))
allocate(symafm(nsym))
allocate(istwfk(nkpt))
allocate(kpt(3,nkpt))
allocate(occ(bantot))
allocate(tnons(3,nsym))
allocate(znucltypat(ntype))
allocate(so_typat(ntype))
allocate(typeat(natom))
allocate(wtk(nkpt))
! 2nd record
if(headform==23 .or. headform==34) then
  read(u) nband(1:nkpt*nsppol), npwarr(1:nkpt), &
&         symrel(1:3,1:3,1:nsym), typeat(1:natom), &
&         istwfk(1:nkpt),& 
&         kpt(1:3,1:nkpt), occ(1:bantot), tnons(1:3,1:nsym),&
&         znucltypat(1:ntype)
else if(headform==40.or.headform==41.or.headform==42.or.headform==44) then
  read(u) istwfk(1:nkpt), nband(1:nkpt*nsppol), npwarr(1:nkpt), &
&         so_typat(1:ntype),&
&         symafm(1:nsym),symrel(1:3,1:3,1:nsym), typeat(1:natom),&
&         kpt(1:3,1:nkpt), occ(1:bantot), tnons(1:3,1:nsym),& 
&         znucltypat(1:ntype)
else if(headform==53 .or. headform==56 .or. headform==57) then
  read(u) istwfk(1:nkpt), nband(1:nkpt*nsppol), npwarr(1:nkpt), &
&         so_typat(1:ntype),&
&         symafm(1:nsym),symrel(1:3,1:3,1:nsym), typeat(1:natom),&
&         kpt(1:3,1:nkpt), occ(1:bantot), tnons(1:3,1:nsym),&
&         znucltypat(1:ntype), wtk(1:nkpt)
endif


do isymin=1,nsym,4
  isymend=isymin+3
  if(isymend>nsym) isymend=nsym
enddo

           
      
if(headform==23) npsp=ntype
if(headform==23 .or. headform==34) symafm = 0
if(npsp/=ntype) stop 'alchemical pseudo not programmed'
! (npsp lines, 1 for each pseudopotential ; npsp=ntype, except if alchemical pseudo-atoms)
allocate(title(132,npsp))
allocate(znuclpsp(npsp),zionpsp(npsp))
allocate(pspso(npsp),pspdat(npsp),pspcod(npsp),pspxc(npsp))
allocate(lmn_size(npsp))
allocate(lmax(npsp),lloc(npsp),mmax(npsp))
lmax(:) = -1
lloc(:) = -1
mmax(:) = -1

! pseudopotential record
do i=1,npsp
  if(headform==23) then
    read(u) title(1:132,i),znuclpsp(i),zionpsp(i),pspso(i),pspdat(i),pspcod(i),pspxc(i),&
&           lmax(i),lloc(i),mmax(i)
  else if(headform==34 .or. headform==40.or.headform==41.or.headform==42) then
    read(u) title(1:132,i),znuclpsp(i),zionpsp(i),pspso(i),pspdat(i),pspcod(i),pspxc(i)
  else if(headform==44 .or. headform==53 .or. headform==56 .or. headform==57)  then
    read(u) title(1:132,i),znuclpsp(i),zionpsp(i),pspso(i),pspdat(i),pspcod(i),&
&    pspxc(i),lmn_size(i)
  endif
enddo

do i=1,npsp
  if (headform<50) then
   if(pspso(i)/=1) then
      write(uout,*) '**ERROR: pspso value not programmed'
      write(uout,*) 'pspso = ', pspso(i)
      stop 'pspso /= 1'
   endif
  else
   if(so_typat(i)/=1) then
     write(uout,*) '**ERROR: so_typat value not programmed'
     write(uout,*) 'so_typat = ', so_typat(i)
     stop 'so_typat /= 1'
   endif
  endif
enddo

allocate(xred(3,natom))
! last record
if(headform==22) then
read(u) residm,xred(1:3,1:natom),etotal
else
read(u) residm,xred(1:3,1:natom),etotal,fermie
endif

!attention: 
!starting from headform=44 at the end of header 
!added rhoij tab for paw method
!here we don't read the last record for paw at all

!end of the reading of the header

crystr%rprimd(:,:) = rprimd(:,:)

if(nsym > 48) stop 'internal error, increase nsymx (48) in ggwdef.F'
allocate(symstr%symafm(nsym))
symstr%nsym = nsym
do i = 1, symstr%nsym
  symstr%sym(i)%r(:,:) = symrel(:,:,i)
  symstr%sym(i)%t(:)   = tnons(:,i)
  symstr%symafm(i) = symafm(i)
enddo

 
call setup_symmetry_structure(symstr)

if(ntype /= npsp) stop 'cannot treat ntype /= npsp'
chemstr%ntype = ntype
chemstr%natom = natom
chemstr%npsa  = npsp
allocate(chemstr%znucl(chemstr%ntype))
allocate(chemstr%typeat(chemstr%natom))
allocate(chemstr%xred(3,chemstr%natom))
allocate(chemstr%psa(chemstr%npsa))
chemstr%znucl(1:chemstr%ntype) = znucltypat(1:ntype)
chemstr%typeat(1:chemstr%natom) = typeat(1:natom)
chemstr%xred(:,1:chemstr%natom) = xred(:,1:natom)
do i = 1, chemstr%npsa
  chemstr%psa(i)%znucl = znuclpsp(i)
  chemstr%psa(i)%zion  = zionpsp(i)
  chemstr%psa(i)%lmax  = lmax(i)
  chemstr%psa(i)%lloc  = lloc(i)
  chemstr%psa(i)%mmax   = mmax(i)
  chemstr%psa(i)%pspso  = pspso(i)
  chemstr%psa(i)%pspdat = pspdat(i)
  chemstr%psa(i)%pspcod = pspcod(i)
  chemstr%psa(i)%pspxc  = pspxc(i)
  chemstr%psa(i)%title(:)  = title(:,i)
enddo
chemstr%nel = 0
do ia = 1, chemstr%natom
  it = chemstr%typeat(ia)
  nel = chemstr%znucl(it)  ! if it's not a pseudoatom, add all electron
  do ipa = 1, chemstr%npsa
    if(chemstr%psa(ipa)%znucl == chemstr%znucl(it)) nel = chemstr%psa(ipa)%zion
  enddo
  chemstr%nel = chemstr%nel + nel
enddo


foustr%nk = nkpt
allocate(foustr%k(3,foustr%nk))
foustr%k(:,1:foustr%nk) = kpt(:,1:nkpt)

elecstr%nsppol = nsppol 

elecstr%nk = nkpt
elecstr%nb = maxval(nband)
allocate(elecstr%occ(elecstr%nk,elecstr%nb,elecstr%nsppol))


!LC ATT reordering to have only 1 index
!LC cfr rdkss abinit
elecstr%occ(:,:,:) = zero
do isppol = 1, elecstr%nsppol
 do ik = 1, nkpt
  do ib = 1, nband(ik+nkpt*(isppol-1))
    i = (ik-1)*nband(ik) + (isppol-1)*nkpt*nband(ik) + ib
    elecstr%occ(ik,ib,isppol) = occ(i)
  enddo
 enddo
enddo !isppol

elecstr%fermie = fermie
elecstr%etotal = etotal
elecstr%residm = residm
elecstr%ecut_eff = ecut_eff
elecstr%xc = ixc

deallocate(nband,npwarr,symrel,symafm,istwfk,kpt,occ,tnons)
deallocate(znucltypat,so_typat,typeat,xred)
deallocate(title,znuclpsp,zionpsp,pspso,pspdat,pspcod,pspxc)
deallocate(lmax,lloc,mmax)
deallocate(lmn_size)
deallocate(wtk)

if(iverbose > 2)call uverbose('e','rdhdr')
end subroutine
!!***


!{\src2tex{textfont=tt}}
!!****p* rdkss/prhdr
!! NAME
!! Print abinit Header
!!
!! FUNCTION
!! Print in the std out the abinit header
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! rdkss
!!
!! SOURCE
!=======================================================================
subroutine prhdr(crystr,chemstr,foustr,elecstr)
!=======================================================================
! print fundamental data
use ggwdef
use dpdef
use verbose
implicit none
type(crystal) crystr
type(chemistry) chemstr
type(transposed_space) foustr
type(electronic_structure) elecstr
character (len=*), parameter :: orb = 'spdfghij'
integer i, ia, ik


if(iverbose > 2)call uverbose('b','pdhdr')

if(iverbose > 1) then
write(uout,*) 'crystallographic data:'
write(uout,'("  r1 ",3f7.2/"  r2 ",3f7.2/"  r3 ",3f7.2)') crystr%rprimd
write(uout,*)
endif
      

if(iverbose > 0) then
write(uout,*) 'atomic composition:'
write(uout,'("  number of types of atom ",i3)') chemstr%ntype

write(uout,'("  atomic numbers          ",10f7.2)') chemstr%znucl(:)
write(uout,'("  number of atoms         ",i3)') chemstr%natom
write(uout,'("  atoms type              ",10i3)') chemstr%typeat(:)
write(uout,'("  number of electrons   ",i5)') chemstr%nel
write(uout,*)
write(uout,*) ' atom type    z  psz   position [reduced coordinates]'
do ia = 1, chemstr%natom
  write(uout,'(i6,i5,i5,i5,2x,3f10.6)') ia,chemstr%typeat(ia), &
&   int(chemstr%znucl(chemstr%typeat(ia))),&
&   int(chemstr%psa(chemstr%typeat(ia))%zion), chemstr%xred(:,ia)
enddo
write(uout,*)
endif

if(iverbose > 2) then
write(uout,*) 'pseudopotentials:'
do i=1,chemstr%npsa
  write(uout,*) ' atom type ', i
  write(uout,'(3x,77a)') chemstr%psa(i)%title(1:77)
  write(uout,'("   atomic number        ",6x,f6.2)') chemstr%psa(i)%znucl
  write(uout,'("   pseudo atomic number ",6x,f6.2)') chemstr%psa(i)%zion
  if(chemstr%psa(i)%lloc > chemstr%psa(i)%lmax) write(uout,'("   WARNING: lmax=",i2," and lloc=",i2, &
     &            " not coherent")') chemstr%psa(i)%lmax, chemstr%psa(i)%lloc
  
    select case(chemstr%psa(i)%lmax)
      case(0)
          if(chemstr%psa(i)%lloc==0) then
            write(uout,'("   local pseudopotential")')
          else
            write(uout,'("   WARNING: lmax=",i2," and lloc=",i2, &
     &            " not coherent")') chemstr%psa(i)%lmax, chemstr%psa(i)%lloc
          endif
      case(1)
        select case(chemstr%psa(i)%lloc)
          case(0)
            write(uout,'("   p nonlocality s reference")')
          case(1)
            write(uout,'("   s nonlocality p reference")')
          case default
            write(uout,'("   WARNING: lmax=",i2," and lloc=",i2, &
     &            " not coherent")') chemstr%psa(i)%lmax, chemstr%psa(i)%lloc
        end select
      case(2)
        select case(chemstr%psa(i)%lloc)
          case(0)
            write(uout,'("   pd nonlocality s reference")')
          case(1)
            write(uout,'("   sd nonlocality p reference")')
          case(2)
            write(uout,'("   sp nonlocality d reference")')
          case default
            write(uout,'("   WARNING: lmax=",i2," and lloc=",i2, &
     &            " not coherent")') chemstr%psa(i)%lmax, chemstr%psa(i)%lloc
        end select
      case(3)
        select case(chemstr%psa(i)%lloc)
          case(0)
            write(uout,'("   pdf nonlocality s reference")')
          case(1)
            write(uout,'("   sdf nonlocality p reference")')
          case(2)
            write(uout,'("   spf nonlocality d reference")')
          case(3)
            write(uout,'("   spd nonlocality f reference")')
          case default
            write(uout,'("   WARNING: lmax=",i2," and lloc=",i2, &
     &            " not coherent")') chemstr%psa(i)%lmax, chemstr%psa(i)%lloc
        end select
    end select

  write(uout,'("   mesh dimension       ",4x,i5)') chemstr%psa(i)%mmax
  write(uout,'("   pseudopotential code ",6x,i3)') chemstr%psa(i)%pspcod
  write(uout,'("   xc functional        ",6x,i3)') chemstr%psa(i)%pspxc
enddo

write(uout,*)
 
endif

if(iverbose > 0) then
 write(uout,*) 'system informations:'
 write(uout,'("  number of k-points ",i5)') foustr%nk
 write(uout,*) ' k-points [reduced coordinates]:'
 write(uout,'(3(1x,f10.6))') ((foustr%k(i,ik),i=1,3),ik=1,foustr%nk)
 write(uout,*)
endif

if(iverbose > 1) then
 write(uout,*) 'convergence data:'
 write(uout,'("  exchange-correlation functional ",i2)') elecstr%xc
 write(uout,'("  effective cut-off ",f6.2)') elecstr%ecut_eff
 write(uout,'("  total energy ",f10.4)') elecstr%etotal
 write(uout,'("  fermi energy ",f10.4)') elecstr%fermie
 write(uout,'("  max residual ",e12.4)') elecstr%residm
 write(uout,*)
endif
      
if(iverbose > 2)call uverbose('e','pdhdr')
end subroutine
!!***


!{\src2tex{textfont=tt}}
!!****f* testlda/fileform
!! NAME
!! fileform
!!
!! FUNCTION
!! Determine the fileformat of the _KSS abinit file.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! testlda, rdkss
!!
!! CHILDREN
!! mesg
!!
!! SOURCE

!=======================================================================
integer function fileform(file,l)
!=======================================================================
! check file type
use verbose
use dpdef
implicit none
integer l
character (len=l) :: file
integer, parameter :: u=10
integer status, status1, status2, status3
character (len=6) :: codvsn
integer headform, fform
character (len=80) :: title
integer i1,i2,i3,i4,i5,i6,i7,i8,i9,i10,i11,i12,i13,i14,i15,i16
integer idum


if(iverbose > 2) write(uout,*) 'trying kss ABINIT form'
open(u,status='unknown',form='unformatted',file=file)
read(u,iostat=status) codvsn, headform, fform
close(u)
fileform = fform
if(fform==502 .and. status==0) then
  if(iverbose > 2) write(uout,*) 'kss ABINIT double precision form'
  return
else if(fform==602 .and. status==0) then
  if(iverbose > 2) write(uout,*) 'kss ABINIT single precision form'
  return
endif
  
if(iverbose > 2) write(uout,*) 'non kss form, status = ', status
if(iverbose > 2) write(uout,*) 'codvsn, headform, fform = ', codvsn, headform, fform

if(iverbose > 2) write(uout,*) 'trying sta and qplda forms'
open(u,status='unknown',form='unformatted',file=file)
read(u,iostat=status1) title
if(iverbose > 2) write(uout,*) title
read(u,iostat=status2) title
if(iverbose > 2) write(uout,*) title
read(u,iostat=status3) i1,i2,i3,i4,i5,i6,i7,i8,i9,i10,i11,i12,i13,i14,i15,i16
close(u)
! flags used so far:
! i1 = 0 if QPLDA file is formatted, 1 if unformatted
! i2 = 0 if RWG format, 1 if BF format, 2 STA format
! i3 = 1 if non-symmorphic operations (+vectors) included, otherwise 0
if(iverbose > 2) write(uout,*) i1,i2,i3
if(iverbose > 2) write(uout,*) status1, status2, status3
if(status1==0.and.status2==0.and.status3==0) then
  if(i2==2) then
    fileform = 2
    if(iverbose > 2) write(uout,*) 'sta double precision form'
    return
  else if(i2==0.or.i2==1) then
    fileform = 1
    if(iverbose > 2) write(uout,*) 'qplda single precision form'
    return
  endif
endif

call mesg('error','unrecognized file format - check the endianness',idum,idum)
write(uout,*)
fileform = -1
return
end function
!!***



!{\src2tex{textfont=tt}}
!!****p* read_structure/rdlda
!! NAME
!! Read Kohn-Sham Structure from the old qplda format
!!
!! FUNCTION
!! Read qplda cp file (wfn, eigenvalues, occupations, etc.)  
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! read_oldformat_file
!!
!! SOURCE
!     ==================================================================
      subroutine rdlda(qpldafile,kl,iparam,nb,nk,ng,tit,i1,i2,i3, & 
                       latt,symstr,trspace,bzstruct,elecstr)
!     ==================================================================
!     read QPLDA type file
      use dpdef
      use ggwdef
      use verbose
      implicit none
      type(dpparam) iparam
      type(lattstruct) latt
      type(symmetry) symstr
      type(bzstructure) bzstruct
      type(electronic_structure)elecstr
      type(transposed_space)trspace
      integer kl
      character (len=kl) :: qpldafile
      integer, parameter :: iunit = 10
      integer nb, nk, ng
      character (len=80) :: tit(2)
      integer i1,i2,i3,i4,i5,i6,i7,i8,i9,i10,i11,i12,i13,i14,i15,i16

!     energuies in single precision
      real, allocatable :: en(:,:)
      
!      integer isppol 

      real b1(3), b2(3), b3(3)
      real ucvol, bzvol

!     nopx: maximum number of symmetry operations, i.e. dimension of op
!     nbx: maximum number of bands, i.e. dimension of en, occ, wf
!     nkx: maximum number of k-points in IBZ, i.e. dim. of kvec, en, occ, wf
!     ngx: maximum number of G-vectors in calculation, i.e. dim. of gvec, wf
!     nwx: maximum number of plane-waves to be used for wave-functions
!     nop: number of symmetry operations contained in file
!     nb: number of bands contained in file
!     nk: number of k-points in IBZ contained in file
!     ng: number of G-vectors in calculation contained in file
!     tit: 2*80-character description of LDA input file
!     i*: flags
!     a1, a2, a3: real-space lattice vectors in au
!     b1, b2, b3: reciprocal-space lattice vectors in au**-1
!     ucvol: unit cell volume in au**3
!     bzvol: reciprocal cell volume
!     op: symmetry operation matrices
!     gvec: RL vectors G
!     kibz: k-points in IBZ
!     en: LDA energies (Hartrees)
!     occ: occupations of bands (2.0=full)
!     wf: LDA wave functions (G) in plane wave order [Norm:|vector|=1]


      integer nbr, nkr, ngr
      integer nbw

!     n*r: number of * read from file
!     if nb < nbx then nbr = nb else nbr = nbx
!     if ng < ngx then ngr = ng else ngr = ngx
!     if ng < nwx then nwr = ng else nwr = nwx
!     if nop < nopx then nopr = nop else nopr = nopx
!     ** if nk < nkx then nkr = nk else ERROR **:
!      due to the structure of LDA file with regard of energies and
!      occupations, (all energies/occupations in one record) it is
!      impossible to read nkx less than nk elements; implicit do in
!      those sections is not synctactically correct and explicit do
!      doesn't go as look for EOR at each loop. 
!     check the return value of nop, nb, ng to see how many elements
!     were loaded


      integer ib, ik, ig, i, j, k
      integer isymin, isymend, isym
      integer idum
      real dum
      complex, allocatable :: wfg(:)


      logical DEBUG_OLD
      parameter(DEBUG_OLD=.TRUE.)
      logical VERBOSE_OLD
      parameter(VERBOSE_OLD=.TRUE.)
      logical FORM
      
#ifdef MPI
include 'mpif.h'
integer ierr,rang,nprocs
call mpi_comm_size(mpi_comm_world,nprocs,ierr)
call mpi_comm_rank(mpi_comm_world,rang,ierr)
if(iparam%mpilocal .and. rang/=0) return
#endif

!     i1=0 -> file formatted -> FORM=.TRUE.
!     i1!=0 -> file unformatted -> FORM=.FALSE.
      if (i1.eq.0) then
        FORM = .TRUE.
      else
        FORM = .FALSE.
      endif


      allocate(en(iparam%nkibz,iparam%nbndmax))


      if (FORM) then
        write (uout,*) 'opening LDA input file formatted unit ',iunit
        open(iunit,status='unknown',form='FORMATTED',file=qpldafile)
      else
        write (uout,*) 'opening LDA input file unformatted unit ',iunit
        open(iunit,status='unknown',form='UNFORMATTED',file=qpldafile)
!  ****** TEMP Add recl for FPS unformatted file
!       open (iunit, status='unknown',form='UNFORMATTED',RECL=500000)
!  ******
      endif


      write (uout,*) 'reading title of file:'
      if (FORM) then
        read (iunit,10) tit(1), tit(2)
10      format (A80/A80)
      else
        read (iunit) tit(1)
        read (iunit) tit(2)
      endif

      write (6,2974) tit(1), tit(2)
2974  format(A80/A80)
      write (uout,*)


!     Flags used so far:
!     i1 = 0 if QPLDA file is formatted, 1 if unformatted
!     i2 = 0 if RWG format, 1 if BF format
!     i3 = 1 if non-symmorphic operations (+vectors) included, otherwise 0

      write (uout,*) 'reading flags'

      if (FORM) then
        read (iunit,20) i1,i2,i3,i4,i5,i6,i7,i8,i9,i10,&
     &               i11,i12,i13,i14,i15,i16
20      format (16I5)
      else
        read (iunit) i1,i2,i3,i4,i5,i6,i7,i8,i9,i10,&
     &               i11,i12,i13,i14,i15,i16
      endif

!     check that flag i1 is consistent with value of FORM assumed
      if ((FORM).and.(i1.ne.0)) then
        write (uout,*) &
     &  'ERROR rdlda 1: input file unformatted read as formatted.'
        stop
      endif
      if ((.not.FORM).and.(i1.ne.1)) then
        write (uout,*) &
     &  'ERROR rdlda 2: input file formatted read as unformatted.'
        stop
      endif

      if (DEBUG_OLD) write (6,2755) i1,i2,i3,i4,i5,i6,i7,i8,i9,i10, &
     &               i11,i12,i13,i14,i15,i16
2755  format (16I4)
      write (uout,*)


      write (uout,*) 'reading real-space lattice vectors'

      if (FORM) then
        read (iunit,30) latt%a1, latt%a2, latt%a3
30      format (3E26.18)
      else
        read (iunit) latt%a1, latt%a2, latt%a3
      endif

      if (VERBOSE_OLD) then
!       calculate Brillouin zone and Unit cell volumes
        ucvol = latt%a1(1)* (latt%a2(2)*latt%a3(3)-latt%a2(3)*latt%a3(2)) &
     &        + latt%a1(2)* (latt%a2(3)*latt%a3(1)-latt%a2(1)*latt%a3(3)) &
     &        + latt%a1(3)* (latt%a2(1)*latt%a3(2)-latt%a2(2)*latt%a3(1))
        bzvol = 8*PI*PI*PI / ucvol

!       calculate reciprocal-space lattice vectors b1-b3
        b1(1) = 2.0 * PI * (latt%a2(2)*latt%a3(3)-latt%a2(3)*latt%a3(2)) / ucvol
        b1(2) = 2.0 * PI * (latt%a2(3)*latt%a3(1)-latt%a2(1)*latt%a3(3)) / ucvol
        b1(3) = 2.0 * PI * (latt%a2(1)*latt%a3(2)-latt%a2(2)*latt%a3(1)) / ucvol
        b2(1) = 2.0 * PI * (latt%a3(2)*latt%a1(3)-latt%a3(3)*latt%a1(2)) / ucvol
        b2(2) = 2.0 * PI * (latt%a3(3)*latt%a1(1)-latt%a3(1)*latt%a1(3)) / ucvol
        b2(3) = 2.0 * PI * (latt%a3(1)*latt%a1(2)-latt%a3(2)*latt%a1(1)) / ucvol
        b3(1) = 2.0 * PI * (latt%a1(2)*latt%a2(3)-latt%a1(3)*latt%a2(2)) / ucvol
        b3(2) = 2.0 * PI * (latt%a1(3)*latt%a2(1)-latt%a1(1)*latt%a2(3)) / ucvol
        b3(3) = 2.0 * PI * (latt%a1(1)*latt%a2(2)-latt%a1(2)*latt%a2(1)) / ucvol

        write (6,8723) latt%a1, latt%a2, latt%a3
8723    format (' real-space lattice vectors in cartesians [au]:' &
     &          /('     ',3F12.6))
        write (6,1022) b1, b2, b3
1022    format (/' calculated reciprocal lattice vectors', &
     &          ' in cartesians [au]:'/ &
     &          ('     ',3F12.6) )
        write (6,1021) bzvol, ucvol
1021    format (/' brillouin zone volume = ', F12.6, ' [au]'/ &
     &           ' unit cell volume      = ', F12.6, ' [au]')
      endif
      write (uout,*) 


!     read symmetry operations (real-space-lattice units) and CONVERT
!     to reciprocal-lattice units   [nop+1 lines]
!     [Note: if R is the matrix in lattice units, then the corresponding
!     matrix in RL units is transpose(R**-1).  But since the set of
!     operations contains inverse pairs, we only need to tranpose
!     the matrices.  Thus op(J,I) rather than op(I,J).]

      write (uout,*) 'reading symmetry operations'

      if (FORM) then
        read (iunit,40) iparam%nopr
40      format (I12)
      else
        read (iunit) iparam%nopr
      endif

      write (uout,*) 'number of translationless symmetry operations', iparam%nopr

      if (iparam%nopr.le.iparam%nop) then
        iparam%nop = iparam%nopr
      else
        write (uout,*) 'WARNING rdlda 3: reading only a number of ', &
     &  'symmetry operations ', iparam%nop
!      iparam%nopr = iparam%nopr
      endif


      if (iparam%nopr.le.iparam%nop) then
        if (FORM) then
          read (iunit,50) (((symstr%op(j,i,k),i=1,3),j=1,3),k=1,iparam%nopr)
50        format (3E26.18)
        else
          read (iunit) (((symstr%op(j,i,k),i=1,3),j=1,3),k=1,iparam%nopr)
        endif
      else
        if (FORM) then
          read (iunit,50) (((symstr%op(j,i,k),i=1,3),j=1,3),k=1,iparam%nop), &
     &                      (((dum,i=1,3),j=1,3),k=iparam%nop+1,iparam%nopr)
        else
          read (iunit) (((symstr%op(j,i,k),i=1,3),j=1,3),k=1,iparam%nop), &
     &                 (((dum,i=1,3),j=1,3),k=iparam%nop+1,iparam%nopr)
        endif
      endif


      if (VERBOSE_OLD) then
        write(uout,*) 'symmetry operations [reciprocal lattice units]:'
        do isymin=1,iparam%nop,3
          isymend=isymin+2
          if(isymend.gt.iparam%nop) isymend=iparam%nop
          do i=1,3
            write(uout,2995) ((symstr%op(i,j,isym),j=1,3),isym=isymin,isymend)
2995        format(2(3f8.4,4x),3f8.4)
          enddo
          write(uout,*)
        enddo
        write(uout,*)
      endif


      if (I3.eq.1) then
!       read (iunit,1020) ((voffset(i,j),i=1,3),j=1,iparam%nopr)
        write (uout,*) 'ERROR rdlda 4: program is not able to handle &
     & non-symorphic translation vectors; run CNVNSY first or check I3'
        stop
      endif

!antiferromagnetic symmetries set by default = 1 (no spin polarized calculations are possible)
      allocate(symstr%symafm(iparam%nop))
      symstr%symafm(:) =+1


      write (uout,*) 'reading reciprocal lattice vectors'
!     ie in the basis of the reciprocal lattice basis vectors

      if (FORM) then
        read (iunit,60) ng
60      format (I12)
      else
        read (iunit) ng
      endif
!      allocate(wfg(ng))
      write (uout,*) 'number of G vectors found, i.e. of plane waves ', ng

      if (ng.le.iparam%npwvec) then
        ngr = ng
      else
        ngr = iparam%npwvec
      endif
      write (uout,*) 'reading a number of G vectors ', ngr

      if (ng.le.iparam%npwvec) then
        if (FORM) then
          read (iunit,70) ((trspace%gvec(i,ig),i=1,3),ig=1,ng)
70        format (3I5)
        else
          read (iunit) ((trspace%gvec(i,ig),i=1,3),ig=1,ng)
        endif
      else
        if (FORM) then
          read (iunit,70) ((trspace%gvec(i,ig),i=1,3),ig=1,iparam%npwvec), &
     &                      ((idum,i=1,3),ig=iparam%npwvec+1,ng)
        else
          read (iunit) ((trspace%gvec(i,ig),i=1,3),ig=1,iparam%npwvec), &
     &                 ((idum,i=1,3),ig=iparam%npwvec+1,ng)
        endif
      endif

      if (VERBOSE_OLD) write (uout,3995) (ig,(trspace%gvec(i,ig),i=1,3),ig=1,ngr)
3995  format (' G vectors [reciprocal lattice units]:'/ &
     &           (5(I5,':',3I3)))
      write (uout,*)


      write (uout,*) 'reading k-points'

      if (FORM) then
        read (iunit,80) nk
80      format (I12)
      else
        read (iunit) nk
      endif

      write (uout,*) 'number of k-points ', nk

      if (nk.le.iparam%nkibz) then
        nkr = nk
      else
        nkr = iparam%nkibz
        write (uout,*)
        write (uout,*) '** ERROR rdlda 5: ', &
     &  'k dimension of vectors initialized at ',iparam%nkibz
        write (uout,*) '                  increase the dimension at ',nk
        stop
      endif

      if (nk.le.iparam%nkibz) then
        if (FORM) then
          read (iunit,90) ((bzstruct%kibz(i,ik),i=1,3),ik=1,nk)
90        format (3E26.18)
        else
          read (iunit) ((bzstruct%kibz(i,ik),i=1,3),ik=1,nk)
        endif
      else
        if (FORM) then
          read (iunit,90) ((bzstruct%kibz(i,ik),i=1,3),ik=1,iparam%nkibz), &
     &                      ((dum,i=1,3),ik=iparam%nkibz+1,nk)
        else
          read (iunit) ((bzstruct%kibz(i,ik),i=1,3),ik=1,iparam%nkibz), &
     &                 ((dum,i=1,3),ik=iparam%nkibz+1,nk)
        endif
      endif

      if (VERBOSE_OLD) write (6,2862) (ik,(bzstruct%kibz(i,ik),i=1,3),ik=1,nkr)
2862  format (' k-points [reciprocal lattice units]:'/ &
     &          (I5,3F12.6))
      write (uout,*)


      write (uout,*) 'reading energies'

      if (FORM) then
        read (iunit,1110) nb
1110    format (I12)
      else
        read (iunit) nb
      endif

      write (uout,*) 'number of bands found ', nb

      if (nb.le.iparam%nbnds) then
        nbr = nb
      else
        nbr = iparam%nbnds
      endif
      write (uout,*) 'reading a number of bands ', nbr


       elecstr%nsppol=iparam%nsppol
       if(elecstr%nsppol == 2) then
        write(uout,*) 'cannot use QPLDA with spin polarized systems'       
        stop 
       endif

!      do isppol = 1, elecstr%nsppol 
       
       if (nb.le.iparam%nbnds) then
         if (nk.le.iparam%nkibz) then
           if (FORM) then
             read (iunit,91) ((en(ik,ib),ik=1,nk),ib=1,nb)
91           format (3E26.18)
           else
             read (iunit) ((en(ik,ib),ik=1,nk),ib=1,nb)
           endif
         else
           if (FORM) then
             do ib = 1, nb
               read (iunit,91) (en(ik,ib),ik=1,iparam%nkibz),(dum,ik=iparam%nkibz+1,nk)
             enddo
           else
             do ib = 1, nb
               read (iunit) (en(ik,ib),ik=1,iparam%nkibz),(dum,ik=iparam%nkibz+1,nk)
             enddo
           endif
         endif
       else
         if (nk.le.iparam%nkibz) then
           if (FORM) then
             read (iunit,91) ((en(ik,ib),ik=1,nk),ib=1,iparam%nbnds), &
     &                         ((dum,ik=1,nk),ib=iparam%nbnds+1,nb)
           else
             read (iunit) ((en(ik,ib),ik=1,nk),ib=1,iparam%nbnds), &
     &                    ((dum,ik=1,nk),ib=iparam%nbnds+1,nb)
           endif
         else
           if (FORM) then
             do ib = 1, iparam%nbnds
               read (iunit,91) (en(ik,ib),ik=1,iparam%nkibz), &
     &                           (dum,ik=iparam%nkibz+1,nk)
             enddo
             do ib = iparam%nbnds+1, nb
               read (iunit,91) (dum,ik=1,nk)
             enddo
           else
             do ib = 1, iparam%nbnds
               read (iunit) (en(ik,ib),ik=1,iparam%nkibz), (dum,ik=iparam%nkibz+1,nk)
             enddo
             do ib = iparam%nbnds+1, nb
               read (iunit) (dum,ik=1,nk)
             enddo
           endif
         endif
       endif
       elecstr%energy(:,:,1) = en(:,:)
!      enddo !isppol
      deallocate(en)

!      do isppol = 1, elecstr%nsppol 

      write (uout,*) 'reading numbers of occupation'
      if (nb.le.iparam%nbnds) then
        if (nk.le.iparam%nkibz) then
          if (FORM) then
            read (iunit,92) ((elecstr%oc(ik,ib,1),ik=1,nk),ib=1,nb)
92          format (3E26.18)
          else
            read (iunit) ((elecstr%oc(ik,ib,1),ik=1,nk),ib=1,nb)
          endif
        else
          if (FORM) then
            do ib = 1, nb
              read (iunit,92) (elecstr%oc(ik,ib,1),ik=1,iparam%nkibz),(dum,ik=iparam%nkibz+1,nk)
            enddo
          else
            do ib = 1, nb
              read (iunit) (elecstr%oc(ik,ib,1),ik=1,iparam%nkibz),(dum,ik=iparam%nkibz+1,nk)
            enddo
          endif
        endif
      else
        if (nk.le.iparam%nkibz) then
          if (FORM) then
            read (iunit,92) ((elecstr%oc(ik,ib,1),ik=1,nk),ib=1,iparam%nbnds), &
     &                        ((dum,ik=1,nk),ib=iparam%nbnds+1,nb)
          else
            read (iunit) ((elecstr%oc(ik,ib,1),ik=1,nk),ib=1,iparam%nbnds), &
     &                   ((dum,ik=1,nk),ib=iparam%nbnds+1,nb)
          endif
        else
          if (FORM) then
            do ib = 1, iparam%nbnds
              read (iunit,92) (elecstr%oc(ik,ib,1),ik=1,iparam%nkibz), &
     &                          (dum,ik=iparam%nkibz+1,nk)
            enddo
            do ib = iparam%nbnds+1, nb
              read (iunit,92) (dum,ik=1,nk)
            enddo
          else
            do ib = 1, iparam%nbnds
              read (iunit) (elecstr%oc(ik,ib,1),ik=1,iparam%nkibz), (dum,ik=iparam%nkibz+1,nk)
            enddo
            do ib = iparam%nbnds+1, nb
              read (iunit) (dum,ik=1,nk)
            enddo
          endif
        endif
      endif
       
!      enddo !isppol
       
      write (uout,*)
      
      if (VERBOSE_OLD) then
!      do isppol=1, elecstr%nsppol
       nbw = 15
       if(nbr.lt.nbw) nbw = nbr
       do ik = 1, nkr
!         write (uout,3324) ik, (en(ik,ib),ib=1,nbw)
          write (uout,3325) ik, (HARTREE*elecstr%energy(ik,ib,1),ib=1,nbw)
          write (uout,3329) ik, (elecstr%oc(ik,ib,1),ib=1,nbw)
       enddo
!      enddo !isppol 
       
      write (uout,*)      
      endif
      
3324    format (/' eigenvalues for k-point',I5,' [Hartrees]:'/ &
     &            (5F12.6))
3325    format (/' eigenvalues for k-point',I5,' [eV]:'/ &
     &            (5F12.6))
3329    format (/' occupations for k-point',I5,':'/ &
     &            (5F12.6))


      write (uout,*) 'closing file'
      close (iunit)
      write (uout,*) 

      end
!!***
      
!{\src2tex{textfont=tt}}
!!****p* read_structure/testvkb
!! NAME
!! Test non-local part of the pseudopotential
!!
!! FUNCTION
!! This is a first reading to the old an deprecated vkb files in order to determine
!! some preliminary values.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! read_oldformat_file
!!
!! SOURCE
!     ==================================================================
      subroutine testvkb(vkbfile,vl,npwwfn,nkibz,ntype,natom,mpsang)
!     ==================================================================
!     test vkb file and return parameter of it
      use dpdef
      implicit none
      integer vl
      character (len=vl) :: vkbfile
      integer npwwfn, nkibz
      integer ntype, natom, mpsang
      integer nspnl, nkibzr, ngvkb
      integer is, ia, il, ik
      integer, allocatable :: na(:), nl(:)
      
      write(uout,*) 'testing file *.vkb'
      open(unit=11,form='unformatted',status='unknown',file=vkbfile)
      read(11) nkibzr,nspnl
      if(nkibzr/=nkibz) then
        write(uout,*) '** ERROR: number of k in file *.vkb not right!'
        stop '*.vkb nkpt'
      endif
      ntype = nspnl
      write(uout,*) 'number of atomic species ', nspnl
      write(uout,*) 'number of k-points ', nkibzr
      allocate(na(nspnl),nl(nspnl))
      natom = 0
      mpsang = 0
      do is = 1, nspnl
        read(11) na(is), nl(is)
        write(uout,*) ' specie ',is
        write(uout,*) '  number of atoms ',na(is)
        write(uout,*) '  number of angular momentum components ',nl(is)
        natom = natom + na(is)
        if(nl(is) > mpsang) mpsang = nl(is)
        do ia = 1, na(is)
          read(11) ! skip atomic position
        enddo
        do il = 1, nl(is)
          read(11) ! skip vkbsign
        enddo
      enddo
      write(uout,*) 'total number of atoms ', natom
      write(uout,*) 'maximum number of angular momentum components ', mpsang
      do ik = 1, nkibz
        read(11) ngvkb
        write(uout,*) 'k-point ', ik, 'number of G ', ngvkb
        if(ngvkb<npwwfn) then
          write(uout,*) 'number of g in out.vnl not enough', ngvkb
          stop 'ngvkb in *.vkb'
        endif
        do is = 1, nspnl
          do il = 1, nl(is)
            read(11) ! skip vkb
            read(11) ! skip vkbd
          enddo
        enddo
      enddo
      close(11)
      write(uout,*)

      deallocate(na,nl)
      return
      end subroutine
!!***
      
      
!{\src2tex{textfont=tt}}
!!****p* read_structure/rdvkb
!! NAME
!! Read non-local part of the pseudopotential
!!
!! FUNCTION
!! reading the old an deprecated vkb files 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! read_oldformat_file
!!
!! SOURCE
!     ==================================================================
!     ==================================================================
      subroutine rdvkb(vkbfile,vl,npwwfn,nkibz,ntype,natom,mpsang, &
     &                 typeat,xcart,vkbsign,vkb,vkbd)
!     ==================================================================
!     read vkb file
      use dpdef
      implicit none
      integer vl
      character (len=vl) :: vkbfile
      integer npwwfn, nkibz
      integer ntype, natom, mpsang
      integer typeat(natom)
      double precision :: xcart(3,natom)
      real vkbsign(mpsang,ntype)
      real vkb(npwwfn,ntype,mpsang,nkibz)
      real vkbd(npwwfn,ntype,mpsang,nkibz)
      integer nspnl, nkibzr, ngvkb
      integer, allocatable :: na(:), nl(:)
      real tau0(3)
      integer is, ia, iar, il, i, ik, ig
      real dum
      logical, parameter :: DEBUG_OLD = .false.
      
      write(uout,*) 'reading Kleinman-Bylander form factors ', &
     &         'and atomic structure factors (file *.vkb)'
      open(unit=11,form='unformatted',status='unknown',file=vkbfile)
      read(11) nkibzr,nspnl
      write(uout,*) 'number of atomic species ', nspnl
      write(uout,*) 'number of k-points ', nkibzr
      if(nkibzr/=nkibz) stop '*.vkb, nkpt'
      if(nspnl/=ntype) stop '*.vkb, nspnl'
      allocate(na(nspnl),nl(nspnl))
      vkbsign(:,:) = 0.0
      ia = 0
      do is = 1, nspnl
        read(11) na(is), nl(is)
        write(uout,*) ' specie ',is
        write(uout,*) '  number of atoms ',na(is)
        write(uout,*) '  number of angular momentum components ',nl(is)
        if(nl(is)>mpsang) stop '*.vkb mpsang'
        if(ia+na(is)>natom) stop '*.vkb natom'
        do iar = 1, na(is)
          ia = ia + 1
          typeat(ia) = is
          read(11) (tau0(i),i=1,3)
          xcart(:,ia) = tau0(:)
          write(uout,3845) iar, (xcart(i,ia),i=1,3)
3845      format('   atom ',i2,' cell shift factor xcart = ',3f10.6)
        enddo
        do il = 1, nl(is)
          read(11) vkbsign(il,is)
          write(uout,'("  vkbsign ",5f4.1)') vkbsign(1:nl(is),is)
        enddo
      enddo
      
      vkb(:,:,:,:) = 0.0
      vkbd(:,:,:,:) = 0.0
      do ik = 1, nkibz
        read(11) ngvkb
        write(uout,*) 'k-point ', ik, 'number of G ', ngvkb
        if(ngvkb<npwwfn) stop 'ngvkb in *.vkb'
        do is = 1, nspnl
          if(DEBUG_OLD) write(uout,*) 'is = ', is
          do il = 1, nl(is)
            if(DEBUG_OLD) write(uout,*) 'il = ', il
            read(11) (vkb(ig,is,il,ik),ig=1,npwwfn), &
     &                (dum,ig=npwwfn+1,ngvkb)
            read(11) (vkbd(ig,is,il,ik),ig=1,npwwfn), &
     &                (dum,ig=npwwfn+1,ngvkb)
            if(DEBUG_OLD) then
              write(uout,'("vkb = ",15f5.2)') vkb(:,is,il,ik)
              write(uout,'("vkbd = ",15f5.2)') vkbd(:,is,il,ik)
            endif
            if(ik == 1) then
              dum = 0.0
              do ig = 1, npwwfn
                dum = dum + vkb(ig,is,il,ik) + vkbd(ig,is,il,ik)
              enddo
              if(dum == 0.0) then
                if(il == 1) then
                  write(uout,*) 'specie ', is, ' reference s'
                else if(il == 2) then
                  write(uout,*) 'specie ', is, ' reference p'
                else if(il == 3) then
                  write(uout,*) 'specie ', is, ' reference d'
                endif
              endif
            endif
          enddo ! il
        enddo ! is
      enddo ! ik
      close(11)
      write(uout,*)
      deallocate(na,nl)
      return
      end subroutine
!!***
