!{\src2tex{textfont=tt}}
!!****p* read_structure/testqps
!! NAME
!! Test Quasiparticle Structure file (QPS file)
!!
!! FUNCTION
!! Test _QPS abinit file (m_lda_to_qp, eigenvalues).   
!! It doesn't read the quasiparticle density.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! read_structure
!!
!! CHILDREN
!!
!! SOURCE
!=======================================================================
subroutine testqps(iparam,qpsfile,ql,nkr,nbr,nsppol)

 use verbose
 use dpdef
 use dpdef
 use interfaces

 implicit none

 type(dpparam) iparam

 integer ql, nkr, nkrqps, nbr, nbrqps, nsppol, nsppolqps, idum 
 integer, parameter :: unitfile=56
 character (len=ql) :: qpsfile
#ifdef MPI
include 'mpif.h'
integer ierr,rang,nprocs
call mpi_comm_size(mpi_comm_world,nprocs,ierr)
call mpi_comm_rank(mpi_comm_world,rang,ierr)
if(iparam%mpilocal .and. rang/=0) return
#endif

 if(iverbose > 2) call uverbose('b','testqps')

 if(iverbose > 0) then
  if(ql<20) then
   headline= "Testing QPS file"//qpsfile
  else
   headline= "Testing QPS file"
  endif
  call print_headline()
 endif
 
 open(unitfile,file=qpsfile,form='formatted',status='old')
 read(unitfile,*) 
 read(unitfile,*) nkrqps
 if(iverbose > 1) write(uout,*) 'number of k-points found in qpsfile', nkrqps
 if (nkrqps/=nkr) then 
  call mesg('error','number of kpoints in qpsfile not matching kssfile',idum,idum)
 endif
 read(unitfile,*) nbrqps
 if(iverbose > 1) write(uout,*) 'number of bandsfound in qpsfile', nbrqps
 if (nbrqps > nbr) then 
  call mesg('error','number of bands in qpsfile greater than kssfile',idum,idum)
 endif
 iparam%nbndqps=nbrqps
 read(unitfile,*) nsppolqps
 if(iverbose > 1) write(uout,*) 'number of spin found in qpsfile', nsppolqps
 if (nsppolqps/=nsppol) then 
  call mesg('error','number of spin polarizations in qpsfile not matching kssfile',idum,idum)
 endif

 close(unitfile)
 if(iverbose > 0) then
  call print_footline()
 endif

 if(iverbose > 2) call uverbose('e','testqps')

 end
!!***

!======================================================================


!{\src2tex{textfont=tt}}
!!****p* quasiparticles/rdqps
!! NAME
!! Read Quasiparticle Structure file (QPS file)
!!
!! FUNCTION
!! Read _QPS abinit file (m_lda_to_qp, eigenvalues).   
!! It doesn't read the quasiparticle density.
!!
!! TODO
!! Case nkibz or nsppol in input differs from dimensions read in kss. 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! quasiparticles
!!
!! CHILDREN
!! uverbose
!!
!! SOURCE
!=======================================================================
 subroutine rdqps(qpsfile,ql,enqp,mldatoqp,nkibz,kibz,nbndqpmin,nbnds,nbndmax,nspin,mpilocal)


 use verbose
 use dpdef
 use ggwdef
 use interfaces

 implicit none
 integer ql,jk
 character (len=ql) :: qpsfile
 integer nkibz,nbnds,nspin,nbndmax,nbndqpmin
 double precision enqp(nkibz,nbnds,nspin)
 complex  mldatoqp(1:nbndmax,nbndqpmin:nbnds,nkibz,nspin)
        
 integer nscf,ib,ik,isppol,nbr,nkr,nsr,nbmin
 integer,parameter :: unitfile=90
 real  k(3),kibz(3,nkibz)
 double precision, allocatable :: entmp(:)
 complex,allocatable :: mtmp(:,:)
  
 logical mpilocal

#ifdef MPI
include 'mpif.h'
integer ierr,rang,nprocs
call mpi_comm_size(mpi_comm_world,nprocs,ierr)
call mpi_comm_rank(mpi_comm_world,rang,ierr)
if(mpilocal .and. rang/=0) return
#endif

  if(iverbose > 2) call uverbose('b','rdqps')

  if(iverbose > 0) then
    headline="Reading QP wavefunctions and energies from QPS file"
    call print_headline()
  endif

  write(uout,*) 
!  if(iverbose > 1) write(uout,*) ' rdqps: looking for file ',qpsfile
  open(unitfile,file=qpsfile,form='formatted',status='old')
  read(unitfile,*) nscf
  if(iverbose > 1) write(uout,*) 'number of iterations performed: ',nscf 

  read(unitfile,*) nkr
  if(iverbose > 1) write(uout,*) 'number of k-points ', nkr

  read(unitfile,*) nbr
  if (nbndqpmin > nbr) then
   write(uout,*) 'number of wfns found in qpsfile ', nbr, ' but lomo is ', nbndqpmin
   write(uout,*) 'cannot use quasiparticle wfns; it will continue with KS ones'
   return
  endif

   nbmin=min(nbr,nbnds)

  if(iverbose > 1) then
   write(uout,*) 'number of wfns found in qpsfile ', nbr
  endif
  if (nbr < nbnds ) then
   write(uout,*) 'warning: in qps file found only ', nbr,' wavefunctions' 
   write(uout,*) 'from ', nbr+1,' to ', nbnds, ' will use Kohn-Sham wavefunctions'
  endif
 
  read(unitfile,*) nsr
  if(iverbose > 1) then
   write(uout,*) 'number of spin component', nsr
  endif

  allocate(mtmp(nbr,nbr))
  allocate(entmp(nbr))


  do isppol=1,nspin 
   do ik=1,nkibz
    read(unitfile,*) k(:)
    if(iverbose > 1) then
     write(uout,1072) ik,k
1072  format (' rdqps: reading ik ',i3,'  k = ',3f10.3)
    endif

! ORDERING OF K POINTS IS THE SAME AS IN KSS FILE
! up to now: k point must in list of k points in irr. Bz (in the kss file)
! equivalent k points in fullBz not reliable in Abinit
! so only check that the k points is in the list:
    if (any(abs(k(:)-kibz(:,ik))>0.001)) then 
      write(uout,1071) (k(jk),jk=1,3)
1071      format ('**ERROR, rdqps: k-point ',3f14.6,' is not in set of k points')
      stop
    endif
!   determine the index number in bz  (=>ikk)
!    ikk = 0
!    do jk = 1, nikbz
!      if(all(abs(k(:)-kbz(:,jk)) < 1.e-4)) ikk = jk
!    enddo
!    ikibz = ktab(jk)
!    if(iverbose > 1) then
!    write(uout,100) ik, ikibz, kibz(:,jk)
!100     format('  k-point, BZ number ',i3,'  BZ number ',i3,'  r.l.c. ',3f6.3)
!    endif
!
!    if(ikk .eq. 0) then
!      write(uout,1071) (k(jk),jk=1,3)
!1071      format ('**ERROR, rdqps: k-point ',3f14.6,' is not in set of k points')
!      stop 
!   endif

    do ib=1,nbr
     read(unitfile,*) entmp(ib)
     read(unitfile,*) mtmp(:,ib)
    end do !ib
    mldatoqp(1:nbr,nbndqpmin:nbmin,ik,isppol)=mtmp(1:nbr,nbndqpmin:nbmin)
    enqp(ik,1:nbmin,isppol)=entmp(1:nbmin)
   end do !ik
  end do !isppol 

!output energies
!    write(uout,*)
!    write(uout,*) 'Quasiparticle energies from QPS file'
!    do isppol=1,nspin
!      write (uout,*) 'spin component', isppol 
!      write (uout,*) 'bands:',nbndqpmin,' to ', nbmin
!      write (uout,*) ' k       eigenvalues [eV]      '       
!      do ik=1,nkibz
!         write(uout,'(i3,7x,10f7.2/50(10x,10f7.2/))') &
!&        ik, (HARTREE*enqp(ik,ib,isppol),ib=nbndqpmin,nbmin)
!      enddo
!    enddo

! doesn't read quasiparticle density
!  read(unitfile,*) rho_p(:,:)

  deallocate(mtmp,entmp)
  close(unitfile)

  if(iverbose > 0) then
   call print_footline()
  endif
  if(iverbose > 2) call uverbose('e','rdqps')

  end
!!***


!{\src2tex{textfont=tt}}
!!****p* main/calc_wf_qp
!! NAME
!! Calculate QuasiParticle WaveFunctions
!!
!! FUNCTION
!! Calculate quasiparticle wfns (wfgqp) from the Kohn-Sham basis.   
!! Deallocate Kohn-Sham wavefunctions (wfg).
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! NOTE 
!! For each k point, spin and g vector component: 
!! psi_QP (j) = sum_i psi_LDA(i) m_lda_to_qp(i,j) 
!!
!! PARENTS
!! main, fft_initial_states
!!
!! CHILDREN
!! cmatxmat
!!
!! SOURCE
!=======================================================================


 subroutine calc_wf_qp(iparam,elecstr)

 use dpdef
 use ggwdef
 use verbose
 use interfaces

 implicit none
 type(dpparam) iparam
 type(electronic_structure) elecstr
 
 integer istat,is,ik
 complex wf_k(iparam%npwvec,1:iparam%nbndmax)
 complex wfqp_k(iparam%npwvec,1:iparam%nbndqpmax-iparam%nbndqpmin+1) 
 complex m_lda_to_qp_k(1:iparam%nbndmax,1:iparam%nbndqpmax-iparam%nbndqpmin+1) 
 complex, allocatable :: wfgqp(:,:,:,:)

  if(iverbose > 2) call uverbose('b','calc_wf_qp')

  if(iverbose > 0) then
   headline="Calculating QP wavefunctions"
   call print_headline()
  endif

 allocate(wfgqp(iparam%npwvec,iparam%nbndqpmin:iparam%nbndqpmax,iparam%nkibz,iparam%nsppol),stat=istat)
 if(istat.ne.0) stop 'Quasiparticle wavefunctions(G) out of memory'
 if(iverbose>2) then
   call memstat('qp wfg  allocated =  ')
 endif


 do is=1,iparam%nsppol
   do ik=1,iparam%nkibz
     wf_k(:,1:iparam%nbndmax)=elecstr%wfg(:,1:iparam%nbndmax,ik,is)
     m_lda_to_qp_k(1:iparam%nbndmax,1:iparam%nbndqpmax-iparam%nbndqpmin+1) =  &
&                                elecstr%m_lda_to_qp(1:iparam%nbndmax,iparam%nbndqpmin:iparam%nbndqpmax,ik,is)

     call cmatxmat(iparam%npwvec,iparam%nbndqpmax-iparam%nbndqpmin+1,iparam%nbndmax,wf_k,iparam%npwvec, &
                   m_lda_to_qp_k,iparam%nbndmax,wfqp_k,iparam%npwvec) 


     wfgqp(:,iparam%nbndqpmin:iparam%nbndqpmax,ik,is) =  wfqp_k(:,1:iparam%nbndqpmax-iparam%nbndqpmin+1)


   enddo
 enddo

 
 deallocate(elecstr%wfg)
 allocate(elecstr%wfg(iparam%npwvec,iparam%nbndqpmin:iparam%nbndqpmax,iparam%nkibz,iparam%nsppol))
 elecstr%wfg=wfgqp
 deallocate(wfgqp)
 deallocate(elecstr%m_lda_to_qp)
! now set iparam%nbndmin = iparam%nbndqpmin which is  = iparam%lomo
 iparam%nbndmin = iparam%nbndqpmin
! iparam%nbndmax = iparam%nbndqpmax =  iparam%nbnds

 if(iverbose>2) then
   call memstat('wfg deallocated =  ')
 endif

  if(iverbose > 0) then
   call print_footline()
  endif
  if(iverbose > 2) call uverbose('e','calc_wf_qp')

 end
!!***

