!{\src2tex{textfont=tt}}
!!****p* main/scalapackinterfaces
!!
!!***

!{\src2tex{textfont=tt}}
!!****p* scalapackinterfaces/diagoparal_ch
!! NAME
!! diagoparal_ch
!!
!! FUNCTION
!! Compute the eigenvalues and the eigenvectors of a complex Hermitian matrix
!! (only the upper triangle of a is used) using the parallel procedure
!! invoking BLACS and SCALAPACK
!! 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!!
!! SOURCE
subroutine diagoparal_ch(n,array,eigenvec,eigenval,files,iparam,trans)
use dpdef
use verbose
use ggwdef
use paral
use interfaces
implicit none
type(dpparam) iparam
type(transition) trans(iparam%nt)
type(dpfiles) files 
integer :: n
complex :: array(lda,ldb), eigenvec(lda,ldb)
real :: eigenval(n)
!about scalapack
integer :: np0, mq0, lwork, lrwork, liwork, idum,anb,nps,nhetrd_lwork
real :: dum, tolw, sqnpc
real, allocatable :: rwork(:)
complex, allocatable :: work(:)
integer, allocatable :: iwork(:)
integer, allocatable :: ifail(:)
integer, allocatable :: iclustr(:)
real, allocatable :: gap(:)
integer, external :: iceil,pslamch,pjlaenv
integer :: i,j,it
#ifdef MPI /* to permit sequential job to compile */
include 'mpif.h'
 call mpi_comm_size(mpi_comm_world,nprocs,ierr)
 call mpi_comm_rank(mpi_comm_world,rang,ierr)
! array descriptors for array and eigenvec (in this case they are the same)
allocate(desca(9))
allocate(descz(9))
write(uout,*)'inside scalapack ',lda,ldb,rang
call descinit( desca, n, n, nb, mb, 0, 0, contxt, lda, info )
call descinit( descz, n, n, nb, mb, 0, 0, contxt, lda, info )


! tolw=pslamch(contxt,'U') this is how it is suggested on the scalapack routine in order to have the best orthogonalization
!                          but it turns out that it doesn't work on power6

tolw = -1.

!np0=numroc(n,nb,0,0,nprow)
!mq0=numroc(n,nb,0,0,npcol)
!lwork=n+(nb+np0+mq0)*nb
!anb = pjlaenv(contxt , 3, 'pchettrd', 'L', 0, 0, 0, 0 )
!sqnpc = sqrt( float( nprow * npcol ) )
!nps=max( numroc( n, 1, 0, 0, sqnpc ), 2*anb )
!nhetrd_lwork = n + 2*( anb+1 )*( 4*nps+2 ) + ( nps + 1 ) * nps
!lwork = max( lwork, nhetrd_lwork )
!lrwork=4*n+max(5*n,np0*mq0)+iceil(n,nprow*npcol)*n
!liwork=6*max(n, nprow*npcol + 1, 4 )
allocate(ifail(n))
allocate(iclustr(2*nprow*npcol))
allocate(gap(nprow*npcol))
lwork = -1
lrwork = -1
liwork = -1
allocate(work(1))
allocate(rwork(1))
allocate(iwork(1))
call pcheevx( 'V', 'A', 'U', n, array, 1, 1, desca, dum, dum, idum, &
             idum, tolw, nout, nzout, eigenval, 0.001, eigenvec, 1, 1, descz, work,&
             lwork, rwork, lrwork, iwork, liwork, ifail, iclustr,&
             gap, info )
lwork=int(abs(work(1)))
lrwork=int(abs(rwork(1)))
liwork=int(abs(iwork(1)))
deallocate(work,rwork,iwork)
allocate(work(lwork))
allocate(rwork(lrwork))
allocate(iwork(liwork))
call memstat('scalapack works allocated')
call pcheevx( 'V', 'A', 'U', n, array, 1, 1, desca, dum, dum, idum, &
             idum, tolw, nout, nzout, eigenval, 0.001, eigenvec, 1, 1, descz, work,&
             lwork, rwork, lrwork, iwork, liwork, ifail, iclustr,&
             gap, info )

! print the eigenvectors
! call prnteigen( n, n, eigenvec, 1, 1, descz, 0, 0, 'z', 6, work,'F')
! create file exceig (eigenval, eigenvec, aux data for plotting excitons)
 write(uout,*) 'write eigenvalues on file out.exeig'
 if (rang==0) then 
  open(25,file=files%exeig,form='unformatted')
  write(25) n
  write(25) cmplx(eigenval(1:n))
 endif
 write(uout,*) 'write eigenvectors on file out.exeig'
 call mpi_barrier(mpi_comm_world,ierr)  ! apparently this barrier is important for power6
 call prnteigen(n, n, eigenvec,1,1,descz,0,0,'z',25, work,'U')
! spin to be added
 if (rang==0) then 
  write(uout,*) 'write trans object on file out.exeig'
  do it = 1, n
    write(25) trans(it)%k,trans(it)%v,trans(it)%c,trans(it)%gwen
  enddo
  write(25) iparam%nkbz, iparam%lomo, iparam%homo, iparam%nbnds
  close(25)
 endif

deallocate(work,rwork,iwork,ifail,iclustr,gap)
 call memstat('after deallocate works')
#endif
end subroutine diagoparal_ch
!!***


!{\src2tex{textfont=tt}}
!!****p* diagoparal_ch/prnteigen
!! NAME
!! prnteigen
!!
!! FUNCTION
!! write the eigenvectors (either on std output or on a file) distributed on 
!! the different processors.
!! Origin of the subroutine: scalapack PCLAPRNT 
!!
!! PARENTS
!! diagoparal_ch
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!!
!! SOURCE
subroutine prnteigen(m,n,a,ia,ja,desca,irprnt,icprnt,cmatnm,nout,work,fform)
!     .. Scalar Arguments ..
 integer            ia, icprnt, irprnt, ja, m, n, nout
 character*1        fform
!     ..
!     .. Array Arguments ..
 character*(*)      cmatnm
 integer            desca( * )
 complex            a( * ), work( * )
 complex, allocatable :: array_tmp(:)
!     .. Parameters ..
 integer  block_cyclic_2d, csrc_, ctxt_, dlen_, dtype_, lld_, mb_, m_, nb_, n_, rsrc_
 parameter          ( block_cyclic_2d = 1, dlen_ = 9, dtype_ = 1,  &
                      ctxt_ = 2, m_ = 3, n_ = 4, mb_ = 5, nb_ = 6, &
                      rsrc_ = 7, csrc_ = 8, lld_ = 9 )
!     .. Local Scalars ..
 integer  h, i, iacol, iarow, ib, ictxt, icurcol,      &
          icurrow, ii, iia, in, j, jb, jj, jja, jn, k, &
          lda, mycol, myrow, npcol, nprow
!     .. External Subroutines ..
 external blacs_barrier, blacs_gridinfo, infog2l,cgerv2d, cgesd2d
!     .. External Functions ..
 integer            iceil
 external           iceil
!     .. Intrinsic Functions ..
 intrinsic          aimag, min, real
 ictxt = desca( ctxt_ )
 call blacs_gridinfo( ictxt, nprow, npcol, myrow, mycol )
!
 call infog2l( ia, ja, desca, nprow, npcol, myrow, mycol, iia, jja, iarow, iacol )
 icurrow = iarow
 icurcol = iacol
 ii = iia
 jj = jja
 lda = desca( lld_ )
!
!handle the first block of column separately
!

 if(myrow.eq.irprnt .and. mycol.eq.icprnt) allocate(array_tmp(m))
 jn = min( iceil( ja, desca( nb_ ) ) * desca( nb_ ), ja+n-1 )
 jb = jn-ja+1
 do h = 0, jb-1
    array_tmp(:)=0
    in = min( iceil( ia, desca( mb_ ) ) * desca( mb_ ), ia+m-1 )
    ib = in-ia+1
    if( icurrow.eq.irprnt .and. icurcol.eq.icprnt ) then
       if( myrow.eq.irprnt .and. mycol.eq.icprnt ) then
          do k = 0, ib-1
            if (fform .eq. 'F') then
             write( nout, fmt = 9999 )  cmatnm, ia+k, ja+h, real( a(ii+k+(jj+h-1)*lda) ), aimag( a(ii+k+(jj+h-1)*lda) )
            elseif (fform .eq. 'U') then
             array_tmp(ia+k)=a(ii+k+(jj+h-1)*lda)
            endif
          enddo
       end if
    else
       if( myrow.eq.icurrow .and. mycol.eq.icurcol ) then
          call cgesd2d( ictxt, ib, 1, a( ii+(jj+h-1)*lda ), lda, irprnt, icprnt )
       else if( myrow.eq.irprnt .and. mycol.eq.icprnt ) then
          call cgerv2d( ictxt, ib, 1, work, desca( mb_ ), icurrow, icurcol )
          do k = 1, ib
            if (fform .eq. 'F') then
             write( nout, fmt = 9999 ) cmatnm, ia+k-1, ja+h, real( work( k ) ),aimag( work( k ) )
            elseif (fform .eq. 'U') then
             array_tmp(ia+k-1)=work(k)
            endif
          enddo
       end if
    end if
    if( myrow.eq.icurrow )ii = ii + ib
    icurrow = mod( icurrow+1, nprow )
    call blacs_barrier( ictxt, 'all' )
!
!   loop over remaining block of rows
!
    do i = in+1, ia+m-1, desca( mb_ )
       ib = min( desca( mb_ ), ia+m-i )
       if( icurrow.eq.irprnt .and. icurcol.eq.icprnt ) then
          if( myrow.eq.irprnt .and. mycol.eq.icprnt ) then
             do k = 0, ib-1
               if (fform .eq. 'F') then
                write( nout, fmt = 9999 ) cmatnm, i+k, ja+h, real( a( ii+k+(jj+h-1)*lda ) ), aimag( a( ii+k+(jj+h-1)*lda ) )
               elseif (fform .eq. 'U') then
                array_tmp(i+k)=a(ii+k+(jj+h-1)*lda)
               endif
             enddo
          end if
       else
          if( myrow.eq.icurrow .and. mycol.eq.icurcol ) then
             call cgesd2d( ictxt, ib, 1, a( ii+(jj+h-1)*lda ), lda, irprnt, icprnt )
          else if( myrow.eq.irprnt .and. mycol.eq.icprnt ) then
             call cgerv2d( ictxt, ib, 1, work, desca( mb_ ), icurrow, icurcol )
             do k = 1, ib
               if (fform .eq. 'F') then
                write( nout, fmt = 9999 ) cmatnm, i+k-1, ja+h, real( work( k ) ), aimag( work( k ) )  
               elseif (fform .eq. 'U') then
                array_tmp(i+k-1)=work(k)
               endif
             enddo
          end if
       end if
       if( myrow.eq.icurrow ) ii = ii + ib
       icurrow = mod( icurrow+1, nprow )
       call blacs_barrier( ictxt, 'all' )
    enddo
!
   ii = iia
   icurrow = iarow
   if (fform .eq. 'U' .and. myrow.eq.irprnt .and. mycol.eq.icprnt) write(nout)array_tmp
 enddo
!
 if( mycol.eq.icurcol ) jj = jj + jb
 icurcol = mod( icurcol+1, npcol )
 call blacs_barrier( ictxt, 'all' )
!
!loop over remaining column blocks
!
 do j = jn+1, ja+n-1, desca( nb_ )
    jb = min(  desca( nb_ ), ja+n-j )
    do h = 0, jb-1
       array_tmp(:)=0
       in = min( iceil( ia, desca( mb_ ) ) * desca( mb_ ), ia+m-1 )
       ib = in-ia+1
       if( icurrow.eq.irprnt .and. icurcol.eq.icprnt ) then
          if( myrow.eq.irprnt .and. mycol.eq.icprnt ) then
             do k = 0, ib-1
               if (fform .eq. 'F') then
                write( nout, fmt = 9999 ) cmatnm, ia+k, j+h, real( a( ii+k+(jj+h-1)*lda ) ), aimag( a( ii+k+(jj+h-1)*lda ) )
               elseif (fform .eq. 'U') then
                array_tmp(ia+k)=a(ii+k+(jj+h-1)*lda)
               endif
             enddo
          end if
       else
          if( myrow.eq.icurrow .and. mycol.eq.icurcol ) then
             call cgesd2d( ictxt, ib, 1, a( ii+(jj+h-1)*lda ), lda, irprnt, icprnt )
          else if( myrow.eq.irprnt .and. mycol.eq.icprnt ) then
             call cgerv2d( ictxt, ib, 1, work, desca( mb_ ), icurrow, icurcol )
             do k = 1, ib
               if (fform .eq. 'F') then
                write( nout, fmt = 9999 ) cmatnm, ia+k-1, j+h, real( work( k ) ),aimag( work( k ) )
               elseif (fform .eq. 'U') then
                array_tmp(ia+k-1)=work(k)
               endif
             enddo
          end if
       end if
       if( myrow.eq.icurrow ) ii = ii + ib
       icurrow = mod( icurrow+1, nprow )
       call blacs_barrier( ictxt, 'all' )
!
!      loop over remaining block of rows
!
       do i = in+1, ia+m-1, desca( mb_ )
          ib = min( desca( mb_ ), ia+m-i )
          if( icurrow.eq.irprnt .and. icurcol.eq.icprnt ) then
             if( myrow.eq.irprnt .and. mycol.eq.icprnt ) then
                do k = 0, ib-1
                  if (fform .eq. 'F') then
                   write( nout, fmt = 9999 ) cmatnm, i+k, j+h, real( a( ii+k+(jj+h-1)*lda ) ), aimag( a( ii+k+(jj+h-1)*lda ) )
                  elseif (fform .eq. 'U') then
                   array_tmp(i+k)=a(ii+k+(jj+h-1)*lda)
                  endif
                enddo
             end if
          else
             if( myrow.eq.icurrow .and. mycol.eq.icurcol ) then
                call cgesd2d( ictxt, ib, 1, a( ii+(jj+h-1)*lda ), lda, irprnt, icprnt )
              else if( myrow.eq.irprnt .and. mycol.eq.icprnt ) then
                call cgerv2d( ictxt, ib, 1, work, desca( mb_ ),icurrow, icurcol )
                do k = 1, ib
                  if (fform .eq. 'F') then
                   write( nout, fmt = 9999 ) cmatnm, i+k-1, j+h, real( work( k ) ), aimag( work( k ) )
                  elseif (fform .eq. 'U') then
                   array_tmp(i+k-1)=work(k)
                  endif
                enddo
             end if
          end if
          if( myrow.eq.icurrow )ii = ii + ib
          icurrow = mod( icurrow+1, nprow )
          call blacs_barrier( ictxt, 'all' )
       enddo
!
       if (fform .eq. 'U' .and. myrow.eq.irprnt .and. mycol.eq.icprnt) write(nout)array_tmp
       ii = iia
       icurrow = iarow
    enddo
!
    if( mycol.eq.icurcol )jj = jj + jb
    icurcol = mod( icurcol+1, npcol )
    call blacs_barrier( ictxt, 'all' )
!
 enddo
!
 if(myrow.eq.irprnt .and. mycol.eq.icprnt) deallocate(array_tmp)
 9999 format(a,'(',i6,',',i6,')=',e16.8, '+i*(',e16.8, ')')
!
 end subroutine prnteigen
!!***
