!{\src2tex{textfont=tt}}
!!****p* main/qpol_and_q
!! NAME
!! qpol_and_q
!!
!! FUNCTION
!! Given Qpol in input write it as Qpol = qpol + G with qpol in first BZ.
!! Setup grid of qpoints.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! CHILDREN
!! fixqpol, findshells, build_qibz, identq, checkqpol, clcqpg, qtabg0, check_qstruct
!!
!! TODO
!! this should take place between testlda and rdkss
!!
!! SOURCE

 subroutine qpol_and_q(iparam,latt,trspace,bzstruct,bzstructp,qstruct,kb,symstr,screen)

 use dpdef
 use ggwdef
 use verbose

 implicit none

 type(dpparam) iparam
 type(symmetry) symstr
 type(lattstruct) latt
 type(bzstructure) bzstruct, bzstructp
 type(transposed_space) trspace
 type(kb_potential) kb
 type(qstructure) qstruct,qstructp
 type(screening) screen

 integer istat
 real temp(3)

#ifdef MPI
 include 'mpif.h'
 integer ierr,rang,nprocs
 call mpi_comm_size(mpi_comm_world,nprocs,ierr)
 call mpi_comm_rank(mpi_comm_world,rang,ierr)
 if (.not.(iparam%mpilocal .and. rang /= 0))  then
#endif

 if(iverbose > 2)call uverbose('b','qpol_and_q')

 if(iverbose > 0) then  
  headline='About qpol and G'
  call print_headline()
 endif

! everything about qpol
 call fixqpol(iparam,latt,trspace,kb)
     
 call findshells(iparam,latt,trspace)

#ifdef MPI
 endif
#endif

! build grid of q-points:

! NOTE: in case of doublegrid & exciton there are 2 different grids of q points
! doublegrid: in any case here build grid of qpol points (not Gamma centered): use nopqpol, ninvqpol
! in case of exciton: then rebuild grid of q points (Gamma centered)

! find q-points q = k - kp in irreducible Bz
  call build_qibz(iparam,bzstruct,bzstructp,qstruct,symstr,latt, iparam%nopqpol,iparam%ninvqpol)

  if (.not. iparam%doublegrid) then  

   if(iparam%exc) then
     if (iparam%nqibz .ne. screen%nqibz) then 
        write(uout,*) 'checkqgrid: ERROR number of q points in screening file not matching q points found'
       write(uout,*) 'from screening file ',screen%nqibz,' now found ', iparam%nqibz
       stop
     endif
   
!!!! TEMPORARY !!!!! now needed in case the two sets don't match (up to a G vector or for symmetries, otherwise wrong)
!                    implement possibility to use symmetries to go from one set to the other ?
    temp(:)=qstruct%qibz(:,1)                                                                                                                               
    qstruct%qibz = screen%qibz
    qstruct%qibz(:,1)=temp(:) 

   endif

  endif

! identify q-points in full Bz 
  call identq(iparam,qstruct,symstr,iparam%nopqpol,iparam%ninvqpol)

! check whether qpol is allowed
  call checkqpol(iparam,qstruct)

  if(.not. iparam%exc) deallocate(qstruct%qtabi,qstruct%qtabo,qstruct%qtab,qstruct%qbz)
  
  if (iparam%exc) then

! in case of doublegrid need to setup again grid of qpoints
   if (iparam%doublegrid) then 
    !now redefine to setup qpoints grid (Gamma centered)
    q_precision = 1.0e-4
    deallocate(qstruct%qibz,qstruct%qbz,qstruct%qtab,qstruct%qtabo,qstruct%qtabi)
    qstructp = qstruct

    !the grid of qpoints is rebuilt using only one of the two grids 
    !(it is needed only for vv' and cc' transitions)
    call build_qibz(iparam,bzstruct,bzstruct,qstruct,symstr,latt,iparam%nopq,iparam%ninvq)
    if (iparam%nqibz .ne. screen%nqibz) then
       write(uout,*) 'qpol_and_q: ERROR number of q points in screening file not matching q points found'
       write(uout,*) 'from screening file ',screen%nqibz,' now found ', iparam%nqibz
       stop
    endif

    !only for check
    call build_qibz(iparam,bzstructp,bzstructp,qstructp,symstr,latt,iparam%nopq,iparam%ninvq)
    if (iparam%nqibz .ne. screen%nqibz) then
       write(uout,*) 'qpol_and_q: ERROR number of q points in screening file not matching q points found'
       write(uout,*) 'from screening file ',screen%nqibz,' now found ', iparam%nqibz
       stop
    endif
    call check_qstruct(iparam%nqibz,qstruct,qstructp)
    deallocate(qstructp%qibz)

! TEMPORARY
    qstruct%qibz = screen%qibz
    call identq(iparam,qstruct,symstr,iparam%nopq,iparam%ninvq)
    
   endif 

   deallocate(symstr%symafm)


!FIXME for mpilocal case
!#ifdef MPI
! call mpi_barrier(mpi_comm_world,ierr)
! call mpi_bcast(qstruct%nqbz,1,mpi_integer,0,mpi_comm_world,ierr)
! call mpi_bcast(qstruct%qbz,3*qstruct%nkbz,mpi_real,0,mpi_comm_world,ierr)
! call mpi_bcast(qstruct%qtab,qstruct%nkbz,mpi_integer,0,mpi_comm_world,ierr)
! call mpi_bcast(qstruct%qtabo,qstruct%nkbz,mpi_integer,0,mpi_comm_world,ierr)
! call mpi_bcast(qstruct%qtabi,qstruct%nkbz,mpi_integer,0,mpi_comm_world,ierr)
!#endif

! check consistency between grid of qpoints and screening file
   call checkqgrid(iparam,screen,qstruct)
 
   !calculate |q+G| for all q(IBZ) and G
   allocate(qstruct%qpg(iparam%npwvec,iparam%nqibz),stat=istat)
   if(istat.ne.0) stop 'out of memory'
   call clcqpg(iparam,trspace%gvec,qstruct%qibz,latt%b1,latt%b2,latt%b3,qstruct%qpg)

   !create table of q=k -kp-g0
   allocate(qstruct%kmkptab(-3*qdenommx:3*qdenommx,-3*qdenommx:3*qdenommx,-3*qdenommx:3*qdenommx))
   qstruct%kmkptab(:,:,:)=0
   call qtabg0(qstruct%nqbz,qstruct%qbz,qstruct%qdenom,qstruct%kmkptab)
 
  endif !exc

  deallocate(qstruct%qibz)

  if(iverbose > 0) then
   call print_footline()
  endif

 if(iverbose > 2)call uverbose('e','qpol_and_q')

#ifdef MPI
 if(.not.(iparam%mpilocal .and. rang/=0)) then
#endif


#ifdef MPI
 endif
if(iparam%mpilocal) then
 call mpi_barrier(mpi_comm_world,ierr)
 call mpi_bcast(latt%b1,3,mpi_real,0,mpi_comm_world,ierr)
 call mpi_bcast(latt%b2,3,mpi_real,0,mpi_comm_world,ierr)
 call mpi_bcast(latt%b3,3,mpi_real,0,mpi_comm_world,ierr)
 call mpi_bcast(latt%ucvol,1,mpi_real,0,mpi_comm_world,ierr)
 call mpi_bcast(iparam%nqpol,1,mpi_integer,0,mpi_comm_world,ierr)
 call mpi_bcast(iparam%qpol,3*6,mpi_real,0,mpi_comm_world,ierr)
! call mpi_bcast(iparam%KP_THEORY,1,mpi_logical,0,mpi_comm_world,ierr)
 call mpi_bcast(iparam%ig,1,mpi_integer,0,mpi_comm_world,ierr)
 call mpi_bcast(iparam%npwvec,1,mpi_integer,0,mpi_comm_world,ierr)
 call mpi_bcast(iparam%npwmat,1,mpi_integer,0,mpi_comm_world,ierr)
 call mpi_bcast(iparam%npwwfn,1,mpi_integer,0,mpi_comm_world,ierr)
endif
#endif


 end
!!***

!{\src2tex{textfont=tt}}
!!****p* qpol_and_q/fixqpol
!! NAME
!! fixqpol
!!
!! FUNCTION
!! Write the q vector along direct bravais
!! lattice coordinates a, reduced coordinates b, etc.
!! If the q vector is outside the 1st Brillouin zone, it is folded back into it
!! and the G-vector is retained.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! qpol_and_q
!!
!! CHILDREN
!! reduceto1bz, identifyg, is_small, is_zero
!!
!! TODO
!! this should take place between testlda and rdkss
!!
!! SOURCE


 subroutine fixqpol(iparam,latt,trspace,kb)

 use dpdef
 use ggwdef
 use verbose

 implicit none

 type(dpparam) iparam
 type(lattstruct) latt
 type(transposed_space) trspace
 type(kb_potential) kb

 integer i,j,idum
 real rnorm
 logical, external:: is_small, is_zero

 if(iverbose > 2)call uverbose('b','fixqpol')
 

 call reduceto1bz(iparam%qpg,iparam%qpol(:,1),iparam%g,latt)
 if(any(iparam%g(:)>9)) call mesg('warning','momentum transfer qpol might be outside 1BZ',idum,idum)
 if(iparam%JDOS) then
   iparam%npwmat = 1
   iparam%npwvec = 1
   iparam%matsh = -1
   iparam%npwwfn = 1
   iparam%wfnsh = -1
   iparam%g(:) = 0
 endif
 call identifyg(iparam%g,iparam%ig,trspace%gvec,iparam%npwvec)
 if(iparam%ig > iparam%npwmat) then
   write(uout,*) 'ERROR: G vector out of limits:'
   write(uout,*) '       G = ', iparam%g
   write(uout,*) '       increase NPWMAT.'
   if(iparam%ig<=iparam%npwvec) then
     write(uout,*) '       NPWMAT should contains at least ', iparam%ig
   endif
   stop 'fixqpol error: g vector out of limits'
 endif

! the decision, if k.p is applied has been already taken in rddpin()
! but in the case there is qpol Q such as Q=q+G with q=0, the commutator is still needed
 if(is_small(iparam%qpol(:,1))) iparam%KP_THEORY = .true.

 if( is_zero(iparam%qpol(:,1))) then
  if (.not.iparam%exciton .or. iparam%ig == 1) then
   iparam%nqpol = 6
   iparam%qpol(1,1) = 1.0 ; iparam%qpol(2,1) = 0.0 ; iparam%qpol(3,1) = 0.0       ! (100)
   iparam%qpol(1,2) = 0.0 ; iparam%qpol(2,2) = 1.0 ; iparam%qpol(3,2) = 0.0       ! (010)
   iparam%qpol(1,3) = 0.0 ; iparam%qpol(2,3) = 0.0 ; iparam%qpol(3,3) = 1.0       ! (001)
   iparam%qpolcc(1,4) = 1.0 ; iparam%qpolcc(2,4) = 0.0 ; iparam%qpolcc(3,4) = 0.0 ! (x)
   iparam%qpolcc(1,5) = 0.0 ; iparam%qpolcc(2,5) = 1.0 ; iparam%qpolcc(3,5) = 0.0 ! (y)
   iparam%qpolcc(1,6) = 0.0 ; iparam%qpolcc(2,6) = 0.0 ; iparam%qpolcc(3,6) = 1.0 ! (z)
   iparam%qpol(:,4) = matmul(latt%bcc2rl,iparam%qpolcc(:,4))
   iparam%qpol(:,5) = matmul(latt%bcc2rl,iparam%qpolcc(:,5))
   iparam%qpol(:,6) = matmul(latt%bcc2rl,iparam%qpolcc(:,6))
   iparam%qpolcc(:,1) = matmul(latt%brl2cc,iparam%qpol(:,1))
   iparam%qpolcc(:,2) = matmul(latt%brl2cc,iparam%qpol(:,2))
   iparam%qpolcc(:,3) = matmul(latt%brl2cc,iparam%qpol(:,3))
   if (iparam%antieps) then ! q -> -q
    iparam%qpol(:,:)   = - iparam%qpol(:,:)
    iparam%qpolcc(:,:) = - iparam%qpolcc(:,:)
   endif
  else
   iparam%nqpol = 1
   iparam%qpol(:,1)=trspace%gvec(:,iparam%ig)
   call mesg('warning','excitonic calculation will be done in ONE direction ONLY;  &
                        &case: Q=0+G, verify your direction G',idum,idum)
   iparam%qpolcc(:,1) = matmul(latt%brl2cc,iparam%qpol(:,1))
  endif

! normalization on cartesian coordinates polarization vectors
   do j = 1, iparam%nqpol
     rnorm = sqrt(dot_product(iparam%qpolcc(:,j),iparam%qpolcc(:,j)))
     iparam%qpolcc(:,j) = iparam%qpolcc(:,j) / rnorm
   enddo
   do j = 1, iparam%nqpol
    iparam%qpol(:,j) = matmul(latt%bcc2rl,iparam%qpolcc(:,j))
   enddo
   iparam%qpolcc2pioa(:,:) = iparam%qpolcc(:,:) / latt%tpula
 
   iparam%qpol(:,:) = iparam%qpol(:,:) * 1.0e-5
   iparam%qpolcc(:,:) = iparam%qpolcc(:,:) * 1.0e-5
   iparam%qpolcc2pioa(:,:) = iparam%qpolcc2pioa(:,:) * 1.0e-5

   if(iverbose > 0) then
    write(uout,*) 'calculating at qpol=0'
    write(uout,*) 'along the following polarization directions:' 
    do j = 1, iparam%nqpol
     write(uout,120) (iparam%qpol(i,j),i=1,3), (iparam%qpolcc(i,j),i=1,3),(iparam%qpolcc2pioa(i,j),i=1,3), latt%tpula
120  format(' qpol = (',f9.6,',',f9.6,',',f9.6,') [r.l.u.]',/, &
            ' qpol = (',f9.6,',',f9.6,',',f9.6,') [c.c.]'/ &
            ' qpol = (',f9.6,',',f9.6,',',f9.6,') ', &
            '[c.c. units of 2pi/a = ',f9.6,' a.u.]'/)
    enddo
   endif

  else !  is_zero(iparam%qpol(:,1))
   iparam%nqpol = 1
   iparam%qpolcc(:,1) = matmul(latt%brl2cc,iparam%qpol(:,1))
   iparam%qpolcc2pioa(:,1) = iparam%qpolcc(:,1) / latt%tpula
   if(iverbose > 0) then
    write(uout,*) 'calculating at the following qpol:'
    write(uout,120) (iparam%qpol(i,1),i=1,3), (iparam%qpolcc(i,1),i=1,3),(iparam%qpolcc2pioa(i,1),i=1,3), latt%tpula
   endif
  endif

  iparam%gcc(:) = real(iparam%g(:))
  iparam%gcc(:) = matmul(latt%brl2cc,iparam%gcc(:))
  iparam%gcc2pioa(:) = iparam%gcc(:) / latt%tpula
  if(iverbose > 0) then
  write(uout,*) 'calculating at the following G:'
  write(uout,125) iparam%g(:), iparam%gcc(:), iparam%gcc2pioa(:), latt%tpula
125   format(' G = (',i2,',',i2,',',i2,') [r.l.u.]',/, &
             ' G = (',f9.6,',',f9.6,',',f9.6,') [c.c.]'/ &
             ' G = (',f9.6,',',f9.6,',',f9.6,') ', &
             '[c.c. units of 2pi/a = ',f9.6,' a.u.]'/) 
  endif

! deallocate the commutator, if there's no need. Problem is that only now we
! know if Q = q + G or not.
if(.not.iparam%JDOS .and. kb%inclvkb .and. .not.iparam%KP_THEORY) deallocate(kb%vkbsign,kb%vkb,kb%vkbd)

if(iparam%FXCMT .and. iparam%nqpol/=1) stop 'fxcmt works with only one qpol'

 if(iverbose > 2)call uverbose('e','fixqpol')

 end
!!***  
