!{\src2tex{textfont=tt}}
!!****p* main/leggi_wfg
!! NAME
!! leggi_wfg
!!
!! FUNCTION
!! Read only the wavefunctions from the kss file
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! CHILDREN
!! test_orthonorm 
!!
!! SOURCE
subroutine leggi_wfg(iparam,files,chemstr,elecstr)

use dpdef
use ggwdef
use verbose

implicit none

type(dpparam) iparam
type(dpfiles) files
type(chemistry) chemstr
type(electronic_structure)elecstr
integer istat, i, ik, ib, is, il, ibp, fform, isppol, ir

integer, parameter :: u = 10


integer :: nsym2, nbndsto, ncomsto, nsh, mpsang, nkpt

complex (kind=8), allocatable :: wfg(:)
complex, allocatable :: wfgs(:)
integer headform, dum, npsp
real cinf, csup
real (kind(0.d0)) :: ddum, einf,esup
character (len=6) :: codvsn
integer, external :: fileform
#ifdef MPI
include 'mpif.h'
integer ierr,rang,nprocs
call mpi_comm_size(mpi_comm_world,nprocs,ierr)
call mpi_comm_rank(mpi_comm_world,rang,ierr)
if(iparam%mpilocal .and. rang/=0) return
#endif


if (iverbose > 2)call uverbose('b','leggi_wfg')

headline="Reading wfg from kss file "//files%kss
call print_headline()

fform = fileform(files%kss,files%kl)
if(fform /= 502 .and. fform /= 602) stop 'not kss form file'
open(u,form='unformatted',file=files%kss)
read(u) codvsn,headform,fform

!write(uout,'(" generated by ABINIT version ",a6," header version ",i1,".",i1)')&
!&     codvsn, headform/10, headform-(headform/10)*10

!if(headform /= 23 .and. headform /= 34 .and. headform /= 40&
!&    .and. headform /= 41 .and. headform /= 42 .and.  headform /= 44 &
!&  .and. headform /= 53 .and. headform/=56 .and. headform/=57) then
!  write(uout,*) '** ERROR: unrecognized abinit header version', headform
!  stop 'unrecognized abinit header version'
!endif

! 1st record
if(headform==23) then
  read(u)  dum,dum,dum,dum,dum,dum,dum,dum,nkpt,dum,dum,dum,dum,npsp
else if(headform==34) then
  read(u) dum,dum,dum,dum,dum,dum,dum,dum,nkpt,dum,dum,dum,dum,npsp
else if(headform==40) then
  read(u) dum,dum,dum,dum,dum,dum,dum,dum,nkpt,dum,dum,dum,dum,npsp
else if(headform==41) then
  read(u) dum,dum,dum,dum,dum,dum,dum,dum,nkpt,dum,dum,dum,dum,npsp
else if(headform==42) then
  read(u) dum,dum,dum,dum,dum,dum,dum,dum,nkpt,dum,dum,dum,dum,npsp
else if(headform==44 .or. headform==53 .or. headform==56) then
  read(u) dum,dum,dum,dum,dum,dum,dum,dum,nkpt,dum,dum,dum,dum,npsp
else if(headform==57) then
  read(u) dum,dum,dum,dum,dum,dum,dum,dum,nkpt,dum,dum,dum,dum,npsp
endif
read(u)
do i=1,npsp
 read(u)
enddo
read(u)


read(u) !tit1 
read(u) !tit2
read(u) nsym2, nbndsto, ncomsto, nsh, mpsang   
read(u) ! symmetries
read(u) ! tnons
read(u) ! gvec
read(u) !(shlim(i),i=1,nsh)
read(u) !vkbsign

if(.not. iparam%wf_distrib) then
 allocate(elecstr%wfg(iparam%npwvec,iparam%nbndmin:iparam%nbndmax,iparam%nkibz,iparam%nsppol),stat=istat)
 if(istat.ne.0) stop 'wavefunctions(G) out of memory'
else 
 allocate(elecstr%wfg2(iparam%npwvec,elecstr%wfdim),stat=istat)
 if(istat.ne.0) stop 'wavefunctions(G) out of memory'
 write(uout,*)!'wfdim=',elecstr%wfdim
endif

 
if(fform==502) then      
  allocate(wfg(ncomsto))
else if(fform==602) then
  allocate(wfgs(ncomsto))
endif


!write(uout,*) ' k       eigenvalues [eV]'
do isppol = 1, elecstr%nsppol 
 do ik = 1, nkpt
 !write(uout,*) 'ik ', ik
  do is = 1, chemstr%ntype
!   write(uout,*) 'is ', is
    do il = 1, mpsang
!     write(uout,*) 'il ', il
      if(fform==502) then      
        read(u) !vkb
        read(u) !vkbd
      else if(fform==602) then
        read(u) !vkb
        read(u) !vkbd
      endif
    enddo !il
  enddo !is 
        
 
  read(u) ! energy
        
  if(fform==502) then    
     do ib = 1, iparam%nbndmax
       read(u) wfg(1:ncomsto)
       if(.not.iparam%JDOS) then
         if (ib .ge. iparam%nbndmin .and. ik .le. iparam%nkibz .and. isppol .le. iparam%nsppol) then 
          if(.not. iparam%wf_distrib) then
           elecstr%wfg(1:iparam%npwvec,ib,ik,isppol) = wfg(1:iparam%npwvec)
          else
           ir=elecstr%wf_count(ik,ib,isppol)
           if(ir/=0) then 
             elecstr%wfg2(1:iparam%npwvec,ir) = wfg(1:iparam%npwvec)
           endif
          endif
         endif
       endif
     enddo
     do ib = iparam%nbndmax+1 , nbndsto
       read(u) ! wfg(1:ncomsto)
     enddo
  else if(fform==602) then
     do ib = 1, iparam%nbndmax
       read(u) wfgs(1:ncomsto)
       if(.not.iparam%JDOS) then
         if (ib .ge. iparam%nbndmin .and. ik .le. iparam%nkibz .and. isppol .le. iparam%nsppol) then
          if(.not. iparam%wf_distrib) then
           elecstr%wfg(1:iparam%npwvec,ib,ik,isppol) = wfgs(1:iparam%npwvec)
          else
           ir=elecstr%wf_count(ik,ib,isppol)
           if(ir/=0) then
             elecstr%wfg2(1:iparam%npwvec,ir) = wfgs(1:iparam%npwvec)
           endif
          endif
         endif
       endif
     enddo
     do ib = iparam%nbndmax+1 , nbndsto
       read(u) ! wfgs(1:ncomsto)
     enddo
  endif

 enddo !ik
enddo !isppol  
if (fform == 502) then
 deallocate(wfg)
else if (fform == 602) then
 deallocate(wfgs)
endif
close(u)

 if(.not.iparam%JDOS) then
  if(iverbose > 1) then
   call test_orthonorm(iparam,elecstr)
  endif
 endif

call print_footline()       
if (iverbose > 2)call uverbose('e','leggi_wfg')

end subroutine leggi_wfg
!!***



