!{\src2tex{textfont=tt}}
!!****p* main/leggi_wfg_qplda
!! NAME
!! leggi_wfg_qplda
!!
!! FUNCTION
!! Read qplda cp file (only wfn)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! CHILDREN
!! test_orthonorm
!!
!! SOURCE
 subroutine leggi_wfg_qplda(iparam,files,elecstr)
!read QPLDA type file
 use dpdef
 use ggwdef
 use verbose
 use interfaces
 implicit none
 type(dpparam) iparam
 type(dpfiles) files
 type(electronic_structure)elecstr
 integer, parameter :: iunit = 10
 integer nb, nk, ng, i1, istat, nwr

 integer isppol, ir


 integer ib, ik, ig,  ibp
 real einf, esup, cinf, csup, sum
 complex cdum
 complex, allocatable :: wfg(:)


 logical FORM
#ifdef MPI
include 'mpif.h'
integer ierr,rang,nprocs
call mpi_comm_size(mpi_comm_world,nprocs,ierr)
call mpi_comm_rank(mpi_comm_world,rang,ierr)
if(iparam%mpilocal .and. rang/=0) return
#endif

 
!i1=0 -> file formatted -> FORM=.TRUE.
!i1!=0 -> file unformatted -> FORM=.FALSE.
 if (files%i1.eq.0) then
   FORM = .TRUE.
 else
   FORM = .FALSE.
 endif


 if (FORM) then
   write (uout,*) 'opening LDA input file formatted unit ',iunit
   open(iunit,status='unknown',form='FORMATTED',file=files%kss)
 else
   write (uout,*) 'opening LDA input file unformatted unit ',iunit
   open(iunit,status='unknown',form='UNFORMATTED',file=files%kss)
 endif

10  format (I12)

 elecstr%nsppol=iparam%nsppol
 if (FORM) then
   read (iunit,*) 
   read (iunit,*) i1
   read (iunit,*) 
   read (iunit,*) 
   read (iunit,*) 
   read (iunit,10) ng 
   read (iunit,*) 
   read (iunit,10) nk
   read (iunit,*) 
   read (iunit,10) nb
   do isppol = 1, elecstr%nsppol 
     read (iunit,*) 
   enddo
   do isppol = 1, elecstr%nsppol 
     read (iunit,*) 
   enddo
 else
   read (iunit) 
   read (iunit)
   read (iunit) i1
   read (iunit) 
   read (iunit) 
   read (iunit) 
   read (iunit) ng
   read (iunit) 
   read (iunit) nk
   read (iunit) 
   read (iunit) nb
   do isppol = 1, elecstr%nsppol 
     read (iunit) 
   enddo
   do isppol = 1, elecstr%nsppol 
     read (iunit) 
   enddo
 endif

 write (uout,*) 'number of k-points ', nk
 write (uout,*) 'number of bands found ', nb

 if (ng.le.iparam%npwwfn) then
   nwr = ng
 else
   nwr = iparam%npwwfn
 endif
 write(uout,*) 'reading a number of plane-waves ', nwr
 write(uout,*) 'reading a number of plane-waves ng', ng
 write(uout,*) 'reading a number of plane-waves iparam%npwwfn', iparam%npwwfn, iparam%npwvec,iparam%nbndmax

 if (.not. iparam%wf_distrib) then
  allocate(elecstr%wfg(iparam%npwvec,iparam%nbndmin:iparam%nbndmax,iparam%nkibz,iparam%nsppol),stat=istat)
  if(istat.ne.0) stop 'wavefunctions(G) out of memory'
  if(iverbose>2) call memstat('wfg allocated =')
 else
  allocate(elecstr%wfg2(iparam%npwvec,elecstr%wfdim),stat=istat)
  if(istat.ne.0) stop 'wavefunctions(G) out of memory'
  if(iverbose>2) call memstat('wfg allocated =')
  write(uout,*) 'wfdim=',elecstr%wfdim
 endif



allocate(wfg(1:ng))

 95   format (3E26.18) ! format for reading text formatted file
!     LC add loop for nsppol
 do isppol=1,elecstr%nsppol
  do ik=1,nk
   do ib=1,iparam%nbndmax
     if (FORM) then
       read (iunit,95) wfg(1:ng)
     else
       read (iunit) wfg(1:ng)
     endif
     if(.not.iparam%JDOS) then
        if (ib .ge. iparam%nbndmin .and. ik .le. iparam%nkibz .and. isppol .le. iparam%nsppol) then
         if (.not. iparam%wf_distrib) then
          elecstr%wfg(1:iparam%npwvec,ib,ik,isppol) = wfg(1:iparam%npwvec)
         else
          ir=elecstr%wf_count(ik,ib,isppol)
          if(ir/=0) then
            elecstr%wfg2(1:iparam%npwvec,ir) = wfg(1:iparam%npwvec)
          endif
         endif
        endif
      endif
   enddo
   do ib=iparam%nbndmax+1,nb
     if (FORM) then
       read (iunit,95) (cdum,ig=1,ng)
     else
       read (iunit) (cdum,ig=1,ng)
     endif
   enddo
  enddo
 enddo !isppol

deallocate(wfg)
 write (uout,*) 'closing file'
 close (iunit)
 write (uout,*) 

 call test_orthonorm(iparam,elecstr)

 end
!!***
