!{\src2tex{textfont=tt}}
!!****p* Multi/memstat
!! NAME
!! memstat
!!
!! FUNCTION
!! subroutine to measure (at this point of the code) the memory use. 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! NOTE
!! This subroutine actually calls a c subroutine (memstatc) that gives different
!! values, depending of which memory we are talking about:
!! * mem1 = info.hblkhd (This is the total size of memory allocated with mmap, in bytes)
!! * mem2 = info.uordblks (This is the total size of memory occupied by chunks handed out by malloc) 
!!
!! PARENTS
!! creation_X, creation_chi0, creation_epsilon, creation_kernelmt, main, fftlda,
!! mesh, prep_trans, prep_trans2, qp_and_fermi, read_structure, xrhotw
!!
!! CHILDREN
!! memstatc
!!
!! SOURCE

subroutine memstatsimple(message)
 use dpdef
 implicit none
 integer mem1,mem2
 character (*) message
#ifdef MPI
 include 'mpif.h'
 integer rang,ierr
 call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif
 call memstatc(mem1,mem2)
#ifdef MPI
 write(78,20)trim(message),mem1,mem2,mem1+mem2,rang
20 format(a60,I9,' + ',I9,' = ',I9,' kb (proc ',I3,')')
#else
 write(78,10)trim(message),mem1,mem2,mem1+mem2
10 format(a60,I9,' + ',I9,' = ',I9,' kb --> ')

#endif
 end


subroutine memstat1(message,typev,val1)
 use dpdef
 implicit none
 integer mem1,mem2
 integer val1
 real valore
 character (*) message
 character (1) typev
#ifdef MPI
 include 'mpif.h'
 integer rang,ierr
 call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif
 call memstatc(mem1,mem2)
 if (typev == 'c') then
  valore = 8.*float(val1)/1024
 elseif (typev == 's') then
  valore = 4.*float(val1)/1024
 elseif (typev == 'd') then
  valore = 8.*float(val1)/1024
 elseif (typev == 'z') then
  valore = 16.*float(val1)/1024
 else ! int
  valore = 2.*float(val1)/1024
 endif
#ifdef MPI
 write(78,20)trim(message),mem1,mem2,mem1+mem2,rang,valore
20 format(a60,I9,' + ',I9,' = ',I9,' kb (proc ',I3,') --> ',f15.2)
#else
 write(78,10)trim(message),mem1,mem2,mem1+mem2,valore
10 format(a60,I9,' + ',I9,' = ',I9,' kb --> ',f15.2)
#endif
 end

subroutine memstat2(message,typev,val1,val2)
 use dpdef
 implicit none
 integer mem1,mem2
 integer val1,val2
 character (*) message
 character (1) typev
 real valore
#ifdef MPI
 include 'mpif.h'
 integer rang,ierr
 call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif
 call memstatc(mem1,mem2)
 if (typev == 'c') then
  valore = 8.*float(val1)*float(val2)/1024
 elseif (typev == 's') then
  valore = 4.*float(val1)*float(val2)/1024
 elseif (typev == 'd') then
  valore = 8.*float(val1)*float(val2)/1024
 elseif (typev == 'z') then
  valore = 16.*float(val1)*float(val2)/1024
 else ! int
  valore = 2.*float(val1)*float(val2)/1024
 endif

#ifdef MPI
 write(78,20)trim(message),mem1,mem2,mem1+mem2,rang,valore
20 format(a60,I9,' + ',I9,' = ',I9,' kb (proc ',I3,') --> ',f15.2)
#else
 write(78,10)trim(message),mem1,mem2,mem1+mem2,valore
10 format(a60,I9,' + ',I9,' = ',I9,' kb --> ',f15.2)
#endif
 end

subroutine memstat3(message,typev,val1,val2,val3)
 use dpdef
 implicit none
 integer mem1,mem2
 integer val1,val2,val3
 character (*) message
 character (1) typev
 real valore
#ifdef MPI
 include 'mpif.h'
 integer rang,ierr
 call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif
 call memstatc(mem1,mem2)
 if (typev == 'c') then
  valore = 8.*float(val1)*float(val2)*float(val3)/1024
 elseif (typev == 's') then
  valore = 4.*float(val1)*float(val2)*float(val3)/1024
 elseif (typev == 'd') then
  valore = 8.*float(val1)*float(val2)*float(val3)/1024
 elseif (typev == 'z') then
  valore = 16.*float(val1)*float(val2)*float(val3)/1024
 else ! int
  valore = 2.*float(val1)*float(val2)*float(val3)/1024
 endif

#ifdef MPI
 write(78,20)trim(message),mem1,mem2,mem1+mem2,rang,valore
20 format(a60,I9,' + ',I9,' = ',I9,' kb (proc ',I3,') --> ',f15.2)
#else
 write(78,10)trim(message),mem1,mem2,mem1+mem2,valore
10 format(a60,I9,' + ',I9,' = ',I9,' kb --> ',f15.2)
#endif
 end

subroutine memstat4(message,typev,val1,val2,val3,val4)
 use dpdef
 implicit none
 integer mem1,mem2
 integer val1,val2,val3,val4
 character (*) message
 character (1) typev
 real valore
#ifdef MPI
 include 'mpif.h'
 integer rang,ierr
 call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif
 call memstatc(mem1,mem2)
 if (typev == 'c') then
  valore = 8.*float(val1)*float(val2)*float(val3)*float(val4)/1024
 elseif (typev == 's') then
  valore = 4.*float(val1)*float(val2)*float(val3)*float(val4)/1024
 elseif (typev == 'd') then
  valore = 8.*float(val1)*float(val2)*float(val3)*float(val4)/1024
 elseif (typev == 'z') then
  valore = 16.*float(val1)*float(val2)*float(val3)*float(val4)/1024
 else ! int
  valore = 2.*float(val1)*float(val2)*float(val3)*float(val4)/1024
 endif

#ifdef MPI
 write(78,20)trim(message),mem1,mem2,mem1+mem2,rang,valore
20 format(a60,I9,' + ',I9,' = ',I9,' kb (proc ',I3,') --> ',f15.2)
#else
 write(78,10)trim(message),mem1,mem2,mem1+mem2,valore
10 format(a60,I9,' + ',I9,' = ',I9,' kb --> ',f15.2)
#endif
 end

subroutine memstat5(message,typev,val1,val2,val3,val4,val5)
 use dpdef
 implicit none
 integer mem1,mem2
 integer val1,val2,val3,val4,val5
 character (1) typev
 character (*) message
 real valore
#ifdef MPI
 include 'mpif.h'
 integer rang,ierr
 call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif
 call memstatc(mem1,mem2)
 if (typev == 'c') then
  valore = 8.*float(val1)*float(val2)*float(val3)*float(val4)*float(val5)/1024
 elseif (typev == 's') then
  valore = 4.*float(val1)*float(val2)*float(val3)*float(val4)*float(val5)/1024
 elseif (typev == 'd') then
  valore = 8.*float(val1)*float(val2)*float(val3)*float(val4)*float(val5)/1024
 elseif (typev == 'z') then
  valore = 16.*float(val1)*float(val2)*float(val3)*float(val4)*float(val5)/1024
 else ! int
  valore = 2.*float(val1)*float(val2)*float(val3)*float(val4)*float(val5)/1024
 endif

#ifdef MPI
 write(78,20)trim(message),mem1,mem2,mem1+mem2,rang,valore
20 format(a60,I9,' + ',I9,' = ',I9,' kb (proc ',I3,') --> ',f15.2)
#else
 write(78,10)trim(message),mem1,mem2,mem1+mem2,valore
10 format(a60,I9,' + ',I9,' = ',I9,' kb --> ',f15.2)
#endif
 end

!!***
