!{\src2tex{textfont=tt}}
!!****p* main/setmesh
!! NAME
!! setmesh
!!
!! FUNCTION
!! creates the real space Mesh for fft
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!!
!! PARENTS
!! main
!!
!! CHILDREN
!! mesg, memstat, modx, allowed_fftsize
!!
!! NOTE
!! the variable addshell is set to 0. This might not be ok for big q
!!
!! SOURCE
subroutine setmesh(iparam,trspace,fftmesh)
  use verbose
  use ggwdef
  use dpdef
  use interfaces
  implicit none
  type(dpparam) iparam
  type(mesh) fftmesh
  type(transposed_space) trspace
  integer addshell
  integer igfft(iparam%npwvec,-2:2,-2:2,-2:2)
  integer mw(3), mm(3), nfftb(3)
  integer ig, ig01, ig02, ig03, gmg01, gmg02, gmg03,istat
  integer, external :: allowed_fftsize, modx
  integer idum
#ifdef MPI
  include 'mpif.h'
  integer ierr,rang,nprocs
  call mpi_comm_size(mpi_comm_world,nprocs,ierr)
  call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif


  if(iverbose > 2) call uverbose('b','setmesh')

! setup FFT real space Mesh and tables for FFT igfft
  allocate(fftmesh%igfft(iparam%npwvec,5,5,5),stat=istat)
  if(istat.ne.0) stop 'out of memory'

#ifdef MPI
  if(.not.(iparam%mpilocal.and.rang/=0)) then
#endif

  if(iverbose>2) then
   call memstat('igfft allocated =')
  endif


! this should be 0 for the KP_THEORY, but not in any case. to verify
  addshell=0
  
! find the Mesh dimension corresponding to nw=npwwfn
  mw(1) = maxval(abs(trspace%gvec(1,1:iparam%npwwfn)))
  mw(2) = maxval(abs(trspace%gvec(2,1:iparam%npwwfn)))
  mw(3) = maxval(abs(trspace%gvec(3,1:iparam%npwwfn)))

! in case we want to avoid the fft aliasing problem, double it
  if(iparam%alias==2) mw(:) = mw(:) * 2

! find the Mesh dimension corresponding to nm=npwmat (arrival values)
  mm(1) = maxval(abs(trspace%gvec(1,1:iparam%npwmat)))
  mm(2) = maxval(abs(trspace%gvec(2,1:iparam%npwmat)))
  mm(3) = maxval(abs(trspace%gvec(3,1:iparam%npwmat)))
! in case the arrival value is required with +- a g0 to bring it into the 1BZ
  if(addshell==1 .or. addshell==2) mm(:) = mm(:) + addshell

! find the fft Mesh which is -m,...,+m that is 2m+1
  nfftb(1) = 2 * max(mw(1),mm(1)) + 1
  nfftb(2) = 2 * max(mw(2),mm(2)) + 1
  nfftb(3) = 2 * max(mw(3),mm(3)) + 1
! verify if are allowed by the fft library, otherwise chose larger values
  fftmesh%nrr(1) = allowed_fftsize(nfftb(1))
  fftmesh%nrr(2) = allowed_fftsize(nfftb(2))
  fftmesh%nrr(3) = allowed_fftsize(nfftb(3))
  if(iverbose > 0) then
  headline= 'FFT tables setup'
  call print_headline()
  write(uout,*) 'selecting fft mesh size'
  write(uout,*) 'alias = ', iparam%alias
  write(uout,'(" mesh size selected: ",i3," x",i3," x",i3)') fftmesh%nrr(:)
  write(uout,'(" total size, nr = ",i6)') fftmesh%nrr(1)*fftmesh%nrr(2)*fftmesh%nrr(3)
  call print_footline()
  endif

  if(any(mw(:)<mm(:))) then
    call mesg('warning','fft mesh to small: increase wfnsh or npwwfn',idum,idum)
  endif

  do ig01 = -2, 2
    do ig02 = -2, 2
      do ig03 = -2, 2
        do ig = 1, iparam%npwvec
          gmg01 = modx( trspace%gvec(1,ig) - ig01 , fftmesh%nrr(1) )
          gmg02 = modx( trspace%gvec(2,ig) - ig02 , fftmesh%nrr(2) )
          gmg03 = modx( trspace%gvec(3,ig) - ig03 , fftmesh%nrr(3) )
          igfft(ig,ig01,ig02,ig03)=1+gmg01+gmg02*fftmesh%nrr(1)+gmg03*fftmesh%nrr(1)*fftmesh%nrr(2)
       enddo
     enddo
   enddo
  enddo
  fftmesh%igfft=igfft
  fftmesh%nr = fftmesh%nrr(1)*fftmesh%nrr(2)*fftmesh%nrr(3)
#ifdef MPI
  endif
  if(iparam%mpilocal) then
   call mpi_barrier(mpi_comm_world,ierr)
   call mpi_bcast(fftmesh%nr,1,mpi_integer,0,mpi_comm_world,ierr)
   call mpi_bcast(fftmesh%nrr,3,mpi_integer,0,mpi_comm_world,ierr)
   call mpi_bcast(fftmesh%igfft,iparam%npwvec*5*5*5,mpi_integer,0,mpi_comm_world,ierr)
  endif
#endif
  call c3dfft_plan(fftmesh%nrr(1),fftmesh%nrr(2),fftmesh%nrr(3),fftmesh%plan,fftmesh%menoplan)
  if(iverbose > 2) call uverbose('e','setmesh')

end
!!***

