!{\src2tex{textfont=tt}}
!!****p* main/get_drude_dim_trans
!! NAME
!! get_drude_dim_trans
!!
!! FUNCTION
!! Calculate the maximum number of transitions for the Drude contribution
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco
!Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see
!COPYING
!!  or
!http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! SOURCE
 subroutine get_drude_dim_trans(iparam,elecstr,bzstruct,trspace,latt,drude_tr,drude_trans,tablekp)
 use dpdef
 use verbose
 use ggwdef
 implicit none
 type(dpparam) iparam
 type(electronic_structure) elecstr
 type(transposed_space) trspace
 type(bzstructure) bzstruct
 type (lattstruct) latt
 type(transgen) drude_tr
 type(transition) drude_trans(iparam%nt)
 type(polarizationq) tablekp

 double precision fact,fact2
 real o1,o2
 real energyderiv(3)
 double precision diffen
 double precision newen1, newen2, kbt
 integer ik,ib,iq,ig
 integer,allocatable :: it(:)
 integer isppol
 double precision, external :: FermiDirac
 integer :: itrans
 real :: qrot(3), opinv(3,3)

#ifdef MPI
 include 'mpif.h'
 integer ierr,rang,nprocs
 call mpi_comm_size(mpi_comm_world,nprocs,ierr)
 call mpi_comm_rank(mpi_comm_world,rang,ierr)
 if (rang == 0)  then
#endif



 if (iverbose > 2) call uverbose('b','get_drude_dim_trans')
 if (iverbose > 0) then
    write(uout,*) '*** Getting maximum number of transitions for Drude term **'
    write(uout,*)
 endif
#ifdef MPI
 endif
#endif

 allocate (it(iparam%nqpol))
 it (:)=0

!start loop to get the numebr of transition
 do isppol = 1, elecstr%nsppol
   do ik=1,iparam%nkbz

!symmetries not yet implemented check consistency
!    if((bzstruct%ktabi(ik).ne.1).and.(bzstruct%ktab(ik).ne.ik))then
!     write (*,*) 'symmetries not yet implemented in drude calculation'
!     stop
!    endif

    do ib=iparam%lomo, iparam%nbnds

     itrans=ik+iparam%nkbz*(ib-iparam%lomo)+(isppol-1)*iparam%nkbz*(iparam%nbnds-iparam%lomo+1)

    
!cycle on q to count transition allowed
     do iq=1,iparam%nqpol

      diffen=real(drude_tr%rhotwxqpol(itrans,iq))

      newen2=elecstr%gwenergy(ik,ib,isppol)+diffen/2
      newen1=elecstr%gwenergy(ik,ib,isppol)-diffen/2
      kbt=iparam%kBT
      o1=FermiDirac(newen1,elecstr%nsppol,elecstr%fermie,kbt)
      o2=FermiDirac(newen2,elecstr%nsppol,elecstr%fermie,kbt)


      if(elecstr%nsppol==2)then
       fact = o1 * (1.0 - o2)
       fact2 =- o2 * (1.0 - o1)
      else
       fact = 0.25 * o1 * (2.0 - o2)
       fact2 =-0.25 * o2 * (2.0 - o1)
      endif
      if (abs(diffen) < 0.0000001) cycle
      if (abs(fact+fact2)   < 0.00001) cycle
      
      if (iparam%td)then
       it(iq)=it(iq)+1
      else
       it(iq)=it(iq)+2
      endif      
#ifdef MPI
 if (rang==0) then
#endif
      if (iverbose > 2) write(uout,200)ik,ib,iq,it(iq),fact,fact2,diffen
#ifdef MPI
 endif
#endif
     enddo
    enddo !ib 
   enddo !ik
 enddo !isppol  

200 format('ik,ib,iq,it,fact,fact2,diffen = ',3i8,2x,1i6,2f15.6,1x,1e15.3)
!look for maximum number of transition

 iparam%ndtmax=0
 do iq=1,iparam%nqpol
  if (it(iq)>iparam%ndtmax)iparam%ndtmax=it(iq)
 enddo 
 deallocate (it)

#ifdef MPI
 if (rang==0) then
#endif
 if (iverbose > 0) print *,' Maximum number of Drude transitions =',iparam%ndtmax

 if (iverbose > 0) then
 write(uout,*) 
  write(uout,*) '************************************************************'
 endif
 if (iverbose > 2) call uverbose('e','get_drude_dim_trans')
#ifdef MPI
 endif
#endif

 end
!!***




!{\src2tex{textfont=tt}}
!!****p* main/prep_drude_trans
!! NAME
!! get_drude_dim_trans
!!
!! FUNCTION
!! Calculate the maximum number of transitions for the Drude contribution
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco
!Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see
!COPYING
!!  or
!http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! SOURCE
 subroutine prep_drude_trans(iparam,elecstr,transdrude,bzstruct,trspace,latt,drude_tr,tablekp)
 use dpdef
 use verbose
 use ggwdef
 implicit none
 type(dpparam) iparam
 type(electronic_structure) elecstr
 type(transposed_space) trspace
 type(bzstructure) bzstruct
 type(lattstruct) latt
 type(transition) transdrude(iparam%ndtmax,iparam%nqpol)
 type(polarizationq) tablekp

 type(transgen) drude_tr

 real fact,fact2
 real o1,o2, vf
 real diffen
 double precision newen1, newen2, kbt
 integer ik,ib,iq,ig
 integer isppol,io
 double precision, external :: FermiDirac 
 integer :: itrans
 real :: qrot(3), opinv(3,3)

 real :: rhotwxgen(3,iparam%nt)

#ifdef MPI
 include 'mpif.h'
 integer ierr,rang,nprocs
 call mpi_comm_size(mpi_comm_world,nprocs,ierr)
 call mpi_comm_rank(mpi_comm_world,rang,ierr)
 if (rang == 0)  then
#endif

 if (iverbose > 2) call uverbose('b','prep_drude_trans')
 if (iverbose > 0) then
    write(uout,*) '********* Preparing transitions for Drude term *****'
    write(uout,*)
 endif
#ifdef MPI
 endif
#endif

iparam%ndrudet(:)=0

!call mpi_gather(matel,iparam%nt,mpi_real,auxmatel,iparam%nt*iparam%nqpol,mpi_real,0,mpi_comm_world,ierr)
!start loop to get the numebr of transition

!write(78+rang,*)'opinv',drude_tr%opinvgen,rang


 do isppol = 1, elecstr%nsppol
   do ik=1,iparam%nkbz

!symmetries not yet implemented check consistency
!    if((bzstruct%ktabi(ik).ne.1).and.(bzstruct%ktab(ik).ne.ik))then
!     write (*,*) 'symmetries not yet implemented in drude calculation'
!     stop
!    endif

    do ib=iparam%lomo, iparam%nbnds

     itrans=ik+iparam%nkbz*(ib-iparam%lomo)+(isppol-1)*iparam%nkbz*(iparam%nbnds-iparam%lomo+1)

!cycle on q 
     do iq=1,iparam%nqpol

      diffen=real(drude_tr%rhotwxqpol(itrans,iq))

      newen2=elecstr%gwenergy(ik,ib,isppol)+diffen/2
      newen1=elecstr%gwenergy(ik,ib,isppol)-diffen/2
      kbt=iparam%kBT
      o1=FermiDirac(newen1,elecstr%nsppol,elecstr%fermie,kbt)
      o2=FermiDirac(newen2,elecstr%nsppol,elecstr%fermie,kbt)



      if(elecstr%nsppol==2)then
       fact = o1 * (1.0 - o2)
       fact2 =- o2 * (1.0 - o1)
      else
       fact = 0.25 * o1 * (2.0 - o2)
       fact2 =-0.25 * o2 * (2.0 - o1)
      endif
      if (abs(diffen) < 0.0000001) cycle
      if (abs(fact+fact2)  < 0.00001 ) cycle

      if (iparam%td)then
       transdrude(iparam%ndrudet(iq)+1,iq)%k = ik
       transdrude(iparam%ndrudet(iq)+1,iq)%v = ib
       transdrude(iparam%ndrudet(iq)+1,iq)%c = ib
       transdrude(iparam%ndrudet(iq)+1,iq)%s = isppol
       transdrude(iparam%ndrudet(iq)+1,iq)%en = abs(diffen)
       transdrude(iparam%ndrudet(iq)+1,iq)%factocc = fact

       iparam%ndrudet(iq)=iparam%ndrudet(iq)+1
      else
       transdrude(iparam%ndrudet(iq)+1,iq)%k = ik
       transdrude(iparam%ndrudet(iq)+1,iq)%v = ib
       transdrude(iparam%ndrudet(iq)+1,iq)%c = ib
       transdrude(iparam%ndrudet(iq)+1,iq)%s = isppol
       transdrude(iparam%ndrudet(iq)+1,iq)%en = diffen

       transdrude(iparam%ndrudet(iq)+2,iq)%k = ik
       transdrude(iparam%ndrudet(iq)+2,iq)%v = ib
       transdrude(iparam%ndrudet(iq)+2,iq)%c = ib
       transdrude(iparam%ndrudet(iq)+2,iq)%s = isppol
       transdrude(iparam%ndrudet(iq)+2,iq)%en = -diffen

        transdrude(iparam%ndrudet(iq)+1,iq)%factocc = fact+fact2
        transdrude(iparam%ndrudet(iq)+2,iq)%factocc = -1.*(fact+fact2)

      iparam%ndrudet(iq)=iparam%ndrudet(iq)+2

      endif
!      write(83+rang,*)'ib,ik,iq,diffen,fact,fact2=',ib,ik,iq,diffen,fact,fact2
     enddo
    enddo !ib 
   enddo !ik
 enddo !isppol  

if (iparam%ONLYDRUDE) then
allocate(iparam%omega(iparam%nomega))
do io = 1, iparam%nomega
  iparam%omega(io) = cmplx(iparam%omegai + (io-1)*iparam%domega , iparam%broad)
enddo
endif


#ifdef MPI
 if (rang==0) then
#endif
 if (iverbose > 0) then
    write(uout,*)'                         done     '
    write(uout,*)'****************************************************'
    write(uout,*)
 endif 
 if (iverbose > 2) call uverbose('e','prep_drude_trans')

#ifdef MPI
 endif
#endif

 end
!!***


!{\src2tex{textfont=tt}}
!!****p* main/calccomm_drude
!! NAME
!! calcomm_drude 
!!
!! FUNCTION
!! Calculate q=0 contribution for Drude term
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! CHILDREN
!! cfnl, vkbme, rginv, dosym
!!
!! SOURCE
 subroutine calccomm_drude(iparam,tr,trspace,bzstruct,fftmesh,elecstr,latt,symstr,&
                       kb,chemstr,trans)

  use ggwdef
  use dpdef
  use verbose
  use interfaces
  implicit none
  type(dpparam) iparam
  type(kb_potential) kb
  type(transition) trans(iparam%nt)
  type(bzstructure) bzstruct
  type(transposed_space) trspace
  type(electronic_structure) elecstr
  type(lattstruct) latt
  type(mesh) fftmesh
  type(symmetry) symstr
  type(chemistry) chemstr
  type(transgen) tr

  integer ::  nrb
  complex :: wfg1(iparam%npwvec),wfg2(iparam%npwvec)
  real :: opinv(3,3)
  integer :: ikbz, ikibz, iop, iinv,  ibv, ibc 
  integer :: it,iq

  complex, external :: pdtqrc

  real :: qrot(3)
  complex :: rhotwx(3)
  real (kind(0.0d0)) :: ediff
  integer istat
  complex :: rhotwg0
  real q(3)
  integer :: startcycle,stopcycle

  integer :: isppol
  complex :: ct
  integer :: ig

#ifdef MPI
  include 'mpif.h'
  integer ierr,rang,nprocs
  real starttime, endtime
  call mpi_comm_size(mpi_comm_world,nprocs,ierr)
  call mpi_comm_rank(mpi_comm_world,rang,ierr)
  starttime = mpi_wtime()
#endif

  if(iverbose > 2) call uverbose('b','calc_comm')
  if(iverbose > 0) then
     write(uout,*)'********************* Calculating commutator DRUDE **********************'
     write(uout,*)
  endif
  


  nrb = fftmesh%nrr(1) * fftmesh%nrr(2) * fftmesh%nrr(3)

  q(:) = iparam%qpol(:,1)

   if(kb%inclvkb) then
    kb%ng = iparam%npwwfn
    kb%nk = iparam%nkibz
    call cfnl(iparam,latt,trspace,bzstruct,kb,chemstr)
   endif

          
#ifdef MPI
!   call split_trans(1,iparam%nt,startcycle,stopcycle)
   startcycle =1
   stopcycle=iparam%nt
#else 
   startcycle =1
   stopcycle=iparam%nt
#endif


   allocate(tr%rhotwxqpol(startcycle:stopcycle,iparam%nqpol),stat=istat)
   if(iverbose > 2) then
     if (istat/=0) then 
      write(uout,*) 'rhotwxqpol not allocated' 
      stop
     endif
    call memstat('MEM STAT (prep_drude) tr%rhotwxqpol allocated =')
   endif


!   set up rho-twiddle(G=0) using small vector q instead of zero
!   and k.p perturbation theory

   do it=startcycle,stopcycle

    ikbz=trans(it)%k
    ikibz = bzstruct%ktab(ikbz)
    ibv = trans(it)%v
    ibc = trans(it)%c
    isppol = trans(it)%s


    ediff =  trans(it)%en   
    wfg1(:)=elecstr%wfg(:,ibc,ikibz,isppol)
    wfg2(:)=elecstr%wfg(:,ibv,ikibz,isppol)

    rhotwx(:) = 0
!   first term <v,k|-iq*grad|c,k>
    do ig = 1, iparam%npwwfn
     ct = conjg(wfg2(ig)) * wfg1(ig)
      rhotwx(:) = rhotwx(:) + (trspace%gvec(:,ig)+bzstruct%kbz(:,ikbz)) * ct
    enddo

!   second term <c,k|[Vnl,iqr]|v,k>
    if(kb%inclvkb) then
     if(iparam%DRUDE .or. iparam%ONLYDRUDE) then
      kb%DRUDE=.true.
     else
      kb%DRUDE=.false.
     endif
     call vkbme(kb,iparam%npwwfn,iparam%npwvec,wfg1,wfg2,ikibz,rhotwx)
    endif

  
  

!   rhotwg0 = R^-1q * rhotwx_ibz
!   rhotwg0 = -R^-1q * conjg(rhotwx_ibz) for inversion
   iop = bzstruct%ktabo(ikbz)
   iinv = (3-bzstruct%ktabi(ikbz))/2
   opinv(:,:) = symstr%op(:,:,iop)
   call rginv(3,opinv,3)

   do iq = 1, iparam%nqpol
     call dosym(opinv,iinv,iparam%qpol(:,iq),qrot)
     rhotwg0 = pdtqrc(qrot,rhotwx,latt%b1,latt%b2,latt%b3)
     if(iinv == 2) rhotwg0 = conjg(rhotwg0)
     tr%rhotwxqpol(it,iq)=rhotwg0
   enddo


  enddo


   if(iverbose > 0) then
      write(uout,*)'*****************************************************************'
      write(uout,*)
   endif
   if(iverbose > 2) call uverbose('e','calc_comm')
  end 
!!***
