!{\src2tex{textfont=tt}}
!!****p* main/prep_paral_distr
!! NAME
!! prep_paral_distr
!!
!! FUNCTION
!! Distribute the wave functions over processors, according to transitions
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! SOURCE

subroutine prep_paral_distr(iparam,iparamp,trans,tablekp,elecstr,elecstrp,kb)
use dpdef
use ggwdef
use verbose
use interfaces
implicit none
type(dpparam) iparam, iparamp
type(transition) trans(iparam%nt)
type(electronic_structure) elecstr,elecstrp
type(polarizationq) tablekp
type(kb_potential) kb
integer ik,ibv,ibc,it,ikbz,ikibz,ikpibz,isppol,ir,irk
integer startcycle,stopcycle
#ifdef MPI
  include 'mpif.h'
  integer ierr,rang,nprocs
  call mpi_comm_size(mpi_comm_world,nprocs,ierr)
  call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif

iparam%wf_distrib=.not.(iparam%fxcmt .or. iparam%qps .or. iparam%LDA &
       .or. iparam%DRUDE .or. iparam%ONLYDRUDE .or. iparam%doublegrid &
       .or. iparam%exciton .or. .not. iparam%savememory)

allocate(kb%k_count(iparam%nkibz))
if(iparam%wf_distrib) then 
 allocate(elecstr%wf_count(iparam%nkibz,iparam%nbnds,iparam%nsppol))
 allocate(elecstrp%wf_count(iparam%nkibz,iparam%nbnds,iparam%nsppol)) 
 elecstr%wf_count(:,:,:)=0
 elecstrp%wf_count(:,:,:)=0
endif
kb%k_count(:)=0
ir=0
irk=0

if(iverbose>2) call memstat('wf_count allocated =')

#ifdef MPI
  headline="Distribution of wavefunctions among procs"
  call print_headline()
  call split_trans(1,iparam%nt,startcycle,stopcycle)
  if (iverbose>1) write(uout,*)'Transitions on proc ',rang,' from ',startcycle, ' to ', stopcycle
#else
  startcycle =1; stopcycle = iparam%nt
#endif
do it=startcycle,stopcycle
 ikbz = trans(it)%k
 ibv = trans(it)%v
 ibc = trans(it)%c
 ikibz = tablekp%ikibz(ikbz)
 ikpibz=tablekp%ikpibz(ikbz)
 isppol =  trans(it)%s
 if (kb%k_count(ikibz)==0) then
  irk = irk+1
  kb%k_count(ikibz)=irk
 endif
 if (iparam%wf_distrib) then
  if (elecstrp%wf_count(ikpibz,ibv,isppol)==0 .and. elecstr%wf_count(ikpibz,ibv,isppol)==0) then
   ir = ir + 1
   elecstrp%wf_count(ikpibz,ibv,isppol)=ir
  endif
  if(elecstr%wf_count(ikibz,ibc,isppol)==0 .and. elecstrp%wf_count(ikibz,ibc,isppol)==0) then
   ir = ir + 1
   elecstr%wf_count(ikibz,ibc,isppol)=ir
  endif
 endif
enddo
 if (iparam%wf_distrib) then
  elecstr%wf_count(:,:,:) = elecstr%wf_count(:,:,:) + elecstrp%wf_count(:,:,:)
  elecstrp%wf_count(:,:,:) = elecstr%wf_count(:,:,:) 
  elecstr%wfdim=ir
  elecstrp%wfdim=ir
 endif

 kb%kdim=irk

!!!!!!MATTEO TEMP!!!!!!!!
do ikibz=1,iparam%nkibz
 kb%k_count(ikibz)=ikibz
enddo
kb%kdim=iparam%nkibz
!!!!!!!!!!!!!!!!!!!!!!!!!

#ifdef MPI
if (iparam%wf_distrib) then
 if (iverbose>1) then
  write(uout,*) 'Number of k points on proc ', rang, ': ',kb%kdim
  write(uout,*) 'Number of wavefunctions on proc ',rang, ': ', elecstr%wfdim
  if(iparam%doublegrid) write(uout,*)'Number of wavefunctions (starting grid) on proc', rang, ': ',elecstrp%wfdim
 endif
endif
 call print_footline()
#endif


end subroutine prep_paral_distr
!!***
