!{\src2tex{textfont=tt}}
!!****p* main/get_dim_trans
!! NAME
!! get_dim_trans
!!
!! FUNCTION
!! Calculate the number of transitions
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! SOURCE
 subroutine get_dim_trans(iparam,elecstr,elecstrp,tablekp)
 use dpdef
 use verbose
 use ggwdef
 implicit none
 type(dpparam) iparam
 type(electronic_structure) elecstr, elecstrp  ! final and initial states
 type(polarizationq),intent(in)::tablekp

 real fact
 double precision gwten
 integer nt_tmp,topv,botc
 integer ik,iv,ic,ikibz,ikpibz
 integer isppol

 if (iverbose > 2) call uverbose('b','get_dim_trans')
 if (iverbose > 0) then
    headline='Getting total number of transitions'
    call print_headline()
 endif

! blocks dimensioning
! ----------------------------- 
! |    SPIN 1   |    SPIN 2   |
! ----------------------------- 
! | RES  | ARES | RES  | ARES | 
! ----------------------------- 
! each block is dimensioned ntblock(ires,isppol) 
! ires=1 or 2 (1= resonant; 2 coupling)
! nsppol=1 or 2 (spin up / down )


 if(iparam%td) then
   allocate(iparam%ntblock(1,iparam%nsppol))
 else
   allocate(iparam%ntblock(2,iparam%nsppol))
 endif

 iparam%ntblock(:,:) = 0
 iparam%nt = 0
 topv = 1 ; botc =100000
 do isppol = 1, iparam%nsppol
  do ik=1,iparam%nkbz
   ikibz = tablekp%ikibz(ik)
   ikpibz = tablekp%ikpibz(ik)
    do iv = iparam%lomo, iparam%nbnds
      do ic = iparam%lomo, iparam%nbnds
        gwten = elecstr%gwenergy(ikibz,ic,isppol)-elecstrp%gwenergy(ikpibz,iv,isppol)
        if (abs(gwten) < 0.0000001) cycle  ! (Drude peak not yet implemented )
        if(iparam%td .and. gwten < 0.) cycle  ! let's count only the resonant transitions
        if(abs(gwten) > iparam%uvcut .or. abs(gwten) < iparam%ircut) cycle ! to be compatible with the exciton
        if(iparam%nsppol==2)then ! spin polarized
         !fact =  elecstr%oc(ikpibz,iv,isppol) * (1.0 - elecstr%oc(ikibz,ic,isppol))
         fact = elecstrp%oc(ikpibz,iv,isppol) - elecstr%oc(ikibz,ic,isppol)
         else  ! spin unpolarized
         !fact = 0.25 * elecstr%oc(ikpibz,iv,isppol) * (2.0 - elecstr%oc(ikibz,ic,isppol))
         fact = 0.5 * (elecstrp%oc(ikpibz,iv,isppol) - elecstr%oc(ikibz,ic,isppol))
        endif      
        if (abs(fact)  < 0.000001 )  cycle
        iparam%nt = iparam%nt + 1
        if (gwten>0.) then
          iparam%ntblock(1,isppol) = iparam%ntblock(1,isppol)+1
        else
          iparam%ntblock(2,isppol) = iparam%ntblock(2,isppol)+1
        endif
        if (iverbose > 2) write(uout,200) isppol,ikpibz,iv,ikibz,ic,fact,gwten*HARTREE
200 format('is,ikp,iv,ik,ic,fact,gwten = ',5i6,2x,2f12.6)
        ! find top valence and bottom conduction
        if (fact > 0.) then !(in this case iv is "more" occupied than ic -> resonant)
          if (iv > topv) topv = iv
          if (ic < botc) botc = ic
        endif
      enddo
    enddo
  enddo
 enddo

 iparam%homo = topv
 iparam%lumo = botc
 if(iverbose>1) then
   if (iverbose > 2) write(uout,*)
   write(uout,'(a,i3)') "Highest occupied band:  ", iparam%homo
   write(uout,'(a,i3)') "Lowest unoccupied band: ", iparam%lumo
   write(uout,*)
 endif

 iparam%nbndv = iparam%homo - iparam%lomo + 1
 iparam%nbndc = iparam%nbnds - iparam%lumo + 1
 nt_tmp = iparam%nkbz * iparam%nbndv * iparam%nbndc * iparam%nsppol
 if (.not. iparam%td) nt_tmp = nt_tmp*2

!FIXME for exciton:
!for the moment all the blocks have the same size
 iparam%ntres=iparam%ntblock(1,1)  
 iparam%ntspin=sum(iparam%ntblock(:,1))
!!!
 
 if (iparam%td) then
  if(iparam%nt .ne. sum(iparam%ntblock(1,:)) ) &
   stop 'get_dim_trans: tamm-dancoff but total number of transitions not equal to number of resonant transititons'
 else 
  if (mod(iparam%nt,2) .eq. 0) then
   if (sum(iparam%ntblock(1,:)) .ne. sum(iparam%ntblock(2,:)) ) then 
     write(uout,*) 'WARNING number of resonant transitions does not match antiresonant ones:', &
                    sum(iparam%ntblock(1,:)),sum(iparam%ntblock(2,:))
     if (.not. (iparam%nop == 1 .and. iparam%ninv == 1)) stop !stop only for non shifted k points
   endif
  else
   write(uout,*) 'WARNING odd number of transitions: that is odd: ',&
   iparam%nt, '( ', sum(iparam%ntblock(1,:)),' are resonant)'
   if (.not. (iparam%nop == 1 .and. iparam%ninv == 1)) stop ! stop only for non shifted k points
  endif
 endif
 
 if (iparam%fxcmt) then
  if (nt_tmp/=iparam%nt) stop 'get_dim_trans: mapping theory not allowed (for now) with metals'
 endif
 
  if (iverbose > 0) then
     write(uout,'(a,i10,a,i10,a)') 'Total number of transitions:', iparam%nt,' (',sum(iparam%ntblock(1,:)),' are resonant)'
     if(iparam%nsppol ==2) then
       write(uout,'(a,i10,a,i10,a)') 'Number of transitions in spin up channel: ', &
                                   sum(iparam%ntblock(:,1)),' (',iparam%ntblock(1,1),' are resonant)'   
       write(uout,'(a,i10,a,i10,a)') 'Number of transitions in spin down channel: ', &
                                   sum(iparam%ntblock(:,2)),' (',iparam%ntblock(1,2),' are resonant)'   
     endif
    call print_footline()
  endif
  if (iverbose > 2) call uverbose('e','get_dim_trans')

 end 
!!*** 



!{\src2tex{textfont=tt}}
!!****p* main/prep_trans
!! NAME
!! prep_trans
!!
!! FUNCTION
!! Create the transitions object
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! SOURCE
 subroutine prep_trans(iparam,elecstr,elecstrp,trans,tablekp,tableantikp,transtab,gwgap)
 use dpdef
 use ggwdef
 use verbose
 use interfaces
 implicit none
 type(dpparam) iparam
 type(transition) trans(iparam%nt)
 type(electronic_structure) elecstr,elecstrp
 type(polarizationq) tablekp,tableantikp
 
 real fact,enmin, enmax
 double precision gwten, gwgap
 real (kind(0.0d0)) :: ten
 integer :: transtab(iparam%nkbz,iparam%lomo:iparam%nbnds,iparam%lomo:iparam%nbnds,iparam%nsppol)
 
 integer ik,iv,ic,it,ikibz,ikpibz,iantikibz,iantikpibz,io,ivv,icc 
 integer isppol,itpos
 integer, allocatable :: itblock(:,:)


 if (iverbose > 2) call uverbose('b','prep_trans')

 if (iverbose > 0) then
    headline='Preparing transition object'
    call print_headline()
 endif
 if (iverbose > 2)  write(uout,*)

 enmax = 0.0
 enmin = 10000000.0
 gwgap= enmin

! Ordering of array trans(it):
! ----------------------------- 
! |    SPIN 1   |    SPIN 2   |
! ----------------------------- 
! | RES  | ARES | RES  | ARES | 
! ----------------------------- 
! each RES/ARES block is dimensioned iparam%ntblock(:,:)

 if(iparam%td) then
  allocate(itblock(1,iparam%nsppol))
 else
  allocate(itblock(2,iparam%nsppol))
 endif

 it = 0
 itblock(:,:) = 0
 do isppol = 1, iparam%nsppol
  do ik=1,iparam%nkbz
    ikibz = tablekp%ikibz(ik)
    ikpibz = tablekp%ikpibz(ik)
    if(iparam%exciton .and. .not. iparam%td )then 
     iantikibz = tableantikp%ikibz(ik) ! end trans
     iantikpibz = tableantikp%ikpibz(ik) ! start trans
    endif
    do iv = iparam%lomo, iparam%nbnds
      do ic = iparam%lomo, iparam%nbnds
        gwten = elecstr%gwenergy(ikibz,ic,isppol)-elecstrp%gwenergy(ikpibz,iv,isppol)
        ten = elecstr%energy(ikibz,ic,isppol)-elecstrp%energy(ikpibz,iv,isppol)
        if (abs(gwten) < 0.000001) cycle  ! (Drude peak not yet implemented )
        if ((iparam%td .or. iparam%exciton) .and. gwten < 0 ) cycle  ! let's count only the resonant transitions
        if(abs(gwten) > iparam%uvcut .or. abs(gwten) < iparam%ircut) cycle ! to be compatible with the exciton
        if(iparam%nsppol==2)then 
         fact = elecstrp%oc(ikpibz,iv,isppol) - elecstr%oc(ikibz,ic,isppol)
        else
         fact = 0.5 * (elecstrp%oc(ikpibz,iv,isppol) - elecstr%oc(ikibz,ic,isppol))
        endif
        if (abs(fact)  < 0.000001 )  cycle

        if (abs(gwten) < gwgap) gwgap = abs(gwten)

        it = it + 1 

        if(gwten>0) then

         itblock(1,isppol) = itblock(1,isppol)+1
         itpos = itblock(1,isppol) + (isppol-1)*sum(iparam%ntblock(:,1)) 
         trans(itpos)%k = ik
         trans(itpos)%v = iv
         trans(itpos)%c = ic
         trans(itpos)%s = isppol
         trans(itpos)%en = ten
         trans(itpos)%gwen = gwten
         trans(itpos)%factocc = fact
         if (iverbose > 2) write(uout,200) isppol,ikpibz,iv,ikibz,ic,itpos,fact,gwten*HARTREE
         transtab(ik,iv,ic,isppol) = itpos

!only EXCITON 
!        exciton requires itpos for res and antires to be related each other)
         if(iparam%exciton)then 
          if (.not. iparam%td) then 
           it = it+1
           itblock(2,isppol) = itblock(2,isppol)+1
           itpos = itblock(2,isppol) + iparam%ntblock(1,isppol) + (isppol-1)*sum(iparam%ntblock(:,1))
           trans(itpos)%k = ik
           trans(itpos)%v = ic
           trans(itpos)%c = iv
           transtab(ik,ic,iv,isppol) = itpos
           ivv =  trans(itpos)%v    !start trans
           icc =  trans(itpos)%c    !end trans
           gwten = elecstr%gwenergy(iantikibz,icc,isppol)-elecstrp%gwenergy(iantikpibz,ivv,isppol) !end - start
           ten = elecstr%energy(iantikibz,icc,isppol)-elecstrp%energy(iantikpibz,ivv,isppol) 
           trans(itpos)%s = isppol
           trans(itpos)%en =  ten
           trans(itpos)%gwen =  gwten
           trans(itpos)%factocc = - fact
           if (iverbose > 2) write(uout,200) isppol,iantikpibz,ivv,iantikibz,icc,itpos,-fact,gwten*HARTREE
          endif  
        endif
!end only EXCITON

!only DP:
      else  ! if gwten < 0 and not coupling it had already cycled
         itblock(2,isppol) = itblock(2,isppol)+1
         itpos = itblock(2,isppol) + iparam%ntblock(1,isppol) + (isppol-1)*sum(iparam%ntblock(:,1))
         trans(itpos)%k = ik
         trans(itpos)%v = iv
         trans(itpos)%c = ic
         trans(itpos)%s = isppol
         trans(itpos)%en = ten
         trans(itpos)%gwen = gwten
         trans(itpos)%factocc = fact
         if (iverbose > 2) write(uout,200) isppol,ikpibz,iv,ikibz,ic,itpos,fact,gwten*HARTREE
         transtab(ik,iv,ic,isppol) = itpos
       endif 
200 format('is,ikp,iv,ik,ic,it,fact,gwten = ',5i6,2x,i6,2f12.6)
!end only DP
       
!max and min for hilbert or matrixelements
        if (iparam%hilbert .or. iparam%matrixel) then
         gwten = gwten - 2.0 * iparam%broad
         if (enmin.gt.gwten) then
          enmin = gwten 
         endif
         gwten = gwten + 2.0 * iparam%broad
         if (enmax .lt. gwten) then
           enmax = gwten 
         endif
        endif 
      enddo
    enddo
  enddo
 enddo !isppol


 do isppol=1,iparam%nsppol
  if(itblock(1,isppol) .ne. iparam%ntblock(1,isppol) ) then
   write(uout,*) 'prep_trans: ERROR: mismatch in counting res transitions for spin', isppol
   stop 
  endif
  if(.not. iparam%td) then
   if(itblock(2,isppol) .ne. iparam%ntblock(2,isppol) ) then
    write(uout,*) 'prep_trans: ERROR: mismatch in counting antires transitions for spin', isppol
    stop 
   endif
  endif
 enddo



 if(iparam%matrixel) then
   iparam%omegai = enmin
   iparam%omegae = enmax
   iparam%nomega = (iparam%omegae - iparam%omegai) / iparam%domega + 0.5 + 1
   write(uout,*) 'omegai and omegae have been re-defined:'  
   write(uout,'(a,f6.2)') 'omegai=',iparam%omegai
   write(uout,'(a,f6.2)') 'omegae=',iparam%omegae
   write(uout,'(a,i5)') 'nomega=',iparam%nomega
  endif

! allocate frequencies
allocate(iparam%omega(iparam%nomega))
do io = 1, iparam%nomega
  iparam%omega(io) = cmplx(iparam%omegai + (io-1)*iparam%domega , iparam%broad)
enddo


 if (.not. (iparam%FXCMT .or. iparam%pes)) then
  deallocate(elecstr%gwenergy)
  if(iparam%doublegrid) deallocate(elecstrp%gwenergy)
 endif 
 if (.not. (iparam%FXCMT .or. iparam%LDA)) then
  deallocate(elecstr%oc)
  if(iparam%doublegrid) deallocate(elecstrp%oc)
 endif
 deallocate(itblock)

  call print_footline()

  if(iparam%hilbert)then
    write(uout,*)'hilbert method used with  '  
    write(uout,'(a,f6.2)') 'enmin =', enmin
    write(uout,'(a,f6.2)') 'enmax =', enmax
  endif


 if(iverbose>2) then
   call memstat('(prep_trans) transition allocated =  ')
 endif

 if (iverbose > 2) call uverbose('e','prep_trans')

 end
!!*** 
 
