/*
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
      
*/

/* 
file rdcl.c

contains common routines to read command line for arguments
to be passed to programs dp, exc, tau

*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>



read_command_line(argc,argv,inputfile,kssfile,kssfilep,gwfile,gwfilep,qpsfile,qpsfilep,scrfile,vkbfile,lambdafile,root)
int argc;
char *argv[];
char **inputfile, **kssfile, **kssfilep, **gwfile, **qpsfile, **gwfilep, **qpsfilep,  **scrfile,  **vkbfile;
char **lambdafile, **root;
{
  int i;
  char *program, *control;
  
  program = argv[0];
  while(--argc>0) {
    argv++;
    if(*argv[0]!='-') {
      /* default argument: kss file */
      *kssfile = *argv;
    } else {
      i=0;
      control = *argv;
      while(control[++i]!='\0') {
        switch(control[i]) {
	  case 'i':
	    if(--argc<0) error_missing_argument(program);
	    *inputfile = *++argv;
	    break;
          case 'k':
	    if(--argc<0) error_missing_argument(program);
	    if(*kssfile==NULL)  *kssfile = *++argv; /* std. kss*/
            else if(*kssfilep==NULL) *kssfilep = *++argv;/* opt. kss for kp=k-qpol*/
            else fprintf(stderr,"WARNING: Ignoring file %s.\n",*++argv);
	    break;
	  case 'g':
	    if(--argc<0) error_missing_argument(program);
 	    if (*gwfile==NULL) *gwfile = *++argv;        /* std. gw*/
            else if (*gwfilep==NULL) *gwfilep = *++argv; /* opt. gw for kp=k-qpol*/          
 	    break;
	  case 'q':
	    if(--argc<0) error_missing_argument(program);
            if (*qpsfile==NULL) *qpsfile = *++argv;        /* std. qps*/
            else if (*qpsfilep==NULL) *qpsfilep = *++argv; /* opt. qps for kp=k-q*/
            break;
	  case 's':
	    if(--argc<0) error_missing_argument(program);
            *scrfile = *++argv;
            break;           
	  case 'e':
	    if(--argc<0) error_missing_argument(program);
            *scrfile = *++argv;
            break; 
	  case 'v':
	    if(--argc<0) error_missing_argument(program);
	    *vkbfile = *++argv;
 	    break;
          case 'l':
	    if(--argc<0) error_missing_argument(program);
	    *lambdafile = *++argv;
 	    break;
	  case 'r':
	    if(--argc<0) error_missing_argument(program);
	    *root = *++argv;
 	    break;
	  default:
            error_option(program,control[i]);
        }
      }
    }
  }
  return(1);    
}

check_file(filename,defaultname,program)
char *program, **filename, *defaultname;
/* return file length if filename or defaultname exists, otherwise 0 */ 
{
  int nl;
  FILE *fp;
  if(*filename!=NULL) {
    nl = strlen(*filename);
    fp = fopen(*filename,"r");
    if(fp==NULL) error_file(program,*filename);
    fclose(fp);
  } else {
    *filename = defaultname; /* default */
    nl = strlen(*filename);
    fp = fopen(*filename,"r");
    if(fp!=NULL) close(fp);
    else *filename=NULL, nl=0;
  }
  return(nl);
}

error_option(program,c)
char *program;
char c;
/* print on stderr option error */
{
  fprintf(stderr,"%s: illegal option --%c\n",program,c);
  usage(program); 
  exit(2);
}

error_missing_argument(program)
char *program;
/* print on stderr missing argument error */
{
  fprintf(stderr,"%s: missing argument\n",program);
  usage(program); 
  exit(3);
}

error_file(program,filename)
char *program, *filename;
/* print on stderr missing file */
{
  fprintf(stderr,"%s: not existing or not readable file %s\n",
    program,filename);
  usage(program);
  exit(4);
}

usage(program)
char *program;
/* print on stderr usage options */
{
  fprintf(stderr,
    "usage: %s [-i inputfile] [-k kssfile] [-g gwfile]  [-q qpsfile] [-s scrfile] \n",
    program); 
  fprintf(stderr,"obsolescences: [-k qpldafile] [-v vkbfile] [-e em1file]\n");
  fprintf(stderr,"default: %s.in in.kss in.gw in.qps in.scr\n",program);
}  
