!{\src2tex{textfont=tt}}
!!****p* main/read_structure
!! NAME
!! Read Kohn-Sham Structure
!!
!! FUNCTION
!! This is the interface between the main and the subroutines reading the kss files 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main, get_initial_states
!!
!! CHILDREN
!! memstat, testlda, testscr, mesg, testvar, print_parameters, rdlda, rdkss, rdscr,
!! testvkb, rdvkb, bcast_kss_data, bcast_kb_parameters, bcast_input_parameters
!!
!! TODO 
!! Ideally here should appear in some form the definitive netcdf reader.
!!
!! SOURCE


subroutine read_structure(iparam,files,chemstr,symstr,bzstruct,trspace,elecstr,screen,latt,kb)

use dpdef
use ggwdef
use verbose
use interfaces

implicit none

type(dpparam) iparam
type(dpfiles) files
type(chemistry) chemstr
type(symmetry) symstr
type(bzstructure) bzstruct 
type(electronic_structure)elecstr
type(screening)screen
type(transposed_space) trspace
type(lattstruct) latt
type(kb_potential) kb
integer ngr, nbr, nkr, nelr, nsppol
integer i1,i2,i3,istat,ia, idum
character (len=80) :: title(2)

!exciton
integer fform
integer nqr,nomegaem1,npwmate,npwwfne,nbndse
!end exciton


#ifdef MPI
include 'mpif.h'
integer ik,ib,isppol
integer ierr,rang,nprocs
call mpi_comm_size(mpi_comm_world,nprocs,ierr)
call mpi_comm_rank(mpi_comm_world,rang,ierr)

call mpi_barrier(mpi_comm_world,ierr) 
!call mpi_bcast(uout,1,mpi_integer,0,mpi_comm_world,ierr)

#endif

if(iverbose>2) call uverbose('b','read_structure')

if(iverbose>2) call memstat('input file read =')


!read parameters of the kss (or old qplda) file
call testlda(iparam,files%kss,files%kl,nkr,nbr,nsppol,ngr,i1,i2, & 
             kb,nelr,elecstr)

!exciton
!read parameters of scr file
if (iparam%exc) then
 call testscr(files%scr,files%sl,nqr,nomegaem1,npwmate,npwwfne,nbndse, &
             fform,iparam%mpilocal)
else
 nqr = -1
endif
!end exciton

if(iparam%qps .or. iparam%qpsenergy) call testqps(iparam,files%qps,files%ql,nkr,nbr,nsppol)

#ifdef MPI
!in case only master reads
!bcast the input parameters from kss/scr to other procs
if(iparam%mpilocal) then
 call bcast_input_parameters(iparam)
 call bcast_kb_parameters(kb)
endif
#endif

call testvar(iparam,nkr,nbr,nsppol,ngr,nelr,nqr)

call print_parameters(iparam)

allocate(symstr%op(3,3,iparam%nopr),stat=istat)
if(istat.ne.0) stop 'out of memory'
allocate(bzstruct%kibz(3,iparam%nkibz),stat=istat)
if(istat.ne.0) stop 'out of memory'
allocate(trspace%gvec(3,iparam%npwvec),stat=istat)
if(istat.ne.0) stop 'out of memory'
allocate(elecstr%oc(iparam%nkibz,iparam%nbnds,iparam%nsppol),stat=istat)
if(istat.ne.0) stop 'out of memory'

if (iparam%LDA .or. iparam%qps .or. iparam%plot_exciton .or. iparam%analysis_exciton) then 
 iparam%nbndmin = 1
else
 iparam%nbndmin = iparam%lomo
endif   
if (iparam%qps .or. iparam%qpsenergy) then
 iparam%nbndmax = max(iparam%nbnds,iparam%nbndqps)
else
 iparam%nbndmax = iparam%nbnds
endif

allocate(elecstr%energy(iparam%nkibz,iparam%nbndmax,iparam%nsppol),stat=istat) 
if(istat.ne.0) stop 'out of memory'

!read kss file
 kb%inclvkb = iparam%vkb
if(kb%exists) then 

 allocate(chemstr%typeat(kb%natom),chemstr%xred(3,kb%natom),chemstr%xcart(3,kb%natom))
 if (.not.iparam%JDOS .and. kb%inclvkb) allocate(kb%vkbsign(kb%mpsang,kb%ntype))
 if (.not.iparam%JDOS .and. kb%inclvkb) allocate(kb%vkb(iparam%npwvec,kb%ntype,kb%mpsang,iparam%nkibz),stat=istat)
 if(iverbose>2) call memstat('vkb allocated =','c',iparam%npwvec,kb%ntype,kb%mpsang,iparam%nkibz)
 if(istat.ne.0) stop 'out of memory'
 if (.not.iparam%JDOS .and. kb%inclvkb) allocate(kb%vkbd(iparam%npwvec,kb%ntype,kb%mpsang,iparam%nkibz),stat=istat)
 if(istat.ne.0) stop 'out of memory'
 if(iverbose>2) call memstat('vkbd allocated =','c',iparam%npwvec,kb%ntype,kb%mpsang,iparam%nkibz)

 call rdkss(files%kss,files%kl,iparam,title,latt,symstr,trspace,bzstruct,elecstr,&
            kb,chemstr)

 if(iverbose>2) call memstat('kss file read (but no wfg) =')
 if(.not.iparam%vkb) call mesg('warning','non local pseudo-potential neglected',idum,idum)

#ifdef MPI
!in case only master reads
 if (iparam%mpilocal .and. rang == 0) then
#endif

deallocate(chemstr%znucl)

#ifdef MPI
 endif
!in case only master reads
 if (iparam%mpilocal) &
 &  call bcast_kss_data(iparam,latt,chemstr,kb,symstr,bzstruct,elecstr,trspace)

#endif

 do ia = 1, kb%natom
   chemstr%xcart(:,ia) = chemstr%xred(1,ia)*latt%a1(:) + &
                         chemstr%xred(2,ia)*latt%a2(:) + &
                         chemstr%xred(3,ia)*latt%a3(:)
 enddo

 deallocate(chemstr%xred)


endif !kb%exists


! read old & deprecated files
if (iparam%oldqplda) then
call read_oldformat_file(files,iparam,nbr,nkr,ngr,title,i1,i2,i3,latt,&
                         symstr,trspace,bzstruct,elecstr,chemstr,kb)
 if (iparam%mpilocal) &
 &  call bcast_kss_data(iparam,latt,chemstr,kb,symstr,bzstruct,elecstr,trspace)

endif

!exciton
if (iparam%exc) then
 screen%nqibz=iparam%nqibz
!read screening file (unit 12) for q-points and parameters
 allocate(screen%qibz(3,screen%nqibz),stat=istat)
 if(istat.ne.0) stop 'out of memory'
 allocate(screen%epsm1q(iparam%npwmat,iparam%npwmat,screen%nqibz),stat=istat)
 if(istat .ne. 0) stop 'out of memory: epsm1q'

 if(iverbose>2) call memstat('screening allocated =')

 if(fform==1002) then
  screen%nomega=1
  call rdscr(files%scr,files%sl,iparam%npwmat,screen%nqibz,screen%nomega,screen%qibz, &
             screen%omega,screen%epsm1q,iparam%mpilocal)
  if(abs(screen%omega(1))>0.0001) stop 'need frequency 0'
 else if(fform==1003) then
  allocate(screen%bigomegatwsq(iparam%npwmat,iparam%npwmat,screen%nqibz),stat=istat)
  if(istat .ne. 0) stop 'out of memory: epsm1q'
  allocate(screen%omegatw(iparam%npwmat,iparam%npwmat,screen%nqibz),stat=istat)
  if(istat .ne. 0) stop 'out of memory: epsm1q'
  call rdepsm1(files%scr,files%sl,iparam%npwmat,screen%nqibz,screen%qibz,screen%epsm1q, &
               screen%bigomegatwsq,screen%omegatw,iparam%mpilocal)
  deallocate(screen%bigomegatwsq,screen%omegatw)
 endif

 if(iverbose>2) call memstat('screening file read =')

 if(iparam%dielconstscreen) then
  screen%epsm1q(:,:,:) = 1.0 / iparam%renormdielconst
  write(uout,*) 'WARNING: constant static dielectric screening'
  write(uout,957) iparam%renormdielconst
957  format(' renormalized dielectric constant = ',f5.2)
 endif 

#ifdef MPI
 if(iparam%mpilocal) then
  call mpi_bcast(screen%qibz,3*screen%nqibz,mpi_real,0,mpi_comm_world,ierr)
  call mpi_bcast(screen%epsm1q,screen%nqibz*iparam%npwmat**2,mpi_complex,0,mpi_comm_world,ierr)
 endif
#endif
endif !exciton

#ifdef MPI
if (iparam%mpilocal .and. rang==0) then
#endif
 if (.not.iparam%oldqplda) then
  deallocate(chemstr%psa)
  deallocate(elecstr%occ)
 endif
#ifdef MPI
endif
#endif

if(iverbose > 2)call uverbose('e','read_structure')


end
!!***


!{\src2tex{textfont=tt}}
!!****p* read_structure/read_oldformat_file
!! NAME
!! Read Kohn-Sham Structure - very old formats
!!
!! FUNCTION
!! This is the interface between the main and the subroutines reading the kss files 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! read_structure
!!
!! CHILDREN
!! rdlda,testvkb,rdvkb
!!
!! TODO 
!! Ideally here should appear in some form the definitive netcdf reader.
!!
!! SOURCE


subroutine read_oldformat_file(files,iparam,nbr,nkr,ngr,title,i1,i2,i3,latt,&
                               symstr,trspace,bzstruct,elecstr,chemstr,kb)


use dpdef
use ggwdef
use verbose

implicit none

type(dpparam) iparam
type(dpfiles) files
type(chemistry) chemstr
type(symmetry) symstr
type(bzstructure) bzstruct 
type(electronic_structure)elecstr
type(transposed_space) trspace
type(lattstruct) latt
type(kb_potential) kb
integer ngr, nbr, nkr
integer i1,i2,i3,istat,idum
character (len=80) :: title(2)

!read old qplda file
if(i2==0.or.i2==1) then 
! file unformatted -> i1=1
 i1 = 1
 call rdlda(files%kss,files%kl,iparam,nbr,nkr,ngr,title,i1,i2,i3,latt,symstr,trspace,bzstruct,elecstr)
 files%i1=i1
endif

if(i2==0.or.i2==1.or.i2==2) then !old qplda
 if(.not.kb%inclvkb) call mesg('warning','non local pseudo-potential neglected',idum,idum)
 if(kb%inclvkb)inquire(file=files%vkb,exist=kb%inclvkb)
 if(kb%inclvkb) then
  call testvkb(files%vkb,files%vl,iparam%npwvec,iparam%nkibz,kb%ntype,kb%natom,kb%mpsang)
 endif
endif
 if(kb%inclvkb) then

  allocate(chemstr%typeat(kb%natom),chemstr%xcart(3,kb%natom))
  allocate(kb%vkbsign(kb%mpsang,kb%ntype))
  allocate(kb%vkb(iparam%npwvec,kb%ntype,kb%mpsang,iparam%nkibz),stat=istat)
  if(istat.ne.0) stop 'out of memory'
  allocate(kb%vkbd(iparam%npwvec,kb%ntype,kb%mpsang,iparam%nkibz),stat=istat)
  if(istat.ne.0) stop 'out of memory'
 endif
 if(kb%inclvkb) then

  call rdvkb(files%vkb,files%vl,iparam%npwvec,iparam%nkibz,kb%ntype,kb%natom,kb%mpsang,chemstr%typeat,& 
             chemstr%xcart,kb%vkbsign,kb%vkb,kb%vkbd)

 endif

end
!!***
