!{\src2tex{textfont=tt}}
!!****p* main/cden
!! NAME
!! cden
!!
!! FUNCTION
!! Calculate the electron density
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! CHILDREN
!! c3dfft
!!
!! SOURCE

  subroutine cden(iparam,fftmesh,rot,latt,bzstruct,elecstr,lda)

  use verbose
  use dpdef
  use ggwdef
  implicit none
  type(dpparam) iparam
  type(ldakernel) lda
  type(mesh) fftmesh
  type(bzstructure) bzstruct
  type(electronic_structure) elecstr
  type(lattstruct) latt
  type(rot_table) rot
  real rho(fftmesh%nr,iparam%nsppol)
  integer ikibz, ig, ir, ib, iop, istat
  integer isppol
  complex, allocatable :: wfr1(:)
  
  real fact

  real tnepuc, rhoav, rs, omegaplasma, tnepucup
  
#ifdef MPI
  include 'mpif.h'
  integer ierr,rang,nprocs
  real, allocatable :: rho_tmp(:,:)
  call mpi_comm_size(mpi_comm_world,nprocs,ierr)
  call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif


  if(iverbose > 2) call uverbose('b','cden')
  if(iverbose > 0) then
   headline='Calculating charge density'
   call print_headline()
  endif

  if(iparam%savememory) allocate(wfr1(fftmesh%nr))
  allocate(lda%rho2(fftmesh%nr,iparam%nsppol),stat=istat)
  if(istat.ne.0) stop 'out of memory'

#ifdef MPI
  if (.not.(iparam%mpilocal .and. rang /= 0))  then
#endif


  rho(:,:) = 0.0
  lda%rho2(:,:) = 0.0

 do isppol = 1, iparam%nsppol
  do ikibz = 1, iparam%nkibz
    do ib = 1, iparam%nbnds
      if (elecstr%oc(ikibz,ib,isppol)< 1.e-5) exit
      if(iparam%savememory) then
        wfr1(:)=0.0
         wfr1(fftmesh%igfft(:,3,3,3)) = elecstr%wfg(:,ib,ikibz,isppol)
        call c3dfft(wfr1,fftmesh%nrr(1),fftmesh%nrr(2),fftmesh%nrr(3),& 
                 fftmesh%nrr(1),fftmesh%nrr(2),fftmesh%nrr(3),1,fftmesh%plan)
        rho(:,isppol) = rho(:,isppol) + elecstr%oc(ikibz,ib,isppol) * bzstruct%wtk(ikibz) * & 
                 conjg(wfr1(:)) * wfr1(:)
      else 
        rho(:,isppol) = rho(:,isppol) + elecstr%oc(ikibz,ib,isppol) * bzstruct%wtk(ikibz) * & 
                 conjg(elecstr%wfr(:,ib,ikibz,isppol)) * elecstr%wfr(:,ib,ikibz,isppol)
      endif
    enddo
  enddo
 enddo  

! loop over symmetry operations, symmetrising rho
  if(iparam%ninv == 1) then
    fact = 1.0 / (iparam%nkbz * latt%ucvol)
  else
!   factor of 2 is for inversion
    fact = 2.0 / (iparam%nkbz * latt%ucvol)
  endif
  do isppol = 1, iparam%nsppol
   do iop = 1, iparam%nop   
     do ir = 1, fftmesh%nr
       lda%rho2(ir,isppol)=lda%rho2(ir,isppol)+fact*rho(rot%irottb(ir,iop),isppol)
     enddo
   enddo
  enddo


! calculate total number of electrons as a check
  tnepuc = 0.0
  do isppol = 1, iparam%nsppol
   do ir = 1, fftmesh%nr
     tnepuc = tnepuc + lda%rho2(ir,isppol)
   enddo
   if(iparam%nsppol == 2) then
    if(isppol == 1) then
      tnepucup=tnepuc   
      write(uout,1111) tnepucup  * latt%ucvol / (fftmesh%nrr(1)*fftmesh%nrr(2)*fftmesh%nrr(3))
1111  format(' number of spin up electrons per unit cell   = ' ,f7.4)
    else
      write(uout,1112) (tnepuc-tnepucup)  * latt%ucvol / (fftmesh%nrr(1)*fftmesh%nrr(2)*fftmesh%nrr(3))
1112  format(' number of spin down electrons per unit cell = ' ,f7.4)    
    endif 
   endif
  enddo
  tnepuc = tnepuc * latt%ucvol / (fftmesh%nrr(1)*fftmesh%nrr(2)*fftmesh%nrr(3))
  if (abs(iparam%nel - tnepuc) > 0.001) then
   write(uout,*) 'WARNING: number of electrons in rho',tnepuc,' while in kss',iparam%nel
  endif
  rhoav = tnepuc / latt%ucvol
  rs = (3.0/(4.0*(4.0*atan(1.0))*rhoav))**(1.0/3.0)
  omegaplasma = sqrt(4.0 * (4.0*atan(1.0)) * rhoav) 
  if(iverbose > 0) then
  write(uout,1110) tnepuc
1110  format (' total number of electrons per unit cell = ',f7.4)
  write(uout,1115) rhoav
1115  format(' average of density, n = ',f7.4)
  write(uout,1120) rs
1120  format(' r_s = ',f7.4)
  write(uout,1125) omegaplasma*HARTREE
1125  format(' omega_plasma = ',f7.4, '[eV]')
   call print_footline()
  endif

  deallocate(rot%irottb)
  deallocate(elecstr%oc)

  if(iparam%savememory) deallocate(wfr1)

  if(iverbose > 2) call uverbose('e','cden')


#ifdef MPI
  endif
#endif

  deallocate(bzstruct%wtk)

  end
!!***
