!{\src2tex{textfont=tt}}
!!****p* main/surot
!! NAME
!! surot
!!
!! FUNCTION
!! set up tables indicating rotations of r-points and g-vectors
!! 
!! NOTE
!! irottb(r,r) contains the index no. of r**-1 r [note inverse!]
!!
!! grottb(g,i,r) contains the index no. of r i g
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!!
!! PARENTS
!! main
!!
!! CHILDREN
!! mesg, dosymr, dosym, modx
!!
!! SOURCE

  subroutine surot(iparam,symstr,fftmesh,trspace,rot)

  use verbose
  use dpdef
  use ggwdef
  implicit none
  type(dpparam) iparam
  type(rot_table) rot
  type(mesh) fftmesh
  type(symmetry) symstr
  type(transposed_space) trspace
  integer nfft(3)
  integer istat, ix, iy, iz, ir, iop, iinv, jx, jy, jz, ig, igp 
  integer ifound
  real r(3), rrot(3), gx(3), gxx(3)
  integer, external :: modx
#ifdef MPI
  include 'mpif.h'
  integer ierr,rang,nprocs
  call mpi_comm_size(mpi_comm_world,nprocs,ierr)
  call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif
  if(iverbose > 2) call uverbose('b','surot')

  allocate(rot%irottb(fftmesh%nr,iparam%nopr),stat=istat)
  if(istat.ne.0) stop 'irottb out of memory'

#ifdef MPI
  if (.not.(iparam%mpilocal .and. rang /= 0))  then
#endif

  if(iverbose > 1) then
   headline="Setting up r- and g-rotation tables"
   call print_headline()
  endif

  nfft(1) = fftmesh%nrr(1)
  nfft(2) = fftmesh%nrr(2)
  nfft(3) = fftmesh%nrr(3)

  do ix = 0, fftmesh%nrr(1)-1
   do iy = 0, fftmesh%nrr(2)-1
    do iz = 0, fftmesh%nrr(3)-1

     r(1) = ix
     r(2) = iy
     r(3) = iz
     ir = 1 + ix + iy*fftmesh%nrr(1) + iz*fftmesh%nrr(1)*fftmesh%nrr(2)

     do iop = 1, iparam%nopr
!     form r**-1 r
      call dosymr(symstr%op(1,1,iop), 1, r, nfft, rrot)
      jx = modx(nint(rrot(1)),fftmesh%nrr(1))
      jy = modx(nint(rrot(2)),fftmesh%nrr(2))
      jz = modx(nint(rrot(3)),fftmesh%nrr(3))
      rot%irottb(ir,iop) = 1 + jx + jy*fftmesh%nrr(1) + jz*fftmesh%nrr(1)*fftmesh%nrr(2)
     enddo
    enddo
   enddo
  enddo

! ensure that any spare space in the fft array (because of
! nfft1a possible being one greater than nfft1) is always
! rotated onto itself so that no harm is done.  note that the
! do-loop is not executed if nfft1a=nfft1
  do ix = fftmesh%nrr(1), fftmesh%nrr(1)-1
   do iy = 0, fftmesh%nrr(2)-1
    do iz = 0, fftmesh%nrr(3)-1
     ir = 1 + ix + iy*fftmesh%nrr(1) + iz*fftmesh%nrr(1)*fftmesh%nrr(2)
     do iop = 1, iparam%nopr
      rot%irottb(ir,iop) = ir
     enddo
    enddo
   enddo
  enddo

 if(iverbose > 1) then
   write(uout,*)' r-rotation tables set up'
 endif

!set up g-rotation table
 if(iparam%FXCMT .or. iparam%exciton) then
  allocate(rot%grottb(iparam%npwvec,2,iparam%nopr),stat=istat)
  if(istat.ne.0) stop 'grottb out of memory'
 loop1: do ig = 1, iparam%npwvec
  gx(1) = trspace%gvec(1,ig)
  gx(2) = trspace%gvec(2,ig)
  gx(3) = trspace%gvec(3,ig)
  do iinv = 1, iparam%ninvq
   do iop = 1, iparam%nopr
    call dosym(symstr%op(1,1,iop),iinv,gx,gxx)
    ifound = 0
    do igp = 1, iparam%npwvec
     if (abs(gxx(1)-trspace%gvec(1,igp)).lt.1.0e-3) then
      if (abs(gxx(2)-trspace%gvec(2,igp)).lt.1.0e-3) then
       if (abs(gxx(3)-trspace%gvec(3,igp)).lt.1.0e-3) then
        ifound = 1
        rot%grottb(ig,iinv,iop) = igp
       endif
      endif
     endif
    enddo
    if (ifound.eq.0) then
     call mesg('shell','plane waves',iparam%npwwfn, ig-1)
     iparam%npwwfn = ig - 1 
     iparam%npwvec = iparam%npwwfn
     if(iparam%npwmat>iparam%npwvec) then
       call mesg('shell','matrix',iparam%npwmat,iparam%npwvec)
       iparam%npwmat = iparam%npwvec
     endif
     exit loop1
!    write(uout,*) ((trspace%gvec(i,j), i=1,3), j=1,iparam%npwvec)
!     stop
    endif
   enddo
  enddo
 enddo loop1
 
  if(iverbose > 1) then
   write(uout,*)' g-rotation tables set up', ig,igp
  endif

 endif !exciton/fxcmt


 call print_footline()

#ifdef MPI
  endif
 if(iparam%mpilocal) then
  call mpi_barrier(mpi_comm_world,ierr)
  call mpi_bcast(iparam%npwvec,1,mpi_integer,0,mpi_comm_world,ierr)
  call mpi_bcast(iparam%npwmat,1,mpi_integer,0,mpi_comm_world,ierr)
  call mpi_bcast(iparam%npwwfn,1,mpi_integer,0,mpi_comm_world,ierr)
  call mpi_bcast(rot%irottb,fftmesh%nr*iparam%nopr,mpi_integer,0,mpi_comm_world,ierr)
  if(iparam%FXCMT .or. iparam%exciton) then
    if(rang /= 0) allocate(rot%grottb(iparam%npwvec,2,iparam%nopr))
    call mpi_bcast(rot%grottb,iparam%npwvec*iparam%nopr,mpi_integer,0,mpi_comm_world,ierr)
  endif
 endif
#endif

 if(iverbose > 2) call uverbose('e','surot')

 end
!!***


!{\src2tex{textfont=tt}}
!!****p* surot/dosymr
!! NAME
!! dosymr
!!
!! FUNCTION
!! perform an inverse symmetry operation on a real-space vector in fft units
!! 
!! NOTE
!!  if the op matrices are in reciprocal-lattice units then
!!  since the inverse-transpose of the op matrix is the
!!  corresponding matrix in real-space-lattice units, nfft(i)/nfft(j)
!!  times the transpose is the matrix corresponding to the inverse
!!  operation in fft units.  thus calling dosymr with op and r1 performs
!!  the operation r2 = r**-1 r1 if op corresponds to r.
!!  iinv is 1 for non-inversion, 2 for inversion                          
!!
!! COPYRIGHT 
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!!
!! PARENTS
!! surot
!!
!! SOURCE

 subroutine dosymr(op, iinv, r1, nfft, r2)

 implicit none 
 integer i, j, iinv, imult
 real r1(3), r2(3), op(3,3)
 integer nfft(3)

 do i=1,3
   r2(i) = 0.0
 enddo

 imult = 3 - 2 * iinv
 do j=1,3
   do i=1,3
     r2(i) = r2(i) + imult * op(j,i) * nfft(i) * r1(j) / float(nfft(j))
   enddo
 enddo

 end
!!***


!{\src2tex{textfont=tt}}
!!****p* surot/dosym
!! NAME
!! dosym
!!
!! FUNCTION
!! perform a symmetry operation k2 = ri k1
!!
!! NOTE
!! if the op matrices are in reciprocal-lattice units then the vectors
!! must be too.
!! iinv is 1 for non-inversion, 2 for inversion                        
!!
!! COPYRIGHT 
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!!
!! PARENTS
!! surot, creation_chi0, identk, identq, create_eps_exc, findnq, findq, prep_rhotw
!!
!! SOURCE

 subroutine dosym(op, iinv, k1, k2)

 implicit none
 integer i, j, iinv, imult
 real k1(3), k2(3), op(3,3)

 do i = 1, 3
   k2(i) = 0.0
 enddo

 imult = 3 - 2 * iinv
 do j = 1, 3
   do i = 1, 3
     k2(i) = k2(i) + imult * op(i,j) * k1(j)
   enddo
 enddo

 end
!!***
