!{\src2tex{textfont=tt}}
!!****p* creation_chi0/split_trans
!! NAME
!! split_trans
!!
!! FUNCTION
!! split transition number (or any other quantity) through procs 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_chi0, creation_chi0drude, creation_epsrpa, creation_epslda, creation_epsalt
!! creation_kernelmt
!!
!! SOURCE 
subroutine split_trans(nstart,nend,istart,iend)

#ifdef OMP
!$ use OMP_LIB
#endif
implicit none

integer nstart, nend, nprocs, rang, istart, iend
integer i1, i2, i3

#ifdef MPI
 integer ierr
 include 'mpif.h'
 call mpi_comm_size(mpi_comm_world,nprocs,ierr)
 call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif
#ifdef OMP
 nprocs = OMP_GET_NUM_THREADS()
 rang = OMP_GET_THREAD_NUM()
#endif


i1     = nend - nstart + 1
i2     = i1/nprocs
i3     = mod(i1,nprocs)
istart = rang*i2 + nstart + min(rang,i3)
iend   = istart + i2 - 1
if(i3.gt.rang) iend = iend + 1

! print *,'rang,nprocs,startcycle,stopcycle= ',rang,nprocs,istart,iend
end subroutine split_trans
!!***

!{\src2tex{textfont=tt}}
!!****p* creation_chi0/split_trans_long
!! NAME
!! split_trans_long
!!
!! FUNCTION
!! split transition number (or any other quantity) through procs 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! creation_chi0, creation_chi0drude, creation_epsrpa, creation_epslda, creation_epsalt
!! creation_kernelmt
!!
!! SOURCE
subroutine split_trans_long(nstart,nend,istart,iend)

use dpdef
#ifdef OMP
!$ use OMP_LIB
#endif

implicit none

integer(ILONG) :: nstart, nend, istart, iend
integer   nprocs, rang
integer(ILONG) :: nprocs_long, rang_long
integer(ILONG) :: i1, i2, i3

#ifdef MPI
 integer ierr
 include 'mpif.h'
 call mpi_comm_size(mpi_comm_world,nprocs,ierr)
 call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif
#ifdef OMP
 nprocs = OMP_GET_NUM_THREADS()
 rang = OMP_GET_THREAD_NUM()
#endif

nprocs_long = nprocs
rang_long = rang
i1     = nend - nstart + 1
i2     = i1/nprocs_long
i3     = mod(i1,nprocs_long)
istart = rang_long*i2 + nstart + min(rang_long,i3)
iend   = istart + i2 - 1
if(i3.gt.rang_long) iend = iend + 1

end subroutine split_trans_long
!!***

