!{\src2tex{textfont=tt}}
!!****p* rdhdr/setup_symmetry_structure
!! NAME
!! Setup symmetry structure
!!
!! FUNCTION
!! Setup symmetry structure
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! rdhdr
!!
!! CHILDREN
!! mtriinv,print_symmetry_structure
!!
!! SOURCE

subroutine setup_symmetry_structure(symstr)
  use ggwdef
  use dpdef
  use verbose
  implicit none
  type(symmetry) symstr
  integer i, j, nsym
  logical, parameter :: onlysim = .false.
  logical, parameter :: intro_time_rev = .false.

  if(iverbose > 2) call uverbose('b','setup_symmetry')
  if(onlysim) then  ! remove non-simmorphic
    nsym = symstr%nsym
    i = 1
    do while(i <= nsym)
      if(any(abs(symstr%sym(i)%t(:)) > 0.0001)) then
        do j = i+1, nsym
          symstr%sym(j-1)%t(:) = symstr%sym(j)%t(:)
          symstr%sym(j-1)%r(:,:) = symstr%sym(j)%r(:,:)
        enddo
        nsym = nsym - 1
        i = i - 1
      endif
      i = i + 1
    enddo
    symstr%nsym = nsym
  endif  ! remove non-simmorphic

  if(intro_time_rev) then! introduce time reversal
    do i = symstr%nsym, 1, -1
      symstr%sym(2*i)%r(:,:) = symstr%sym(i)%r(:,:)
      symstr%sym(2*i)%t(:) = symstr%sym(i)%t(:)
      symstr%sym(2*i)%tr = -1
      symstr%sym(2*i-1)%r(:,:) = symstr%sym(i)%r(:,:)
      symstr%sym(2*i-1)%t(:) = symstr%sym(i)%t(:)
      symstr%sym(2*i-1)%tr = +1
    enddo
    symstr%nsym = symstr%nsym * 2
  endif
  
  do i = 1, symstr%nsym
    ! the inverse of { R , t} is { R^-1 , - R^-1 t }
    call mtriinv(symstr%isym(i)%r,symstr%sym(i)%r)
  ! BUG: the next should be negative but for a strange reason does not work???
  ! probably it is due to the fact that the sign of the FFT is reversed
  ! symstr%isym(i)%t = - matmul(symstr%isym(i)%r,symstr%sym(i)%t)
    symstr%isym(i)%t = matmul(symstr%isym(i)%r,symstr%sym(i)%t)
    symstr%isym(i)%tr = symstr%sym(i)%tr
    ! the inverse transpose
    symstr%itsym(i)%r = transpose(symstr%isym(i)%r)
    symstr%itsym(i)%t(:) = symstr%isym(i)%t(:)
    symstr%itsym(i)%tr = symstr%isym(i)%tr
  enddo
!!  if(iverbose > 2) then
!!    call print_symmetry_structure(symstr)
!!  endif
  if(iverbose > 2) call uverbose('e','setup_symmetry')
end
!!***

!{\src2tex{textfont=tt}}
!!****p* setup_symmetry_structure/print_symmetry_structure
!! NAME
!! Print symmetry structure
!!
!! FUNCTION
!! Print symmetry structure
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! setup_symmetry_structure
!!
!! SOURCE
subroutine print_symmetry_structure(symstr)
  use ggwdef
  use dpdef
  use verbose
  implicit none
  type(symmetry) symstr
  integer i, j, isym, isymin, isymend
  if(iverbose > 2) call uverbose('b','print_symmetry')
  write(uout,*) 'symmetries of the system:'
  write(uout,'("  number of symmetry operations ",i2)') symstr%nsym
  write(uout,*) ' symmetry operations (R t)'
  do isymin=1,symstr%nsym,4
    isymend=isymin+3
    if(isymend>symstr%nsym) isymend=symstr%nsym
    do i=1,3
      write(uout,'(4(3i3,f8.3,3x))') &
&      ((symstr%sym(isym)%r(i,j),j=1,3),symstr%sym(isym)%t(i),isym=isymin,isymend)
    enddo
    write(uout,*)
  enddo
  write(uout,*) ' inverse symmetry operations (R^-1 , -R^-1 t)'
  do isymin=1,symstr%nsym,4
    isymend=isymin+3
    if(isymend>symstr%nsym) isymend=symstr%nsym
    do i=1,3
      write(uout,'(4(3i3,f8.3,3x))') &
&      ((symstr%isym(isym)%r(i,j),j=1,3),symstr%isym(isym)%t(i),isym=isymin,isymend)
    enddo
    write(uout,*)
  enddo
  write(uout,*) ' inverse transpose symmetry operations (R^-1^t , -R^-1 t)'
  do isymin=1,symstr%nsym,4
    isymend=isymin+3
    if(isymend>symstr%nsym) isymend=symstr%nsym
    do i=1,3
      write(uout,'(4(3i3,f8.3,3x))') &
&      ((symstr%itsym(isym)%r(i,j),j=1,3),symstr%itsym(isym)%t(i),isym=isymin,isymend)
    enddo
    write(uout,*)
  enddo
  if(iverbose > 2) call uverbose('e','print_symmetry')
end
!!***


!{\src2tex{textfont=tt}}
!!****f* mtriinv/mtridet
!! NAME
!! mtridet
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! mtriinv
!!
!! SOURCE

integer function mtridet(a)
  implicit none
  integer a(3,3)

  mtridet =   a(1,1) * a(2,2) * a(3,3) &
  &         - a(1,1) * a(2,3) * a(3,2) &
  &         - a(1,2) * a(2,1) * a(3,3) &
  &         + a(1,2) * a(2,3) * a(3,1) &
  &         + a(1,3) * a(2,1) * a(3,2) &
  &         - a(1,3) * a(2,2) * a(3,1)
  return
end
!!***

!{\src2tex{textfont=tt}}
!!****p* setup_symmetry_structure/mtriinv
!! NAME
!! mtriinv
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! setup_symmetry_structure
!!
!! CHILDREN
!! mtridet
!!
!! SOURCE
subroutine mtriinv(a,b)
  implicit none

  integer a(0:2,0:2), b(0:2,0:2), c(0:2,0:2)
  integer det
  integer, external :: mtridet
  det = mtridet(b)
  a(0,0) = (  b(1,1) * b(2,2) - b(1,2) * b(2,1)) / det
  a(1,0) = (- b(1,0) * b(2,2) + b(1,2) * b(2,0)) / det 
  a(2,0) = (  b(1,0) * b(2,1) - b(1,1) * b(2,0)) / det
  a(0,1) = (- b(0,1) * b(2,2) + b(0,2) * b(2,1)) / det
  a(1,1) = (  b(0,0) * b(2,2) - b(0,2) * b(2,0)) / det 
  a(2,1) = (- b(0,0) * b(2,1) + b(0,1) * b(2,0)) / det
  a(0,2) = (  b(0,1) * b(1,2) - b(0,2) * b(1,1)) / det
  a(1,2) = (- b(0,0) * b(1,2) + b(0,2) * b(1,0)) / det 
  a(2,2) = (  b(0,0) * b(1,1) - b(0,1) * b(1,0)) / det
  
  c = matmul(a,b)
  if(c(0,0)/=1 .or. c(1,1)/=1 .or. c(2,2)/=1) stop 'wrong inverse'
  if(c(0,1)/=0 .or. c(0,2)/=0 .or. c(1,2)/=0) stop 'wrong inverse'
  if(c(1,0)/=0 .or. c(2,0)/=0 .or. c(2,1)/=0) stop 'wrong inverse'
end
!!***
