!{\src2tex{textfont=tt}}
!!****p* leggi_wfg/test_orthonorm
!! NAME
!! test_orthonorm
!!
!! FUNCTION
!! test on the normalization of the wavefunctions
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! leggi_wfg, leggi_wfg_qplda
!!
!! SOURCE
subroutine test_orthonorm(iparam,elecstr)
use dpdef
use ggwdef
use verbose
implicit none
type(dpparam) iparam
type(electronic_structure) elecstr
real cinf, csup
real (kind(0.d0)) :: ddum, einf,esup
integer isppol, ik, ib, ibp, ir, ir1, ir2
complex, allocatable :: wfg(:)
#ifdef MPI
include 'mpif.h'
integer ierr,rang,nprocs
call mpi_comm_size(mpi_comm_world,nprocs,ierr)
call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif



if (iverbose > 2)call uverbose('b','test_orthonorm')

 allocate(wfg(iparam%npwvec))
 write(uout,*) 'test on the normalization of the wavefunctions'
 einf = 1.0e24; esup = 0.0
 cinf = 1.0e24; csup = 0.0
 do isppol = 1, iparam%nsppol
  do ik = 1, iparam%nkibz
!  test on the normalization of wavefunctions
   do ib = iparam%nbndmin, iparam%nbndmax
!   here it can occur a
!   forrtl: error (74): floating underflow
!   it means that the imag part of the dot_product is really small
    if (.not. iparam%wf_distrib) then
     wfg(1:iparam%npwvec) = elecstr%wfg(1:iparam%npwvec,ib,ik,isppol)
     ddum = abs(dot_product(wfg(:),wfg(:)))
    else
     ir=elecstr%wf_count(ik,ib,isppol)
     if(ir/=0) then
       wfg(1:iparam%npwvec) = elecstr%wfg2(1:iparam%npwvec,ir)
       ddum = abs(dot_product(wfg(:),wfg(:)))
     else
       cycle
     endif
    endif
    if(ddum<einf) einf = ddum
    if(ddum>esup) esup = ddum
   enddo
! test on the orthogonalization of wavefunctions
! only serial
#if !defined MPI
   do ib = iparam%nbndmin, iparam%nbndmax
    do ibp = ib+1, iparam%nbndmax
     if (.not. iparam%wf_distrib) then
      ddum = abs(dot_product(elecstr%wfg(:,ib,ik,isppol),elecstr%wfg(:,ibp,ik,isppol)))
     else
      ir1=elecstr%wf_count(ik,ib,isppol)
      ir2=elecstr%wf_count(ik,ibp,isppol)
      ddum = abs(dot_product(elecstr%wfg2(:,ir1),elecstr%wfg2(:,ir2)))
     endif
     if(ddum<cinf) cinf = ddum
     if(ddum>csup) csup = ddum
    enddo
   enddo
#endif
  enddo
 enddo !isppol

#ifdef MPI
 call mpi_barrier(mpi_comm_world,ierr)
 call mpi_reduce(einf,ddum,1,mpi_double_precision,mpi_min,0,mpi_comm_world,ierr)
 einf=ddum
 call mpi_bcast(einf,1,mpi_double_precision,0,mpi_comm_world,ierr)
 call mpi_reduce(esup,ddum,1,mpi_double_precision,mpi_max,0,mpi_comm_world,ierr)
 esup=ddum
 call mpi_bcast(esup,1,mpi_double_precision,0,mpi_comm_world,ierr)
#endif

 write(uout,5693) einf, esup
 5693  format(' min sum_G |a(n,k,G)| = ',f9.6/ &
     &       ' max sum_G |a(n,k,G)| = ',f9.6)
#if !defined MPI
 write(uout,*) 'test on the orthogonalization of the wavefunctions'
 write(uout,5694) cinf, csup
 5694  format(' min sum_G a(n,k,G)* a(n",k,G) = ',f9.6/ &
     &       ' max sum_G a(n,k,G)* a(n",k,G) = ',f9.6)
#endif

 deallocate(wfg)
 if (iverbose > 2)call uverbose('e','test_orthonorm')
end subroutine test_orthonorm
!!***
