!{\src2tex{textfont=tt}}
!!****p* read_structure/testvar
!! NAME
!! testvar
!!
!! FUNCTION
!! to test if the input parameters are coherently chosen 
!! fiele.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! NOTE
!! whenever is possible default values are used. when a parameter is badly 
!! chosen, either the code tries to correct it, according to the Kohn-Sham
!! structure file, or the run stops with some error message.
!!
!! PARENTS
!! read_structure
!!
!! CHILDREN
!! mesg
!!
!! SOURCE

subroutine testvar(iparam,nkr,nbr,nsppol,ngr,nelr,nqr)
use dpdef
use ggwdef
use verbose

implicit none
type(dpparam) iparam
integer ngr, nbr, nkr, nqr, nelr, nsppol
integer ish,idum

if(iverbose > 2)call uverbose('b','testvar')


!now test input parameters from input and kss file
!and assume some defults
if(iparam%nkibz == -1) iparam%nkibz = nkr
if(nkr < iparam%nkibz) then
  call mesg('over','k-points',nkr,iparam%nkibz)
  iparam%nkibz = nkr
else if(nkr > iparam%nkibz) then
  call mesg('unmatch','k-points',nkr,iparam%nkibz)
endif

if(iparam%nsppol == -1) iparam%nsppol = nsppol
if(nsppol < iparam%nsppol) then
  call mesg('over','spins',nsppol,iparam%nsppol)
  iparam%nsppol = nsppol
else if(nsppol > iparam%nsppol) then
  call mesg('unmatch','spins',nsppol,iparam%nsppol)
endif

!exciton
if(iparam%exc) then
 if(iparam%nqibz == -1) iparam%nqibz = nqr
 if(nqr < iparam%nqibz) then
   call mesg('over','q-points',nqr,iparam%nqibz)
   iparam%nqibz = nqr
 else if(nqr > iparam%nqibz) then
   call mesg('unmatch','q-points',nqr,iparam%nqibz)
 endif
endif
!end exciton


if (iparam%npwmat < 0 .and. iparam%matsh > 0) then
   if (iparam%matsh > iparam%nsh) then
     call mesg('over','shells',iparam%nsh,iparam%matsh)
     iparam%matsh = iparam%nsh
   endif 
   iparam%npwmat=iparam%shlim(iparam%matsh)
endif
if (iparam%npwwfn < 0 .and. iparam%wfnsh > 0) then
   if (iparam%wfnsh > iparam%nsh) then
     call mesg('over','shells',iparam%nsh,iparam%wfnsh)
     iparam%wfnsh = iparam%nsh
   endif 
   iparam%npwwfn=iparam%shlim(iparam%wfnsh)
endif

! if(iparam%npwwfn == -1) iparam%npwwfn = ngr
if (iparam%npwwfn < 0 .and. iparam%wfnsh < 0)  then
  iparam%npwwfn = ngr
  iparam%wfnsh = iparam%nsh
endif
if (iparam%npwmat < 0 .and. iparam%matsh < 0)  then 
  iparam%npwmat = ngr
  iparam%matsh = iparam%nsh
endif
! test if shells are closed
do ish=1,iparam%nsh
 if (iparam%npwmat > iparam%shlim(ish)) cycle
 if (iparam%npwmat == iparam%shlim(ish)) then
   exit
 else  ! that means npwmat is in between two closed shells
   call mesg('shell','npwmat',iparam%npwmat,iparam%shlim(ish-1))
   iparam%npwmat=iparam%shlim(ish-1)
   iparam%matsh=ish-1
   exit
 endif
enddo
do ish=1,iparam%nsh
 if (iparam%npwwfn > iparam%shlim(ish)) cycle
 if (iparam%npwwfn == iparam%shlim(ish)) then
   exit
 else  ! that means npwwfn is in between two closed shells
   call mesg('shell','npwwfn',iparam%npwwfn,iparam%shlim(ish-1))
   iparam%npwwfn=iparam%shlim(ish-1)
   iparam%wfnsh=ish-1
   exit
 endif
enddo

!calculate NPWVEC as the biggest between NPWMAT and NPWWFN
if(iparam%npwwfn > iparam%npwmat) then
  iparam%npwvec = iparam%npwwfn
  if(ngr < iparam%npwvec) then
   call mesg('over','plane-waves(npwwfn)',ngr,iparam%npwwfn)
   iparam%npwvec = ngr
   iparam%npwwfn = ngr
  endif
else
  iparam%npwvec = iparam%npwmat
  if(ngr < iparam%npwvec) then
   call mesg('over','plane-waves(npwmat)',ngr,iparam%npwmat)
   iparam%npwvec = ngr
   iparam%npwmat = ngr
  endif
endif
if(iparam%npwmat > iparam%ngw) then
  iparam%ngw = iparam%npwmat
else
  iparam%npwmat = iparam%ngw
endif

if(nbr < iparam%nbnds) then
  call mesg('over','nbands',nbr,iparam%nbnds)
  iparam%nbnds = nbr
endif

if(iparam%nop == -1) iparam%nop = iparam%nopr
if(iparam%nopr < iparam%nop) then
  call mesg('over','symmetries',iparam%nopr,iparam%nop)
  iparam%nop = iparam%nopr
else if(iparam%nopr > iparam%nop) then
  call mesg('unmatch','symmetries',iparam%nopr,iparam%nop)
endif

! the grid of q points for screening (for exc)
! is always Gamma centered
  iparam%nopq  = iparam%nopr
  iparam%ninvq = 2

! these are used only to verify that qpol is allowed (see routine qpol_and_q)
if (iparam%doublegrid) then
  ! in this case qpol belongs to a shifted qpoint grid
  iparam%nopqpol  = 1
  iparam%ninvqpol = 1
else
  ! in the normal case the grid of qpoints is always Gamma centered
  iparam%nopqpol  = iparam%nopr
  iparam%ninvqpol = 2
endif

if(iparam%nel == -1) iparam%nel = nelr
if(iparam%nel == -1) then
 call mesg('error','you must specify nel (maybe it is an old QPLDA format?',idum,idum)
endif 
if(iparam%nel /= nelr) then
  call mesg('unmatch','electrons',nelr,iparam%nel)
endif 

if(iverbose > 2)call uverbose('e','testvar')


end
!!***

