!{\src2tex{textfont=tt}}
!!****p* main/wreps
!! NAME
!! wreps
!!
!! FUNCTION
!! Prepare to write all the spectra files of the dp calculation, doing the averages
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! CHILDREN
!! vlngth, calc_eps0, wrfiles
!!
!! SOURCE

  subroutine wreps(iparam,latt,res,files)

  use ggwdef
  use dpdef
  use verbose
   
  implicit none
   
  type(dpparam) iparam 
  type(lattstruct) latt
  type(response) res
  type (dpfiles) files
   
  real domega
  real domev
  real dsffact
  integer io, iq, nnq
  integer isppol
  real, external :: vlngth
  logical :: lf,spin,dyson
#ifdef MPI
  include 'mpif.h'
  integer ierr,rang,nprocs
  call mpi_comm_size(mpi_comm_world,nprocs,ierr)
  call mpi_comm_rank(mpi_comm_world,rang,ierr)
  if (rang==0) then
#endif
  
  if(iverbose>2)call uverbose('b','wreps')

  dsffact = vlngth(iparam%qpg,latt%b1,latt%b2,latt%b3)**2 / (4 * PI**2 * iparam%nel/latt%ucvol)
    
  domega = real(iparam%omega(iparam%nomega) - iparam%omega(1)) / (iparam%nomega - 1)
  domev = domega * HARTREE
   

  if(iparam%nqpol == 1) then
    nnq = 1
  else
    nnq = 8        
    if (iparam%NOFXC) nnq = 11
!   calculating reciprocal lattice and cartesian averages
    do io = 1, iparam%nomega
      res%epsmlf(io,7)  = (res%epsmlf(io,1) + res%epsmlf(io,2) + res%epsmlf(io,3)) / 3.0
      res%epsmnlf(io,7) = (res%epsmnlf(io,1)+ res%epsmnlf(io,2)+ res%epsmnlf(io,3)) / 3.0
      res%epsmlf(io,8)  = (res%epsmlf(io,4) + res%epsmlf(io,5) + res%epsmlf(io,6)) / 3.0
      res%epsmnlf(io,8) = (res%epsmnlf(io,4)+ res%epsmnlf(io,5)+ res%epsmnlf(io,6)) / 3.0
      if ( iparam%nsppol == 2) then
        res%epsmnlfspin(io,7,:)  = (res%epsmnlfspin(io,1,:) + res%epsmnlfspin(io,2,:) + res%epsmnlfspin(io,3,:)) / 3.0
        res%epsmnlfspin(io,8,:)  = (res%epsmnlfspin(io,4,:) + res%epsmnlfspin(io,5,:) + res%epsmnlfspin(io,6,:)) / 3.0
        if (.not.iparam%NOFXC) then
         res%epsmlfspin(io,7,:)  = (res%epsmlfspin(io,1,:) + res%epsmlfspin(io,2,:) + res%epsmlfspin(io,3,:)) / 3.0
         res%epsmlfspin(io,8,:)  = (res%epsmlfspin(io,4,:) + res%epsmlfspin(io,5,:) + res%epsmlfspin(io,6,:)) / 3.0
        endif
      endif
    enddo
  endif

! evaluation of the dielctric constant (only if q=0 and at w=0)
  if(iparam%KP_THEORY .and. iparam%ig == 1 .and. iparam%omegai < 0.1/HARTREE .and. .not.iparam%antieps) then
    call calc_eps0(iparam,res)
  endif

! complex conj. for eps*(-q,-w)
  if (iparam%antieps) then
    res%epsmlf(:,:) = conjg(res%epsmlf(:,:))
    res%epsmnlf(:,:)= conjg(res%epsmnlf(:,:))
    if ( iparam%nsppol == 2) then
      res%epsmnlfspin(:,:,:) = conjg( res%epsmnlfspin(:,:,:) )
      if (.not.iparam%NOFXC) res%epsmlfspin(:,:,:) = conjg( res%epsmlfspin(:,:,:) )
    endif
  endif

! write the spectra output files 
  spin=.false.
  dyson=.false.
  isppol=0
  if (iparam%nqpol==1) then
   iq=1
  else
   iq=8
  endif
! write the local fields file 
  lf=.true.
  call wrfiles(iparam,lf,latt,dsffact,latt%ucvol,nnq,iq,spin,isppol,dyson,res,files%rl,files%root) 
! write the non local fields file 
  lf=.false.
  call wrfiles(iparam,lf,latt,dsffact,latt%ucvol,nnq,iq,spin,isppol,dyson,res,files%rl,files%root) 
  if (iparam%nsppol==2) then
   spin=.true.
!  write the non local fields file for spin up 
   lf=.false.
   isppol=1
   call wrfiles(iparam,lf,latt,dsffact,latt%ucvol,nnq,iq,spin,isppol,dyson,res,files%rl,files%root) 
!  write the non local fields file for spin down
   isppol=2
   call wrfiles(iparam,lf,latt,dsffact,latt%ucvol,nnq,iq,spin,isppol,dyson,res,files%rl,files%root) 
!  also obtain the eps^-1_up and eps^-1_down (possible for eps^-1 = 1 + v chi)
   if (.not.iparam%NOFXC) then
    lf=.true.
    dyson=.true.
    isppol=1
    call wrfiles(iparam,lf,latt,dsffact,latt%ucvol,nnq,iq,spin,isppol,dyson,res,files%rl,files%root) 
    isppol=2
    call wrfiles(iparam,lf,latt,dsffact,latt%ucvol,nnq,iq,spin,isppol,dyson,res,files%rl,files%root) 
   endif
  endif

  if(iparam%nsppol==2) deallocate(res%epsmnlfspin,res%epsmlfspin)

  if(iverbose>2)call uverbose('e','wreps')
#ifdef MPI
  endif
#endif

  end
!!***

