!{\src2tex{textfont=tt}}
!!****p* wreps/wrfiles
!! NAME
!! wrfiles
!!
!! FUNCTION
!! Write all the spectra files of the dp calculation. 
!! In particular :: outlf.mdf and outnlf.mdf. 
!! 6 polarizations will also generate ouclf.mdf and oucnlf.mdf.
!! Spin polarized case will also generate outlnf_up.mdf and outlnf_down.mdf
!! 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! wreps
!! 
!! CHILDREN
!! wrtitle, wrq
!! 
!! SOURCE

 subroutine wrfiles(iparam,lf,latt,dsffact,ucvol,nq,iq,spin,isppol,dyson,res,rl,root)
 use dpdef
 use ggwdef
 use verbose
 implicit none
 type(dpparam)iparam
 type(lattstruct) latt
 type(response) res
 real :: dsffact, omegaev, ucvol
 integer :: io,iq,isppol,nq,iiq, rl
 character (len=50) :: lftitle,mdf,spinname,ave,lfname,outfile,ext
 character (len=rl) :: root
 logical :: lf,spin,dyson

 if(iverbose>2)call uverbose('b','wrfiles')


 if(lf) then 
  lftitle = '# with local fields'
  lfname = 'lf'
 else
  if (dyson) then
   lftitle = '# without local fields and without kernel'
  else
   lftitle = '# without local fields'
  endif
  lfname = 'nlf'
 endif
 if (.not.spin) then
  mdf = '# macroscopic dielectric function' 
 else
  if(.not.dyson)then
   if (isppol==1) then 
    mdf = '# macroscopic dielectric function spin UP'  
   else
    mdf = '# macroscopic dielectric function spin DOWN'  
   endif
  else
   if (isppol==1) then 
    mdf = '# macroscopic inverse dielectric function spin UP'  
   else
    mdf = '# macroscopic inverse dielectric function spin DOWN'  
   endif
  endif
 endif
 if (iq==1) then
  ave='t'
 else
  ave='c'
 endif
 if (isppol/=0) then
  if (isppol==1) then
   spinname='_up' 
  else
   spinname='_down' 
  endif
 else
  spinname=''
 endif
 ext='.mdf'
 if(lf .and. spin .and. dyson) ext='.eel'
 outfile=trim(root)//'ou'//trim(ave)//trim(lfname)//trim(spinname)//trim(ext)
 open(22,file=trim(outfile),form='formatted')
 write(22,'(a)')trim(mdf)
 write(22,'(a)')trim(lftitle)
 call wrtitle(22,iparam,latt%tpula,dsffact,ucvol)
 call wrq(22,iparam,latt%tpula,iq)
 write(22,'("# omega       eps_1       eps_2")')
 do io = 1, iparam%nomega
   omegaev = real(iparam%omega(io))*HARTREE
   if (lf) then 
    if(.not.spin) then
     write(22,'(f7.3,2e12.4)') omegaev, res%epsmlf(io,iq)
    else 
     if(dyson) then
      write(22,'(f7.3,2e12.4)') omegaev,  res%epsmlfspin(io,iq,isppol)
     endif
    endif
   else
    if(.not.spin) then
     write(22,'(f7.3,2e12.4)') omegaev, res%epsmnlf(io,iq)
    else
     write(22,'(f7.3,2e12.4)') omegaev,  res%epsmnlfspin(io,iq,isppol)
    endif
   endif
 enddo
 close(22)
 if (nq>1) then
  ave='t'
  outfile=trim(root)//'ou'//trim(ave)//trim(lfname)//trim(spinname)//trim(ext)
  open(22,file=trim(outfile),form='formatted')
  write(22,'(a)')trim(mdf)
  write(22,'(a)')trim(lftitle)
  call wrtitle(22,iparam,latt%tpula,dsffact,ucvol)
  write(22,'("# 9 polarizations + 2 averages:")')
  call wrq(22,iparam,latt%tpula,7)
  call wrq(22,iparam,latt%tpula,8)
  do iiq = 1, iparam%nqpol
    call wrq(22,iparam,latt%tpula,iiq)
  enddo
  write(22,'("# omega       eps_1       eps_2  for r c b1 b2 b3 xx yy zz xy xz yz")')
  do io = 1, iparam%nomega
   omegaev = real(iparam%omega(io))*HARTREE
   if (lf) then
    if (.not.spin) then
     write(22,'(f7.3,22e12.4)') omegaev, res%epsmlf(io,7),res%epsmlf(io,8), &
                                res%epsmlf(io,1), res%epsmlf(io,2), res%epsmlf(io,3), &
                                res%epsmlf(io,4), res%epsmlf(io,5), res%epsmlf(io,6), &
                                res%epsmlf(io,9), res%epsmlf(io,10), res%epsmlf(io,11)
    else
     if(dyson) then
      write(22,'(f7.3,22e12.4)') omegaev, res%epsmlfspin(io,7,isppol),res%epsmlfspin(io,8,isppol), &
                                res%epsmlfspin(io,1,isppol), res%epsmlfspin(io,2,isppol), res%epsmlfspin(io,3,isppol), &
                                res%epsmlfspin(io,4,isppol), res%epsmlfspin(io,5,isppol), res%epsmlfspin(io,6,isppol), &
                                res%epsmlfspin(io,9,isppol), res%epsmlfspin(io,10,isppol), res%epsmlfspin(io,11,isppol)

     endif
    endif
   else
    if(.not.spin) then
     write(22,'(f7.3,22e12.4)') omegaev, res%epsmnlf(io,7),res%epsmnlf(io,8), &
                                res%epsmnlf(io,1), res%epsmnlf(io,2), res%epsmnlf(io,3), &
                                res%epsmnlf(io,4), res%epsmnlf(io,5), res%epsmnlf(io,6), &
                                res%epsmnlf(io,9), res%epsmnlf(io,10), res%epsmnlf(io,11)
    else
     write(22,'(f7.3,22e12.4)') omegaev, res%epsmnlfspin(io,7,isppol),res%epsmnlfspin(io,8,isppol), &
                                res%epsmnlfspin(io,1,isppol), res%epsmnlfspin(io,2,isppol), res%epsmnlfspin(io,3,isppol), &
                                res%epsmnlfspin(io,4,isppol), res%epsmnlfspin(io,5,isppol), res%epsmnlfspin(io,6,isppol), &
                                res%epsmnlfspin(io,9,isppol), res%epsmnlfspin(io,10,isppol), res%epsmnlfspin(io,11,isppol)
    endif
   endif
  enddo
  close(22)
 endif
  
 if(iverbose>2)call uverbose('e','wrfiles')

 end subroutine wrfiles
!!***


!{\src2tex{textfont=tt}}
!!****p* wrfiles/wrtitle
!! NAME
!! wrtitle
!!
!! FUNCTION
!! Write title for the files of the dp calculation
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! wrfiles, wreps_exciton
!!
!! SOURCE

  subroutine wrtitle(unita,iparam,tpula,dsffact,ucvol)

  use ggwdef
  use dpdef
  implicit none
  integer unita
  type(dpparam) iparam
  real tpula
  real dsffact,ucvol
  
  if(.not. iparam%exciton) write(unita,'("# ",a12,2x,a20)') iparam%dieltype,iparam%approxtype
  if(iparam%SO) & 
     write(unita,'("# soenergy = ",f5.2," [eV] - valence/conduction stretching",f5.2," / ", f5.2)') &
     iparam%soenergy*HARTREE, iparam%stretchvalence, iparam%stretchconduction
  if(iparam%FXCALPHA) write(unita,'("# alpha = ",f5.2)') iparam%alphadiv
  if(iparam%FXCALPHABETA) write(unita,'("# alpha = ",f5.2," beta = ",f6.2)') iparam%alphadiv, iparam%beta
  
  
  write(unita,300) iparam%npwmat,iparam%npwwfn,iparam%nbnds,iparam%lomo,iparam%nkibz,iparam%nkbz
300 format('# npwmat =',i5,'  npwwfn =',i5,'  nbands =',i5, &
        '  lomo =',i5,'  nkibz =',i5,'  nkbz =',i5)
  write(unita,'("# energy range ",2f6.2," [eV]")')&
   iparam%omegai*HARTREE, iparam%omegae*HARTREE
  write(unita,1000) iparam%broad*HARTREE
1000  format('# lorentzian broadening = ',f6.4,' [eV]')

  write(unita,'("# dsf factor ",f12.6," [a.u.] ")') dsffact
  write(unita,'("# unit cell volume  ",f12.6," [a.u.] ")') ucvol

  write(unita,700) iparam%g(:)
700   format('# G = (',i3,',',i3,',',i3,') [r.l.u.]')
  write(unita,800) iparam%gcc(:)
800   format('# G = (',f9.6,',',f9.6,',',f9.6,') [c.c.]')
  write(unita,900) iparam%gcc2pioa(:), tpula
900   format('# G = (',f9.6,',',f9.6,',',f9.6,') ', &
             '[c.c. units of 2pi/a = ',f9.6,' a.u.]')

  end
!!***

!{\src2tex{textfont=tt}}
!!****p* wrfiles/wrq
!! NAME
!! wrq
!!
!! FUNCTION
!! Write q for the files of the dp calculation
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! wrfiles, wreps_exciton
!!
!! SOURCE

  subroutine wrq(unita,iparam,tpula,iq)

  use ggwdef
  use dpdef
  implicit none
  integer unita
  type(dpparam) iparam
  real tpula
  integer iq

  if(iq <= 6) then
    write(unita,400) iparam%qpol(:,iq)
400   format('# q = (',f9.6,',',f9.6,',',f9.6,') [r.l.u.]')
    write(unita,500) iparam%qpolcc(:,iq)
500   format('# q = (',f9.6,',',f9.6,',',f9.6,') [c.c.]')
    write(unita,600) iparam%qpolcc2pioa(:,iq), tpula
600   format('# q = (',f9.6,',',f9.6,',',f9.6,') ', &
         '[c.c. units of 2pi/a = ',f9.6,' a.u.]')
  else if(iq == 7) then
    write(unita,410)
410     format('# q -> 0 ', &
      'average over the three reciprocal lattice directions')
  else if(iq == 8) then
    write(unita,420)
420     format('# q -> 0 ', &
      'average over the three cartesian directions')
  endif

  end
!!***

