!{\src2tex{textfont=tt}}
!!****p* main/prep_rhotw
!! NAME
!! prep_rhotw
!!
!! FUNCTION
!! This works only for the kernel MT case. it creates the rhotw object, for any G and transition 
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile
!!  A list of contributors is available, see CONTRIBUTORS
!!  This file is distributed under the terms of the
!!  Scientific Software Open Source Academic For Free License, see COPYING
!!  or http://lab-neel.grenoble.cnrs.fr/etsf/dp/home/scientificsoftware.html
!!
!! PARENTS
!! main
!!
!! CHILDREN
!! split_trans, get_wfr, rho_tw_g
!!
!! SOURCE
 subroutine prep_rhotw(iparam,bzstruct,tablekp,fftmesh,elecstr,trans,tr)

  use ggwdef
  use dpdef
  use verbose
  use interfaces
  implicit none
  type(dpparam) iparam
  type(transition) trans(iparam%nt)
  type(transgen) tr
  type(bzstructure) bzstruct
  type(polarizationq) tablekp
  type(electronic_structure) elecstr
  type(mesh) fftmesh

  integer ::  nrb
  integer :: ikbz, ikibz, iinv, iinv2, ibv, ibc,isppol
  integer :: it,  ig01a, ig02a, ig03a, ikp, ikpibz

  logical, external :: findkp
  complex, external :: pdtqrc

  complex, allocatable :: rhotwg(:)
  real q(3)

  complex :: wfr2(fftmesh%nr), wfr1(fftmesh%nr)

#ifdef MPI
  include 'mpif.h'
  integer istat 
  integer ierr,rang,nprocs,startcycle,stopcycle
  call mpi_comm_size(mpi_comm_world,nprocs,ierr)
  call mpi_comm_rank(mpi_comm_world,rang,ierr)
#endif

  if(iverbose > 2) call uverbose('b','prep_rhotw')
  if(iverbose > 0) then
     headline="Calculating rho_t(G)"
     call print_headline()
     write(uout,*)'    -----> q/=0 term'
  endif

  nrb = fftmesh%nrr(1) * fftmesh%nrr(2) * fftmesh%nrr(3)

  allocate(rhotwg(iparam%npwmat))

  q(:) = iparam%qpol(:,1)

#ifdef MPI
  call split_trans(1,iparam%nt,startcycle,stopcycle)
  allocate(tr%rhotwgen(startcycle:stopcycle,iparam%npwmat),stat=istat)
  if (istat/=0) then 
    write(0,*) 'rhotwgen not allocated' 
    stop
  endif
   if(iverbose > 2) then
     call memstat('(xrhotw) tr%rhotwgen allocated =')
   endif
   do it=startcycle,stopcycle
#else 
  allocate(tr%rhotwgen(iparam%nt,iparam%npwmat))
  if(iverbose > 2) then
   call memstat('(xrhotw) tr%rhotwgen allocated =')
  endif
  do it=1,iparam%nt
#endif
   if(iverbose > 2) write(uout,*)'it=',it

   ikbz=trans(it)%k
   ikibz=tablekp%ikibz(ikbz)
   ikpibz=tablekp%ikpibz(ikbz)
   ikp=tablekp%ikp(ikbz)
   iinv=tablekp%iinv(ikbz)
   iinv2=tablekp%iinv2(ikbz)
   ig01a=tablekp%ig01a(ikbz)
   ig02a=tablekp%ig02a(ikbz)
   ig03a=tablekp%ig03a(ikbz)
   ibv=trans(it)%v
   ibc=trans(it)%c
   isppol=trans(it)%s

    call get_wfr(iparam,elecstr,elecstr,fftmesh,ibv,ikpibz,ibc,ikibz,isppol,wfr1,wfr2)

   call rho_tw_g(iparam%npwmat,fftmesh%nr,nrb, &
               fftmesh%nrr(1),fftmesh%nrr(1),fftmesh%nrr(2),fftmesh%nrr(3), &
               fftmesh%igfft(:,ig01a,ig02a,ig03a), &
               wfr1,iinv,bzstruct%ktabr(:,ikp), &
               wfr2,iinv2,bzstruct%ktabr(:,ikbz),fftmesh%menoplan,rhotwg) 

   tr%rhotwgen(it,:)=rhotwg(:)
   if(iparam%KP_THEORY) tr%rhotwgen(it,1)=tr%rhotwxqpol(it,1)

  enddo

  deallocate(rhotwg)


!! test low memory
!  deallocate(elecstr%wfr,bzstruct%ktabr)
!! test low memory

  if (iverbose > 2) then
    call memstat('(xrhotw) tr%wfr deallocated =')
  endif
  
  if(iverbose > 0) then
   call print_footline()
  endif
  if(iverbose > 2) call uverbose('e','prep_rhotw')

  end 
!!***
