/*
   program dputil
   
   used to
   add broadening;
   and to 
   calculate energy loss function
   refraction index
   reflectivity
   dynamic structure factor
   from dielectric macroscopic function

   use:
   mdf2eel inputfile [outputfile]
   mdf2rfr inputfile [outputfile]
   mdf2rfl inputfile [outputfile]
   mdf2dsf inputfile [outputfile]
   broad   inputfile [outputfile]
   
   Valerio Olevano June 2002
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#define eel(mdf1,mdf2) ((mdf2) / ((mdf1)*(mdf1) + (mdf2)*(mdf2)))

float nrfr(float mdf1, float mdf2)
{
  float r;
  r = sqrt(mdf1*mdf1 + mdf2*mdf2);
  return( sqrt((r+mdf1)/2.0));
}

float krfr(float mdf1, float mdf2)
{
  float r, sign;
  r = sqrt(mdf1*mdf1 + mdf2*mdf2);
  sign = (mdf2 < 0.0) ? -1.0 : 1.0;
  return( sign * sqrt((r-mdf1)/2.0));
}
float rfl(float mdf1,float mdf2)
{
  float k2, nm12, np12;
  float n1, k1;
  n1 = nrfr(mdf1,mdf2);
  k1 = krfr(mdf1,mdf2);
  nm12 = (n1-1.0)*(n1-1.0);
  np12 = (n1+1.0)*(n1+1.0);
  k2 = k1*k1;
  return( (nm12+k2) / (np12+k2) );
}

enum ctype { EEL, RFR, RFL, DSF, BROAD, SIGMACS};

main(int argc, char *argv[])
{
  int n, ny, nz;
  float *x, *y, *z, *zt;
  int i, j, k;
  float mdf1, mdf2, dx;
  float etaa, etar, sigmaa, sigmar;
  float etarp, sigmarp;
  float  eta, eta2, norm, lor, sigma, gauss;
  char *infile, *outfile;
  char *changeext(), *addext();
  float dsffact;
  float ucvol, c;
  int type;
  char string[200];
 
  if(strstr(argv[0],"mdf2eel")!=NULL) type = EEL;
  if(strstr(argv[0],"mdf2rfr")!=NULL) type = RFR;
  if(strstr(argv[0],"mdf2rfl")!=NULL) type = RFL;
  if(strstr(argv[0],"mdf2dsf")!=NULL) type = DSF;
  if(strstr(argv[0],"mdf2sigma")!=NULL) type = SIGMACS;
  if(strstr(argv[0],"broad")!=NULL)   type = BROAD;


 
  if(argc<2) exit(2);
  infile = argv[1];
  readfile(infile,&n,&ny,&x,&y);

  /*
  printf("%f %f\n",x[0],y[0]);
  printf("%f %f\n",x[n-1],y[n*ny-1]);
  */

  switch(type) {
    case EEL:
      outfile = changeext(infile,"eel");
      modifyfirstcomment("# electron energy loss\n");
      modifylastcomment("# omega         eel\n");
      break;
    case RFR:
      outfile = changeext(infile,"rfr");
      modifyfirstcomment("# index of refraction function\n");
      modifylastcomment("# omega           n           k\n");
      break;
    case RFL:
      outfile = changeext(infile,"rfl");
      modifyfirstcomment("# reflectance and its logarithmic derivative\n");
      modifylastcomment("# omega         rfl       ldrfl\n");
      break;
    case DSF:
      outfile = changeext(infile,"dsf");
      modifyfirstcomment("# dynamic structure factor\n");
      modifylastcomment("# omega [eV] - (q+G)^2 / (4PI^2 e^2 n) * Im epsilon^-1 [1/eV]\n");
      break;
    case SIGMACS:
      outfile = changeext(infile,"sigma");
      modifyfirstcomment("# photoabsorption cross section\n");
      modifylastcomment("# omega [eV] - sigma \n");
      break;
    case BROAD:
      outfile = addext(infile,"brd");
      break;
    default:
      ;
  }
  if(argc>=3) outfile = argv[2];


  switch(type) {
    case SIGMACS:
      readucvol(&ucvol);
      nz = ny / 2;
      z = calloc(n*nz,sizeof(float));
      for(i=0;i<n;i++) {
        for(j=0;j<nz;j++) {
          mdf1 = y[i*ny+j*2];
          mdf2 = y[i*ny+j*2+1];
          z[i*nz+j] = mdf2*x[i]*ucvol/137.036;
        }
      }
      break;
    case EEL:
      nz = ny / 2;
      z = calloc(n*nz,sizeof(float));
      for(i=0;i<n;i++) {
        for(j=0;j<nz;j++) {
          mdf1 = y[i*ny+j*2];
          mdf2 = y[i*ny+j*2+1];
          z[i*nz+j] = eel(mdf1,mdf2);
        }
      }
      break;
    case RFR:
      nz = ny;
      z = calloc(n*nz,sizeof(float));
      for(i=0;i<n;i++) {
        for(j=0;j<ny/2;j++) {
          mdf1 = y[i*ny+j*2];
          mdf2 = y[i*ny+j*2+1];
          z[i*nz+j*2]   = nrfr(mdf1,mdf2);
          z[i*nz+j*2+1] = krfr(mdf1,mdf2);
        }
      }
      break;
    case RFL:
      nz = ny;
      z = calloc(n*nz,sizeof(float));
      for(i=0;i<n;i++) {
        for(j=0;j<ny/2;j++) {
          mdf1 = y[i*ny+j*2];
          mdf2 = y[i*ny+j*2+1];
          z[i*nz+j*2]   = rfl(mdf1,mdf2);

          if(i==0) { 
            z[i*nz+j*2+1] = 0.0;
          } else {
            dx = x[i] - x[i-1];
            z[i*nz+j*2+1] = ( 1.0 - z[(i-1)*nz+j*2] / z[i*nz+j*2] ) / dx;
          }
        }
      }
      break;
    case DSF:
      readdsff(&dsffact);
      nz = ny / 2;
      z = calloc(n*nz,sizeof(float));
      for(i=0;i<n;i++) {
        for(j=0;j<nz;j++) {
          mdf1 = y[i*ny+j*2];
          mdf2 = y[i*ny+j*2+1];
          z[i*nz+j] = dsffact * eel(mdf1,mdf2) / 27.2116 ;
        }
      }
      break;
    case BROAD:
      printf("absolute lorentzian broadening [eV] ?\n");
      printf("eta = ");
      scanf("%f",&etaa);
      printf("relative lorentzian broadening (1. => 100%) ?\n");
      printf("etar = ");
      scanf("%f",&etar);
      printf("absolute gaussian broadening [eV] ?\n");
      printf("sigma = ");
      scanf("%f",&sigmaa);
      printf("relative gaussian broadening (1. => 100%) ?\n");
      printf("sigmar = ");
      scanf("%f",&sigmar);
      if(etaa != 0.0) {
        sprintf(string,"# Lorentzian absolute broadening %5.2f [eV]\n",etaa);
        addcomment(string);
      }
      if(etar != 0.0) {
        etarp = etar*100;
        sprintf(string,"# Lorentzian relative broadening %5.2f % \n",etarp);
        addcomment(string);
      }
      if(sigmaa != 0.0) {
        sprintf(string,"# Gaussian absolute broadening %5.2f [eV]\n",sigmaa);
        addcomment(string);
      }
      if(sigmar != 0.0) {
        sigmarp = sigmar*100;
        sprintf(string,"# Gaussian relative broadening %5.2f % \n",sigmarp);
        addcomment(string);
      }

      nz = ny;
      z = calloc(n*nz,sizeof(float));
      zt = calloc(nz,sizeof(float));
      /* first lorentzian */
      if(etaa != 0.0 || etar != 0.0) {
        for(i=0;i<n;i++) {
          eta = x[i] * etar + etaa;
          eta2 = eta * eta;
          if(eta != 0.0) {
            for(k=0;k<ny;k++) zt[k] = 0.0;
            norm = 0.0;
            for(j=0;j<n;j++) {
              lor = x[i]-x[j];
              lor = eta / (lor*lor + eta2);
              for(k=0;k<ny;k++) zt[k] += lor * y[j*ny+k];
              norm += lor;
            }
            for(k=0;k<ny;k++) z[i*ny+k] = zt[k] / norm;
          }else{
            for(k=0;k<ny;k++) z[i*ny+k] = y[i*ny+k];
          }
        }
        for(i=0;i<n;i++) {for(k=0;k<ny;k++) y[i*ny+k] = z[i*ny+k];}
      }
      /* now gaussian */
      for(i=0;i<n;i++) {
        sigma = sigmaa + x[i] * sigmar;
        if(sigma != 0.0) {
          for(k=0;k<ny;k++) zt[k] = 0.0;
          norm = 0.0;
          for(j=0;j<n;j++) {
            gauss = (x[i]-x[j]) / sigma;
            gauss = exp(- gauss * gauss / 2);
            for(k=0;k<ny;k++) zt[k] += gauss * y[j*ny+k];
            norm += gauss;
          }
          for(k=0;k<ny;k++) z[i*ny+k] = zt[k] / norm;
        }else{
          for(k=0;k<ny;k++) z[i*ny+k] = y[i*ny+k];
        }
      }
      break;
    default:
      ;
  }

  writefile(outfile,n,nz,x,z);
  exit(1);
}
